@@ -150,7 +150,7 @@ namespace rtengine {
 
 void CurveFactory::updatechroma (
 		const std::vector<double>& cccurvePoints,
-		LUTu & histogramC, LUTu & histogramCroppedC, LUTu & outBeforeCCurveHistogramC,//for chroma
+		LUTu & histogramC, LUTu & outBeforeCCurveHistogramC,//for chroma
 		int skip)
 {
 	LUTf dCcurve(65536,0);
@@ -184,7 +184,7 @@ void CurveFactory::curveLightBrightColor (
 		procparams::ColorAppearanceParams::eTCModeId curveMode2, const std::vector<double>& curvePoints2,
 		procparams::ColorAppearanceParams::eCTCModeId curveMode3, const std::vector<double>& curvePoints3,
 		LUTu & histogram, LUTu & histogramCropped, LUTu & outBeforeCCurveHistogram,//for Luminance  
-		LUTu & histogramC, LUTu & histogramCroppedC, LUTu & outBeforeCCurveHistogramC,//for chroma
+		LUTu & histogramC, LUTu & outBeforeCCurveHistogramC,//for chroma
 		ColorAppearance & customColCurve1,
 		ColorAppearance & customColCurve2,
 		ColorAppearance & customColCurve3,
@@ -288,11 +288,112 @@ void CurveFactory::curveLightBrightColor (
 
 }
 
+void CurveFactory::curveBW (
+		const std::vector<double>& curvePointsbw, const std::vector<double>& curvePointsbw2,
+		LUTu & histogrambw, LUTu & outBeforeCCurveHistogrambw,//for Luminance
+		ToneCurve & customToneCurvebw1, ToneCurve & customToneCurvebw2, int skip)
+{
+	LUTf dcurve(65536,0);
+	
+	float val;
+	for (int i=0; i<32768; i++) {
+			val = (double)i / 32767.0;
+			dcurve[i] = CLIPD(val);
+		}
+
+	outBeforeCCurveHistogrambw.clear();
+	bool histNeeded = false;
+	
+	DiagonalCurve* tcurve = NULL;
+	customToneCurvebw2.Reset();
+
+	if (!curvePointsbw2.empty() && curvePointsbw2[0]>DCT_Linear && curvePointsbw2[0]<DCT_Unchanged) {
+		tcurve = new DiagonalCurve (curvePointsbw2, CURVES_MIN_POLY_POINTS/skip);
+		if (outBeforeCCurveHistogrambw /*&& histogramCropped*/)
+				histNeeded = true;
+		
+	}
+	if (tcurve) {
+		if (!tcurve->isIdentity())
+			customToneCurvebw2.Set(tcurve);
+		delete tcurve;
+		tcurve = NULL;
+	}
+
+	customToneCurvebw1.Reset();
+
+	if (!curvePointsbw.empty() && curvePointsbw[0]>DCT_Linear && curvePointsbw[0]<DCT_Unchanged) {
+		tcurve = new DiagonalCurve (curvePointsbw, CURVES_MIN_POLY_POINTS/skip);
+		if (outBeforeCCurveHistogrambw /*&& histogramCropped*/)
+				histNeeded = true;
+		
+	}
+	if (tcurve) {
+		if (!tcurve->isIdentity())
+			customToneCurvebw1.Set(tcurve);
+		delete tcurve;
+		tcurve = NULL;
+	}
+	// create first curve if needed
+
+	for (int i=0; i<=32768; i++) {
+		float val;
+
+			if (histNeeded) {
+				float hval = dcurve[i];
+				int hi = (int)(255.0*CLIPD(hval));
+				outBeforeCCurveHistogrambw[hi] += histogrambw[i] ;
+			}
+	}
+	
+	if (tcurve) delete tcurve;
+
+}
+// add curve Lab : C=f(L)
+void CurveFactory::curveCL ( bool & clcutili,const std::vector<double>& clcurvePoints, LUTf & clCurve, LUTu & histogramcl, LUTu & outBeforeCLurveHistogram,int skip){
+		bool needed;
+		DiagonalCurve* dCurve = NULL;
+		LUTf dCcurve(65536,0);
+	
+		float val;
+		for (int i=0; i<50000; i++) {  //# 32768*1.414  approximation maxi for chroma
+				dCcurve[i] = (float)i / 49999.0;
+		}
+		
+		if (outBeforeCLurveHistogram)		
+			outBeforeCLurveHistogram.clear();
+		bool histNeededCL = false;
+
+		needed = false;
+		if (!clcurvePoints.empty() && clcurvePoints[0]!=0) {
+			dCurve = new DiagonalCurve (clcurvePoints, CURVES_MIN_POLY_POINTS/skip);
+			if (outBeforeCLurveHistogram)
+				histNeededCL = true;
+			
+			if (dCurve && !dCurve->isIdentity())
+				{needed = true;clcutili=true;}
+		}
+		for (int i=0; i<=50000; i++) {//32768*1.414  + ...
+			float val;
+			if (histNeededCL) {
+				float hval = dCcurve[i];
+				int hi = (int)(255.0*CLIPD(hval)); //
+				outBeforeCLurveHistogram[hi] += histogramcl[i] ;
+			}
+		}
+		
+		fillCurveArray(dCurve, clCurve, skip, needed);
+		if (dCurve) {
+			delete dCurve;
+			dCurve = NULL;
+		}
+}
+
 	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 	void CurveFactory::complexsgnCurve ( bool & autili,  bool & butili, bool & ccutili, bool & cclutili, double saturation, double rstprotection,
 										const std::vector<double>& acurvePoints, const std::vector<double>& bcurvePoints,const std::vector<double>& cccurvePoints,
 										const std::vector<double>& lccurvePoints, LUTf & aoutCurve, LUTf & boutCurve, LUTf & satCurve, LUTf & lhskCurve,
-										LUTu & histogramC, LUTu & histogramCroppedC, LUTu & outBeforeCCurveHistogram,//for chroma
+										LUTu & histogramC, LUTu & histogramLC, LUTu & outBeforeCCurveHistogram,LUTu & outBeforeLCurveHistogram, //for chroma
 										int skip) {
 		
 		
@@ -302,13 +403,17 @@ void CurveFactory::curveLightBrightColor (
 		DiagonalCurve* dCurve = NULL;
 		LUTf dCcurve(65536,0);
 	
-		float val;
 		for (int i=0; i<48000; i++) {  //# 32768*1.414  approximation maxi for chroma
 				dCcurve[i] = (float)i / 47999.0;
 		}
 		if (outBeforeCCurveHistogram)		
 			outBeforeCCurveHistogram.clear();
 		bool histNeededC = false;
+		
+		if (outBeforeLCurveHistogram)		
+			outBeforeLCurveHistogram.clear();
+		bool histNeededLC = false;
+		
 		//-----------------------------------------------------
 
 		needed = false;
@@ -372,16 +477,28 @@ void CurveFactory::curveLightBrightColor (
 		needed = false;
 		if (!lccurvePoints.empty() && lccurvePoints[0]!=0) {
 			dCurve = new DiagonalCurve (lccurvePoints, CURVES_MIN_POLY_POINTS/skip);
+			if (outBeforeLCurveHistogram /*&& histogramCropped*/)
+				histNeededLC = true;
+			
 			if (dCurve && !dCurve->isIdentity())
 				{needed = true;cclutili=true;}
 		}
+		for (int i=0; i<=48000; i++) {//32768*1.414  + ...
+			float val;
+			if (histNeededLC) {
+				float hval = dCcurve[i];
+				int hi = (int)(255.0*CLIPD(hval)); //
+				outBeforeLCurveHistogram[hi] += histogramLC[i] ;
+			}
+		}
+		
+		
 		fillCurveArray(dCurve, lhskCurve, skip, needed);
 		if (dCurve) {
 			delete dCurve;
 			dCurve = NULL;
 		}
 		
-		
 	}
 
 	//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -397,6 +514,7 @@ void CurveFactory::curveLightBrightColor (
 									 LUTu & outBeforeCCurveHistogram,
 									 ToneCurve & customToneCurve1,
 									 ToneCurve & customToneCurve2,
+									 
 									 int skip) {
 		
 		
@@ -595,6 +713,8 @@ void CurveFactory::curveLightBrightColor (
 
 		//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+		//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 		// create first curve if needed
 		customToneCurve1.Reset();
 
@@ -614,7 +734,48 @@ void CurveFactory::curveLightBrightColor (
 				tcurve = NULL;
 			}
 		}
+		//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+		// create curve bw
+		// curve 2
+/*		DiagonalCurve* tbwcurve = NULL;
+		customToneCurvebw2.Reset();
+
+		if (!curvePointsbw2.empty() && curvePointsbw2[0]>DCT_Linear && curvePointsbw2[0]<DCT_Unchanged) {
+			tbwcurve = new DiagonalCurve (curvePointsbw2, CURVES_MIN_POLY_POINTS/skip);
+			if (outBeforeCCurveHistogram )
+				histNeeded = true;
+		}
+		if (tbwcurve) {
+			if (tbwcurve->isIdentity()) {
+				delete tbwcurve;
+				tbwcurve = NULL;
+			}
+			else
+				customToneCurvebw2.Set(tbwcurve);
+			delete tbwcurve;
+			tbwcurve = NULL;
+		}
+		
+		customToneCurvebw1.Reset();
+
+		if (!curvePointsbw.empty() && curvePointsbw[0]>DCT_Linear && curvePointsbw[0]<DCT_Unchanged) {
+			tbwcurve = new DiagonalCurve (curvePointsbw, CURVES_MIN_POLY_POINTS/skip);
+			if (outBeforeCCurveHistogram )
+				histNeeded = true;
+		}
+		if (tbwcurve) {
+			if (tbwcurve->isIdentity()) {
+				delete tbwcurve;
+				tbwcurve = NULL;
+			}
+			else if (curveModeb != procparams::BlackWhiteParams::TC_MODE_STD_BW) {
+				customToneCurvebw1.Set(tbwcurve);
+				delete tbwcurve;
+				tbwcurve = NULL;
+			}
+		}
+	*/	
 		//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 		for (int i=0; i<=0xffff; i++) {
