@@ -61,8 +61,8 @@ class CurveFactory {
             return 0.0;
         double k = sqrt ((m1-1.0)*(m1-m2)/2) / (1.0-m2);
         double l = (m1-m2) / (1.0-m2) + k;
-        double lx = log(x);
-        return m2*x + (1.0-m2)*(2.0 - exp(k*lx))*exp(l*lx);
+        double lx = xlog(x);
+        return m2*x + (1.0-m2)*(2.0 - xexp(k*lx))*xexp(l*lx);
     }
     // basic concave function between (0,0) and (1,1). m1 and m2 controls the slope at the start and end point
     static inline double baseu (double x, double m1, double m2) {
@@ -180,18 +180,27 @@ class CurveFactory {
 
   public:
     static void complexCurve (double ecomp, double black, double hlcompr, double hlcomprthresh, double shcompr, double br, double contr,
-							  double gamma_, bool igamma_, procparams::ToneCurveParams::eTCModeId curveMode, const std::vector<double>& curvePoints, procparams::ToneCurveParams::eTCModeId curveMode2, const std::vector<double>& curvePoints2, LUTu & histogram, LUTu & histogramCropped,
-							  LUTf & hlCurve, LUTf & shCurve,LUTf & outCurve, LUTu & outBeforeCCurveHistogram, ToneCurve & outToneCurve, ToneCurve & outToneCurve2, int skip=1);
-	static void complexsgnCurve ( bool & autili,  bool & butili, bool & ccutili, bool & cclutili,  double saturation, double rstprotection, const std::vector<double>& acurvePoints,
-								 const std::vector<double>& bcurvePoints,const std::vector<double>& cccurvePoints,const std::vector<double>& cclurvePoints, LUTf & aoutCurve, LUTf & boutCurve, LUTf & satCurve, LUTf & lhskCurve, 
-								LUTu & histogramC, LUTu & histogramCroppedC, LUTu & outBeforeCCurveHistogramC,//for chroma
+							  double gamma_, bool igamma_, procparams::ToneCurveParams::eTCModeId curveMode, const std::vector<double>& curvePoints, procparams::ToneCurveParams::eTCModeId curveMode2, const std::vector<double>& curvePoints2, 
+							  
+							  LUTu & histogram, LUTu & histogramCropped,
+							  LUTf & hlCurve, LUTf & shCurve,LUTf & outCurve, LUTu & outBeforeCCurveHistogram, ToneCurve & outToneCurve, ToneCurve & outToneCurve2, 
+				  
+							  int skip=1);
+	static void curveBW (const std::vector<double>& curvePointsbw, const std::vector<double>& curvePointsbw2, LUTu & histogrambw, LUTu & outBeforeCCurveHistogrambw,
+						 ToneCurve & customToneCurvebw1, ToneCurve & customToneCurvebw2, int skip);
+		
+	static void curveCL ( bool & clcutili, const std::vector<double>& clcurvePoints, LUTf & clCurve, LUTu & histogramcl, LUTu & outBeforeCLurveHistogram, int skip);
+						  
+	static void complexsgnCurve ( bool & autili,  bool & butili, bool & ccutili, bool & clcutili, double saturation, double rstprotection, const std::vector<double>& acurvePoints,
+								 const std::vector<double>& bcurvePoints,const std::vector<double>& cccurvePoints,const std::vector<double>& lccurvePoints, LUTf & aoutCurve, LUTf & boutCurve, LUTf & satCurve, LUTf & lhskCurve, 
+								 LUTu & histogramC, LUTu & histogramLC, LUTu & outBeforeCCurveHistogram,LUTu & outBeforeLCurveHistogram,///for chroma
 								int skip=1);
 	static void complexLCurve (double br, double contr, const std::vector<double>& curvePoints, LUTu & histogram, LUTu & histogramCropped,
 							   LUTf & outCurve, LUTu & outBeforeCCurveHistogram, int skip, bool & utili); 
 
 	static void updatechroma (
 					const std::vector<double>& cccurvePoints,
-					LUTu & histogramC, LUTu & histogramCroppedC, LUTu & outBeforeCCurveHistogramC,//for chroma
+					LUTu & histogramC, LUTu & outBeforeCCurveHistogramC,//for chroma
 					int skip=1);
 
 	static void curveLightBrightColor (
@@ -199,7 +208,7 @@ class CurveFactory {
 					procparams::ColorAppearanceParams::eTCModeId curveMode2, const std::vector<double>& curvePoints2,
 					procparams::ColorAppearanceParams::eCTCModeId curveMode3, const std::vector<double>& curvePoints3,
 					LUTu & histogram, LUTu & histogramCropped, LUTu & outBeforeCCurveHistogram,
-					LUTu & histogramC, LUTu & histogramCroppedC, LUTu & outBeforeCCurveHistogramC,
+					LUTu & histogramC, LUTu & outBeforeCCurveHistogramC,
 					ColorAppearance & outColCurve1,
 					ColorAppearance & outColCurve2,
 					ColorAppearance & outColCurve3,
@@ -220,6 +229,12 @@ class Curve {
     int ppn;			// targeted polyline point number
     double* x;
     double* y;
+    // begin of variables used in Parametric curves only
+    double mc;
+    double mfc;
+    double msc;
+    double mhc;
+    // end of variables used in Parametric curves only
     std::vector<double> poly_x;		// X points of the faceted curve
     std::vector<double> poly_y;		// Y points of the faceted curve
     std::vector<HashEntry> hash;
@@ -385,6 +400,10 @@ class StandardToneCurve : public ToneCurve {
   public:
     void Apply(float& r, float& g, float& b) const;
 };
+class StandardToneCurvebw : public ToneCurve {
+  public:
+    void Apply(float& r, float& g, float& b) const;
+};
 
 class AdobeToneCurve : public ToneCurve {
   private:
@@ -394,11 +413,24 @@ class AdobeToneCurve : public ToneCurve {
     void Apply(float& r, float& g, float& b) const;
 };
 
+class AdobeToneCurvebw : public ToneCurve {
+  private:
+    void RGBTone(float& r, float& g, float& b) const;  // helper for tone curve
+
+  public:
+    void Apply(float& r, float& g, float& b) const;
+};
+
 class SatAndValueBlendingToneCurve : public ToneCurve {
   public:
     void Apply(float& r, float& g, float& b) const;
 };
 
+class SatAndValueBlendingToneCurvebw : public ToneCurve {
+  public:
+    void Apply(float& r, float& g, float& b) const;
+};
+
 class WeightedStdToneCurve : public ToneCurve {
   private:
     float Triangle(float refX, float refY, float X2) const;
@@ -406,6 +438,12 @@ class WeightedStdToneCurve : public ToneCurve {
     void Apply(float& r, float& g, float& b) const;
 };
 
+class WeightedStdToneCurvebw : public ToneCurve {
+  private:
+    float Triangle(float refX, float refY, float X2) const;
+  public:
+    void Apply(float& r, float& g, float& b) const;
+};
 
 // Standard tone curve
 inline void StandardToneCurve::Apply (float& r, float& g, float& b) const {
@@ -416,6 +454,15 @@ inline void StandardToneCurve::Apply (float& r, float& g, float& b) const {
     g = lutToneCurve[g];
     b = lutToneCurve[b];
 }
+// Standard tone curve
+inline void StandardToneCurvebw::Apply (float& r, float& g, float& b) const {
+
+    assert (lutToneCurve);
+
+    r = lutToneCurve[r];
+    g = lutToneCurve[g];
+    b = lutToneCurve[b];
+}
 
 // Tone curve according to Adobe's reference implementation
 // values in 0xffff space
@@ -440,6 +487,26 @@ inline void AdobeToneCurve::Apply (float& r, float& g, float& b) const {
         else             RGBTone (g, b, r); // Case 7: g >= b >  r
     }
 }
+inline void AdobeToneCurvebw::Apply (float& r, float& g, float& b) const {
+
+    assert (lutToneCurve);
+
+    if (r >= g) {
+        if      (g > b) RGBTone (r, g, b); // Case 1: r >= g >  b
+        else if (b > r) RGBTone (b, r, g); // Case 2: b >  r >= g
+        else if (b > g) RGBTone (r, b, g); // Case 3: r >= b >  g
+        else {                             // Case 4: r >= g == b
+            r = lutToneCurve[r];
+            g = lutToneCurve[g];
+            b = g;
+        }
+    }
+    else {
+        if      (r >= b) RGBTone (g, r, b); // Case 5: g >  r >= b
+        else if (b >  g) RGBTone (b, g, r); // Case 6: b >  g >  r
+        else             RGBTone (g, b, r); // Case 7: g >= b >  r
+    }
+}
 
 inline void AdobeToneCurve::RGBTone (float& r, float& g, float& b) const {
     float rold=r,gold=g,bold=b;
@@ -448,6 +515,13 @@ inline void AdobeToneCurve::RGBTone (float& r, float& g, float& b) const {
     b = lutToneCurve[bold];
     g = b + ((r - b) * (gold - bold) / (rold - bold));
 }
+inline void AdobeToneCurvebw::RGBTone (float& r, float& g, float& b) const {
+    float rold=r,gold=g,bold=b;
+
+    r = lutToneCurve[rold];
+    b = lutToneCurve[bold];
+    g = b + ((r - b) * (gold - bold) / (rold - bold));
+}
 
 inline float WeightedStdToneCurve::Triangle(float a, float a1, float b) const {
 	if (a != b) {
@@ -459,6 +533,16 @@ inline float WeightedStdToneCurve::Triangle(float a, float a1, float b) const {
 	}
 	return a1;
 }
+inline float WeightedStdToneCurvebw::Triangle(float a, float a1, float b) const {
+	if (a != b) {
+		float b1;
+		float a2 = a1 - a;
+		if (b < a) { b1 = b + a2 *      b  /     a ; }
+		else       { b1 = b + a2 * (65535.f-b) / (65535.f-a); }
+		return b1;
+	}
+	return a1;
+}
 
 // Tone curve modifying the value channel only, preserving hue and saturation
 // values in 0xffff space
@@ -483,6 +567,27 @@ inline void WeightedStdToneCurve::Apply (float& r, float& g, float& b) const {
     b = CLIP<float>(b1*0.25f + b2*0.25f + b3*0.50f);
 }
 
+inline void WeightedStdToneCurvebw::Apply (float& r, float& g, float& b) const {
+
+    assert (lutToneCurve);
+
+    float r1 = lutToneCurve[r];
+    float g1 = Triangle(r, r1, g);
+    float b1 = Triangle(r, r1, b);
+
+    float g2 = lutToneCurve[g];
+    float r2 = Triangle(g, g2, r);
+    float b2 = Triangle(g, g2, b);
+
+    float b3 = lutToneCurve[b];
+    float r3 = Triangle(b, b3, r);
+    float g3 = Triangle(b, b3, g);
+
+    r = CLIP<float>( r1*0.50f + r2*0.25f + r3*0.25f);
+    g = CLIP<float>(g1*0.25f + g2*0.50f + g3*0.25f);
+    b = CLIP<float>(b1*0.25f + b2*0.25f + b3*0.50f);
+}
+
 // Tone curve modifying the value channel only, preserving hue and saturation
 // values in 0xffff space
 inline void SatAndValueBlendingToneCurve::Apply (float& r, float& g, float& b) const {
@@ -514,6 +619,35 @@ inline void SatAndValueBlendingToneCurve::Apply (float& r, float& g, float& b) c
     }
 }
 
+inline void SatAndValueBlendingToneCurvebw::Apply (float& r, float& g, float& b) const {
+
+    assert (lutToneCurve);
+
+    float h, s, v;
+    float lum = (r+g+b)/3.f;
+    //float lum = Color::rgbLuminance(r, g, b);
+    float newLum = lutToneCurve[lum];
+    if (newLum == lum)
+    	return;
+    bool increase = newLum > lum;
+
+    Color::rgb2hsv(r, g, b, h, s, v);
+
+    if (increase) {
+    	// Linearly targeting Value = 1 and Saturation = 0
+        float coef = (newLum-lum)/(65535.f-lum);
+        float dV = (1.f-v)*coef;
+        s *= 1.f-coef;
+    	Color::hsv2rgb(h, s, v+dV, r, g, b);
+    }
+    else {
+    	// Linearly targeting Value = 0
+        float coef = (lum-newLum)/lum ;
+        float dV = v*coef;
+    	Color::hsv2rgb(h, s, v-dV, r, g, b);
+    }
+}
+
 }
 
 #undef CLIPI
