@@ -52,6 +52,9 @@ public:
     ,bright(1.),threshold(0.)
     ,half_size(0),four_color_rgb(0),document_mode(0),highlight(0)
     ,verbose(0)
+    ,RT_whitelevel_from_constant(0)
+    ,RT_blacklevel_from_constant(0)
+    ,RT_matrix_from_constant(0)
     ,use_auto_wb(0),use_camera_wb(0),use_camera_matrix(-1)
     ,output_color(1),output_bps(8),output_tiff(0),med_passes(0),no_auto_bright(0)
 	,getbithuff(this,ifp,zero_after_ff)
@@ -71,7 +74,7 @@ protected:
     FILE *ofp;
     short order;
     const char *ifname;
-    char *meta_data;
+    char *meta_data, xtrans[6][6];
     char cdesc[5], desc[512], make[64], model[64], model2[64], artist[64];
     float flash_used, canon_ev, iso_speed, shutter, aperture, focal_len;
     time_t timestamp;
@@ -100,6 +103,9 @@ protected:
     int output_color, output_bps, output_tiff, med_passes;
     int no_auto_bright;
     unsigned greybox[4] ;
+    int RT_matrix_from_constant;
+    int RT_blacklevel_from_constant;
+    int RT_whitelevel_from_constant;
 
     float cam_mul[4], pre_mul[4], cmatrix[3][4], rgb_cam[3][4];
     
@@ -264,11 +270,13 @@ private:
 };
 pana_bits_t pana_bits;
 
+void canon_rmf_load_raw();
 void panasonic_load_raw();
 void olympus_load_raw();
 void minolta_rd175_load_raw();
 void quicktake_100_load_raw();
 void kodak_radc_load_raw();
+void samsung_load_raw();
 
 void kodak_jpeg_load_raw();
 void lossy_dng_load_raw();
@@ -317,6 +325,9 @@ void foveon_make_curves(short **curvep, float dq[3], float div[3], float filt);
 int foveon_apply_curve (short *curve, int i);
 void foveon_interpolate();
 
+void xtrans_interpolate (int passes);
+void cielab (ushort rgb[3], short lab[3]);
+
 void remove_zeroes();
 void bad_pixels (const char *cfname);
 void subtract (const char *fname);
@@ -350,7 +361,7 @@ int  parse_tiff (int base);
 void apply_tiff();
 void parse_external_jpeg();
 void ciff_block_1030();
-void parse_ciff (int offset, int length);
+void parse_ciff (int offset, int length, int depth);
 void parse_rollei();
 void parse_sinar_ia();
 void parse_phase_one (int base);
@@ -369,6 +380,7 @@ float find_green (int bps, int bite, int off0, int off1);
 void identify();
 void apply_profile (const char *input, const char *output);
 void jpeg_thumb() {}  // not needed
+bool dcraw_coeff_overrides(const char make[], const char model[], int iso_speed, short trans[12], int *black_level, int *white_level);
 
 };
 
