@@ -251,9 +251,9 @@ dfInfo *DFManager::addFileInfo(const Glib::ustring &filename ,bool pool )
 {
 	Glib::RefPtr<Gio::File> file = Gio::File::create_for_path(filename);
     if (!file )
-        return false;
+        return 0;
     if( !file->query_exists())
-    	return false;
+    	return 0;
     Glib::RefPtr<Gio::FileInfo> info = safe_query_file_info(file);
     if (info && info->get_file_type() != Gio::FILE_TYPE_DIRECTORY && (!info->is_hidden() || !options.fbShowHidden)) {
 	size_t lastdot = info->get_name().find_last_of ('.');
@@ -402,8 +402,11 @@ int DFManager::scanBadPixelsFile( Glib::ustring filename )
 	size_t dirpos2 = filename.find_last_of ('\\');
 	if( dirpos1 == Glib::ustring::npos && dirpos2== Glib::ustring::npos )
 		dirpos1 =0;
-	else
+	else if( dirpos1 != Glib::ustring::npos && dirpos2 != Glib::ustring::npos )
 		dirpos1= (dirpos1> dirpos2 ? dirpos1: dirpos2);
+	else if( dirpos1 == Glib::ustring::npos )
+		dirpos1= dirpos2;
+
 	std::string makmodel(filename,dirpos1+1,lastdot-(dirpos1+1) );
 	std::list<badPix> bp;
 	char line[256];
