@@ -29,7 +29,9 @@
 
 
 #include "rt_math.h"
+#include "rawimagesource.h"
 
+namespace rtengine {
 
 //void green_equilibrate()//for dcraw implementation
 void RawImageSource::green_equilibrate(float thresh)
@@ -40,7 +42,8 @@ void RawImageSource::green_equilibrate(float thresh)
 	int height=H, width=W; 
 
 	// local variables
-	array2D<float> cfa (width,height,rawData);
+	float** rawptr = rawData;
+	array2D<float> cfa (width,height,rawptr);
 	//array2D<int> checker (width,height,ARRAY2D_CLEAR_DATA);
 	
 	
@@ -141,4 +144,6 @@ void RawImageSource::green_equilibrate(float thresh)
 	
 	
 }
+}
+
 #undef TS
