@@ -16,12 +16,12 @@
  *  You should have received a copy of the GNU General Public License
  *  along with RawTherapee.  If not, see <http://www.gnu.org/licenses/>.
  */
-#include "iccstore.h"
 #ifdef WIN32
 #include <winsock2.h>
 #else
 #include <netinet/in.h>
 #endif
+#include "iccstore.h"
 #include "iccmatrices.h"
 #include <glib/gstdio.h>
 #include "safegtk.h"
@@ -33,7 +33,7 @@ namespace rtengine {
 
 const double (*wprofiles[])[3]  = {xyz_sRGB, xyz_adobe, xyz_prophoto, xyz_widegamut, xyz_bruce, xyz_beta, xyz_best};
 const double (*iwprofiles[])[3] = {sRGB_xyz, adobe_xyz, prophoto_xyz, widegamut_xyz, bruce_xyz, beta_xyz, best_xyz};
-const char* wpnames[] = {"sRGB", "Adobe RGB", "ProPhoto", "WideGamut", "BruceRGB", "Beta RGB", "BestRGB"};         
+const char* wpnames[] = {"sRGB", "Adobe RGB", "ProPhoto", "WideGamut", "BruceRGB", "Beta RGB", "BestRGB"};
 const char* wpgamma[] = {"default","BT709_g2.2_s4.5", "sRGB_g2.4_s12.92", "linear_g1.0", "standard_g2.2", "standard_g1.8", "High_g1.3_s3.35","Low_g2.6_s6.9"};  //gamma free
 //default = gamma inside profile
 //BT709 g=2.22 s=4.5  sRGB g=2.4 s=12.92  
@@ -41,50 +41,138 @@ const char* wpgamma[] = {"default","BT709_g2.2_s4.5", "sRGB_g2.4_s12.92", "linea
 //std22 g=2.2   std18 g=1.8
 // high  g=1.3 s=3.35  for high dynamic images
 //low  g=2.6 s=6.9  for low contrast images
-       
 
-std::vector<std::string> getGamma () {//return gamma
 
-    std::vector<std::string> res;
+std::vector<Glib::ustring> getGamma () {//return gamma
+
+    std::vector<Glib::ustring> res;
     for (unsigned int i=0; i<sizeof(wpgamma)/sizeof(wpgamma[0]); i++)
         res.push_back (wpgamma[i]);
     return res;
 }
 
 
-std::vector<std::string> getWorkingProfiles () {
+std::vector<Glib::ustring> getWorkingProfiles () {
 
-    std::vector<std::string> res;
+    std::vector<Glib::ustring> res;
     for (unsigned int i=0; i<sizeof(wpnames)/sizeof(wpnames[0]); i++)
         res.push_back (wpnames[i]);
     return res;
 }
 
-std::vector<std::string> ICCStore::getOutputProfiles () {
+std::vector<Glib::ustring> ICCStore::getOutputProfiles () {
 
-	Glib::Mutex::Lock lock(mutex_);
+	MyMutex::MyLock lock(mutex_);
 
-    std::vector<std::string> res;
-    for (std::map<std::string, cmsHPROFILE>::iterator i=fileProfiles.begin(); i!=fileProfiles.end(); i++){
-    	std::string name(i->first);
-    	std::string::size_type  i2 = name.find_last_of('/');
-    	if( i2 == std::string::npos )
-    		i2 = name.find_last_of('\\');
-    	if( i2 == std::string::npos )
+    std::vector<Glib::ustring> res;
+    for (std::map<Glib::ustring, cmsHPROFILE>::iterator i=fileProfiles.begin(); i!=fileProfiles.end(); i++){
+        Glib::ustring name(i->first);
+        std::string::size_type  i2 = name.find_last_of('/');
+        if( i2 == std::string::npos )
+            i2 = name.find_last_of('\\');
+        if( i2 == std::string::npos )
            res.push_back ( name ); // list only profiles inside selected profiles directory
     }
     return res;
 }
 
 
+cmsHPROFILE
+ICCStore::makeStdGammaProfile(cmsHPROFILE iprof)
+{
+    // forgive me for the messy code, quick hack to change gamma of an ICC profile to the RT standard gamma
+    void *buf = NULL;
+    if (!iprof) {
+        return NULL;
+    }
+    cmsUInt32Number bytesNeeded = 0;
+    cmsSaveProfileToMem(iprof, 0, &bytesNeeded);
+    if (bytesNeeded == 0) {
+        return NULL;
+    }
+    uint8_t *data = new uint8_t[bytesNeeded+1];
+    cmsSaveProfileToMem(iprof, data, &bytesNeeded);
+    const size_t len = (int)bytesNeeded;
+    const uint8_t *p = &data[128]; // skip 128 byte header
+    uint32_t tag_count;
+    memcpy(&tag_count, p, 4);
+    p += 4;
+    tag_count = ntohl(tag_count);
+
+    struct icctag {
+        uint32_t sig;
+        uint32_t offset;
+        uint32_t size;
+    } tags[tag_count];
+
+    const uint32_t gamma = 0x239;
+    int gamma_size = (gamma == 0 || gamma == 256) ? 12 : 14;
+    int data_size = (gamma_size + 3) & ~3;
+    for (int i = 0; i < tag_count; i++) {
+        memcpy(&tags[i], p, 12);
+        tags[i].sig = ntohl(tags[i].sig);
+        tags[i].offset = ntohl(tags[i].offset);
+        tags[i].size = ntohl(tags[i].size);
+        p += 12;
+        if (tags[i].sig != 0x62545243 && // bTRC
+            tags[i].sig != 0x67545243 && // gTRC
+            tags[i].sig != 0x72545243 && // rTRC
+            tags[i].sig != 0x6B545243) // kTRC
+        {
+            data_size += (tags[i].size + 3) & ~3;
+        }
+    }
+    uint32_t sz = 128 + 4 + tag_count * 12 + data_size;
+    uint8_t *nd = new uint8_t[sz];
+    memset(nd, 0, sz);
+    memcpy(nd, data, 128 + 4);
+    sz = htonl(sz);
+    memcpy(nd, &sz, 4);
+    uint32_t offset = 128 + 4 + tag_count * 12;
+    uint32_t gamma_offset = 0;
+    for (int i = 0; i < tag_count; i++) {
+        struct icctag tag;
+        tag.sig = htonl(tags[i].sig);
+        if (tags[i].sig == 0x62545243 || // bTRC
+            tags[i].sig == 0x67545243 || // gTRC
+            tags[i].sig == 0x72545243 || // rTRC
+            tags[i].sig == 0x6B545243) // kTRC
+        {
+            if (gamma_offset == 0) {
+                gamma_offset = offset;
+                uint32_t pcurve[] = { htonl(0x63757276), htonl(0), htonl(gamma_size == 12 ? 0 : 1) };
+                memcpy(&nd[offset], pcurve, 12);
+                if (gamma_size == 14) {
+                    uint16_t gm = htons(gamma);
+                    memcpy(&nd[offset+12], &gm, 2);
+                }
+                offset += (gamma_size + 3) & ~3;
+            }
+            tag.offset = htonl(gamma_offset);
+            tag.size = htonl(gamma_size);
+        } else {
+            tag.offset = htonl(offset);
+            tag.size = htonl(tags[i].size);
+            memcpy(&nd[offset], &data[tags[i].offset], tags[i].size);
+            offset += (tags[i].size + 3) & ~3;
+        }
+        memcpy(&nd[128 + 4 + i * 12], &tag, 12);
+    }
+
+    cmsHPROFILE oprof = cmsOpenProfileFromMem (nd, ntohl(sz));
+    delete [] nd;
+    delete [] data;
+    return oprof;
+}
+
 ICCStore*
 ICCStore::getInstance(void)
 {
 	static ICCStore* instance_ = 0;
 	if ( instance_ == 0 )
 	{
-		static Glib::Mutex smutex_;
-		Glib::Mutex::Lock lock(smutex_);
+		static MyMutex smutex_;
+		MyMutex::MyLock lock(smutex_);
 		if ( instance_ == 0 )
 		{
 			instance_ = new ICCStore();
@@ -117,7 +205,7 @@ int ICCStore::numOfWProfiles () {
 
 TMatrix ICCStore::workingSpaceMatrix (Glib::ustring name) {
 
-    std::map<std::string, TMatrix>::iterator r = wMatrices.find (name);
+    std::map<Glib::ustring, TMatrix>::iterator r = wMatrices.find (name);
     if (r!=wMatrices.end()) 
         return r->second;
     else
@@ -126,7 +214,7 @@ TMatrix ICCStore::workingSpaceMatrix (Glib::ustring name) {
 
 TMatrix ICCStore::workingSpaceInverseMatrix (Glib::ustring name) {
 
-    std::map<std::string, TMatrix>::iterator r = iwMatrices.find (name);
+    std::map<Glib::ustring, TMatrix>::iterator r = iwMatrices.find (name);
     if (r!=iwMatrices.end()) 
         return r->second;
     else 
@@ -135,7 +223,7 @@ TMatrix ICCStore::workingSpaceInverseMatrix (Glib::ustring name) {
 
 cmsHPROFILE ICCStore::workingSpace (Glib::ustring name) {
 
-    std::map<std::string, cmsHPROFILE>::iterator r = wProfiles.find (name);
+    std::map<Glib::ustring, cmsHPROFILE>::iterator r = wProfiles.find (name);
     if (r!=wProfiles.end()) 
         return r->second;
     else
@@ -144,7 +232,7 @@ cmsHPROFILE ICCStore::workingSpace (Glib::ustring name) {
 
 cmsHPROFILE ICCStore::workingSpaceGamma (Glib::ustring name) {
 
-    std::map<std::string, cmsHPROFILE>::iterator r = wProfilesGamma.find (name);
+    std::map<Glib::ustring, cmsHPROFILE>::iterator r = wProfilesGamma.find (name);
     if (r!=wProfilesGamma.end()) 
         return r->second;
     else
@@ -153,9 +241,9 @@ cmsHPROFILE ICCStore::workingSpaceGamma (Glib::ustring name) {
 
 cmsHPROFILE ICCStore::getProfile (Glib::ustring name) {
 
-	Glib::Mutex::Lock lock(mutex_);
+	MyMutex::MyLock lock(mutex_);
 
-    std::map<std::string, cmsHPROFILE>::iterator r = fileProfiles.find (name);
+    std::map<Glib::ustring, cmsHPROFILE>::iterator r = fileProfiles.find (name);
     if (r!=fileProfiles.end()) 
         return r->second;
     else {
@@ -176,10 +264,10 @@ cmsHPROFILE ICCStore::getProfile (Glib::ustring name) {
 
 cmsHPROFILE ICCStore::getStdProfile (Glib::ustring name) {
 
-	Glib::Mutex::Lock lock(mutex_);
+	MyMutex::MyLock lock(mutex_);
 
 
-    std::map<std::string, cmsHPROFILE>::iterator r = fileStdProfiles.find (name.uppercase());
+    std::map<Glib::ustring, cmsHPROFILE>::iterator r = fileStdProfiles.find (name.uppercase());
     if (r==fileStdProfiles.end()) return NULL;
     
     return r->second;
@@ -187,7 +275,7 @@ cmsHPROFILE ICCStore::getStdProfile (Glib::ustring name) {
 
 ProfileContent ICCStore::getContent (Glib::ustring name) {
 
-	Glib::Mutex::Lock lock(mutex_);
+	MyMutex::MyLock lock(mutex_);
 
     return fileProfileContents[name];
 }
@@ -195,7 +283,7 @@ ProfileContent ICCStore::getContent (Glib::ustring name) {
 // Reads all profiles from the given profiles dir
 void ICCStore::init (Glib::ustring usrICCDir, Glib::ustring rtICCDir) {
 
-	Glib::Mutex::Lock lock(mutex_);
+	MyMutex::MyLock lock(mutex_);
 
 	//
     fileProfiles.clear();
@@ -211,7 +299,7 @@ void ICCStore::init (Glib::ustring usrICCDir, Glib::ustring rtICCDir) {
     loadICCs(Glib::build_filename(rtICCDir, "input"), true, fileStdProfiles, fileStdProfileContents);
 }
 
-void ICCStore::loadICCs(Glib::ustring rootDirName, bool nameUpper, std::map<std::string, cmsHPROFILE>& resultProfiles, std::map<std::string, ProfileContent> &resultProfileContents) {
+void ICCStore::loadICCs(Glib::ustring rootDirName, bool nameUpper, std::map<Glib::ustring, cmsHPROFILE>& resultProfiles, std::map<Glib::ustring, ProfileContent> &resultProfileContents) {
     if (rootDirName!="") {
         std::deque<Glib::ustring> qDirs;
 
