@@ -23,6 +23,7 @@
 #include <glibmm.h>
 #include <map>
 #include <string>
+#include "../rtgui/threadutils.h"
 
 namespace rtengine {
 
@@ -45,30 +46,31 @@ class ProfileContent {
 
 class ICCStore {
 
-        std::map<std::string, cmsHPROFILE> wProfiles;
-        std::map<std::string, cmsHPROFILE> wProfilesGamma;
-        std::map<std::string, TMatrix> wMatrices;
-        std::map<std::string, TMatrix> iwMatrices;
+        std::map<Glib::ustring, cmsHPROFILE> wProfiles;
+        std::map<Glib::ustring, cmsHPROFILE> wProfilesGamma;
+        std::map<Glib::ustring, TMatrix> wMatrices;
+        std::map<Glib::ustring, TMatrix> iwMatrices;
         
         // these contain profiles from user/system directory (supplied on init)
-        std::map<std::string, cmsHPROFILE>    fileProfiles;
-        std::map<std::string, ProfileContent> fileProfileContents;
+        std::map<Glib::ustring, cmsHPROFILE>    fileProfiles;
+        std::map<Glib::ustring, ProfileContent> fileProfileContents;
 
         // these contain standard profiles from RT. keys are all in uppercase
-        std::map<std::string, cmsHPROFILE>    fileStdProfiles;
-        std::map<std::string, ProfileContent> fileStdProfileContents;
+        std::map<Glib::ustring, cmsHPROFILE>    fileStdProfiles;
+        std::map<Glib::ustring, ProfileContent> fileStdProfileContents;
 
         cmsHPROFILE xyz;
         cmsHPROFILE srgb;
 
-        Glib::Mutex mutex_;
+        MyMutex mutex_;
 
         ICCStore (); 
-        void             loadICCs(Glib::ustring rootDirName, bool nameUpper, std::map<std::string, cmsHPROFILE>& resultProfiles, std::map<std::string, ProfileContent> &resultProfileContents);
+        void             loadICCs(Glib::ustring rootDirName, bool nameUpper, std::map<Glib::ustring, cmsHPROFILE>& resultProfiles, std::map<Glib::ustring, ProfileContent> &resultProfileContents);
         
     public:
 
         static ICCStore* getInstance(void);
+        static cmsHPROFILE makeStdGammaProfile(cmsHPROFILE iprof);
 
         Glib::ustring    defaultMonitorProfile;  // Main monitors standard profile name, from OS
         void             findDefaultMonitorProfile();
@@ -88,7 +90,7 @@ class ICCStore {
 
         cmsHPROFILE      getXYZProfile ()  { return xyz;  }
         cmsHPROFILE      getsRGBProfile () { return srgb; }
-        std::vector<std::string> getOutputProfiles ();
+        std::vector<Glib::ustring> getOutputProfiles ();
 };
 
 #define iccStore ICCStore::getInstance()
