@@ -23,6 +23,7 @@
 #include <glib/gstdio.h>
 #include <glibmm.h>
 #include <vector>
+#include "../rtgui/threadutils.h"
 #include "rt_math.h"
 #include "alignedbuffer.h"
 #include "imagedimensions.h"
@@ -84,7 +85,7 @@ namespace rtengine {
         virtual void getSpotWBData (double &reds, double &greens, double &blues, int &rn, int &gn, int &bn,
                                     std::vector<Coord2D> &red, std::vector<Coord2D> &green, std::vector<Coord2D> &blue,
                                     int tran) {}
-        virtual ColorTemp getAutoWB () { return ColorTemp(); }
+        virtual void getAutoWBMultipliers (double &rm, double &gm, double &bm) { rm=gm=bm=1.0; }
         virtual const char* getType () const { return "unknown"; }
 
     };
@@ -476,7 +477,7 @@ namespace rtengine {
                 }
         }
 
-        ColorTemp getAutoWB () {
+        void getAutoWBMultipliers (double &rm, double &gm, double &bm) {
 
             double avg_r = 0.;
             double avg_g = 0.;
@@ -500,8 +501,9 @@ namespace rtengine {
                     avg_b += intpow( (double)b(i, j), p);*/
                     n++;
                 }
-            return ColorTemp (avg_r/double(n), avg_g/double(n), avg_b/double(n));
-            //return ColorTemp (pow(avg_r/n, 1.0/p), pow(avg_g/n, 1.0/p), pow(avg_b/n, 1.0/p));
+            rm = avg_r/double(n);
+            gm = avg_g/double(n);
+            bm = avg_b/double(n);
         }
 
         void transformPixel (int x, int y, int tran, int& tx, int& ty) {
@@ -687,9 +689,7 @@ namespace rtengine {
             assert (dest!=NULL);
             // Make sure that the size is the same, reallocate if necessary
             dest->allocate(width, height);
-            for (int i=0; i<height; i++) {
-                memcpy (dest->data, data, 3*width*height*sizeof(T));
-            }
+            memcpy (dest->data, data, 3*width*height*sizeof(T));
         }
 
         void rotate (int deg) {
@@ -905,7 +905,7 @@ namespace rtengine {
                 }
         }
 
-        ColorTemp getAutoWB () {
+        void getAutoWBMultipliers (double &rm, double &gm, double &bm) {
 
             double avg_r = 0.;
             double avg_g = 0.;
@@ -929,8 +929,9 @@ namespace rtengine {
                     avg_b += intpow( (double)b(i, j), p);*/
                     n++;
                 }
-            return ColorTemp (avg_r/double(n), avg_g/double(n), avg_b/double(n));
-            //return ColorTemp (pow(avg_r/n, 1.0/p), pow(avg_g/n, 1.0/p), pow(avg_b/n, 1.0/p));
+            rm = avg_r/double(n);
+            gm = avg_g/double(n);
+            bm = avg_b/double(n);
         }
 
         void transformPixel (int x, int y, int tran, int& tx, int& ty) {
@@ -1019,7 +1020,7 @@ namespace rtengine {
         virtual ~IImage() {}
         /** Returns a mutex that can is useful in many situations. No image operations shuold be performed without locking this mutex.
           * @return The mutex */
-            virtual Glib::Mutex& getMutex ()=0;
+            virtual MyMutex& getMutex ()=0;
             virtual cmsHPROFILE getProfile ()=0;
         /** Returns the bits per pixel of the image.
           * @return The bits per pixel of the image */
