@@ -43,7 +43,7 @@ class Image16 : public IImage16, public ImageIO {
         Image8*              to8();
         Imagefloat*          tofloat();
 
-        virtual void         getStdImage (ColorTemp ctemp, int tran, Imagefloat* image, PreviewProps pp, bool first, procparams::HRecParams hrp);
+        virtual void         getStdImage (ColorTemp ctemp, int tran, Imagefloat* image, PreviewProps pp, bool first, procparams::ToneCurveParams hrp);
 
         virtual const char*  getType     () const { return sImage16; }
         virtual int          getBPS      () { return 8*sizeof(unsigned short); }
@@ -51,7 +51,7 @@ class Image16 : public IImage16, public ImageIO {
         virtual void         setScanline (int row, unsigned char* buffer, int bps, float *minValue=NULL, float *maxValue=NULL);
 
         // functions inherited from IImage16:
-        virtual Glib::Mutex& getMutex () { return mutex (); }
+        virtual MyMutex&     getMutex () { return mutex (); }
         virtual cmsHPROFILE  getProfile () { return getEmbeddedProfile (); }
         virtual int          getBitsPerPixel () { return 8*sizeof(unsigned short); }
         virtual int          saveToFile (Glib::ustring fname) { return save (fname); }
