@@ -34,6 +34,7 @@
 #include "../rtexif/rtexif.h"
 #include "imagedimensions.h"
 #include "iimage.h"
+#include "../rtgui/threadutils.h"
 
 namespace rtengine {
 
@@ -69,7 +70,7 @@ class ImageIO : virtual public ImageDatas {
         procparams::ExifPairs exifChange;
         IptcData* iptc;
         const rtexif::TagDirectory* exifRoot;
-        Glib::Mutex imutex;
+        MyMutex imutex;
         IIOSampleFormat sampleFormat;
         IIOSampleArrangement sampleArrangement;
 
@@ -89,7 +90,7 @@ class ImageIO : virtual public ImageDatas {
         void                 setSampleArrangement(IIOSampleArrangement sArrangement) { sampleArrangement = sArrangement; }
         IIOSampleArrangement getSampleArrangement() { return sampleArrangement; }
 
-        virtual void    getStdImage (ColorTemp ctemp, int tran, Imagefloat* image, PreviewProps pp, bool first, procparams::HRecParams hrp) {
+        virtual void    getStdImage (ColorTemp ctemp, int tran, Imagefloat* image, PreviewProps pp, bool first, procparams::ToneCurveParams hrp) {
             printf("getStdImage NULL!\n");
         }
 
@@ -122,7 +123,7 @@ class ImageIO : virtual public ImageDatas {
         void setMetadata (const rtexif::TagDirectory* eroot);
         void setMetadata (const rtexif::TagDirectory* eroot, const rtengine::procparams::ExifPairs& exif, const rtengine::procparams::IPTCPairs& iptcc);
         void setOutputProfile  (char* pdata, int plen);
-        Glib::Mutex& mutex () { return imutex; }
+        MyMutex& mutex () { return imutex; }
 };
 
 }
