@@ -27,6 +27,7 @@
 #include "procevents.h"
 #include "dcrop.h"
 #include "LUT.h"
+#include "../rtgui/threadutils.h"
 
 namespace rtengine {
 
@@ -34,8 +35,17 @@ using namespace procparams;
 
 class Crop;
 
-// Manages the image processing, espc. of the preview windows
-// There is one ImProcCoordinator per edit panel
+/** @brief Manages the image processing, espc. of the preview windows
+  *
+  * There is one ImProcCoordinator per edit panel.
+  *
+  * The ImProcCoordinator handle an sized down image representation of the full image, that is used when paning
+  * and in the Navigator object.
+  *
+  * Each ImProcCoordinator handles an rtengine::Crop list, which process images too with their own pipeline,
+  * but using this class' LUT and other precomputed parameters. The main preview area is displaying a non framed Crop object,
+  * while detail windows are framed Crop objects.
+  */
 class ImProcCoordinator : public StagedImageProcessor {
 
     friend class Crop;
@@ -43,11 +53,11 @@ class ImProcCoordinator : public StagedImageProcessor {
     protected:
         Imagefloat *orig_prev;
         Imagefloat *oprevi;
-        LabImage *oprevl;    
-        LabImage *nprevl;    
+        LabImage *oprevl;
+        LabImage *nprevl;
         Image8 *previmg;
         Image8 *workimg;
-        CieImage *ncie;    
+        CieImage *ncie;
 
         ImageSource* imgsrc;
 
@@ -56,7 +66,7 @@ class ImProcCoordinator : public StagedImageProcessor {
         ColorTemp currWB;
         ColorTemp autoWB;
 
-        bool awbComputed;
+        double lastAwbEqual;
 
         ImProcFunctions ipf;
 
@@ -67,6 +77,11 @@ class ImProcCoordinator : public StagedImageProcessor {
 
         void freeAll ();
 
+        // Precomputed values used by DetailedCrop ----------------------------------------------
+
+        float bwAutoR, bwAutoG, bwAutoB;
+        float CAMMean;
+
         LUTf hltonecurve;
         LUTf shtonecurve;
         LUTf tonecurve;
@@ -76,39 +91,48 @@ class ImProcCoordinator : public StagedImageProcessor {
         LUTf chroma_bcurve;
         LUTf satcurve;
         LUTf lhskcurve;
+        LUTf clcurve;
 
-        LUTu vhist16;
+        LUTu vhist16,vhist16bw;
         LUTu lhist16,lhist16Cropped;
         LUTu lhist16CAM,lhist16CroppedCAM;
-        LUTu lhist16CCAM,lhist16CroppedCCAM;
-		LUTu lhist16CCAMAF;
- 		LUTu lhist16ClabAF;
-		LUTu histCropped;
-		LUTu lhist16Clad,lhist16CroppedClad;
+        LUTu lhist16CCAM;
+        LUTu lhist16CCAMAF;
+        LUTu lhist16ClabAF;
+        LUTu histCropped;
+        LUTu lhist16Clad,lhist16CLlad,lhist16LClad,lhist16LLClad;
         LUTu histRed, histRedRaw;
         LUTu histGreen, histGreenRaw;
         LUTu histBlue, histBlueRaw;
-        LUTu histLuma, histToneCurve, histLCurve, histCCurve, histLCAM, histCCAM, histClad, bcabhist, histChroma;
+        LUTu histLuma, histToneCurve, histToneCurveBW, histLCurve, histCCurve, histCLurve;
+        LUTu histLLCurve, histLCAM, histCCAM, histClad, bcabhist, histChroma;
+
+        LUTf CAMBrightCurveJ, CAMBrightCurveQ;
 
         LUTf rCurve;
         LUTf gCurve;
         LUTf bCurve;
-		bool utili;
-		bool autili;
-		bool butili;
-		bool ccutili;
-		bool cclutili;		
+        bool utili;
+        bool autili;
+        bool butili;
+        bool ccutili;
+        bool cclutili;
+        bool clcutili;
 
         ToneCurve customToneCurve1;
         ToneCurve customToneCurve2;
         ColorAppearance customColCurve1;
         ColorAppearance customColCurve2;
         ColorAppearance customColCurve3;
+        ToneCurve beforeToneCurveBW;
+        ToneCurve afterToneCurveBW;
 
         LUTu rcurvehist, rcurvehistCropped, rbeforehist;
         LUTu gcurvehist, gcurvehistCropped, gbeforehist;
         LUTu bcurvehist, bcurvehistCropped, bbeforehist;
 
+        // ------------------------------------------------------------------------------------
+
         int fw, fh, tr, fullw, fullh;
         int pW, pH;
 
@@ -116,6 +140,8 @@ class ImProcCoordinator : public StagedImageProcessor {
         PreviewImageListener* imageListener;
         AutoExpListener* aeListener;
         AutoCamListener* acListener;
+        AutoBWListener* abwListener;
+
         HistogramListener* hListener;
         std::vector<SizeListener*> sizeListeners;
 
@@ -123,7 +149,7 @@ class ImProcCoordinator : public StagedImageProcessor {
 
         bool resultValid;
 
-        Glib::Mutex minit;
+        MyMutex minit;  // to gain mutually exclusive access to ... to what exactly?
 
         void progress (Glib::ustring str, int pr);
         void reallocAll ();
@@ -131,13 +157,13 @@ class ImProcCoordinator : public StagedImageProcessor {
         void setScale (int prevscale);
         void updatePreviewImage (int todo, Crop* cropCall= NULL);
 
-        Glib::Mutex mProcessing;
+        MyMutex mProcessing;
         ProcParams params;
 
         // members of the updater:
         Glib::Thread* thread;
-        Glib::Mutex updaterThreadStart;
-        Glib::Mutex paramsUpdateMutex;
+        MyMutex updaterThreadStart;
+        MyMutex paramsUpdateMutex;
         int  changeSinceLast;
         bool updaterRunning;
         ProcParams nextParams;
@@ -174,7 +200,7 @@ class ImProcCoordinator : public StagedImageProcessor {
 
         DetailedCrop* createCrop  ();
 
-        bool getAutoWB   (double& temp, double& green);
+        bool getAutoWB   (double& temp, double& green, double equal);
         void getCamWB    (double& temp, double& green);
         void getSpotWB   (int x, int y, int rectSize, double& temp, double& green);
         void getAutoCrop (double ratio, int &x, int &y, int &w, int &h);
@@ -186,9 +212,10 @@ class ImProcCoordinator : public StagedImageProcessor {
         void setAutoExpListener  (AutoExpListener* ael)  {aeListener = ael; }
         void setHistogramListener(HistogramListener *h)  {hListener = h; }
         void setAutoCamListener  (AutoCamListener* acl)  {acListener = acl; }
+        void setAutoBWListener  (AutoBWListener* abw)  {abwListener = abw; }
 
         void saveInputICCReference (const Glib::ustring& fname);
-        
+
         InitialImage*  getInitialImage () { return imgsrc; }
 };
 }
