@@ -51,9 +51,9 @@ class ImProcFunctions {
 
         void calcVignettingParams(int oW, int oH, const VignettingParams& vignetting, double &w2, double &h2, double& maxRadius, double &v, double &b, double &mul);
 
-		void transformPreview      (Imagefloat* original, Imagefloat* transformed, int cx, int cy, int sx, int sy, int oW, int oH, const LCPMapper *pLCPMap);
-		void transformVignetteOnly (Imagefloat* original, Imagefloat* transformed, int cx, int cy, int oW, int oH);
-		void transformHighQuality  (Imagefloat* original, Imagefloat* transformed, int cx, int cy, int sx, int sy, int oW, int oH, const LCPMapper *pLCPMap, bool fullImage);
+		void transformPreview       (Imagefloat* original, Imagefloat* transformed, int cx, int cy, int sx, int sy, int oW, int oH, int fW, int fH, const LCPMapper *pLCPMap);
+		void transformLuminanceOnly (Imagefloat* original, Imagefloat* transformed, int cx, int cy, int oW, int oH, int fW, int fH);
+		void transformHighQuality   (Imagefloat* original, Imagefloat* transformed, int cx, int cy, int sx, int sy, int oW, int oH, int fW, int fH, const LCPMapper *pLCPMap, bool fullImage);
 
 		void sharpenHaloCtrl    (LabImage* lab, float** blurmap, float** base, int W, int H);
 		void sharpenHaloCtrlcam (CieImage* ncie, float** blurmap, float** base, int W, int H);
@@ -64,9 +64,109 @@ class ImProcFunctions {
 		bool needsDistortion    ();
 		bool needsRotation      ();
 		bool needsPerspective   ();
+		bool needsGradient      ();
 		bool needsVignetting    ();
         bool needsLCP           ();
- //   static cmsUInt8Number* Mempro = NULL;		
+ //   static cmsUInt8Number* Mempro = NULL;
+
+		inline void interpolateTransformCubic (Imagefloat* src, int xs, int ys, double Dx, double Dy, float *r, float *g, float *b, double mul) {
+			const double A=-0.85;
+
+			double w[4];
+
+			{
+			double t1, t2;
+			t1 = -A*(Dx-1.0)*Dx;
+			t2 = (3.0-2.0*Dx)*Dx*Dx;
+			w[3] = t1*Dx;
+			w[2] = t1*(Dx-1.0) + t2;
+			w[1] = -t1*Dx + 1.0 - t2;
+			w[0] = -t1*(Dx-1.0);
+			}
+
+			double rd, gd, bd;
+			double yr[4], yg[4], yb[4];
+
+			for (int k=ys, kx=0; k<ys+4; k++, kx++) {
+				rd = gd = bd = 0.0;
+				for (int i=xs, ix=0; i<xs+4; i++, ix++) {
+					rd += src->r(k,i) * w[ix];
+					gd += src->g(k,i) * w[ix];
+					bd += src->b(k,i) * w[ix];
+				}
+				yr[kx] = rd; yg[kx] = gd; yb[kx] = bd;
+			}
+
+
+			{
+			double t1, t2;
+
+			t1 = -A*(Dy-1.0)*Dy;
+			t2 = (3.0-2.0*Dy)*Dy*Dy;
+			w[3] = t1*Dy;
+			w[2] = t1*(Dy-1.0) + t2;
+			w[1] = -t1*Dy + 1.0 - t2;
+			w[0] = -t1*(Dy-1.0);
+			}
+
+			rd = gd = bd = 0.0;
+			for (int i=0; i<4; i++) {
+				rd += yr[i] * w[i];
+				gd += yg[i] * w[i];
+				bd += yb[i] * w[i];
+			}
+
+			*r = rd * mul;
+			*g = gd * mul;
+			*b = bd * mul;
+
+			//  if (xs==100 && ys==100)
+			//    printf ("r=%g, g=%g\n", *r, *g);
+		}
+
+		inline void interpolateTransformChannelsCubic (float** src, int xs, int ys, double Dx, double Dy, float *r, double mul) {
+			const double A=-0.85;
+
+			double w[4];
+
+			{
+			double t1, t2;
+			t1 = -A*(Dx-1.0)*Dx;
+			t2 = (3.0-2.0*Dx)*Dx*Dx;
+			w[3] = t1*Dx;
+			w[2] = t1*(Dx-1.0) + t2;
+			w[1] = -t1*Dx + 1.0 - t2;
+			w[0] = -t1*(Dx-1.0);
+			}
+
+			double rd;
+			double yr[4];
+
+			for (int k=ys, kx=0; k<ys+4; k++, kx++) {
+				rd = 0.0;
+				for (int i=xs, ix=0; i<xs+4; i++, ix++) {
+					rd += src[k][i] * w[ix];
+				}
+				yr[kx] = rd;
+			}
+
+
+			{
+			double t1, t2;
+			t1 = -A*(Dy-1.0)*Dy;
+			t2 = (3.0-2.0*Dy)*Dy*Dy;
+			w[3] = t1*Dy;
+			w[2] = t1*(Dy-1.0) + t2;
+			w[1] = -t1*Dy + 1.0 - t2;
+			w[0] = -t1*(Dy-1.0);
+			}
+
+			rd = 0.0;
+			for (int i=0; i<4; i++)
+				rd += yr[i] * w[i];
+
+			*r = rd * mul;
+		}
 
 
 	public:
@@ -91,23 +191,30 @@ class ImProcFunctions {
 		void setScale         (double iscale);
 
 		bool needsTransform   ();
+		bool needsPCVignetting ();
 
 		void firstAnalysis    (Imagefloat* working, const ProcParams* params, LUTu & vhist16, double gamma);
 		void rgbProc          (Imagefloat* working, LabImage* lab, LUTf & hltonecurve, LUTf & shtonecurve, LUTf & tonecurve,
-		                       SHMap* shmap, int sat, LUTf & rCurve, LUTf & gCurve, LUTf & bCurve, const ToneCurve & customToneCurve1, const ToneCurve & customToneCurve2);
+		                       SHMap* shmap, int sat, LUTf & rCurve, LUTf & gCurve, LUTf & bCurve, const ToneCurve & customToneCurve1, const ToneCurve & customToneCurve2,
+		                       const ToneCurve & customToneCurvebw1,const ToneCurve & customToneCurvebw2, double &rrm, double &ggm, double &bbm, float &autor, float &autog, float &autob);
 		void rgbProc          (Imagefloat* working, LabImage* lab, LUTf & hltonecurve, LUTf & shtonecurve, LUTf & tonecurve,
 		                       SHMap* shmap, int sat, LUTf & rCurve, LUTf & gCurve, LUTf & bCurve, const ToneCurve & customToneCurve1, const ToneCurve & customToneCurve2,
+		                       const ToneCurve & customToneCurvebw1,const ToneCurve & customToneCurvebw2, double &rrm, double &ggm, double &bbm, float &autor, float &autog, float &autob,
 		                       double expcomp, int hlcompr, int hlcomprthresh);
 		void luminanceCurve   (LabImage* lold, LabImage* lnew, LUTf &curve);
-		void ciecam_02float	 (CieImage* ncie, float adap, int begh, int endh,  int pW, int pwb, LabImage* lab, const ProcParams* params , const ColorAppearance & customColCurve1, const ColorAppearance & customColCurve, const ColorAppearance & customColCurve3, LUTu &histLCAM, LUTu &histCCAM,int Iterates, int scale, float** buffer, bool execsharp, float &d);
-		void ciecam_02		 (CieImage* ncie, double adap, int begh, int endh,  int pW, int pwb, LabImage* lab, const ProcParams* params , const ColorAppearance & customColCurve1, const ColorAppearance & customColCurve, const ColorAppearance & customColCurve3, LUTu &histLCAM, LUTu &histCCAM,int Iterates, int scale, float** buffer, bool execsharp, double &d);
-		void chromiLuminanceCurve (int pW, LabImage* lold, LabImage* lnew, LUTf &acurve, LUTf &bcurve, LUTf & satcurve,LUTf & satclcurve, LUTf &curve, bool utili, bool autili, bool butili, bool ccutili, bool cclutili, LUTu &histCCurve);
-		void vibrance 		  (LabImage* lab);//Jacques' vibrance
+		void ciecam_02float   (CieImage* ncie, float adap, int begh, int endh,  int pW, int pwb, LabImage* lab, const ProcParams* params,
+		                       const ColorAppearance & customColCurve1, const ColorAppearance & customColCurve, const ColorAppearance & customColCurve3,
+		                       LUTu &histLCAM, LUTu &histCCAM, LUTf & CAMBrightCurveJ, LUTf & CAMBrightCurveQ, float &mean, int Iterates, int scale, float** buffer, bool execsharp, float &d);
+		void ciecam_02        (CieImage* ncie, double adap, int begh, int endh,  int pW, int pwb, LabImage* lab, const ProcParams* params,
+		                       const ColorAppearance & customColCurve1, const ColorAppearance & customColCurve, const ColorAppearance & customColCurve3,
+		                       LUTu &histLCAM, LUTu &histCCAM, LUTf & CAMBrightCurveJ, LUTf & CAMBrightCurveQ, float &mean, int Iterates, int scale, float** buffer, bool execsharp, double &d);
+		void chromiLuminanceCurve (int pW, LabImage* lold, LabImage* lnew, LUTf &acurve, LUTf &bcurve, LUTf & satcurve,LUTf & satclcurve, LUTf &clcurve, LUTf &curve, bool utili, bool autili, bool butili, bool ccutili, bool cclutili, bool clcutili, LUTu &histCCurve, LUTu &histCLurve, LUTu &histLCurve, LUTu &histLurve);
+		void vibrance         (LabImage* lab);//Jacques' vibrance
 		void colorCurve       (LabImage* lold, LabImage* lnew);
 		void sharpening       (LabImage* lab, float** buffer);
 		void sharpeningcam    (CieImage* ncie, float** buffer);
-		void transform        (Imagefloat* original, Imagefloat* transformed, int cx, int cy, int sx, int sy, int oW, int oH,
-                               double focalLen, double focalLen35mm, float focusDist, int rawRotationDeg, bool fullImage);
+		void transform        (Imagefloat* original, Imagefloat* transformed, int cx, int cy, int sx, int sy, int oW, int oH, int fW, int fH,
+		                       double focalLen, double focalLen35mm, float focusDist, int rawRotationDeg, bool fullImage);
 		void lab2monitorRgb   (LabImage* lab, Image8* image);
 		void resize           (Image16* src, Image16* dst, float dScale);
 		void deconvsharpening (LabImage* lab, float** buffer);
@@ -148,10 +255,10 @@ class ImProcFunctions {
 	//	void WaveletDenoiseAll(wavelet_decomposition &WaveletCoeffs_L, wavelet_decomposition &WaveletCoeffs_a, 
 	//						   wavelet_decomposition &WaveletCoeffs_b, float noisevar_L, float noisevar_ab, wavelet_decomposition &wch, NoiImage * noi );
 		void WaveletDenoiseAll(wavelet_decomposition &WaveletCoeffs_L, wavelet_decomposition &WaveletCoeffs_a, 
-							   wavelet_decomposition &WaveletCoeffs_b, float noisevar_L, float noisevar_ab, LabImage * noi );
+							   wavelet_decomposition &WaveletCoeffs_b, float noisevar_L, float noisevar_abr, float noisevar_abb, LabImage * noi );
 							   
 		void WaveletDenoiseAll_BiShrink(wavelet_decomposition &WaveletCoeffs_L, wavelet_decomposition &WaveletCoeffs_a, 
-										wavelet_decomposition &WaveletCoeffs_b, float noisevar_L, float noisevar_ab );
+										wavelet_decomposition &WaveletCoeffs_b, float noisevar_L, float noisevar_abr, float noisevar_abb, LabImage * noi);
 		//void BiShrink(float * ReCoeffs, float * ImCoeffs, float * ReParents, float * ImParents, 
 		//			  int W, int H, int level, int padding, float noisevar);
 		//void Shrink(float ** WavCoeffs, int W, int H, int level, float noisevar);
@@ -159,15 +266,15 @@ class ImProcFunctions {
 	//				   int W_L, int H_L, int W_ab, int H_ab, int W_h, int H_h, int skip_L, int skip_ab, int skip_h, float noisevar_L, float noisevar_ab, float **WavCoeffs_h, LabImage * noi);
 
 		void ShrinkAll(float ** WavCoeffs_L, float ** WavCoeffs_a, float ** WavCoeffs_b, int level, 
-					   int W_L, int H_L, int W_ab, int H_ab, int skip_L, int skip_ab, float noisevar_L, float noisevar_ab, LabImage * noi);
+					   int W_L, int H_L, int W_ab, int H_ab, int skip_L, int skip_ab, float noisevar_L, float noisevar_abr, float noisevar_abb,LabImage * noi);
 
 		float MadMax(float * HH_Coeffs, int &max, int datalen);
 		
 		// pyramid equalizer
-		void dirpyr_equalizer    (float ** src, float ** dst, int srcwidth, int srcheight, const double * mult);//Emil's directional pyramid equalizer
-		void dirpyr_equalizercam    (CieImage* ncie, float ** src, float ** dst, int srcwidth, int srcheight, const double * mult, bool execdir );//Emil's directional pyramid equalizer
-		void dirpyr_channel      (float ** data_fine, float ** data_coarse, int width, int height, LUTf & rangefn, int level, int scale, const double * mult  );
-		void idirpyr_eq_channel  (float ** data_coarse, float ** data_fine, float ** buffer, int width, int height, int level, const double * mult );
+		void dirpyr_equalizer    (float ** src, float ** dst, int srcwidth, int srcheight, const double * mult, const double dirpyrThreshold);//Emil's directional pyramid equalizer
+		void dirpyr_equalizercam    (CieImage* ncie, float ** src, float ** dst, int srcwidth, int srcheight, const double * mult, const double dirpyrThreshold, bool execdir );//Emil's directional pyramid equalizer
+		void dirpyr_channel      (float ** data_fine, float ** data_coarse, int width, int height, int level, int scale );
+		void idirpyr_eq_channel  (float ** data_coarse, float ** data_fine, float ** buffer, int width, int height, int level, const double * mult, const double dirpyrThreshold );
 
 		void defringe       (LabImage* lab);
 		void defringecam    (CieImage* ncie);
@@ -177,7 +284,7 @@ class ImProcFunctions {
 		void PF_correct_RTcam (CieImage * src, CieImage * dst, double radius, int thresh);
 		void Badpixelscam(CieImage * src, CieImage * dst, double radius, int thresh, int mode);
 
-		Image8*     lab2rgb   (LabImage* lab, int cx, int cy, int cw, int ch, Glib::ustring profile);
+		Image8*     lab2rgb   (LabImage* lab, int cx, int cy, int cw, int ch, Glib::ustring profile, bool standard_gamma);
 		Image16*    lab2rgb16b (LabImage* lab, int cx, int cy, int cw, int ch, Glib::ustring profile, Glib::ustring profi, Glib::ustring gam, bool freegamma, double gampos, double slpos, double &ga0, double &ga1, double &ga2, double &ga3, double &ga4, double &ga5, double &ga6);// for gamma output		
 		Image16*    lab2rgb16 (LabImage* lab, int cx, int cy, int cw, int ch, Glib::ustring profile);//without gamma ==>default
        // CieImage *ciec;    
