@@ -19,20 +19,23 @@
 #include "rtengine.h"
 #include "iccstore.h"
 #include "dcp.h"
+#include "camconst.h"
+#include "rawimagesource.h"
 #include "improcfun.h"
 #include "improccoordinator.h"
 #include "dfmanager.h"
 #include "ffmanager.h"
 #include "rtthumbnail.h"
 #include "../rtgui/profilestore.h"
+#include "../rtgui/threadutils.h"
 
 namespace rtengine {
 
 const Settings* settings;
 
-Glib::Mutex* lcmsMutex = NULL;
+MyMutex* lcmsMutex = NULL;
 
-int init (const Settings* s, Glib::ustring baseDir) {
+int init (const Settings* s, Glib::ustring baseDir, Glib::ustring userSettingsDir) {
 
     settings = s;
     iccStore->init (s->iccDirectory, baseDir + "/iccprofiles");
@@ -40,13 +43,15 @@ int init (const Settings* s, Glib::ustring baseDir) {
 
     dcpStore->init (baseDir + "/dcpprofiles");
 
+    CameraConstantsStore::initCameraConstants (baseDir, userSettingsDir);
     profileStore.init ();
     ProcParams::init ();
-    Color::init();
+    Color::init ();
+    RawImageSource::init ();
     ImProcFunctions::initCache ();
     Thumbnail::initGamma ();
     delete lcmsMutex;
-    lcmsMutex = new Glib::Mutex;
+    lcmsMutex = new MyMutex;
     dfm.init( s->darkFramesPath );
     ffm.init( s->flatFieldsPath );
 	return 0;
@@ -58,6 +63,7 @@ void cleanup () {
     Color::cleanup ();
     ImProcFunctions::cleanupCache ();
     Thumbnail::cleanupGamma ();
+    RawImageSource::cleanup ();
 }
 
 StagedImageProcessor* StagedImageProcessor::create (InitialImage* initialImage) {
