@@ -70,7 +70,7 @@ enum ProcEvent {
     EvCDNEnabled=45,  // obsolete
     EvBlendCMSMatrix=46,
     EvDCPToneCurve=47,
-    EvPrefProfile=48,
+    EvDCPIlluminant=48,
     EvSHEnabled=49,
     EvSHHighlights=50,
     EvSHShadows=51,
@@ -88,12 +88,12 @@ enum ProcEvent {
     EvCrop=63,
     EvCACorr=64,
     EvHREnabled=65,
-    EvHRAmount=66,
+    EvHRAmount=66,   //obsolete
     EvHRMethod=67,
     EvWProfile=68,
     EvOProfile=69,
     EvIProfile=70,
-    EvVignetting=71,
+    EvVignettingAmount=71,
     EvChMixer=72,
     EvResizeScale=73,
     EvResizeMethod=74,
@@ -188,7 +188,7 @@ enum ProcEvent {
 	EvRGBgCurve=163,
 	EvRGBbCurve=164,
 	EvNeutralExp=165,
-	EvLBWtoning=166,
+//	EvLBWtoning=166,  -- can be reused --
 	EvLCCCurve=167,
 	EvLCHCurve=168,
 	EvVibranceSkinTonesCurve=169,
@@ -229,8 +229,52 @@ enum ProcEvent {
 	EvDemosaicLMMSEIter=203,
 	EvCATbadpix=204,
 	EvCATAutoAdap=205,
-	EvPFCurve=206,	
-	NUMOFEVENTS=207
+	EvPFCurve=206,
+	EvWBequal=207,
+	EvWBequalbo=208,
+	EvGradientDegree=209,
+	EvGradientEnabled=210,
+	EvPCVignetteStrength=211,
+	EvPCVignetteEnabled=212,
+	EvBWChmixEnabled=213,
+	EvBWred=214,
+	EvBWgreen=215,
+	EvBWblue=216,
+	EvBWredgam=217,
+	EvBWgreengam=218,
+	EvBWbluegam=219,
+	EvBWfilter=220,
+	EvBWsetting=221,
+	EvBWoran=222,
+	EvBWyell=223,
+	EvBWcyan=224,
+	EvBWmag=225,
+	EvBWpur=226,
+	EvBWLuminanceEqual=227,
+	EvBWChmixEnabledLm=228,
+	EvBWmethod=229,
+	EvBWBeforeCurve=230,
+	EvBWBeforeCurveMode=231,
+	EvBWAfterCurve=232,
+	EvBWAfterCurveMode=233,
+	EvAutoch=234,
+//	EvFixedch=235,   -- can be reused --
+	EvNeutralBW=236,
+	EvGradientFeather=237,
+	EvGradientStrength=238,
+	EvGradientCenter=239,
+	EvPCVignetteFeather=240,
+	EvPCVignetteRoundness=241,
+	EvVignettingRadius=242,
+	EvVignettingStrenght=243,
+	EvVignettingCenter=244,
+	EvLCLCurve=245,
+	EvLLHCurve=246,
+	EvLHHCurve=247,
+	EvDirPyrEqualizerThreshold=248,
+	EvDPDNenhance=249,	
+	
+	NUMOFEVENTS=250
 };
 }
 #endif
