@@ -24,6 +24,7 @@
 #include <cassert>
 #include <cstdio>
 #include <cmath>
+#include "LUT.h"
 
 class ParamsEdited;
 
@@ -190,6 +191,8 @@ class ToneCurveParams {
 
         bool        autoexp;
         double      clip;
+        bool        hrenabled;  // Highlight Reconstruction enabled
+        Glib::ustring method;   // Highlight Reconstruction's method
         double      expcomp;
         std::vector<double>   curve;
         std::vector<double>   curve2;
@@ -200,8 +203,10 @@ class ToneCurveParams {
         int         contrast;
         int         saturation;
         int         shcompr;
-        int         hlcompr;
-        int         hlcomprthresh;
+        int         hlcompr;        // Highlight Recovery's compression
+        int         hlcomprthresh;  // Highlight Recovery's threshold
+
+        static bool HLReconstructionNecessary(LUTu &histRedRaw, LUTu &histGreenRaw, LUTu &histBlueRaw);
 };
 
 /**
@@ -215,14 +220,16 @@ class LCurveParams {
         std::vector<double>   bcurve;
         std::vector<double>   cccurve;
         std::vector<double>   chcurve;
+        std::vector<double>   lhcurve;
+        std::vector<double>   hhcurve;
         std::vector<double>   lccurve;
+        std::vector<double>   clcurve;
         int     brightness;
         int     contrast;
         int     chromaticity;
         bool    avoidcolorshift;
         double  rstprotection;
-        bool    bwtoning;
-		bool    lcredsk;
+        bool    lcredsk;
 };
 
 /**
@@ -315,6 +322,7 @@ enum WBTypes {
     WBT_DAYLIGHT,
     WBT_CLOUDY,
     WBT_SHADE,
+    WBT_WATER,
     WBT_TUNGSTEN,
     WBT_FLUORESCENT,
     WBT_LAMP,
@@ -330,8 +338,10 @@ public:
     enum WBTypes type;
     Glib::ustring GUILabel;
     int temperature;
+    double green;
+    double equal;
 
-    WBEntry(Glib::ustring p, enum WBTypes t, Glib::ustring l, int temp) : ppLabel(p), type(t), GUILabel(l), temperature(temp) {};
+    WBEntry(Glib::ustring p, enum WBTypes t, Glib::ustring l, int temp, double green, double equal) : ppLabel(p), type(t), GUILabel(l), temperature(temp), green(green), equal(equal) {};
 };
 
 class WBParams {
@@ -341,6 +351,7 @@ class WBParams {
         Glib::ustring   method;
         int             temperature;
         double          green;
+        double          equal;
 
         static void     init();
         static void     cleanup();
@@ -439,7 +450,7 @@ class DefringeParams {
     public:
         bool    enabled;
         double  radius;
-        int     threshold;
+        float   threshold;
         std::vector<double> huecurve;
 };
 
@@ -461,6 +472,8 @@ class DirPyrDenoiseParams {
 
     public:
         bool    enabled;
+        bool    enhance;
+		
         bool    perform;
         double  luma;
         double  Ldetail;
@@ -568,8 +581,34 @@ public:
 class PerspectiveParams {
 
     public:
-		double  horizontal;
-		double  vertical;
+        double  horizontal;
+        double  vertical;
+};
+
+/**
+  * Parameters of the gradient filter
+  */
+class GradientParams {
+
+    public:
+        bool   enabled;
+        double degree;
+        int    feather;
+        double strength;
+        int    centerX;
+        int    centerY;
+};
+
+/**
+  * Parameters of the post-crop vignette filter
+  */
+class PCVignetteParams {
+
+    public:
+        bool   enabled;
+        double strength;
+        int    feather;
+        int    roundness;
 };
 
 /**
@@ -580,9 +619,9 @@ class VignettingParams {
     public:
         int  amount;
         int  radius;
-	int  strength;
-	int  centerX;
-	int  centerY;
+        int  strength;
+        int  centerX;
+        int  centerY;
 };
 
 /**
@@ -596,6 +635,41 @@ class ChannelMixerParams {
         int blue[3];
 };
 
+class BlackWhiteParams {
+
+    public:
+        enum eTCModeId {
+            TC_MODE_STD_BW,               // Standard modes, the curve is applied on all component individually
+            TC_MODE_WEIGHTEDSTD_BW,       // Weighted standard mode
+            TC_MODE_FILMLIKE_BW,          // Film-like mode, as defined in Adobe's reference code
+            TC_MODE_SATANDVALBLENDING_BW  // Modify the Saturation and Value channel
+        };
+
+        std::vector<double> beforeCurve;
+        eTCModeId beforeCurveMode;
+        std::vector<double> afterCurve;
+        eTCModeId afterCurveMode;
+
+        std::vector<double> luminanceCurve;
+        bool autoc;
+        bool enabledcc;
+        bool enabled;
+        Glib::ustring filter;
+        Glib::ustring setting;
+        Glib::ustring method;
+        int mixerRed;
+        int mixerOrange;
+        int mixerYellow;
+        int mixerGreen;
+        int mixerCyan;
+        int mixerBlue;
+        int mixerMagenta;
+        int mixerPurple;
+        int gammaRed;
+        int gammaGreen;
+        int gammaBlue;
+};
+
 /**
   * Parameters of the c/a correction
   */
@@ -609,13 +683,14 @@ class CACorrParams {
 /**
   * Parameters of the highlight recovery
   */
+  /*
 class HRecParams {
 
     public:
         bool enabled;
-		Glib::ustring method;
+       Glib::ustring method;
 };
-
+*/
 /**
   * Parameters of the resizing
   */
@@ -639,8 +714,8 @@ class ColorManagementParams {
     public:
         Glib::ustring input;
         bool          toneCurve;
-        bool          blendCMSMatrix;
-        short preferredProfile;
+        bool          blendCMSMatrix; // setting no longer used
+        int dcpIlluminant;
         Glib::ustring working;
         Glib::ustring output;
         static const Glib::ustring NoICMString;      
@@ -669,7 +744,8 @@ class DirPyrEqualizerParams {
 	
 	public:
 		bool enabled;
-		double mult[8];
+		double mult[5];
+		double threshold;
 };
 	
 /**
@@ -681,6 +757,7 @@ class HSVEqualizerParams {
 		std::vector<double>   hcurve;
 		std::vector<double>   scurve;
 		std::vector<double>   vcurve;
+		
 };
 
 
@@ -767,10 +844,12 @@ class ProcParams {
         DistortionParams        distortion;      ///< Lens distortion correction parameters
         LensProfParams          lensProf;        ///< Lens correction profile parameters
         PerspectiveParams       perspective;     ///< Perspective correction parameters
+        GradientParams          gradient;        ///< Gradient filter parameters
+        PCVignetteParams        pcvignette;      ///< Post-crop vignette filter parameters
         CACorrParams            cacorrection;    ///< Lens c/a correction parameters
         VignettingParams        vignetting;      ///< Lens vignetting correction parameters
         ChannelMixerParams      chmixer;         ///< Channel mixer parameters
-        HRecParams              hlrecovery;      ///< Highlight recovery parameters
+        BlackWhiteParams        blackwhite;      ///< Black & White parameters
         ResizeParams            resize;          ///< Resize parameters
         ColorManagementParams   icm;             ///< profiles/color spaces used during the image processing
         RAWParams               raw;             ///< RAW parameters before demosaicing
@@ -798,10 +877,12 @@ class ProcParams {
         * save the same file in two different location, i.e. the cache and the image's directory
         * @param fname   the name of the first file (can be an empty string)
         * @param fname2  the name of the second file (can be an empty string) (optional)
+        * @param fnameAbsolute set to false if embedded filenames (if any, darkframe/flatfield) should be stored as relative
+        * filenames if they are inside the same directory or in a sub-directory to fname's directory.
         * @param pedited pointer to a ParamsEdited object (optional) to store which values has to be saved
         * @return Error code (=0 if all supplied filenames where created correctly)
         */
-        int     save        (Glib::ustring fname, Glib::ustring fname2 = "", ParamsEdited* pedited=NULL) const;
+        int     save        (Glib::ustring fname, Glib::ustring fname2 = "", bool fnameAbsolute = true, ParamsEdited* pedited=NULL) const;
       /**
         * Loads the parameters from a file.
         * @param fname the name of the file
@@ -859,6 +940,17 @@ class PartialProfile {
         const void applyTo  (ProcParams *destParams) const ;
 };
 
+/**
+  * This class automatically create the pparams and pedited instance in the constructor,
+  * and automatically delete them in the destructor. This class has been mostly created
+  * to be used with vectors, which use the default constructor/destructor
+  */
+class AutoPartialProfile : public PartialProfile {
+    public:
+        AutoPartialProfile() : PartialProfile(true) {}
+        ~AutoPartialProfile() { deleteInstance(); }
+};
+
 }
 }
 #endif
