@@ -7,7 +7,7 @@
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
- * 
+ *
  *  RawTherapee is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -28,6 +28,7 @@
 #include "color.h"
 #include "iimage.h"
 #include "../rtgui/cacheimagedata.h"
+#include "../rtgui/threadutils.h"
 
 #define HR_SCALE 2
 
@@ -62,23 +63,27 @@ template<class T> void freeArray2 (T** a, int H) {
 class RawImageSource : public ImageSource {
 
     private:
+        static DiagonalCurve *phaseOneIccCurve;
+        static DiagonalCurve *phaseOneIccCurveInv;
         static LUTf invGrad;  // for fast_demosaic
         static LUTf initInvGrad ();
         static bool findInputProfile(Glib::ustring inProfile, cmsHPROFILE embedded, std::string camName, DCPProfile **dcpProf, cmsHPROFILE& in);
-        static void colorSpaceConversion (Imagefloat* im, ColorManagementParams &cmp, float rawWhitePoint, cmsHPROFILE embedded, cmsHPROFILE camprofile, double cam[3][3], const std::string &camName);
+        static void colorSpaceConversion (Imagefloat* im, ColorManagementParams &cmp, ColorTemp &wb, float rawWhitePoint, cmsHPROFILE embedded, cmsHPROFILE camprofile, double cam[3][3], const std::string &camName);
 
     protected:
-        Glib::Mutex getImageMutex;  // locks getImage
+        MyMutex getImageMutex;  // locks getImage
 
         int W, H;
-        ColorTemp wb;
+        ColorTemp camera_wb;
         ProgressListener* plistener;
         float scale_mul[4]; // multiplier for each color
         float c_black[4]; // copy of cblack Dcraw for black level
+        float c_white[4];
         float cblacksom[4];
-        double camwb_red;
-        double camwb_green;
-        double camwb_blue;
+        float ref_pre_mul[4];
+        double refwb_red;
+        double refwb_green;
+        double refwb_blue;
         double rgb_cam[3][3];
         double cam_rgb[3][3];
         double xyz_cam[3][3];
@@ -92,6 +97,7 @@ class RawImageSource : public ImageSource {
         int max_3[3];
         float chmax[4],hlmax[4];
         double initialGain; // initial gain calculated after scale_colors
+        double camInitialGain;
         double defGain;
         bool full;
         cmsHPROFILE camProfile;
@@ -108,7 +114,7 @@ class RawImageSource : public ImageSource {
         array2D<float> rawData;  // holds preprocessed pixel values, rowData[i][j] corresponds to the ith row and jth column
 
         // the interpolated green plane:
-        array2D<float> green; 
+        array2D<float> green;
         // the interpolated red plane:
         array2D<float> red;
         // the interpolated blue plane:
@@ -141,7 +147,7 @@ class RawImageSource : public ImageSource {
         void        demosaic    (const RAWParams &raw);
         void        flushRawData      ();
         void        flushRGB          ();
-        void        HLRecovery_Global (HRecParams hrp);
+        void        HLRecovery_Global (ToneCurveParams hrp);
         void        refinement_lassus (int PassCount);
         void        refinement(int PassCount);
 
@@ -151,10 +157,10 @@ class RawImageSource : public ImageSource {
         void        cfaboxblur  (RawImage *riFlatFile, float* cfablur, int boxH, int boxW );
         void        scaleColors (int winx,int winy,int winw,int winh, const RAWParams &raw);// raw for cblack
 
-        void        getImage    (ColorTemp ctemp, int tran, Imagefloat* image, PreviewProps pp, HRecParams hrp, ColorManagementParams cmp, RAWParams raw);
-        ColorTemp   getWB       () { return wb; }
-        ColorTemp   getAutoWB   ();
-        ColorTemp   getSpotWB   (std::vector<Coord2D> &red, std::vector<Coord2D> &green, std::vector<Coord2D> &blue, int tran);
+        void        getImage    (ColorTemp ctemp, int tran, Imagefloat* image, PreviewProps pp, ToneCurveParams hrp, ColorManagementParams cmp, RAWParams raw);
+        ColorTemp   getWB       () { return camera_wb; }
+        void        getAutoWBMultipliers (double &rm, double &gm, double &bm);
+        ColorTemp   getSpotWB   (std::vector<Coord2D> &red, std::vector<Coord2D> &green, std::vector<Coord2D> &blue, int tran, double equal);
         bool        isWBProviderReady () { return rawData; }
 
         double      getDefGain  () { return defGain; }
@@ -173,13 +179,13 @@ class RawImageSource : public ImageSource {
         void        getAutoExpHistogram (LUTu & histogram, int& histcompr);
         void        getRAWHistogram (LUTu & histRedRaw, LUTu & histGreenRaw, LUTu & histBlueRaw);
 
-        void convertColorSpace(Imagefloat* image, ColorManagementParams cmp, RAWParams raw);
+        void convertColorSpace(Imagefloat* image, ColorManagementParams cmp, ColorTemp &wb, RAWParams raw);
         //static void colorSpaceConversion16 (Image16*    im, ColorManagementParams cmp, cmsHPROFILE embedded, cmsHPROFILE camprofile, double cam[3][3], std::string camName);
-        static void colorSpaceConversion   (Imagefloat* im, ColorManagementParams cmp, cmsHPROFILE embedded, cmsHPROFILE camprofile, double cam[3][3], std::string camName) {
-            colorSpaceConversion (im, cmp, 0.0f, embedded, camprofile, cam, camName);
+        static void colorSpaceConversion   (Imagefloat* im, ColorManagementParams cmp, ColorTemp &wb, cmsHPROFILE embedded, cmsHPROFILE camprofile, double cam[3][3], std::string camName) {
+            colorSpaceConversion (im, cmp, wb, 0.0f, embedded, camprofile, cam, camName);
         }
-        static void colorSpaceConversion   (Imagefloat* im, ColorManagementParams cmp, RAWParams raw, cmsHPROFILE embedded, cmsHPROFILE camprofile, double cam[3][3], std::string camName) {
-            colorSpaceConversion (im, cmp, float(raw.expos), embedded, camprofile, cam, camName);
+        static void colorSpaceConversion   (Imagefloat* im, ColorManagementParams cmp, ColorTemp &wb, RAWParams raw, cmsHPROFILE embedded, cmsHPROFILE camprofile, double cam[3][3], std::string camName) {
+            colorSpaceConversion (im, cmp, wb, float(raw.expos), embedded, camprofile, cam, camName);
         }
         static void inverse33 (const double (*coeff)[3], double (*icoeff)[3]);
 
@@ -194,6 +200,8 @@ class RawImageSource : public ImageSource {
         static void HLRecovery_Luminance (float* rin, float* gin, float* bin, float* rout, float* gout, float* bout, int width, float maxval);
         static void HLRecovery_CIELab (float* rin, float* gin, float* bin, float* rout, float* gout, float* bout, int width, float maxval, double cam[3][3], double icam[3][3]);
         static void HLRecovery_blend (float* rin, float* gin, float* bin, int width, float maxval, float* pre_mul, const RAWParams &raw, float* hlmax);
+	static void init ();
+	static void cleanup ();
 
     protected:
         typedef unsigned short ushort;
@@ -206,9 +214,9 @@ class RawImageSource : public ImageSource {
         inline  void interpolate_row_rb     (float* ar, float* ab, float* pg, float* cg, float* ng, int i);
         inline  void interpolate_row_rb_mul_pp (float* ar, float* ab, float* pg, float* cg, float* ng, int i, double r_mul, double g_mul, double b_mul, int x1, int width, int skip);
 
-        int  LinEqSolve( int nDim, float* pfMatr, float* pfVect, float* pfSolution);//Emil's CA auto correction
+        int  LinEqSolve( int nDim, double* pfMatr, double* pfVect, double* pfSolution);//Emil's CA auto correction
         void CA_correct_RT	(double cared, double cablue);
-        void ddct8x8s(int isgn, float **a);
+        void ddct8x8s(int isgn, float a[8][8]);
         void processRawWhitepoint (float expos, float preser);  // exposure before interpolation
 
         int  cfaCleanFromMap( PixelsMap &bitmapBads );
