@@ -68,7 +68,7 @@ RGBCURVE,         // EvToneCurveMode2,
 0,                // EvCDNEnabled:obsolete,
 ALL,              // EvBlendCMSMatrix,
 ALL,              // EvDCPToneCurve,
-ALL,              // EvPrefProfile,
+ALL,              // EvDCPIlluminant,
 RETINEX,          // EvSHEnabled,
 RGBCURVE,         // EvSHHighlights,
 RGBCURVE,         // EvSHShadows,
@@ -91,7 +91,7 @@ ALLNORAW,         // EvHRMethod,
 ALL,              // EvWProfile,
 OUTPUTPROFIL,     // EvOProfile,
 ALL,              // EvIProfile,
-TRANSFORM,        // EvVignetting,
+TRANSFORM,        // EvVignettingAmount,
 RGBCURVE,         // EvChMixer,
 RESIZE,           // EvResizeScale,
 RESIZE,           // EvResizeMethod,
@@ -186,7 +186,7 @@ RGBCURVE,         // EvRGBrCurve
 RGBCURVE,         // EvRGBgCurve
 RGBCURVE,         // EvRGBbCurve
 RGBCURVE,         // EvNeutralExp
-LUMINANCECURVE,   // EvLBWtoning
+NONE,             // --unused--,
 LUMINANCECURVE,   // EvLCCurve
 LUMINANCECURVE,   // EvLCHCurve
 RGBCURVE,         // EvVibranceSkinTonesCurve
@@ -198,35 +198,79 @@ LUMINANCECURVE,   // EvCATDegree
 LUMINANCECURVE,   // EvCATMethodsur
 LUMINANCECURVE,   // EvCATAdapscen
 LUMINANCECURVE,   // EvCATAdapLum
-LUMINANCECURVE,    // EvCATMethodWB
+LUMINANCECURVE,   // EvCATMethodWB
 LUMINANCECURVE,   // EvCATJLight
 LUMINANCECURVE,   // EvCATChroma
-LUMINANCECURVE,    // EvCATAutoDegree
-LUMINANCECURVE,    // EvCATContrast
-LUMINANCECURVE,    // EvCATSurr
-LUMINANCECURVE,    // EvCATgamut
-LUMINANCECURVE,    // EvCATmethodalg
-LUMINANCECURVE,    // EvCATRstpro
-LUMINANCECURVE,    // EvCATQbright
-LUMINANCECURVE,    // EvCATQContrast
-LUMINANCECURVE,    // EvCATSChroma
-LUMINANCECURVE,    // EvCATMchroma
-LUMINANCECURVE,    // EvCAThue
-LUMINANCECURVE,    // EvCATcurve1
-LUMINANCECURVE,    // EvCATcurve2
-LUMINANCECURVE,    // EvCATcurvemode1
-LUMINANCECURVE,    // EvCATcurvemode2
-LUMINANCECURVE,    // EvCATcurve3
-LUMINANCECURVE,    // EvCATcurvemode3
-LUMINANCECURVE,    // EvCATdatacie
-LUMINANCECURVE,    // EvCATtonecie
+LUMINANCECURVE,   // EvCATAutoDegree
+LUMINANCECURVE,   // EvCATContrast
+LUMINANCECURVE,   // EvCATSurr
+LUMINANCECURVE,   // EvCATgamut
+LUMINANCECURVE,   // EvCATmethodalg
+LUMINANCECURVE,   // EvCATRstpro
+LUMINANCECURVE,   // EvCATQbright
+LUMINANCECURVE,   // EvCATQContrast
+LUMINANCECURVE,   // EvCATSChroma
+LUMINANCECURVE,   // EvCATMchroma
+LUMINANCECURVE,   // EvCAThue
+LUMINANCECURVE,   // EvCATcurve1
+LUMINANCECURVE,   // EvCATcurve2
+LUMINANCECURVE,   // EvCATcurvemode1
+LUMINANCECURVE,   // EvCATcurvemode2
+LUMINANCECURVE,   // EvCATcurve3
+LUMINANCECURVE,   // EvCATcurvemode3
+LUMINANCECURVE,   // EvCATdatacie
+LUMINANCECURVE,   // EvCATtonecie
 ALLNORAW,         // EvDPDNbluechro
 ALLNORAW,         // EvDPDNperform
-ALLNORAW,         //EvDPDNmet
+ALLNORAW,         // EvDPDNmet
 DEMOSAIC,         // EvDemosaicLMMSEIter
-LUMINANCECURVE,    // EvCATbadpix
-LUMINANCECURVE,    // EvCATAutoadap
-DEFRINGE         // EvPFCurve,
+LUMINANCECURVE,   // EvCATbadpix
+LUMINANCECURVE,   // EvCATAutoadap
+DEFRINGE,         // EvPFCurve
+WHITEBALANCE,     // EvWBequal
+WHITEBALANCE,     // EvWBequalbo
+TRANSFORM,        // EvGradientDegree
+TRANSFORM,        // EvGradientEnabled
+TRANSFORM,        // EvPCVignetteStrength
+TRANSFORM,        // EvPCVignetteEnabled
+RGBCURVE,         // EvBWChmixEnabled
+RGBCURVE,         // EvBWred
+RGBCURVE,         // EvBWgreen
+RGBCURVE,         // EvBWblue
+RGBCURVE,         // EvBWredgam
+RGBCURVE,         // EvBWgreengam
+RGBCURVE,         // EvBWbluegam
+RGBCURVE,         // EvBWfilter
+RGBCURVE,         // EvBWsetting
+RGBCURVE,         // EvBWoran
+RGBCURVE,         // EvBWyell
+RGBCURVE,         // EvBWcyan
+RGBCURVE,         // EvBWmag
+RGBCURVE,         // EvBpur
+RGBCURVE,         // EvBWLuminanceEqual
+RGBCURVE,         // EvBWChmixEnabledLm
+RGBCURVE,         // EvBWmethod
+RGBCURVE,         // EvBWBeforeCurve
+RGBCURVE,         // EvBWBeforeCurveMode
+RGBCURVE,         // EvBWAfterCurve
+RGBCURVE,         // EvBWAfterCurveMode
+RGBCURVE,         // EvAutoch
+NONE,             // --unused--
+RGBCURVE,         // EvNeutralBW
+TRANSFORM,        // EvGradientFeather
+TRANSFORM,        // EvGradientStrength
+TRANSFORM,        // EvGradientCenter
+TRANSFORM,        // EvPCVignetteFeather
+TRANSFORM,        // EvPCVignetteRoundness
+TRANSFORM,        // EvVignettingRadius,
+TRANSFORM,        // EvVignettingStrength
+TRANSFORM,        // EvVignettingCenter
+LUMINANCECURVE,   // EvLCLCurve
+LUMINANCECURVE,   // EvLLHCurve
+LUMINANCECURVE,   // EvLHHCurve
+DIRPYREQUALIZER,  // EvDirPyrEqualizerThreshold
+ALLNORAW          // EvDPDNenhance
+
 
 //LUMINANCECURVE    // EvCATsharpcie
 
