@@ -30,6 +30,7 @@
 #include "rawmetadatalocation.h"
 #include "iimage.h"
 #include "utils.h"
+#include "../rtgui/threadutils.h"
 #include "settings.h"
 #include "LUT.h"
 /**
@@ -231,7 +232,7 @@ namespace rtengine {
               * @param histBlue is the array of size 256 containing the histogram of the blue channel
               * @param histLuma is the array of size 256 containing the histogram of the luminance channel
               * other for curves backgrounds, histRAW is RAW without colors */
-            virtual void histogramChanged (LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma, LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histLCAM, LUTu & histCCAM,
+            virtual void histogramChanged (LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma, LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histCLurve, LUTu & histLLCurve, LUTu & histLCAM, LUTu & histCCAM,
                 LUTu & histRedRaw, LUTu & histGreenRaw, LUTu & histBlueRaw, LUTu & histChroma) {}
     };
 
@@ -241,20 +242,30 @@ namespace rtengine {
             virtual ~AutoExpListener() {}
             /** This member function is called when the auto exposure has been recomputed.
               * @param brightness is the new brightness value (in logarithmic scale)
-              * @param, bright is the new ...
+              * @param bright is the new ...
               * @param black is the new black level (measured in absolute pixel data)
               * @param contrast is the new contrast values
               * @param hlcompr is the new highlight recovery amount 
-              * #param hlcomprthresh is the new threshold for hlcompr*/
-            virtual void autoExpChanged (double brightness, int bright, int contrast, int black, int hlcompr, int hlcomprthresh) {}
+              * @param hlcomprthresh is the new threshold for hlcompr
+              * @param hlrecons set to true if HighLight Reconstruction is enabled */
+            virtual void autoExpChanged (double brightness, int bright, int contrast, int black, int hlcompr, int hlcomprthresh, bool hlrecons) {}
     };
 
-	class AutoCamListener {
+    class AutoCamListener {
+        public :
+            virtual ~AutoCamListener() {}
+            virtual void autoCamChanged (double ccam) {}
+            virtual void adapCamChanged (double cadap) {}
+    };
+	
+	class AutoBWListener {
 		public :
-			virtual void autoCamChanged (double ccam) {}
-			virtual void adapCamChanged (double cadap) {}
+            virtual ~AutoBWListener() {}
+			virtual void BWChanged (double redbw, double greenbw, double bluebw) {}
 			
-	};		
+ 	};		
+	
+
     /** This class represents a detailed part of the image (looking through a kind of window).
       * It can be created and destroyed with the appropriate members of StagedImageProcessor.
       * Several crops can be assigned to the same image.   */
@@ -321,7 +332,7 @@ namespace rtengine {
             /** Creates and returns a Crop instance that acts as a window on the image */
             virtual DetailedCrop* createCrop  () =0;
 
-            virtual bool        getAutoWB   (double& temp, double& green) =0;
+            virtual bool        getAutoWB   (double& temp, double& green, double equal) =0;
             virtual void        getCamWB    (double& temp, double& green) =0;
             virtual void        getSpotWB  (int x, int y, int rectSize, double& temp, double& green) =0;
             virtual void        getAutoCrop (double ratio, int &x, int &y, int &w, int &h) =0;
@@ -335,6 +346,7 @@ namespace rtengine {
             virtual void        setHistogramListener    (HistogramListener *l) =0;
             virtual void        setPreviewImageListener (PreviewImageListener* l) =0;
             virtual void        setAutoCamListener      (AutoCamListener* l) =0;
+            virtual void        setAutoBWListener     	(AutoBWListener* l) =0;
 
             virtual ~StagedImageProcessor () {}
 
@@ -347,18 +359,21 @@ namespace rtengine {
 
 
 /** 
-  *  Initializes the RT engine
-  * @param s is a struct of basic settings, baseDir of RT */
-    int init (const Settings* s, Glib::ustring baseDir);
+  * @brief Initializes the RT engine
+  * @param s is a struct of basic settings
+  * @param baseDir base directory of RT's installation dir
+  * @param userSettingsDir RT's base directory in the user's settings dir */
+    int init (const Settings* s, Glib::ustring baseDir, Glib::ustring userSettingsDir);
 
 /** Cleanup the RT engine (static variables) */
     void cleanup ();
 
 /** Returns the available working profile names
   * @return a vector of the available working profile names */
-    std::vector<std::string> getWorkingProfiles ();
-/** return gamma	*/
-    std::vector<std::string> getGamma ();
+    std::vector<Glib::ustring> getWorkingProfiles ();
+/** Returns the available output gammas
+  * @return a vector of the available gamma names */
+    std::vector<Glib::ustring> getGamma ();
 
     /** This class  holds all the necessary informations to accomplish the full processing of the image */
     class ProcessingJob {
@@ -406,6 +421,7 @@ namespace rtengine {
                         * @param img is the result of the last ProcessingJob 
                         * @return the next ProcessingJob to process */  
             virtual ProcessingJob* imageReady (IImage16* img) =0;
+            virtual void error(Glib::ustring message) =0;
     };
 /** This function performs all the image processinf steps corresponding to the given ProcessingJob. It runs in the background, thus it returns immediately,
    * When it finishes, it calls the BatchProcessingListener with the resulting image and asks for the next job. It the listener gives a new job, it goes on 
@@ -417,7 +433,7 @@ namespace rtengine {
     void startBatchProcessing (ProcessingJob* job, BatchProcessingListener* bpl, bool tunnelMetaData);
 
 
-    extern Glib::Mutex* lcmsMutex;
+    extern MyMutex* lcmsMutex;
 }
 
 #endif
