@@ -18,6 +18,7 @@
  */
 #include "rtengine.h"
 #include "rtthumbnail.h"
+#include "../rtgui/options.h"
 #include "image8.h"
 #include <lcms2.h>
 #include "curves.h"
@@ -38,9 +39,11 @@
 #include "jpeg.h"
 #include "../rtgui/ppversion.h"
 
+extern Options options;
+
 namespace rtengine {
 
-Thumbnail* Thumbnail::loadFromImage (const Glib::ustring& fname, int &w, int &h, int fixwh, int deg) {
+Thumbnail* Thumbnail::loadFromImage (const Glib::ustring& fname, int &w, int &h, int fixwh, double wbEq) {
 
     StdImageSource imgSrc;
     if (imgSrc.load(fname)) {
@@ -48,33 +51,16 @@ Thumbnail* Thumbnail::loadFromImage (const Glib::ustring& fname, int &w, int &h,
     }
 
     ImageIO* img = imgSrc.getImageIO();
-
-    if (deg) {
-        img->rotate(deg);
-    }
   
     Thumbnail* tpp = new Thumbnail ();
 
-    tpp->camwbRed = 1.0;
-    tpp->camwbGreen = 1.0;
-    tpp->camwbBlue = 1.0;
-
-    tpp->embProfileLength = 0;
     unsigned char* data;
     img->getEmbeddedProfileData (tpp->embProfileLength, data);
     if (data && tpp->embProfileLength) {
         tpp->embProfileData = new unsigned char [tpp->embProfileLength];
         memcpy (tpp->embProfileData, data, tpp->embProfileLength);
     }
-    else {
-        tpp->embProfileLength = 0;
-        tpp->embProfileData = NULL;
-    }
     
-    tpp->redMultiplier = 1.0;
-    tpp->greenMultiplier = 1.0;
-    tpp->blueMultiplier = 1.0;
-
     tpp->scaleForSave = 8192;
     tpp->defGain = 1.0;
     tpp->gammaCorrected = false;
@@ -124,7 +110,13 @@ Thumbnail* Thumbnail::loadFromImage (const Glib::ustring& fname, int &w, int &h,
 
     if (n>0) {
         ColorTemp cTemp;
-        cTemp.mul2temp (avg_r/double(n), avg_g/double(n), avg_b/double(n), tpp->autowbTemp, tpp->autowbGreen);
+
+        tpp->redAWBMul   = avg_r/double(n);
+        tpp->greenAWBMul = avg_g/double(n);
+        tpp->blueAWBMul  = avg_b/double(n);
+        tpp->wbEqual = wbEq;
+
+        cTemp.mul2temp (tpp->redAWBMul, tpp->greenAWBMul, tpp->blueAWBMul, tpp->wbEqual, tpp->autoWBTemp, tpp->autoWBGreen);
     }
 
     tpp->init ();
@@ -178,21 +170,6 @@ Thumbnail* Thumbnail::loadQuickFromRaw (const Glib::ustring& fname, RawMetaDataL
 
     Thumbnail* tpp = new Thumbnail ();
 
-    tpp->camwbRed = 1.0;
-    tpp->camwbGreen = 1.0;
-    tpp->camwbBlue = 1.0;
-
-    tpp->embProfileLength = 0;
-    tpp->embProfile = NULL;
-    tpp->embProfileData = NULL;
-
-    tpp->redMultiplier = 1.0;
-    tpp->greenMultiplier = 1.0;
-    tpp->blueMultiplier = 1.0;
-
-    tpp->scaleForSave = 8192;
-    tpp->defGain = 1.0;
-    tpp->gammaCorrected = false;
     tpp->isRaw = 1;
     memset (tpp->colorMatrix, 0, sizeof(tpp->colorMatrix));
     tpp->colorMatrix[0][0] = 1.0;
@@ -212,13 +189,16 @@ Thumbnail* Thumbnail::loadQuickFromRaw (const Glib::ustring& fname, RawMetaDataL
     tpp->thumbImg = resizeTo<Image8>(w, h, TI_Nearest, img);
     delete img;
 
-    tpp->autowbTemp=2700;
-    tpp->autowbGreen=1.0;
-
     if (rotate && ri->get_rotateDegree() > 0) {
-        // Leaf .mos, Mamiya .mef and Phase One files have thumbnails already rotated.
-        if (ri->get_maker() != "Leaf" && ri->get_maker() != "Mamiya" && ri->get_maker() != "Phase One")  {
+        std::string fname = ri->get_filename();
+        std::string suffix = fname.length() > 4 ? fname.substr(fname.length()-3) : "";
+        for (int i = 0; i < suffix.length(); i++) suffix[i] = std::tolower(suffix[i]);
+        // Leaf .mos, Mamiya .mef and Phase One .iiq files have thumbnails already rotated.
+        if (suffix != "mos" && suffix != "mef" && suffix != "iiq")  {
             tpp->thumbImg->rotate(ri->get_rotateDegree());
+            // width/height may have changed after rotating
+            w = tpp->thumbImg->width;
+            h = tpp->thumbImg->height;
         }
     }
 
@@ -235,7 +215,24 @@ Thumbnail* Thumbnail::loadQuickFromRaw (const Glib::ustring& fname, RawMetaDataL
 #define FISBLUE(filter,row,col) \
 	((filter >> ((((row) << 1 & 14) + ((col) & 1)) << 1) & 3)==2 || !filter)
 
-Thumbnail* Thumbnail::loadFromRaw (const Glib::ustring& fname, RawMetaDataLocation& rml, int &w, int &h, int fixwh, bool rotate)
+RawMetaDataLocation Thumbnail::loadMetaDataFromRaw (const Glib::ustring& fname)
+{
+	RawMetaDataLocation rml;
+	rml.exifBase = -1;
+	rml.ciffBase = -1;
+	rml.ciffLength = -1;
+
+	RawImage ri(fname);
+	int r = ri.loadRaw(false);
+	if( !r ){
+		rml.exifBase = ri.get_exifBase();
+		rml.ciffBase = ri.get_ciffBase();
+		rml.ciffLength = ri.get_ciffLen();
+	}
+	return rml;
+}
+
+Thumbnail* Thumbnail::loadFromRaw (const Glib::ustring& fname, RawMetaDataLocation& rml, int &w, int &h, int fixwh, double wbEq, bool rotate)
 {
 	RawImage *ri= new RawImage (fname);
 	int r = ri->loadRaw(1,0);
@@ -307,6 +304,7 @@ Thumbnail* Thumbnail::loadFromRaw (const Glib::ustring& fname, RawMetaDataLocati
 
 	Imagefloat* tmpImg = new Imagefloat(tmpw, tmph);
 	if (ri->isBayer()) {
+		// demosaicing! (sort of)
 		for (int row = 1, y = 0; row < height - 1 && y < tmph; row += vskip, y++) {
 			rofs = row * width;
 			for (int col = firstgreen, x = 0; col < width - 1 && x < tmpw; col+= hskip, x++) {
@@ -314,10 +312,10 @@ Thumbnail* Thumbnail::loadFromRaw (const Glib::ustring& fname, RawMetaDataLocati
 				int g = image[ofs][1];
 				int r, b;
 				if (FISRED(filter,row,col+1)) {
-					r = (image[ofs + 1][0] + image[ofs - 1][0]) >> 1;
+					r = (image[ofs + 1    ][0] + image[ofs - 1    ][0]) >> 1;
 					b = (image[ofs + width][2] + image[ofs - width][2]) >> 1;
 				} else {
-					b = (image[ofs + 1][2] + image[ofs - 1][2]) >> 1;
+					b = (image[ofs + 1    ][2] + image[ofs - 1    ][2]) >> 1;
 					r = (image[ofs + width][0] + image[ofs - width][0]) >> 1;
 				}
 				tmpImg->r(y,x) = r;
@@ -354,18 +352,9 @@ Thumbnail* Thumbnail::loadFromRaw (const Glib::ustring& fname, RawMetaDataLocati
 					continue;
 				double fr = r - ur;
 				double fc = c - uc;
-				fImg->r(row,col) = (tmpImg->r(ur,uc) * (1 - fc)
-						+ tmpImg->r(ur,uc + 1) * fc) * (1 - fr)
-						+ (tmpImg->r(ur + 1,uc) * (1 - fc)
-								+ tmpImg->r(ur + 1,uc + 1) * fc) * fr;
-				fImg->g(row,col) = (tmpImg->g(ur,uc) * (1 - fc)
-						+ tmpImg->g(ur,uc + 1) * fc) * (1 - fr)
-						+ (tmpImg->g(ur + 1,uc) * (1 - fc)
-								+ tmpImg->g(ur + 1,uc + 1) * fc) * fr;
-				fImg->b(row,col) = (tmpImg->b(ur,uc) * (1 - fc)
-						+ tmpImg->b(ur,uc + 1) * fc) * (1 - fr)
-						+ (tmpImg->b(ur + 1,uc) * (1 - fc)
-								+ tmpImg->b(ur + 1,uc + 1) * fc) * fr;
+				fImg->r(row,col) = (tmpImg->r(ur,uc)*(1-fc) + tmpImg->r(ur,uc + 1)*fc) * (1-fr)   +   (tmpImg->r(ur + 1,uc)*(1-fc) + tmpImg->r(ur + 1,uc + 1)*fc) * fr;
+				fImg->g(row,col) = (tmpImg->g(ur,uc)*(1-fc) + tmpImg->g(ur,uc + 1)*fc) * (1-fr)   +   (tmpImg->g(ur + 1,uc)*(1-fc) + tmpImg->g(ur + 1,uc + 1)*fc) * fr;
+				fImg->b(row,col) = (tmpImg->b(ur,uc)*(1-fc) + tmpImg->b(ur,uc + 1)*fc) * (1-fr)   +   (tmpImg->b(ur + 1,uc)*(1-fc) + tmpImg->b(ur + 1,uc + 1)*fc) * fr;
 			}
 		delete tmpImg;
 		tmpImg = fImg;
@@ -463,12 +452,13 @@ Thumbnail* Thumbnail::loadFromRaw (const Glib::ustring& fname, RawMetaDataLocati
 	double greens = avg_g / gn * tpp->camwbGreen;
 	double blues = avg_b / bn * tpp->camwbBlue;
 
-	double rm = ri->get_rgb_cam(0, 0) * reds + ri->get_rgb_cam(0, 1) * greens + ri->get_rgb_cam(0, 2) * blues;
-	double gm = ri->get_rgb_cam(1, 0) * reds + ri->get_rgb_cam(1, 1) * greens + ri->get_rgb_cam(1, 2) * blues;
-	double bm = ri->get_rgb_cam(2, 0) * reds + ri->get_rgb_cam(2, 1) * greens + ri->get_rgb_cam(2, 2) * blues;
+	tpp->redAWBMul   = ri->get_rgb_cam(0, 0) * reds + ri->get_rgb_cam(0, 1) * greens + ri->get_rgb_cam(0, 2) * blues;
+	tpp->greenAWBMul = ri->get_rgb_cam(1, 0) * reds + ri->get_rgb_cam(1, 1) * greens + ri->get_rgb_cam(1, 2) * blues;
+	tpp->blueAWBMul  = ri->get_rgb_cam(2, 0) * reds + ri->get_rgb_cam(2, 1) * greens + ri->get_rgb_cam(2, 2) * blues;
+	tpp->wbEqual = wbEq;
 
 	ColorTemp cTemp;
-	cTemp.mul2temp(rm, gm, bm, tpp->autowbTemp, tpp->autowbGreen);
+	cTemp.mul2temp(tpp->redAWBMul, tpp->greenAWBMul, tpp->blueAWBMul, tpp->wbEqual, tpp->autoWBTemp, tpp->autoWBGreen);
 
 	if (rotate && ri->get_rotateDegree() > 0) {
 		tpp->thumbImg->rotate(ri->get_rotateDegree());
@@ -517,7 +507,15 @@ void Thumbnail::init () {
 }
 
 Thumbnail::Thumbnail () :
-    camProfile(NULL), thumbImg(NULL),  embProfileData(NULL), embProfile(NULL) {
+    camProfile(NULL), thumbImg(NULL),
+    camwbRed(1.0), camwbGreen(1.0), camwbBlue(1.0),
+    redAWBMul(-1.0), greenAWBMul(-1.0), blueAWBMul(-1.0),
+    autoWBTemp(2700), autoWBGreen(1.0), wbEqual(-1.0),
+    embProfileLength(0), embProfileData(NULL), embProfile(NULL),
+    redMultiplier(1.0), greenMultiplier(1.0), blueMultiplier(1.0),
+    defGain(1.0),
+    scaleForSave(8192),
+    gammaCorrected(false) {
 }
 
 Thumbnail::~Thumbnail () {
@@ -559,17 +557,25 @@ IImage8* Thumbnail::quickProcessImage (const procparams::ProcParams& params, int
 IImage8* Thumbnail::processImage (const procparams::ProcParams& params, int rheight, TypeInterpolation interp, std::string camName, 
     double focalLen, double focalLen35mm, float focusDist, float shutter, float fnumber, float iso,std::string expcomp_, double& myscale) {
 
+    // check if the WB's equalizer value has changed
+    if (wbEqual < (params.wb.equal-5e-4) || wbEqual > (params.wb.equal+5e-4)) {
+        wbEqual = params.wb.equal;
+        // recompute the autoWB
+        ColorTemp cTemp;
+        cTemp.mul2temp (redAWBMul, greenAWBMul, blueAWBMul, wbEqual, autoWBTemp, autoWBGreen);
+    }
+
     // compute WB multipliers
-    ColorTemp currWB = ColorTemp (params.wb.temperature, params.wb.green, params.wb.method);
+    ColorTemp currWB = ColorTemp (params.wb.temperature, params.wb.green, params.wb.equal,params.wb.method);
     if (params.wb.method=="Camera") {
         //recall colorMatrix is rgb_cam
         double cam_r = colorMatrix[0][0]*camwbRed + colorMatrix[0][1]*camwbGreen + colorMatrix[0][2]*camwbBlue;
         double cam_g = colorMatrix[1][0]*camwbRed + colorMatrix[1][1]*camwbGreen + colorMatrix[1][2]*camwbBlue;
         double cam_b = colorMatrix[2][0]*camwbRed + colorMatrix[2][1]*camwbGreen + colorMatrix[2][2]*camwbBlue;
-        currWB = ColorTemp (cam_r, cam_g, cam_b);
+        currWB = ColorTemp (cam_r, cam_g, cam_b, params.wb.equal);
     }
     else if (params.wb.method=="Auto")
-        currWB = ColorTemp (autowbTemp, autowbGreen, "Custom");
+        currWB = ColorTemp (autoWBTemp, autoWBGreen, wbEqual, "Custom");
     double r, g, b;
     currWB.getMultipliers (r, g, b);
     //iColorMatrix is cam_rgb
@@ -583,7 +589,7 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, int rhei
     double logDefGain = log(defGain) / log(2.0);
     int rmi, gmi, bmi;
     // Since HL recovery is not rendered in thumbs
-//    if (!isRaw || !params.hlrecovery.enabled) {
+//    if (!isRaw || !params.toneCurve.hrenabled) {
         logDefGain = 0.0;
         rmi = 1024.0 * rm * defGain / mul_lum;
         gmi = 1024.0 * gm * defGain / mul_lum;
@@ -607,10 +613,10 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, int rhei
     int rwidth;
     if (params.coarse.rotate==90 || params.coarse.rotate==270) {
         rwidth = rheight;
-        rheight = thumbImg->height * rwidth / thumbImg->width;
+        rheight = int(size_t(thumbImg->height) * size_t(rwidth) / size_t(thumbImg->width));
     }
     else 
-        rwidth = thumbImg->width * rheight / thumbImg->height;
+        rwidth = int(size_t(thumbImg->width) * size_t(rheight) / size_t(thumbImg->height));
 
     Imagefloat* baseImg = resizeTo<Imagefloat>(rwidth, rheight, interp, thumbImg);
 
@@ -646,13 +652,13 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, int rhei
         }
 
 /*
-    // apply highlight recovery, if needed		-- CURRENTLY BROKEN DUE TO INCOMPATIBLE DATA TYPES; DO WE CARE???
-    if (isRaw && params.hlrecovery.enabled) {
+    // apply highlight recovery, if needed		-- CURRENTLY BROKEN DUE TO INCOMPATIBLE DATA TYPES, BUT HL RECOVERY AREN'T COMPUTED FOR THUMBNAILS ANYWAY...
+    if (isRaw && params.toneCurve.hrenabled) {
         int maxval = 65535 / defGain;
-        if (params.hlrecovery.method=="Luminance" || params.hlrecovery.method=="Color") 
+        if (params.toneCurve.method=="Luminance" || params.toneCurve.method=="Color")
             for (int i=0; i<rheight; i++)
                 RawImageSource::HLRecovery_Luminance (baseImg->r[i], baseImg->g[i], baseImg->b[i], baseImg->r[i], baseImg->g[i], baseImg->b[i], rwidth, maxval);
-        else if (params.hlrecovery.method=="CIELab blending") {
+        else if (params.toneCurve.method=="CIELab blending") {
             double icamToD50[3][3];
             RawImageSource::inverse33 (cam2xyz, icamToD50);
             for (int i=0; i<rheight; i++)
@@ -665,7 +671,7 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, int rhei
 
     // perform color space transformation
     if (isRaw)
-        RawImageSource::colorSpaceConversion (baseImg, params.icm, embProfile, camProfile, cam2xyz, camName );
+        RawImageSource::colorSpaceConversion (baseImg, params.icm, currWB, embProfile, camProfile, cam2xyz, camName );
     else
         StdImageSource::colorSpaceConversion (baseImg, params.icm, embProfile, thumbImg->getSampleFormat());
 
@@ -685,7 +691,11 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, int rhei
     // perform transform
     if (ipf.needsTransform()) {
         Imagefloat* trImg = new Imagefloat (fw, fh);
-        ipf.transform (baseImg, trImg, 0, 0, 0, 0, fw, fh, focalLen, focalLen35mm, focusDist, 0, true);  // Raw rotate degree not detectable here
+        int origFW;
+        int origFH;
+        double tscale;
+        getDimensions(origFW, origFH, tscale);
+        ipf.transform (baseImg, trImg, 0, 0, 0, 0, fw, fh, origFW*tscale+0.5, origFH*tscale+0.5, focalLen, focalLen35mm, focusDist, 0, true);  // Raw rotate degree not detectable here
         delete baseImg;
         baseImg = trImg;
     }
@@ -718,6 +728,7 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, int rhei
 	LUTf curve (65536);
 	LUTf satcurve (65536);
 	LUTf lhskcurve (65536);
+	LUTf clcurve (65536);
 	
 	LUTf rCurve (65536);
 	LUTf gCurve (65536);
@@ -729,6 +740,8 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, int rhei
     ColorAppearance customColCurve1;
     ColorAppearance customColCurve2;
     ColorAppearance customColCurve3;
+	ToneCurve customToneCurvebw1;
+	ToneCurve customToneCurvebw2;
 
 	ipf.g = gamma;
 	ipf.iGamma = true;
@@ -744,8 +757,13 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, int rhei
 	
 	LabImage* labView = new LabImage (fw,fh);
 	CieImage* cieView = new CieImage (fw,fh);
-
-    ipf.rgbProc (baseImg, labView, curve1, curve2, curve, shmap, params.toneCurve.saturation, rCurve, gCurve, bCurve, customToneCurve1, customToneCurve2, expcomp, hlcompr, hlcomprthresh);
+	
+    CurveFactory::curveBW (params.blackwhite.beforeCurve, params.blackwhite.afterCurve, hist16, dummy, customToneCurvebw1, customToneCurvebw2, 16);
+	
+	double rrm, ggm, bbm;
+    float autor, autog, autob;
+    autor = autog = autob = -9000.f; // This will ask to compute the "auto" values for the B&W tool
+    ipf.rgbProc (baseImg, labView, curve1, curve2, curve, shmap, params.toneCurve.saturation, rCurve, gCurve, bCurve, customToneCurve1, customToneCurve2, customToneCurvebw1, customToneCurvebw2,rrm, ggm, bbm, autor, autog, autob, expcomp, hlcompr, hlcomprthresh);
 
     if (shmap)
         delete shmap;
@@ -755,7 +773,7 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, int rhei
     for (int i=0; i<fh; i++)
         for (int j=0; j<fw; j++){
             hist16[CLIP((int)((labView->L[i][j])))]++;
-            hist16C=CLIP((int)sqrt(labView->a[i][j]*labView->a[i][j] + labView->b[i][j]*labView->b[i][j]));
+            hist16C[CLIP((int)sqrt(labView->a[i][j]*labView->a[i][j] + labView->b[i][j]*labView->b[i][j]))]++;
 			}
     // luminance processing
 //	ipf.EPDToneMap(labView,0,6);
@@ -765,14 +783,19 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, int rhei
 	bool butili=false;
 	bool ccutili=false;
 	bool cclutili=false;
+	bool clcutili=false;
+	
     CurveFactory::complexLCurve (params.labCurve.brightness, params.labCurve.contrast, params.labCurve.lcurve, 
         hist16, hist16, curve, dummy, 16, utili);
-    CurveFactory::complexsgnCurve (autili, butili, ccutili, cclutili, params.labCurve.chromaticity, params.labCurve.rstprotection,
-								   params.labCurve.acurve, params.labCurve.bcurve,params.labCurve.cccurve,params.labCurve.lccurve, curve1, curve2, satcurve,lhskcurve, 
-									hist16C, hist16C, dummy,  
+	
+	CurveFactory::curveCL(clcutili, params.labCurve.clcurve, clcurve, hist16C, dummy, 16);
+	
+    CurveFactory::complexsgnCurve (autili, butili, ccutili, cclutili,params.labCurve.chromaticity, params.labCurve.rstprotection,
+								   params.labCurve.acurve, params.labCurve.bcurve,params.labCurve.cccurve,params.labCurve.lccurve, curve1, curve2, satcurve,lhskcurve,
+									hist16C, hist16C, dummy, dummy,
 								   16);
     //ipf.luminanceCurve (labView, labView, curve);
-    ipf.chromiLuminanceCurve (1,labView, labView, curve1, curve2, satcurve,lhskcurve, curve, utili, autili, butili, ccutili,cclutili, dummy);
+    ipf.chromiLuminanceCurve (1,labView, labView, curve1, curve2, satcurve,lhskcurve, clcurve, curve, utili, autili, butili, ccutili,cclutili, clcutili, dummy, dummy, dummy, dummy);
 	
 	ipf.vibrance(labView);
 	int begh = 0, endh = labView->H;
@@ -786,46 +809,48 @@ IImage8* Thumbnail::processImage (const procparams::ProcParams& params, int rhei
 					params.colorappearance.curveMode2, params.colorappearance.curve2,
 					params.colorappearance.curveMode3, params.colorappearance.curve3,
 					hist16, hist16, dummy,
-					hist16C, hist16C, dummy,
+					hist16C, dummy,
 					customColCurve1,
 					customColCurve2, 
 					customColCurve3, 
 					16);
 
-	int f_h=2,f_w=2;
 	if(params.colorappearance.enabled){
-	float** buffer = new float*[fh];
-	for (int i=0; i<fh; i++)
-		buffer[i] = new float[fw];
-	bool execsharp=false;
-	float d;
-	float fnum = fnumber;// F number
-	float fiso = iso;// ISO
-	float fspeed = shutter;//speed 
-	char * writ = new char[expcomp_.size() + 1];//convert expcomp_ to char
-	std::copy(expcomp_.begin(), expcomp_.end(), writ);
-	writ[expcomp_.size()] = '\0'; 
-	float fcomp = atof(writ); //compensation + -
-	delete[] writ;
-	float adap2,adap;
-	double ada, ada2;
-	if(fnum < 0.3f || fiso < 5.f || fspeed < 0.00001f) {adap=adap=2000.f;ada=2000.;}//if no exif data or wrong
-	else {
-	float E_V = fcomp + log2 ((fnum*fnum) / fspeed / (fiso/100.f));
-	float expo2= params.toneCurve.expcomp;// exposure compensation in tonecurve ==> direct EV
-	E_V += expo2;
-	float expo1;//exposure raw white point
-	expo1=log2(params.raw.expos);//log2 ==>linear to EV
-	E_V += expo1;
-	adap2 = adap= powf(2.f, E_V-3.f);//cd / m2
-	ada=ada2=(double) adap;
-	//end calculation adaptation scene luminosity
-	}
+		float** buffer = new float*[fh];
+		for (int i=0; i<fh; i++)
+			buffer[i] = new float[fw];
+		bool execsharp=false;
+		float d;
+		float fnum = fnumber;// F number
+		float fiso = iso;// ISO
+		float fspeed = shutter;//speed
+		char * writ = new char[expcomp_.size() + 1];//convert expcomp_ to char
+		std::copy(expcomp_.begin(), expcomp_.end(), writ);
+		writ[expcomp_.size()] = '\0';
+		float fcomp = atof(writ); //compensation + -
+		delete[] writ;
+		float adap;
+		if(fnum < 0.3f || fiso < 5.f || fspeed < 0.00001f)
+			//if no exif data or wrong
+			adap=2000.f;
+		else {
+			float E_V = fcomp + log2 ((fnum*fnum) / fspeed / (fiso/100.f));
+			float expo2= params.toneCurve.expcomp;// exposure compensation in tonecurve ==> direct EV
+			E_V += expo2;
+			float expo1;//exposure raw white point
+			expo1=log2(params.raw.expos);//log2 ==>linear to EV
+			E_V += expo1;
+			adap= powf(2.f, E_V-3.f);//cd / m2
+			//end calculation adaptation scene luminosity
+		}
 
-	ipf.ciecam_02float (cieView, adap, begh, endh, 1, 2, labView, &params,customColCurve1,customColCurve2,customColCurve3, dummy, dummy, 5, 6, (float**)buffer, execsharp, d);
-	for (int i=0; i<fh; i++)
-		delete [] buffer[i];
-	delete [] buffer; buffer=NULL;
+		LUTf CAMBrightCurveJ;
+		LUTf CAMBrightCurveQ;
+		float CAMMean;
+		ipf.ciecam_02float (cieView, adap, begh, endh, 1, 2, labView, &params,customColCurve1,customColCurve2,customColCurve3, dummy, dummy, CAMBrightCurveJ, CAMBrightCurveQ, CAMMean, 5, 6, (float**)buffer, execsharp, d);
+		for (int i=0; i<fh; i++)
+			delete [] buffer[i];
+		delete [] buffer; buffer=NULL;
 	}
     // color processing
     //ipf.colorCurve (labView, labView);
@@ -888,15 +913,28 @@ void Thumbnail::getCamWB (double& temp, double& green) {
     double cam_r = colorMatrix[0][0]*camwbRed + colorMatrix[0][1]*camwbGreen + colorMatrix[0][2]*camwbBlue;
     double cam_g = colorMatrix[1][0]*camwbRed + colorMatrix[1][1]*camwbGreen + colorMatrix[1][2]*camwbBlue;
     double cam_b = colorMatrix[2][0]*camwbRed + colorMatrix[2][1]*camwbGreen + colorMatrix[2][2]*camwbBlue;
-    ColorTemp currWB = ColorTemp (cam_r, cam_g, cam_b);
+    ColorTemp currWB = ColorTemp (cam_r, cam_g, cam_b, 1.0);  // we do not take the equalizer into account here, because we want camera's WB
     temp = currWB.getTemp ();
     green = currWB.getGreen ();
 }
 
-void Thumbnail::getAutoWB (double& temp, double& green) {
-    
-    temp = autowbTemp;
-    green = autowbGreen;
+void Thumbnail::getAutoWB (double& temp, double& green, double equal) {
+
+    if (equal != wbEqual) {
+        // compute the values depending on equal
+        ColorTemp cTemp;
+        wbEqual = equal;
+        // compute autoWBTemp and autoWBGreen
+        cTemp.mul2temp(redAWBMul, greenAWBMul, blueAWBMul, wbEqual, autoWBTemp, autoWBGreen);
+    }
+    temp = autoWBTemp;
+    green = autoWBGreen;
+}
+
+void Thumbnail::getAutoWBMultipliers (double& rm, double& gm, double& bm) {
+    rm = redAWBMul;
+    gm = greenAWBMul;
+    bm = blueAWBMul;
 }
 
 void Thumbnail::applyAutoExp (procparams::ProcParams& params) {
@@ -941,7 +979,7 @@ void Thumbnail::getSpotWB (const procparams::ProcParams& params, int xp, int yp,
     double gm = colorMatrix[1][0]*reds + colorMatrix[1][1]*greens + colorMatrix[1][2]*blues;
     double bm = colorMatrix[2][0]*reds + colorMatrix[2][1]*greens + colorMatrix[2][2]*blues;
 
-    ColorTemp ct (rm, gm, bm);
+    ColorTemp ct (rm, gm, bm, params.wb.equal);
     rtemp = ct.getTemp ();
     rgreen = ct.getGreen ();
 }
@@ -1279,7 +1317,7 @@ bool Thumbnail::readData  (const Glib::ustring& fname) {
     SafeKeyFile keyFile;
     
     try {
-        Glib::Mutex::Lock thmbLock(thumbMutex);
+        MyMutex::MyLock thmbLock(thumbMutex);
         if (!keyFile.load_from_file (fname)) 
             return false;
 
@@ -1287,8 +1325,9 @@ bool Thumbnail::readData  (const Glib::ustring& fname) {
             if (keyFile.has_key ("LiveThumbData", "CamWBRed"))          camwbRed            = keyFile.get_double ("LiveThumbData", "CamWBRed");
             if (keyFile.has_key ("LiveThumbData", "CamWBGreen"))        camwbGreen          = keyFile.get_double ("LiveThumbData", "CamWBGreen");
             if (keyFile.has_key ("LiveThumbData", "CamWBBlue"))         camwbBlue           = keyFile.get_double ("LiveThumbData", "CamWBBlue");
-            if (keyFile.has_key ("LiveThumbData", "AutoWBTemp"))        autowbTemp          = keyFile.get_double ("LiveThumbData", "AutoWBTemp");
-            if (keyFile.has_key ("LiveThumbData", "AutoWBGreen"))       autowbGreen         = keyFile.get_double ("LiveThumbData", "AutoWBGreen");
+            if (keyFile.has_key ("LiveThumbData", "RedAWBMul"))         redAWBMul           = keyFile.get_double ("LiveThumbData", "RedAWBMul");
+            if (keyFile.has_key ("LiveThumbData", "GreenAWBMul"))       greenAWBMul         = keyFile.get_double ("LiveThumbData", "GreenAWBMul");
+            if (keyFile.has_key ("LiveThumbData", "BlueAWBMul"))        blueAWBMul          = keyFile.get_double ("LiveThumbData", "BlueAWBMul");
             if (keyFile.has_key ("LiveThumbData", "AEHistCompression")) aeHistCompression   = keyFile.get_integer ("LiveThumbData", "AEHistCompression");
             if (keyFile.has_key ("LiveThumbData", "RedMultiplier"))     redMultiplier       = keyFile.get_double ("LiveThumbData", "RedMultiplier");
             if (keyFile.has_key ("LiveThumbData", "GreenMultiplier"))   greenMultiplier     = keyFile.get_double ("LiveThumbData", "GreenMultiplier");
@@ -1305,30 +1344,45 @@ bool Thumbnail::readData  (const Glib::ustring& fname) {
                         colorMatrix[i][j] = cm[ix++];
             }
         }
+        return true;
     }
     catch (Glib::Error &err) {
-        return false;
+        if (options.rtSettings.verbose)
+            printf("Thumbnail::readData / Error code %d while reading values from \"%s\":\n%s\n", err.code(), fname.c_str(), err.what().c_str());
+    }
+    catch (...) {
+        if (options.rtSettings.verbose)
+            printf("Thumbnail::readData / Unknown exception while trying to load \"%s\"!\n", fname.c_str());
     }
 
-    return true;
+    return false;
 }
 
 bool Thumbnail::writeData  (const Glib::ustring& fname) {
 
     SafeKeyFile keyFile;
 
-    Glib::Mutex::Lock thmbLock(thumbMutex);
+    MyMutex::MyLock thmbLock(thumbMutex);
 
     try {
-    if( safe_file_test(fname,Glib::FILE_TEST_EXISTS) )
-        keyFile.load_from_file (fname); 
-    } catch (...) {}
+        if( safe_file_test(fname,Glib::FILE_TEST_EXISTS) )
+            keyFile.load_from_file (fname);
+    }
+    catch (Glib::Error &err) {
+        if (options.rtSettings.verbose)
+            printf("Thumbnail::writeData / Error code %d while reading values from \"%s\":\n%s\n", err.code(), fname.c_str(), err.what().c_str());
+    }
+    catch (...) {
+        if (options.rtSettings.verbose)
+            printf("Thumbnail::writeData / Unknown exception while trying to save \"%s\"!\n", fname.c_str());
+    }
 
     keyFile.set_double  ("LiveThumbData", "CamWBRed", camwbRed);
     keyFile.set_double  ("LiveThumbData", "CamWBGreen", camwbGreen);
     keyFile.set_double  ("LiveThumbData", "CamWBBlue", camwbBlue);
-    keyFile.set_double  ("LiveThumbData", "AutoWBTemp", autowbTemp);
-    keyFile.set_double  ("LiveThumbData", "AutoWBGreen", autowbGreen);
+    keyFile.set_double  ("LiveThumbData", "RedAWBMul", redAWBMul);
+    keyFile.set_double  ("LiveThumbData", "GreenAWBMul", greenAWBMul);
+    keyFile.set_double  ("LiveThumbData", "BlueAWBMul", blueAWBMul);
     keyFile.set_integer ("LiveThumbData", "AEHistCompression", aeHistCompression);
     keyFile.set_double  ("LiveThumbData", "RedMultiplier", redMultiplier);
     keyFile.set_double  ("LiveThumbData", "GreenMultiplier", greenMultiplier);
@@ -1341,8 +1395,11 @@ bool Thumbnail::writeData  (const Glib::ustring& fname) {
     keyFile.set_double_list ("LiveThumbData", "ColorMatrix", cm);
 
     FILE *f = safe_g_fopen (fname, "wt");
-    if (!f)
+    if (!f) {
+        if (options.rtSettings.verbose)
+            printf("Thumbnail::writeData / Error: unable to open file \"%s\" with write access!\n", fname.c_str());
         return false;
+    }
     else {
         fprintf (f, "%s", keyFile.to_data().c_str());
         fclose (f);
