@@ -82,10 +82,49 @@ Glib::RefPtr<Gio::FileInfo> safe_query_file_info (Glib::RefPtr<Gio::File> &file)
 		return info;
 }
 
+Glib::RefPtr<Gio::FileInfo> safe_next_file (Glib::RefPtr<Gio::FileEnumerator> &dirList)
+{
+		Glib::RefPtr<Gio::FileInfo> info;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+		bool retry;
+		Glib::ustring last_error = "";
+		do {
+			retry = false;
+			try {
+				info = dirList->next_file();
+			} catch (Glib::Exception& ex) {
+				printf ("%s\n", ex.what().c_str());
+				// API problem: not possible to differ between error looking at particular entry or
+				// general error scanning the directory. We do a hack by retrying and see if the
+				// error changes, if it does we can assume it's about the current filename and we
+				// should look at the next. More likely if a single file error is that the next
+				// retry is okay of course.
+				retry = (ex.what() != last_error);
+				last_error = ex.what();
+			}
+		} while (retry);
+#else
+		bool retry;
+		Glib::ustring last_error = "";
+		do {
+			retry = false;
+			std::auto_ptr<Glib::Error> error;
+			Glib::RefPtr<Gio::Cancellable> cancellable;
+			info = dirList->next_file(cancellable, error);
+			if (!info && error.get()) {
+				printf ("%s\n", error.what().c_str());
+				retry = (error.what() != last_error);
+				last_error = error.what();
+			}
+		} while (retry);
+#endif
+		return info;
+}
+
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
 # define SAFE_ENUMERATOR_CODE_START \
 				do{try {	if ((dirList = dir->enumerate_children ())) \
-						for (Glib::RefPtr<Gio::FileInfo> info = dirList->next_file(); info; info = dirList->next_file()) {
+						for (Glib::RefPtr<Gio::FileInfo> info = safe_next_file(dirList); info; info = safe_next_file(dirList)) {
 						
 # define SAFE_ENUMERATOR_CODE_END \
 				}}	catch (Glib::Exception& ex) {	printf ("%s\n", ex.what().c_str());	}}while(0)
@@ -93,7 +132,7 @@ Glib::RefPtr<Gio::FileInfo> safe_query_file_info (Glib::RefPtr<Gio::File> &file)
 # define SAFE_ENUMERATOR_CODE_START \
 				do{std::auto_ptr<Glib::Error> error;	Glib::RefPtr<Gio::Cancellable> cancellable; \
 					if ((dirList = dir->enumerate_children (cancellable, "*", Gio::FILE_QUERY_INFO_NONE, error))) \
-						for (Glib::RefPtr<Gio::FileInfo> info = dirList->next_file(cancellable, error); !error.get() && info; info = dirList->next_file(cancellable, error)) {
+						for (Glib::RefPtr<Gio::FileInfo> info = safe_next_file(dirList); info; info = safe_next_file(dirList)) {
 						
 # define SAFE_ENUMERATOR_CODE_END 	} if (error.get())	printf ("%s\n", error->what().c_str());}while (0)
 #endif
@@ -395,7 +434,7 @@ Glib::ustring safe_get_user_picture_dir() {
     // get_user_special_dir/pictures crashes on some Windows configurations.
     // so we use the safe native functions here
     WCHAR pathW[MAX_PATH]={0};
-	if (SHGetSpecialFolderPathW(NULL,pathW,CSIDL_MYPICTURES,false)) {
+    if (SHGetSpecialFolderPathW(NULL,pathW,CSIDL_MYPICTURES,false)) {
         char pathA[MAX_PATH];
         WideCharToMultiByte(CP_UTF8,0,pathW,-1,pathA,MAX_PATH,0,0);
         return Glib::ustring(pathA);
@@ -408,14 +447,49 @@ Glib::ustring safe_get_user_picture_dir() {
     #endif
 }
 
+Glib::ustring safe_get_user_home_dir() {
+    #ifdef WIN32
+    // get_user_special_dir/pictures crashes on some Windows configurations.
+    // so we use the safe native functions here
+    WCHAR pathW[MAX_PATH]={0};
+    if (SHGetSpecialFolderPathW(NULL,pathW,CSIDL_PERSONAL,false)) {
+        char pathA[MAX_PATH];
+        WideCharToMultiByte(CP_UTF8,0,pathW,-1,pathA,MAX_PATH,0,0);
+        return Glib::ustring(pathA);
+    } else return Glib::ustring("C:\\");
+
+    #else
+
+    return Glib::get_home_dir();
+
+    #endif
+}
+
+Glib::ustring safe_get_user_desktop_dir() {
+    #ifdef WIN32
+    // get_user_special_dir/pictures crashes on some Windows configurations.
+    // so we use the safe native functions here
+    WCHAR pathW[MAX_PATH]={0};
+    if (SHGetSpecialFolderPathW(NULL,pathW,CSIDL_DESKTOP,false)) {
+        char pathA[MAX_PATH];
+        WideCharToMultiByte(CP_UTF8,0,pathW,-1,pathA,MAX_PATH,0,0);
+        return Glib::ustring(pathA);
+    } else return Glib::ustring("C:\\");
+
+    #else
+
+    return Glib::get_user_special_dir (G_USER_DIRECTORY_DESKTOP);
+
+    #endif
+}
+
 #ifdef WIN32
 /*
  * Test if the path is a root path based on the content of the string
  *
  * Warning: this function is a workaround for Windows platform, and not necessarily bullet proof
  */
-bool safe_is_root_dir (const Glib::ustring& path) {
-	return PathIsRootA(path.c_str());
+bool safe_is_shortcut_dir (const Glib::ustring& path) {
+    return PathIsRootA(path.c_str()) || safe_get_user_home_dir() == path || safe_get_user_desktop_dir() == path; // || safe_get_user_picture_dir() == path;
 }
 #endif
-
