@@ -47,13 +47,15 @@ namespace rtengine {
 			Glib::ustring   srgb;					// default name of SRGB space profile
 			Glib::ustring   srgb10;					// default name of SRGB space profile
 			
-			bool            gamutICC;
+			bool            gamutICC; // no longer used
 			bool            gamutLch;
 			bool			ciecamfloat;
+			int				amchroma;
 			int             protectred;
 			double          protectredh;
 			bool			ciebadpixgauss;
 			int             CRI_color; // N for display Lab value  ; 0 disabled
+		//	bool			bw_complementary;
 
         /** Creates a new instance of Settings.
           * @return a pointer to the new Settings instance. */
