@@ -28,6 +28,7 @@
 #include <iostream>
 #include "rawimagesource.h"
 #include "../rtgui/ppversion.h"
+#include "../rtgui/multilangmgr.h"
 //#include "mytime.h"
 
 #undef THREAD_PRIORITY_NORMAL
@@ -99,30 +100,42 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
 
     PreviewProps pp (0, 0, fw, fh, 1);
     imgsrc->preprocess( params.raw, params.lensProf, params.coarse);
-	if (pl) pl->setProgress (0.20);
+
+    if (params.toneCurve.autoexp) {// this enabled HLRecovery
+        LUTu histRedRaw(256), histGreenRaw(256), histBlueRaw(256);
+        imgsrc->getRAWHistogram(histRedRaw, histGreenRaw, histBlueRaw);
+        if (ToneCurveParams::HLReconstructionNecessary(histRedRaw, histGreenRaw, histBlueRaw) && !params.toneCurve.hrenabled) {
+            params.toneCurve.hrenabled=true;
+            // WARNING: Highlight Reconstruction is being forced 'on', should we force a method here too?
+        }
+    }
+
+    if (pl) pl->setProgress (0.20);
     imgsrc->demosaic( params.raw);
     if (pl) pl->setProgress (0.30);
-    imgsrc->HLRecovery_Global( params.hlrecovery );
+    imgsrc->HLRecovery_Global( params.toneCurve );
     if (pl) pl->setProgress (0.40);
 	// set the color temperature
-    ColorTemp currWB = ColorTemp (params.wb.temperature, params.wb.green, params.wb.method);
+    ColorTemp currWB = ColorTemp (params.wb.temperature, params.wb.green, params.wb.equal, params.wb.method);
     if (params.wb.method=="Camera")
         currWB = imgsrc->getWB ();
-    else if (params.wb.method=="Auto")
-        currWB = imgsrc->getAutoWB ();
+    else if (params.wb.method=="Auto") {
+        double rm, gm, bm;
+        imgsrc->getAutoWBMultipliers(rm, gm, bm);
+        currWB.update(rm, gm, bm, params.wb.equal);
+    }
     Imagefloat* baseImg = new Imagefloat (fw, fh);
-    imgsrc->getImage (currWB, tr, baseImg, pp, params.hlrecovery, params.icm, params.raw);
+    imgsrc->getImage (currWB, tr, baseImg, pp, params.toneCurve, params.icm, params.raw);
     if (pl) pl->setProgress (0.45);
 
 
     // perform luma/chroma denoise
 //	CieImage *cieView;    
 
-    LabImage* labView = new LabImage (fw,fh);
     if (params.dirpyrDenoise.enabled) {
 		ipf.RGB_denoise(baseImg, baseImg, imgsrc->isRAW(), params.dirpyrDenoise, params.defringe, imgsrc->getDirPyrDenoiseExpComp());
     }
-    imgsrc->convertColorSpace(baseImg, params.icm, params.raw);
+    imgsrc->convertColorSpace(baseImg, params.icm, currWB, params.raw);
 
     // perform first analysis
     LUTu hist16 (65536);
@@ -133,7 +146,7 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
     // perform transform (excepted resizing)
     if (ipf.needsTransform()) {
         Imagefloat* trImg = new Imagefloat (fw, fh);
-        ipf.transform (baseImg, trImg, 0, 0, 0, 0, fw, fh, imgsrc->getMetaData()->getFocalLen(), imgsrc->getMetaData()->getFocalLen35mm(),
+        ipf.transform (baseImg, trImg, 0, 0, 0, 0, fw, fh, fw, fh, imgsrc->getMetaData()->getFocalLen(), imgsrc->getMetaData()->getFocalLen35mm(),
                        imgsrc->getMetaData()->getFocusDist(), imgsrc->getRotateDegree(), true);
         delete baseImg;
         baseImg = trImg;
@@ -175,6 +188,7 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
 	LUTf satcurve (65536,0);
 	LUTf lhskcurve (65536,0);
 	LUTf lumacurve(65536,0);
+	LUTf clcurve (65536,0);
 
 	LUTf rCurve (65536,0);
 	LUTf gCurve (65536,0);
@@ -183,18 +197,29 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
 
 	ToneCurve customToneCurve1, customToneCurve2;
 	ColorAppearance customColCurve1, customColCurve2,customColCurve3 ;
+	ToneCurve customToneCurvebw1;
+	ToneCurve customToneCurvebw2;
 
 	ipf.g = imgsrc->getGamma();
 	ipf.iGamma = true;
 	CurveFactory::complexCurve (expcomp, black/65535.0, hlcompr, hlcomprthresh, params.toneCurve.shcompr, bright, contr, ipf.g, !ipf.iGamma,
 	                            params.toneCurve.curveMode, params.toneCurve.curve, params.toneCurve.curveMode2, params.toneCurve.curve2,
-	                            hist16, dummy, curve1, curve2, curve, dummy, customToneCurve1, customToneCurve2);
-	
+	                            hist16, dummy, curve1, curve2, curve, dummy, customToneCurve1, customToneCurve2 );
+
 	CurveFactory::RGBCurve (params.rgbCurves.rcurve, rCurve, 1);
 	CurveFactory::RGBCurve (params.rgbCurves.gcurve, gCurve, 1);
 	CurveFactory::RGBCurve (params.rgbCurves.bcurve, bCurve, 1);
 
-    ipf.rgbProc (baseImg, labView, curve1, curve2, curve, shmap, params.toneCurve.saturation, rCurve, gCurve, bCurve, customToneCurve1, customToneCurve2, expcomp, hlcompr, hlcomprthresh);
+    LabImage* labView = new LabImage (fw,fh);
+
+
+	CurveFactory::curveBW (params.blackwhite.beforeCurve, params.blackwhite.afterCurve, hist16, dummy, customToneCurvebw1, customToneCurvebw2, 1);
+	double rrm, ggm, bbm;
+    float autor, autog, autob;
+    autor = -9000.f; // This will ask to compute the "auto" values for the B&W tool (have to be inferior to -5000)
+    ipf.rgbProc (baseImg, labView, curve1, curve2, curve, shmap, params.toneCurve.saturation, rCurve, gCurve, bCurve, customToneCurve1, customToneCurve2,customToneCurvebw1, customToneCurvebw2, rrm, ggm, bbm, autor, autog, autob, expcomp, hlcompr, hlcomprthresh);
+    if (settings->verbose)
+        printf("Output image / Auto B&W coefs:   R=%.2f   G=%.2f   B=%.2f\n", autor, autog, autob);
 
     // Freeing baseImg because not used anymore
     delete baseImg;
@@ -215,16 +240,24 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
 	if(params.labCurve.contrast !=0) {//only use hist16 for contrast
 	
 #ifdef _OPENMP
-#pragmaompparallelshared(hist16,labView, fh, fw)
+#pragma omp parallel shared(hist16,labView, fh, fw)
 #endif
 {
-#ifdef _OPENMP	
-#pragma omp for schedule(static)
+    LUTu hist16thr (65536);  // one temporary lookup table per thread
+    hist16thr.clear();
+#ifdef _OPENMP
+#pragma omp for schedule(static) nowait
 #endif
     for (int i=0; i<fh; i++)
         for (int j=0; j<fw; j++){
-            hist16[CLIP((int)((labView->L[i][j])))]++;
-			}
+            hist16thr[CLIP((int)((labView->L[i][j])))]++;
+        }
+
+#pragma omp critical
+{
+    for(int i=0;i<65536;i++)
+        hist16[i] += hist16thr[i];
+}
 }
 	
 	
@@ -234,14 +267,17 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
 	bool butili=false;
 	bool ccutili=false;
 	bool cclutili=false;	
+	bool clcutili=false;	
+
 	CurveFactory::complexLCurve (params.labCurve.brightness, params.labCurve.contrast, params.labCurve.lcurve,hist16, hist16, lumacurve, dummy, 1, utili);
+	CurveFactory::curveCL(clcutili, params.labCurve.clcurve, clcurve, hist16C, dummy, 1);
 
 	CurveFactory::complexsgnCurve (autili, butili, ccutili, cclutili, params.labCurve.chromaticity, params.labCurve.rstprotection,
 								   params.labCurve.acurve, params.labCurve.bcurve, params.labCurve.cccurve,params.labCurve.lccurve,curve1, curve2, satcurve,lhskcurve, 
-								   hist16C, hist16C,dummy,	
+								   hist16C, hist16C, dummy,dummy,
 								   1);
 
-	ipf.chromiLuminanceCurve (1,labView, labView, curve1, curve2, satcurve,lhskcurve,lumacurve, utili, autili, butili, ccutili,cclutili,dummy);
+	ipf.chromiLuminanceCurve (1,labView, labView, curve1, curve2, satcurve,lhskcurve,clcurve, lumacurve, utili, autili, butili, ccutili,cclutili, clcutili, dummy, dummy, dummy, dummy);
 	
  	if((params.colorappearance.enabled && !params.colorappearance.tonecie) || (!params.colorappearance.enabled))ipf.EPDToneMap(labView,5,1);
 	
@@ -288,58 +324,60 @@ IImage16* processImage (ProcessingJob* pjob, int& errorCode, ProgressListener* p
 					params.colorappearance.curveMode2, params.colorappearance.curve2,
 					params.colorappearance.curveMode3, params.colorappearance.curve3,
 					hist16, hist16,dummy,
-					hist16C, hist16C,dummy,	
+					hist16C, dummy,
 					customColCurve1,
 					customColCurve2,
-					customColCurve3, 					
+					customColCurve3,
 					1);
 	float adap2,adap;
 	double ada, ada2;
-	if(params.colorappearance.enabled){	
-	float fnum = imgsrc->getMetaData()->getFNumber  ();// F number
-	float fiso = imgsrc->getMetaData()->getISOSpeed () ;// ISO
-	float fspeed = imgsrc->getMetaData()->getShutterSpeed () ;//speed 
-	float fcomp = imgsrc->getMetaData()->getExpComp  ();//compensation + -
-	if(fnum < 0.3f || fiso < 5.f || fspeed < 0.00001f) {adap=adap2=2000.f;ada=ada2=2000.;}//if no exif data or wrong
-	else {
-	float E_V = fcomp + log2 ((fnum*fnum) / fspeed / (fiso/100.f));
-	float expo2= params.toneCurve.expcomp;// exposure compensation in tonecurve ==> direct EV
-	E_V += expo2;
-	float expo1;//exposure raw white point
-	expo1=log2(params.raw.expos);//log2 ==>linear to EV
-	E_V += expo1;
-	adap2 = adap= powf(2.f, E_V-3.f);//cd / m2
-	ada=ada2=(double) adap;
-	}			
-    if (params.sharpening.enabled) {
-		float d;
-		double dd;
-		
-        float** buffer = new float*[fh];
-        for (int i=0; i<fh; i++)
-            buffer[i] = new float[fw];	
-	if(settings->ciecamfloat) ipf.ciecam_02float (cieView, adap, begh, endh,1,2, labView, &params,customColCurve1,customColCurve2,customColCurve3, dummy, dummy, 5, 1, (float**)buffer, true, d);
-	else ipf.ciecam_02 (cieView, ada, begh, endh,1,2, labView, &params,customColCurve1,customColCurve2,customColCurve3, dummy, dummy, 5, 1, (float**)buffer, true, dd);
+	if(params.colorappearance.enabled){
+		float fnum = imgsrc->getMetaData()->getFNumber  ();// F number
+		float fiso = imgsrc->getMetaData()->getISOSpeed () ;// ISO
+		float fspeed = imgsrc->getMetaData()->getShutterSpeed () ;//speed
+		float fcomp = imgsrc->getMetaData()->getExpComp  ();//compensation + -
+		if(fnum < 0.3f || fiso < 5.f || fspeed < 0.00001f) {adap=adap2=2000.f;ada=ada2=2000.;}//if no exif data or wrong
+		else {
+			float E_V = fcomp + log2 ((fnum*fnum) / fspeed / (fiso/100.f));
+			float expo2= params.toneCurve.expcomp;// exposure compensation in tonecurve ==> direct EV
+			E_V += expo2;
+			float expo1;//exposure raw white point
+			expo1=log2(params.raw.expos);//log2 ==>linear to EV
+			E_V += expo1;
+			adap2 = adap= powf(2.f, E_V-3.f);//cd / m2
+			ada=ada2=(double) adap;
+		}
+		LUTf CAMBrightCurveJ;
+		LUTf CAMBrightCurveQ;
+		float CAMMean;
+		if (params.sharpening.enabled) {
+			float d;
+			double dd;
 
-	        for (int i=0; i<fh; i++)
-            delete [] buffer[i];
+			float** buffer = new float*[fh];
+			for (int i=0; i<fh; i++)
+			buffer[i] = new float[fw];
+			if(settings->ciecamfloat) ipf.ciecam_02float (cieView, adap, begh, endh,1,2, labView, &params,customColCurve1,customColCurve2,customColCurve3, dummy, dummy, CAMBrightCurveJ, CAMBrightCurveQ, CAMMean, 5, 1, (float**)buffer, true, d);
+			else ipf.ciecam_02 (cieView, ada, begh, endh,1,2, labView, &params,customColCurve1,customColCurve2,customColCurve3, dummy, dummy, CAMBrightCurveJ, CAMBrightCurveQ, CAMMean, 5, 1, (float**)buffer, true, dd);
+			for (int i=0; i<fh; i++)
+			delete [] buffer[i];
 			delete [] buffer;
-			}
-			else {
+		}
+		else {
 			int f_h=2,f_w=2;
 			float d;
-			
-			double dd;
-	        float** buffer = new float*[f_h];
-			for (int i=0; i<f_h; i++)
-            buffer[i] = new float[f_w];
-if(settings->ciecamfloat) ipf.ciecam_02float (cieView, adap, begh, endh,1,2, labView, &params,customColCurve1,customColCurve2,customColCurve3, dummy, dummy, 5, 1, (float**)buffer, true, d);
-else ipf.ciecam_02 (cieView, adap, begh, endh,1, 2, labView, &params,customColCurve1,customColCurve2,customColCurve3, dummy, dummy, 5, 1, (float**)buffer, true, dd);
 
-	        for (int i=0; i<f_h; i++)
-            delete [] buffer[i];
+			double dd;
+			float** buffer = new float*[f_h];
+			for (int i=0; i<f_h; i++)
+			buffer[i] = new float[f_w];
+			if(settings->ciecamfloat) ipf.ciecam_02float (cieView, adap, begh, endh,1,2, labView, &params,customColCurve1,customColCurve2,customColCurve3, dummy, dummy, CAMBrightCurveJ, CAMBrightCurveQ, CAMMean, 5, 1, (float**)buffer, true, d);
+			else ipf.ciecam_02 (cieView, adap, begh, endh,1, 2, labView, &params,customColCurve1,customColCurve2,customColCurve3, dummy, dummy, CAMBrightCurveJ, CAMBrightCurveQ, CAMMean, 5, 1, (float**)buffer, true, dd);
+
+			for (int i=0; i<f_h; i++)
+			delete [] buffer[i];
 			delete [] buffer;
-			}
+		}
 	}	
     delete cieView;
     cieView = NULL;
@@ -392,7 +430,7 @@ else ipf.ciecam_02 (cieView, adap, begh, endh,1, 2, labView, &params,customColCu
         bool pro=false;
         Glib::ustring chpro, outProfile;
         bool present_space[9]={false,false,false,false,false,false,false,false,false};
-        std::vector<std::string> opnames = iccStore->getOutputProfiles ();
+        std::vector<Glib::ustring> opnames = iccStore->getOutputProfiles ();
         //test if files are in system
         for (int j=0; j<9; j++) {
             // one can modify "option" [Color Management] to adapt the profile's name if they are different for windows, MacOS, Linux ??
@@ -671,9 +709,17 @@ void batchProcessingThread (ProcessingJob* job, BatchProcessingListener* bpl, bo
     while (currentJob) {
         int errorCode;
         IImage16* img = processImage (currentJob, errorCode, bpl, tunnelMetaData);
-        if (errorCode) 
-            bpl->error ("Can not load input image.");
-        currentJob = bpl->imageReady (img);
+        if (errorCode) {
+            bpl->error (M("MAIN_MSG_CANNOTLOAD"));
+            currentJob = NULL;
+        } else {
+            try {
+                currentJob = bpl->imageReady (img);
+            } catch (Glib::Exception& ex) {
+                bpl->error (ex.what());
+                currentJob = NULL;
+            }
+        }
     }
 }
 
