@@ -188,13 +188,13 @@ int StdImageSource::load (Glib::ustring fname, bool batch) {
         plistener->setProgress (1.0);
     }
 
-    wb = ColorTemp (1.0,1.0,1.0);
+    wb = ColorTemp (1.0,1.0,1.0,1.0);
     //this is probably a mistake if embedded profile is not D65
 
     return 0;
 }
 
-void StdImageSource::getImage (ColorTemp ctemp, int tran, Imagefloat* image, PreviewProps pp, HRecParams hrp, ColorManagementParams cmp, RAWParams raw) {
+void StdImageSource::getImage (ColorTemp ctemp, int tran, Imagefloat* image, PreviewProps pp,ToneCurveParams  hrp, ColorManagementParams cmp, RAWParams raw) {
 
     // the code will use OpenMP as of now.
 
@@ -211,7 +211,7 @@ void StdImageSource::getImage (ColorTemp ctemp, int tran, Imagefloat* image, Pre
         image->vflip();
 }
 
-void StdImageSource::convertColorSpace(Imagefloat* image, ColorManagementParams cmp, RAWParams raw) {
+void StdImageSource::convertColorSpace(Imagefloat* image, ColorManagementParams cmp, ColorTemp &wb, RAWParams raw) {
     colorSpaceConversion (image, cmp, embProfile, img->getSampleFormat());
 }
 
@@ -292,11 +292,22 @@ void StdImageSource::getAutoExpHistogram (LUTu & histogram, int& histcompr) {
     }
 }
 
-ColorTemp StdImageSource::getAutoWB () {
-    return img->getAutoWB();
+void StdImageSource::getAutoWBMultipliers (double &rm, double &gm, double &bm) {
+    if (redAWBMul != -1.) {
+        rm = redAWBMul;
+        gm = greenAWBMul;
+        bm = blueAWBMul;
+        return;
+    }
+
+    img->getAutoWBMultipliers(rm, gm, bm);
+
+    redAWBMul   = rm;
+    greenAWBMul = gm;
+    blueAWBMul  = bm;
 }
 
-ColorTemp StdImageSource::getSpotWB (std::vector<Coord2D> &red, std::vector<Coord2D> &green, std::vector<Coord2D>& blue, int tran) {
+ColorTemp StdImageSource::getSpotWB (std::vector<Coord2D> &red, std::vector<Coord2D> &green, std::vector<Coord2D>& blue, int tran, double equal) {
     int rn, gn, bn;
     double reds, greens, blues;
     img->getSpotWBData(reds, greens, blues, rn, gn, bn, red, green, blue, tran);
@@ -305,7 +316,7 @@ ColorTemp StdImageSource::getSpotWB (std::vector<Coord2D> &red, std::vector<Coor
     if( settings->verbose )
         printf ("AVG: %g %g %g\n", reds/rn, greens/gn, blues/bn);
 
-    return ColorTemp (reds/rn*img_r, greens/gn*img_g, blues/bn*img_b);
+    return ColorTemp (reds/rn*img_r, greens/gn*img_g, blues/bn*img_b, equal);
 }
 
 }
