@@ -27,7 +27,11 @@
 
 #define MIN_RESET_BUTTON_HEIGHT 17
 
-Adjuster::Adjuster (Glib::ustring vlabel, double vmin, double vmax, double vstep, double vdefault, bool editedcb) {
+static double one2one(double val) { return val; }
+
+Adjuster::Adjuster (Glib::ustring vlabel, double vmin, double vmax, double vstep, double vdefault, Gtk::Image *imgIcon, double2double_fun slider2value_, double2double_fun value2slider_) {
+
+  Gtk::HBox *hbox2=NULL;
 
   adjusterListener = NULL;
   afterReset = false;
@@ -35,10 +39,11 @@ Adjuster::Adjuster (Glib::ustring vlabel, double vmin, double vmax, double vstep
   automatic = NULL;
   eventPending = false;
 
+  slider2value = slider2value_ ? slider2value_ : one2one;
+  value2slider = value2slider_ ? value2slider_ : one2one;
   vMin = vmin;
   vMax = vmax;
   vStep = vstep;
-  initialDefaultVal = vdefault;
   addMode = false;
 
   // TODO: let the user chose the default value of Adjuster::delay, for slow machines
@@ -51,120 +56,58 @@ Adjuster::Adjuster (Glib::ustring vlabel, double vmin, double vmax, double vstep
   hbox->set_border_width(0);
   hbox->set_spacing(2);
 
-  adjustmentName = Glib::ustring(vlabel);
+  editedCheckBox = NULL;
 
-  if (editedcb) {
-    label = NULL;
-    editedCheckBox = Gtk::manage (new Gtk::CheckButton (adjustmentName));
-    editedChange = editedCheckBox->signal_toggled().connect( sigc::mem_fun(*this, &Adjuster::editedToggled) );
-    hbox->pack_start (*editedCheckBox);
+  if (!vlabel.empty()) {
+    adjustmentName = vlabel;
+    label = Gtk::manage (new Gtk::Label (adjustmentName, Gtk::ALIGN_LEFT));
+  }
+
+  reset = Gtk::manage (new Gtk::Button ());
+  reset->add (*Gtk::manage (new RTImage ("gtk-undo-ltr-small.png", "gtk-undo-rtl-small.png")));
+  reset->set_relief (Gtk::RELIEF_NONE);
+  reset->set_border_width (0);
+  reset->set_tooltip_text (M("ADJUSTER_RESET_TO_DEFAULT"));
+
+  spin = Gtk::manage (new MySpinButton ());
+  spin->set_has_frame(false);
+  spin->set_name("FramelessSpinButton");
+
+  reset->set_size_request (-1, spin->get_height() > MIN_RESET_BUTTON_HEIGHT ? spin->get_height(): MIN_RESET_BUTTON_HEIGHT);
+
+  slider = Gtk::manage (new MyHScale ());
+  slider->set_draw_value (false);
+
+  pack_start (*hbox, true, true);
+
+  if (vlabel.empty()) {
+    // No label, everything goes in hbox
+    if (imgIcon) hbox->pack_start (*imgIcon, Gtk::PACK_SHRINK);
+    hbox->pack_end (*reset, Gtk::PACK_SHRINK, 0); 
+    hbox->pack_end (*spin, Gtk::PACK_SHRINK, 0);
+    hbox->pack_start (*slider, true, true);
   }
   else {
-    editedCheckBox = NULL;
-    label = Gtk::manage (new Gtk::Label (adjustmentName, Gtk::ALIGN_LEFT));
+    // A label is provided, spreading the widgets in 2 rows
     hbox->pack_start (*label);
+    hbox->pack_end (*reset, Gtk::PACK_SHRINK, 0);
+    hbox->pack_end (*spin, Gtk::PACK_SHRINK, 0);
+    if (!imgIcon) {
+      pack_start (*slider, true, true);
+    }
+    else {
+      // A second HBox is necessary
+      hbox2 = Gtk::manage (new Gtk::HBox());
+      hbox2->pack_start (*imgIcon, Gtk::PACK_SHRINK);
+      hbox2->pack_start (*slider, true, true);
+      pack_start (*hbox2, true, true);
+    }
   }
 
-  reset = Gtk::manage (new Gtk::Button ());
-  reset->add (*Gtk::manage (new RTImage ("gtk-undo-ltr-small.png", "gtk-undo-rtl-small.png")));
-  reset->set_relief (Gtk::RELIEF_NONE);
-  reset->set_border_width (0);
-  reset->set_tooltip_text (M("ADJUSTER_RESET_TO_DEFAULT"));
-
-  hbox->pack_end (*reset, Gtk::PACK_SHRINK, 0); 
-  
-  spin = Gtk::manage (new MySpinButton ());
-  spin->set_has_frame(false);
-  spin->set_name("FramelessSpinButton");
-
-  hbox->pack_end (*spin, Gtk::PACK_SHRINK, 0);
-
-  reset->set_size_request (-1, spin->get_height() > MIN_RESET_BUTTON_HEIGHT ? spin->get_height(): MIN_RESET_BUTTON_HEIGHT);
-
-  slider = Gtk::manage (new MyHScale ());
-  slider->set_draw_value (false);
-
-  pack_start (*hbox, false, false);
-  pack_start (*slider, false, false);
-
   setLimits (vmin, vmax, vstep, vdefault);
   
   defaultVal = shapeValue (vdefault);
-  initialDefaultVal = shapeValue (vdefault);
-  editedState = defEditedState = Irrelevant;
-  autoState = Irrelevant;
-
-  sliderChange = slider->signal_value_changed().connect( sigc::mem_fun(*this, &Adjuster::sliderChanged) );
-  spinChange = spin->signal_value_changed().connect ( sigc::mem_fun(*this, &Adjuster::spinChanged), true);
-  reset->signal_button_release_event().connect_notify( sigc::mem_fun(*this, &Adjuster::resetPressed) );
-  slider->set_update_policy (Gtk::UPDATE_CONTINUOUS);
-  
-  show_all ();
-}
-
-Adjuster::Adjuster (Gtk::Image *imgIcon, double vmin, double vmax, double vstep, double vdefault, bool editedcb) {
-
-  adjusterListener = NULL;
-  afterReset = false;
-  blocked = false;
-  automatic = NULL;
-  eventPending = false;
-
-  vMin = vmin;
-  vMax = vmax;
-  vStep = vstep;
-  initialDefaultVal = vdefault;
-  addMode = false;
-
-  // TODO: let the user chose the default value of Adjuster::delay, for slow machines
-  delay = options.adjusterDelay;		// delay is no more static, so we can set the delay individually (useful for the RAW editor tab)
-
-  set_border_width (0);
-  set_spacing (2);
-
-  hbox = Gtk::manage (new Gtk::HBox ());
-  hbox->set_border_width(0);
-  hbox->set_spacing(2);
-
-  if (editedcb) {
-    editedCheckBox = Gtk::manage (new Gtk::CheckButton ());
-    editedCheckBox->set_border_width (0);
-    editedChange = editedCheckBox->signal_toggled().connect( sigc::mem_fun(*this, &Adjuster::editedToggled) );
-	hbox->pack_start (*editedCheckBox);
-  }
-  else
-    editedCheckBox = NULL;
-    
-
-  reset = Gtk::manage (new Gtk::Button ());
-  reset->add (*Gtk::manage (new RTImage ("gtk-undo-ltr-small.png", "gtk-undo-rtl-small.png")));
-  reset->set_relief (Gtk::RELIEF_NONE);
-  reset->set_border_width (0);
-  reset->set_tooltip_text (M("ADJUSTER_RESET_TO_DEFAULT"));
-
-  hbox->pack_start (*imgIcon, Gtk::PACK_SHRINK);
-
-  hbox->pack_end (*reset, Gtk::PACK_SHRINK, 0); 
-  
-  spin = Gtk::manage (new MySpinButton ());
-  spin->set_has_frame(false);
-  spin->set_name("FramelessSpinButton");
-
-  reset->set_size_request (-1, spin->get_height() > MIN_RESET_BUTTON_HEIGHT ? spin->get_height(): MIN_RESET_BUTTON_HEIGHT);
-
-  
-  slider = Gtk::manage (new MyHScale ());
-  slider->set_draw_value (false);
-
-  hbox->pack_end (*spin, Gtk::PACK_SHRINK, 0);
-  hbox->pack_start (*slider);
-
-  pack_start (*hbox, false, false);
-
-  setLimits (vmin, vmax, vstep, vdefault);
-  
-  defaultVal = shapeValue (vdefault);
-  initialDefaultVal = shapeValue (vdefault);
+  ctorDefaultVal = shapeValue (vdefault);
   editedState = defEditedState = Irrelevant;
   autoState = Irrelevant;
 
@@ -272,9 +215,7 @@ void Adjuster::spinReleased (GdkEventButton* event) {
     }
 }
 
-// Please note that it won't change the "Auto" CheckBox's state, if there
-void Adjuster::resetPressed (GdkEventButton* event) {
-
+void Adjuster::resetValue (bool toInitial) {
     if (editedState!=Irrelevant) {
         editedState = defEditedState;
         if (editedCheckBox) {
@@ -285,12 +226,26 @@ void Adjuster::resetPressed (GdkEventButton* event) {
         refreshLabelStyle ();
     }
     afterReset = true;
+    if (toInitial) {
+        // resetting to the initial editing value, when the image has been loaded
+	slider->set_value (addMode ? defaultVal : value2slider(defaultVal));
+    }
+    else {
+        // resetting to the slider default value
+        if (addMode)
+            slider->set_value (0.);
+        else
+            slider->set_value (value2slider(ctorDefaultVal));
+    }
+}
+
+// Please note that it won't change the "Auto" CheckBox's state, if there
+void Adjuster::resetPressed (GdkEventButton* event) {
+
     if ((event != NULL) && (event->state & GDK_CONTROL_MASK) && (event->button == 1))
-        // CTRL pressed : resetting to current default value
-        slider->set_value (defaultVal);
+        resetValue(true);
     else
-        // no modifier key or addMode=true : resetting to initial default value
-        slider->set_value (initialDefaultVal);
+        resetValue(false);
 }
 
 double Adjuster::shapeValue (double a) {
@@ -310,8 +265,8 @@ void Adjuster::setLimits (double vmin, double vmax, double vstep, double vdefaul
   spin->set_value (shapeValue(vdefault));
   slider->set_digits (digits);
   slider->set_increments (vstep, 2.0*vstep);
-  slider->set_range (vmin, vmax);
-  slider->set_value (shapeValue(vdefault));
+  slider->set_range (addMode ? vmin : value2slider(vmin), addMode ? vmax : value2slider(vmax));
+  slider->set_value (addMode ? shapeValue(vdefault) : value2slider(shapeValue(vdefault)));
   //defaultVal = shapeValue (vdefault);
   sliderChange.block (false);
   spinChange.block (false);
@@ -320,6 +275,7 @@ void Adjuster::setLimits (double vmin, double vmax, double vstep, double vdefaul
 void Adjuster::setAddMode(bool addM) {
 	if (addM != addMode) {
 		// Switching the Adjuster to the new mode
+		addMode = addM;
 		if (addM) {
 			// Switching to the relative mode
 			double range = -vMin + vMax;
@@ -330,7 +286,6 @@ void Adjuster::setAddMode(bool addM) {
 			// Switching to the absolute mode
 			setLimits(vMin, vMax, vStep, defaultVal);
 		}
-		addMode = addM;
 	}
 }
 
@@ -340,7 +295,7 @@ void Adjuster::spinChanged () {
     delayConnection.disconnect ();
 
   sliderChange.block (true);
-  slider->set_value (spin->get_value ());
+  slider->set_value (addMode ? spin->get_value () : value2slider(spin->get_value ()));
   sliderChange.block (false);
 
   if (delay==0) {
@@ -375,7 +330,7 @@ void Adjuster::sliderChanged () {
     delayConnection.disconnect ();
 
   spinChange.block (true);
-  spin->set_value (slider->get_value ());
+  spin->set_value (addMode ? slider->get_value () : slider2value(slider->get_value ()));
   spinChange.block (false);
 
   if (delay==0 || afterReset) {
@@ -409,7 +364,7 @@ void Adjuster::setValue (double a) {
   spinChange.block (true);
   sliderChange.block (true);
   spin->set_value (shapeValue (a));
-  slider->set_value (shapeValue (a));
+  slider->set_value (addMode ? shapeValue(a) : value2slider(shapeValue (a)));
   sliderChange.block (false);
   spinChange.block (false);
   afterReset = false;
