@@ -32,6 +32,7 @@ class AdjusterListener {
     virtual void adjusterAutoToggled (Adjuster* a, bool newval) {}
 };
 
+typedef double(*double2double_fun)(double val);
 
 class Adjuster : public Gtk::VBox {
 
@@ -53,7 +54,7 @@ class Adjuster : public Gtk::VBox {
     sigc::connection buttonReleaseSpin;
     bool listenerReady;
     double defaultVal;			// current default value (it can change when switching from ADD or SET mode)
-    double initialDefaultVal;	// default value at construction time
+    double ctorDefaultVal;		// default value at construction time
     EditedState editedState;
     EditedState defEditedState;
     EditedState autoState;
@@ -70,13 +71,13 @@ class Adjuster : public Gtk::VBox {
 
     double shapeValue (double a);
     void   refreshLabelStyle ();
+    double2double_fun value2slider, slider2value;
 
   public:
 
     int delay;
 
-    Adjuster (Glib::ustring label, double vmin, double vmax, double vstep, double vdefault, bool editedCheckBox=false);
-    Adjuster (Gtk::Image *imgIcon, double vmin, double vmax, double vstep, double vdefault, bool editedCheckBox=false);
+    Adjuster (Glib::ustring vlabel, double vmin, double vmax, double vstep, double vdefault, Gtk::Image *imgIcon=NULL, double2double_fun slider2value=NULL, double2double_fun value2slider=NULL);
     virtual ~Adjuster ();
 
     // Add an "Automatic" checkbox next to the reset button.
@@ -122,6 +123,7 @@ class Adjuster : public Gtk::VBox {
     bool notifyListener ();
     void sliderReleased (GdkEventButton* event);
     void spinReleased (GdkEventButton* event);
+    void resetValue (bool toInitial);
     void resetPressed (GdkEventButton* event);
     void editedToggled ();
     double trimValue (double& val);
