@@ -25,6 +25,29 @@
 #include "../rtengine/safegtk.h"
 #include "rtimage.h"
 
+struct BQProcessLoaded {
+    BatchQueue* bq;
+};
+
+int processLoadedBatchQueueUIThread (void* data) {
+
+    BatchQueue* bq = static_cast<BatchQueue*>(data);
+    bq->resizeLoadedQueue();
+    return 0;
+}
+
+static Glib::ustring makeFolderLabel(Glib::ustring path)
+{
+    if (!safe_file_test (path, Glib::FILE_TEST_IS_DIR))
+        return "(" + M("GENERAL_NONE") + ")";
+    if (path.size() > 40) {
+        size_t last_ds = path.find_last_of (G_DIR_SEPARATOR);
+        if (last_ds != Glib::ustring::npos && last_ds > 10) {
+            path = "..." + path.substr(last_ds);
+        }
+    }
+    return path;
+}
 
 BatchQueuePanel::BatchQueuePanel () {
 
@@ -65,10 +88,32 @@ BatchQueuePanel::BatchQueuePanel () {
     Gtk::HBox* hb3 = Gtk::manage (new Gtk::HBox ());
     useFolder = Gtk::manage (new Gtk::RadioButton (M("PREFERENCES_OUTDIRFOLDER")+":"));
     hb3->pack_start (*useFolder, Gtk::PACK_SHRINK,4);
+
+#if defined(__APPLE__) || defined(__linux__)
+    // At the time of writing (2013-11-11) the gtkmm FileChooserButton with ACTION_SELECT_FOLDER
+    // is so buggy on these platforms (OS X and Linux) that we rather employ this ugly button hack.
+    // When/if GTKMM gets fixed we can go back to use the FileChooserButton, like we do on Windows.
+    outdirFolderButton = Gtk::manage (new Gtk::Button("(" + M("GENERAL_NONE") + ")"));
+    outdirFolderButton->set_alignment(0.0, 0.0);
+    hb3->pack_start (*outdirFolderButton);
+    outdirFolderButton->signal_pressed().connect( sigc::mem_fun(*this, &BatchQueuePanel::pathFolderButtonPressed) );
+    outdirFolderButton->set_tooltip_markup (M("PREFERENCES_OUTDIRFOLDERHINT"));
+    outdirFolderButton->set_label(makeFolderLabel(options.savePathFolder));
+    Gtk::Image* folderImg = Gtk::manage (new Gtk::Image (Gtk::Stock::DIRECTORY, Gtk::ICON_SIZE_MENU));
+    folderImg->show ();
+    outdirFolderButton->set_image (*folderImg);
+    outdirFolder = 0;
+#else
     outdirFolder = Gtk::manage (new MyFileChooserButton (M("PREFERENCES_OUTDIRFOLDER"), Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER));
     hb3->pack_start (*outdirFolder);
-    odvb->pack_start (*hb3, Gtk::PACK_SHRINK, 4);
+    outdirFolder->signal_current_folder_changed().connect (sigc::mem_fun(*this, &BatchQueuePanel::pathFolderChanged));
     outdirFolder->set_tooltip_markup (M("PREFERENCES_OUTDIRFOLDERHINT"));
+    if (safe_file_test (options.savePathFolder, Glib::FILE_TEST_IS_DIR)) 
+        outdirFolder->set_current_folder (options.savePathFolder);
+    outdirFolderButton = 0;
+#endif
+
+    odvb->pack_start (*hb3, Gtk::PACK_SHRINK, 4);
     useFolder->set_tooltip_markup (M("PREFERENCES_OUTDIRFOLDERHINT"));
     Gtk::RadioButton::Group g = useTemplate->get_group();
     useFolder->set_group (g);
@@ -81,14 +126,11 @@ BatchQueuePanel::BatchQueuePanel () {
 
     saveFormatPanel->init (options.saveFormatBatch);
     outdirTemplate->set_text (options.savePathTemplate);
-    if (safe_file_test (options.savePathFolder, Glib::FILE_TEST_IS_DIR)) 
-        outdirFolder->set_current_folder (options.savePathFolder);
     useTemplate->set_active (options.saveUsePathTemplate);
     useFolder->set_active (!options.saveUsePathTemplate);
 
     // setup signal handlers
     outdirTemplate->signal_changed().connect (sigc::mem_fun(*this, &BatchQueuePanel::saveOptions));    
-    outdirFolder->signal_current_folder_changed().connect (sigc::mem_fun(*this, &BatchQueuePanel::pathFolderChanged));
     useTemplate->signal_toggled().connect (sigc::mem_fun(*this, &BatchQueuePanel::saveOptions));    
     useFolder->signal_toggled().connect (sigc::mem_fun(*this, &BatchQueuePanel::saveOptions));    
     saveFormatPanel->setListener (this);
@@ -132,7 +174,9 @@ BatchQueuePanel::BatchQueuePanel () {
     batchQueue->setBatchQueueListener (this);
 
     show_all ();
-    batchQueue->loadBatchQueue ();
+    if (batchQueue->loadBatchQueue ()) {
+        g_idle_add_full (G_PRIORITY_LOW, processLoadedBatchQueueUIThread, batchQueue, NULL);
+    }
 }
 
 
@@ -179,17 +223,21 @@ void BatchQueuePanel::updateTab (int qsize)
     }
 }
 
-void BatchQueuePanel::queueSizeChanged (int qsize, bool queueEmptied)
+void BatchQueuePanel::queueSizeChanged (int qsize, bool queueEmptied, bool queueError, Glib::ustring queueErrorMessage)
 {
 	updateTab ( qsize);
 
-    if (queueEmptied) {
+    if (queueEmptied || queueError) {
         stopBatchProc ();
         fdir->set_sensitive (true);
         fformat->set_sensitive (true);
 
         SoundManager::playSoundAsync(options.sndBatchQueueDone);
     }
+    if (queueError) {
+        Gtk::MessageDialog msgd (queueErrorMessage, true, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
+        msgd.run ();
+    }
 }
 
 void BatchQueuePanel::startBatchProc () {
@@ -250,6 +298,22 @@ void BatchQueuePanel::saveOptions () {
     options.procQueueEnabled    = autoStart->get_active ();
 }
 
+void BatchQueuePanel::pathFolderButtonPressed () {
+
+    Gtk::FileChooserDialog fc(M("PREFERENCES_OUTDIRFOLDER"),Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER );
+    fc.add_button( Gtk::StockID("gtk-cancel"), Gtk::RESPONSE_CANCEL);
+    fc.add_button( Gtk::StockID("gtk-ok"), Gtk::RESPONSE_OK);
+    fc.set_filename(options.savePathFolder);
+    fc.set_transient_for(*parent);
+    int result = fc.run();
+    if (result == Gtk::RESPONSE_OK) {
+        if (safe_file_test(fc.get_current_folder(), Glib::FILE_TEST_IS_DIR)) {
+            options.savePathFolder = fc.get_current_folder();
+            outdirFolderButton->set_label(makeFolderLabel(options.savePathFolder));
+        }
+    }
+}
+
 // We only want to save the following when it changes,
 // since these settings are shared with editorpanel : 
 void BatchQueuePanel::pathFolderChanged () {
@@ -262,3 +326,7 @@ void BatchQueuePanel::formatChanged (Glib::ustring f) {
     options.saveFormatBatch = saveFormatPanel->getFormat ();
     
 }
+
+bool BatchQueuePanel::handleShortcutKey (GdkEventKey* event) {
+	return batchQueue->keyPressed (event);
+}
