@@ -27,6 +27,7 @@ using namespace rtengine::procparams;
 
 BatchToolPanelCoordinator::BatchToolPanelCoordinator (FilePanel* parent) : ToolPanelCoordinator(), parent(parent) {
 
+	blockedUpdate = false;
 	// remove exif panel and iptc panel
 	std::vector<ToolPanel*>::iterator epi = std::find (toolPanels.begin(), toolPanels.end(), exifpanel);
 	if (epi!=toolPanels.end())
@@ -56,13 +57,13 @@ void BatchToolPanelCoordinator::selectionChanged (const std::vector<Thumbnail*>&
 
 void BatchToolPanelCoordinator::closeSession (bool save) {
 
-    pparamsEdited.set (false);        
+    pparamsEdited.set (false);
 
     for (size_t i=0; i<selected.size(); i++)
         selected[i]->removeThumbnailListener (this);
 
     if (somethingChanged && save) {
-        
+
         // read new values from the gui
         for (size_t i=0; i<toolPanels.size(); i++)
             toolPanels[i]->write (&pparams, &pparamsEdited);
@@ -73,11 +74,11 @@ void BatchToolPanelCoordinator::closeSession (bool save) {
             newParams = initialPP[i];
             pparamsEdited.combine (newParams, pparams, selected.size()==1);
 
-    		// trim new adjuster's values to the adjuster's limits
-    		for (unsigned int j=0; j<toolPanels.size(); j++)
-    			toolPanels[j]->trimValues (&newParams);
+            // trim new adjuster's values to the adjuster's limits
+            for (unsigned int j=0; j<toolPanels.size(); j++)
+                toolPanels[j]->trimValues (&newParams);
 
-    		selected[i]->setProcParams (newParams, NULL, BATCHEDITOR, true);
+            selected[i]->setProcParams (newParams, NULL, BATCHEDITOR, true);
         }
     }
     for (size_t i=0; i<paramcListeners.size(); i++)
@@ -125,13 +126,15 @@ void BatchToolPanelCoordinator::initSession () {
 
 			toneCurve->setAdjusterBehavior (false, false, false, false, false, false, false, false);
 			lcurve->setAdjusterBehavior (false, false, false);
-			whitebalance->setAdjusterBehavior (false, false);
-			vibrance->setAdjusterBehavior (false, false, false);
+			whitebalance->setAdjusterBehavior (false, false, false);
+			vibrance->setAdjusterBehavior (false, false);
 			vignetting->setAdjusterBehavior (false, false, false, false);
 			colorappearance->setAdjusterBehavior (false, false, false, false, false, false, false, false, false, false, false, false, false);
 			rotate->setAdjusterBehavior (false);
 			distortion->setAdjusterBehavior (false);
 			perspective->setAdjusterBehavior (false);
+			gradient->setAdjusterBehavior (false, false, false, false);
+			pcvignette->setAdjusterBehavior (false, false, false);
 			cacorrection->setAdjusterBehavior (false);
 			sharpening->setAdjusterBehavior (false);
 			sharpenEdge->setAdjusterBehavior (false, false);
@@ -139,8 +142,10 @@ void BatchToolPanelCoordinator::initSession () {
 			icm->setAdjusterBehavior (false, false);
 			
 			chmixer->setAdjusterBehavior (false);
+			blackwhite->setAdjusterBehavior (false,false);
+			
 			shadowshighlights->setAdjusterBehavior (false, false, false);
-			dirpyrequalizer->setAdjusterBehavior (false);
+			dirpyrequalizer->setAdjusterBehavior (false, false);
 			dirpyrdenoise->setAdjusterBehavior (false, false,false,false,false,false);
 			preprocess->setAdjusterBehavior (false, false);
 			rawcacorrection->setAdjusterBehavior (false);
@@ -153,21 +158,25 @@ void BatchToolPanelCoordinator::initSession () {
 
 			toneCurve->setAdjusterBehavior (options.baBehav[ADDSET_TC_EXPCOMP], options.baBehav[ADDSET_TC_HLCOMPAMOUNT],options.baBehav[ADDSET_TC_HLCOMPTHRESH], options.baBehav[ADDSET_TC_BRIGHTNESS], options.baBehav[ADDSET_TC_BLACKLEVEL],options.baBehav[ADDSET_TC_SHCOMP], options.baBehav[ADDSET_TC_CONTRAST], options.baBehav[ADDSET_TC_SATURATION]);
 			lcurve->setAdjusterBehavior (options.baBehav[ADDSET_LC_BRIGHTNESS], options.baBehav[ADDSET_LC_CONTRAST], options.baBehav[ADDSET_LC_CHROMATICITY]);
-			whitebalance->setAdjusterBehavior (options.baBehav[ADDSET_WB_TEMPERATURE], options.baBehav[ADDSET_WB_GREEN]);
+			whitebalance->setAdjusterBehavior (options.baBehav[ADDSET_WB_TEMPERATURE], options.baBehav[ADDSET_WB_GREEN], options.baBehav[ADDSET_WB_EQUAL]);
+			vibrance->setAdjusterBehavior (options.baBehav[ADDSET_VIBRANCE_PASTELS], options.baBehav[ADDSET_VIBRANCE_SATURATED]);
 			vignetting->setAdjusterBehavior (options.baBehav[ADDSET_VIGN_AMOUNT], options.baBehav[ADDSET_VIGN_RADIUS], options.baBehav[ADDSET_VIGN_STRENGTH], options.baBehav[ADDSET_VIGN_CENTER]);
 			colorappearance->setAdjusterBehavior (options.baBehav[ADDSET_CAT_DEGREE], options.baBehav[ADDSET_CAT_ADAPTSCENE], options.baBehav[ADDSET_CAT_ADAPTVIEWING],options.baBehav[ADDSET_CAT_BADPIX], options.baBehav[ADDSET_CAT_LIGHT], options.baBehav[ADDSET_CAT_CHROMA],options.baBehav[ADDSET_CAT_CONTRAST],options.baBehav[ADDSET_CAT_RSTPRO],options.baBehav[ADDSET_CAT_BRIGHT],options.baBehav[ADDSET_CAT_CONTRAST_Q],options.baBehav[ADDSET_CAT_CHROMA_S],options.baBehav[ADDSET_CAT_CHROMA_M],options.baBehav[ADDSET_CAT_HUE]);
 			rotate->setAdjusterBehavior (options.baBehav[ADDSET_ROTATE_DEGREE]);
 			distortion->setAdjusterBehavior (options.baBehav[ADDSET_DIST_AMOUNT]);
 			perspective->setAdjusterBehavior (options.baBehav[ADDSET_PERSPECTIVE]);
+			gradient->setAdjusterBehavior (options.baBehav[ADDSET_GRADIENT_DEGREE], options.baBehav[ADDSET_GRADIENT_FEATHER], options.baBehav[ADDSET_GRADIENT_STRENGTH], options.baBehav[ADDSET_GRADIENT_CENTER]);
+			pcvignette->setAdjusterBehavior (options.baBehav[ADDSET_PCVIGNETTE_STRENGTH], options.baBehav[ADDSET_PCVIGNETTE_FEATHER], options.baBehav[ADDSET_PCVIGNETTE_ROUNDNESS]);
 			cacorrection->setAdjusterBehavior (options.baBehav[ADDSET_CA]);
 			sharpening->setAdjusterBehavior (options.baBehav[ADDSET_SHARP_AMOUNT]);
 			sharpenEdge->setAdjusterBehavior (options.baBehav[ADDSET_SHARPENEDGE_AMOUNT],options.baBehav[ADDSET_SHARPENEDGE_PASS]);
 			sharpenMicro->setAdjusterBehavior (options.baBehav[ADDSET_SHARPENMICRO_AMOUNT],options.baBehav[ADDSET_SHARPENMICRO_UNIFORMITY]);
 			icm->setAdjusterBehavior (options.baBehav[ADDSET_FREE_OUPUT_GAMMA],options.baBehav[ADDSET_FREE_OUTPUT_SLOPE]);
 			
-			chmixer->setAdjusterBehavior (options.baBehav[ADDSET_CHMIXER]);
+			chmixer->setAdjusterBehavior (options.baBehav[ADDSET_CHMIXER] );
+			blackwhite->setAdjusterBehavior (options.baBehav[ADDSET_BLACKWHITE_HUES],options.baBehav[ADDSET_BLACKWHITE_GAMMA]);
 			shadowshighlights->setAdjusterBehavior (options.baBehav[ADDSET_SH_HIGHLIGHTS], options.baBehav[ADDSET_SH_SHADOWS], options.baBehav[ADDSET_SH_LOCALCONTRAST]);
-			dirpyrequalizer->setAdjusterBehavior (options.baBehav[ADDSET_DIRPYREQ]);
+			dirpyrequalizer->setAdjusterBehavior (options.baBehav[ADDSET_DIRPYREQ], options.baBehav[ADDSET_DIRPYREQ_THRESHOLD]);
 			dirpyrdenoise->setAdjusterBehavior (options.baBehav[ADDSET_DIRPYRDN_LUMA],options.baBehav[ADDSET_DIRPYRDN_LUMDET],options.baBehav[ADDSET_DIRPYRDN_CHROMA],options.baBehav[ADDSET_DIRPYRDN_CHROMARED],options.baBehav[ADDSET_DIRPYRDN_CHROMABLUE], options.baBehav[ADDSET_DIRPYRDN_GAMMA]);
 			preprocess->setAdjusterBehavior (options.baBehav[ADDSET_PREPROCESS_LINEDENOISE], options.baBehav[ADDSET_PREPROCESS_GREENEQUIL]);
 			rawcacorrection->setAdjusterBehavior (options.baBehav[ADDSET_RAWCACORR]);
@@ -196,10 +205,16 @@ void BatchToolPanelCoordinator::initSession () {
 			if (options.baBehav[ADDSET_SHARPENMICRO_UNIFORMITY])  pparams.sharpenMicro.uniformity = 0;
 
 			if (options.baBehav[ADDSET_CHMIXER]) for (int i=0; i<3; i++) pparams.chmixer.red[i] = pparams.chmixer.green[i] = pparams.chmixer.blue[i] = 0;
+			if (options.baBehav[ADDSET_BLACKWHITE_HUES]) pparams.blackwhite.mixerRed=pparams.blackwhite.mixerOrange=pparams.blackwhite.mixerYellow=
+														 pparams.blackwhite.mixerGreen=pparams.blackwhite.mixerCyan=pparams.blackwhite.mixerBlue=
+														 pparams.blackwhite.mixerMagenta=pparams.blackwhite.mixerPurple=0;
+			if (options.baBehav[ADDSET_BLACKWHITE_GAMMA]) pparams.blackwhite.gammaRed=pparams.blackwhite.gammaGreen=pparams.blackwhite.gammaBlue = 0;
+
 			//if (options.baBehav[ADDSET_LD_EDGETOLERANCE])  pparams.lumaDenoise.edgetolerance = 0;
 
 			if (options.baBehav[ADDSET_WB_TEMPERATURE])  pparams.wb.temperature = 0;
 			if (options.baBehav[ADDSET_WB_GREEN])  pparams.wb.green = 0;
+			if (options.baBehav[ADDSET_WB_EQUAL])  pparams.wb.equal = 0;
 
 			if (options.baBehav[ADDSET_VIBRANCE_PASTELS])  pparams.vibrance.pastels = 0;
 			if (options.baBehav[ADDSET_VIBRANCE_SATURATED])  pparams.vibrance.saturated = 0;
@@ -229,6 +244,14 @@ void BatchToolPanelCoordinator::initSession () {
 			if (options.baBehav[ADDSET_ROTATE_DEGREE])  pparams.rotate.degree = 0;
 			if (options.baBehav[ADDSET_DIST_AMOUNT])  pparams.distortion.amount = 0;
 			if (options.baBehav[ADDSET_PERSPECTIVE])  pparams.perspective.horizontal = pparams.perspective.vertical = 0;
+			if (options.baBehav[ADDSET_GRADIENT_DEGREE])  pparams.gradient.degree = 0;
+			if (options.baBehav[ADDSET_GRADIENT_FEATHER])  pparams.gradient.feather = 0;
+			if (options.baBehav[ADDSET_GRADIENT_STRENGTH])  pparams.gradient.strength = 0;
+			if (options.baBehav[ADDSET_GRADIENT_CENTER])  pparams.gradient.centerX = 0;
+			if (options.baBehav[ADDSET_GRADIENT_CENTER])  pparams.gradient.centerY = 0;
+			if (options.baBehav[ADDSET_PCVIGNETTE_STRENGTH])  pparams.pcvignette.strength = 0;
+			if (options.baBehav[ADDSET_PCVIGNETTE_FEATHER])  pparams.pcvignette.feather = 0;
+			if (options.baBehav[ADDSET_PCVIGNETTE_ROUNDNESS])  pparams.pcvignette.roundness = 0;
 			if (options.baBehav[ADDSET_CA])  pparams.cacorrection.red = 0;
 			if (options.baBehav[ADDSET_CA])  pparams.cacorrection.blue = 0;
 			if (options.baBehav[ADDSET_VIGN_AMOUNT])  pparams.vignetting.amount = 0;
@@ -238,6 +261,8 @@ void BatchToolPanelCoordinator::initSession () {
 			if (options.baBehav[ADDSET_VIGN_CENTER])  pparams.vignetting.centerY = 0;
 
 			if (options.baBehav[ADDSET_DIRPYREQ]) for (int i=0; i<5; i++) pparams.dirpyrequalizer.mult[i] = 0;
+			if (options.baBehav[ADDSET_DIRPYREQ_THRESHOLD]) pparams.dirpyrequalizer.threshold = 0;
+
 			if (options.baBehav[ADDSET_DIRPYRDN_LUMA]) pparams.dirpyrDenoise.luma = 0;
 			
 			if (options.baBehav[ADDSET_DIRPYRDN_CHROMA]) pparams.dirpyrDenoise.chroma = 0;
@@ -285,6 +310,11 @@ void BatchToolPanelCoordinator::panelChanged (rtengine::ProcEvent event, const G
         for (size_t i=0; i<selected.size(); i++) {
             initialPP[i].toneCurve.autoexp = pparams.toneCurve.autoexp;
             initialPP[i].toneCurve.clip = pparams.toneCurve.clip;
+
+            // at this stage, we don't know if HL Reconstruction will be enabled or not (depending on the raw histogram),
+            // so we're forcing its paramseditd value to false (i.e. mixed state)
+            pparamsEdited.toneCurve.hrenabled = false;
+
             selected[i]->applyAutoExp (initialPP[i]);
         }
 
@@ -313,10 +343,10 @@ void BatchToolPanelCoordinator::panelChanged (rtengine::ProcEvent event, const G
         paramcListeners[i]->procParamsChanged (&pparams, event, descr, &pparamsEdited);
 }
 
-void BatchToolPanelCoordinator::getAutoWB (double& temp, double& green) {
+void BatchToolPanelCoordinator::getAutoWB (double& temp, double& green, double equal) {
 
     if (!selected.empty())
-        selected[0]->getAutoWB (temp, green);       
+        selected[0]->getAutoWB (temp, green, equal);
 }
 
 void BatchToolPanelCoordinator::getCamWB (double& temp, double& green) {
@@ -333,12 +363,28 @@ void BatchToolPanelCoordinator::optionsChanged () {
 
 void BatchToolPanelCoordinator::procParamsChanged (Thumbnail* thm, int whoChangedIt) {
 
-    if (whoChangedIt!=BATCHEDITOR) {
+    if (whoChangedIt!=BATCHEDITOR && !blockedUpdate) {
         closeSession (false);
         initSession ();
     }
 }
 
+void BatchToolPanelCoordinator::beginBatchPParamsChange(int numberOfEntries) {
+
+    blockedUpdate = true;
+    if (numberOfEntries > 50) // Arbitrary amount
+        parent->set_sensitive(false);
+}
+
+// The end of a batch pparams change triggers a close/initsession
+void BatchToolPanelCoordinator::endBatchPParamsChange() {
+    //printf("BatchToolPanelCoordinator::endBatchPParamsChange  /  Nouvelle session!\n");
+    closeSession (false);
+    initSession ();
+    blockedUpdate = false;
+    parent->set_sensitive(true);
+}
+
 /*
  * WARNING: profileChange is actually called by the History only.
  *          Using a Profile panel in the batch tool panel editor is actually
@@ -372,7 +418,7 @@ void BatchToolPanelCoordinator::profileChange  (const rtengine::procparams::Part
     ProcParams newParams;
     for (size_t i=0; i<selected.size(); i++) {
         newParams = initialPP[i];
-        pparamsEdited.combine (newParams, pparams, true);
+        pparamsEdited.combine (newParams, pparams, selected.size()==1);
         selected[i]->setProcParams (newParams, NULL, BATCHEDITOR, false);
     }
 
