@@ -33,8 +33,8 @@ CacheManager::getInstance(void)
 	static CacheManager* instance_ = 0;
 	if ( instance_ == 0 )
 	{
-		static Glib::Mutex smutex_;
-		Glib::Mutex::Lock lock(smutex_);
+		static MyMutex smutex_;
+		MyMutex::MyLock lock(smutex_);
 		if ( instance_ == 0 )
 		{
 			instance_ = new CacheManager();
@@ -45,7 +45,7 @@ CacheManager::getInstance(void)
 
 void CacheManager::init () {
 
-    Glib::Mutex::Lock lock(mutex_);
+    MyMutex::MyLock lock(mutex_);
 
     openEntries.clear ();
     baseDir = options.cacheBaseDir;
@@ -71,7 +71,7 @@ Thumbnail* CacheManager::getEntry (const Glib::ustring& fname) {
     // take manager lock and search for entry, if found return it else release
     // lock and create it
 	{
-        Glib::Mutex::Lock lock(mutex_);
+        MyMutex::MyLock lock(mutex_);
 
 		string_thumb_map::iterator r = openEntries.find (fname);
 		// if it is open, return it
@@ -93,7 +93,7 @@ Thumbnail* CacheManager::getEntry (const Glib::ustring& fname) {
     if (safe_file_test (cfname, Glib::FILE_TEST_EXISTS)) {
         CacheImageData* cfs = new CacheImageData ();
         int e = cfs->load (cfname);
-        if (!e && cfs->supported==true) 
+        if (!e && cfs->supported==true)
             res = new Thumbnail (this, fname, cfs);
         if (res && !res->isSupported ()) {
             delete res;
@@ -115,7 +115,7 @@ Thumbnail* CacheManager::getEntry (const Glib::ustring& fname) {
     // was use it over our version. if not added we create the cache entry
     if (res)
 	{
-		Glib::Mutex::Lock lock(mutex_);
+		MyMutex::MyLock lock(mutex_);
 
 		string_thumb_map::iterator r = openEntries.find (fname);
 		if (r!=openEntries.end()) {
@@ -134,7 +134,7 @@ Thumbnail* CacheManager::getEntry (const Glib::ustring& fname) {
 
 void CacheManager::deleteEntry (const Glib::ustring& fname) {
 
-    Glib::Mutex::Lock lock(mutex_);
+    MyMutex::MyLock lock(mutex_);
 
     // check if it is opened
     string_thumb_map::iterator r = openEntries.find (fname);
@@ -196,7 +196,7 @@ void CacheManager::clearFromCache (const Glib::ustring& fname, bool leavenotrace
 
 void CacheManager::renameEntry (const std::string& oldfilename, const std::string& oldmd5, const std::string& newfilename) {
 
-    Glib::Mutex::Lock lock(mutex_);
+    MyMutex::MyLock lock(mutex_);
 
     std::string newmd5 = getMD5 (newfilename);
 
@@ -224,7 +224,7 @@ void CacheManager::renameEntry (const std::string& oldfilename, const std::strin
 
 void CacheManager::closeThumbnail (Thumbnail* t) {
 
-    Glib::Mutex::Lock lock(mutex_);
+    MyMutex::MyLock lock(mutex_);
 
     t->updateCache ();
     string_thumb_map::iterator r = openEntries.find (t->getFileName());
@@ -235,14 +235,14 @@ void CacheManager::closeThumbnail (Thumbnail* t) {
 
 void CacheManager::closeCache () {
 
-    Glib::Mutex::Lock lock(mutex_);
+    MyMutex::MyLock lock(mutex_);
 
     applyCacheSizeLimitation ();
 }
 
 void CacheManager::clearAll () {
 
-    Glib::Mutex::Lock lock(mutex_);
+    MyMutex::MyLock lock(mutex_);
 
     deleteDir ("images");
     deleteDir ("aehistograms");
@@ -260,7 +260,7 @@ void CacheManager::clearAll () {
 }
 void CacheManager::clearThumbImages () {
 
-    Glib::Mutex::Lock lock(mutex_);
+    MyMutex::MyLock lock(mutex_);
 
     deleteDir ("images");
     deleteDir ("aehistograms");
@@ -268,7 +268,7 @@ void CacheManager::clearThumbImages () {
 }
 
 void CacheManager::clearProfiles () {
-    Glib::Mutex::Lock lock(mutex_);
+    MyMutex::MyLock lock(mutex_);
 
     deleteDir ("profiles");
 }
