@@ -893,14 +893,14 @@ void ColorAppearance::setDefaults (const ProcParams* defParams, const ParamsEdit
 	}
 }
 int autoCamChangedUI (void* data) {
+    GThreadLock lock; // All GUI acces from idle_add callbacks or separate thread HAVE to be protected
     (static_cast<ColorAppearance*>(data))->autoCamComputed_ ();
     return 0;
 }
 void ColorAppearance::autoCamChanged (double ccam) 
-{
+{  
     nextCcam = ccam;
-	g_idle_add (autoCamChangedUI, this);
-  //  Glib::signal_idle().connect (sigc::mem_fun(*this, &ColorAppearance::autoCamComputed_));
+    g_idle_add (autoCamChangedUI, this);
 }
 
 bool ColorAppearance::autoCamComputed_ () {
@@ -913,14 +913,14 @@ bool ColorAppearance::autoCamComputed_ () {
     return false;
 }
 int adapCamChangedUI (void* data) {
+    GThreadLock lock; // All GUI acces from idle_add callbacks or separate thread HAVE to be protected
     (static_cast<ColorAppearance*>(data))->adapCamComputed_ ();
     return 0;
 }
 void ColorAppearance::adapCamChanged (double cadap) 
 {
     nextCadap = cadap;
-	g_idle_add (adapCamChangedUI, this);
-  //  Glib::signal_idle().connect (sigc::mem_fun(*this, &ColorAppearance::autoCamComputed_));
+    g_idle_add (adapCamChangedUI, this);
 }
 
 bool ColorAppearance::adapCamComputed_ () {
@@ -1175,7 +1175,7 @@ void ColorAppearance::setBatchMode (bool batchMode) {
 //	curveEditorG3->setBatchMode (batchMode);
 }
 
-void ColorAppearance::updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histLCAM,  LUTu & histCCAM, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma){
+void ColorAppearance::updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histCLurve, LUTu & histLLCurve, LUTu & histLCAM,  LUTu & histCCAM, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma){
 
 	shape->updateBackgroundHistogram (histLCAM);
 	shape3->updateBackgroundHistogram (histCCAM);
