@@ -469,11 +469,13 @@ void Crop::enabledChanged () {
 }
 
 int notifyListenerUI (void* data) {
+    GThreadLock lock; // All GUI acces from idle_add callbacks or separate thread HAVE to be protected
     (static_cast<Crop*>(data))->notifyListener ();
     return 0;
 }
 
 int refreshSpinsUI (void* data) {
+    GThreadLock lock; // All GUI acces from idle_add callbacks or separate thread HAVE to be protected
     RefreshSpinHelper* rsh = static_cast<RefreshSpinHelper*>(data);
     rsh->crop->refreshSpins (rsh->notify);
     delete rsh;
@@ -587,23 +589,23 @@ void Crop::ratioChanged () {
 
 // Correct current crop if it doesn't fit
 void Crop::adjustCropToRatio() {
-  if (fixr->get_active() && !fixr->get_inconsistent()) {
+    if (fixr->get_active() && !fixr->get_inconsistent()) {
  
-//    int W = w->get_value ();
-//    int H = h->get_value ();
-    int W = nw;
-    int H = nh;
-    int X = nx;
-    int Y = ny;
-    if (W>=H)
-      cropWidth2Resized (X, Y, W, H);
-    else 
-      cropHeight2Resized (X, Y, W, H);
+//        int W = w->get_value ();
+//        int H = h->get_value ();
+        int W = nw;
+        int H = nh;
+        int X = nx;
+        int Y = ny;
+        if (W>=H)
+          cropWidth2Resized (X, Y, W, H);
+        else
+          cropHeight2Resized (X, Y, W, H);
     }
 
-    // This will safe the options
+    // This will save the options
     g_idle_add (refreshSpinsUI, new RefreshSpinHelper (this, true));
-  }
+}
 
 void Crop::refreshSize () {
 
@@ -667,6 +669,7 @@ struct setdimparams {
 };
 
 int sizeChangedUI (void* data) {
+    GThreadLock lock; // All GUI acces from idle_add callbacks or separate thread HAVE to be protected
     setdimparams* params = static_cast<setdimparams*>(data);
     params->crop->setDimensions (params->x, params->y);
     delete params;
@@ -738,126 +741,194 @@ void Crop::cropMoved (int &X, int &Y, int &W, int &H) {
 
 void Crop::cropWidth1Resized (int &X, int &Y, int &W, int &H) {
 
-  if (W<0)
-    W = 0;
-  if (H<0)
-    H = 0;
-
-  if (X<0) {
-    W += X;
-    X = 0;
-  }
-  if (fixr->get_active()) {
-    double r = getRatio();
-    int W2max = (int)round(r*(maxh-Y));
-    if (W>W2max) {
-      X += W - W2max;
-      W = W2max;  
+    int oldXR = nx + nw;
+    if (W < 0) W = 0;
+    if (W > oldXR) W = oldXR;
+    if (fixr->get_active()) {
+        double r = getRatio();
+        H = (int)round(W / r);
+        if (H > maxh) {
+            H = maxh;
+            W = H * r;
+        }
+        ny = ny - (H - nh) / 2.0;
+        if (ny < 0) ny = 0;
+        if (ny + H > maxh) ny = maxh - H;
     }
-    H = (int)round(W / r);
-  }
+    X = oldXR - W;
+    Y = ny;
+    nx = X;
+    nw = W;
+    nh = H;
 
-  nx = X;
-  ny = Y;
-  nw = W;
-  nh = H;
-
-  g_idle_add (refreshSpinsUI, new RefreshSpinHelper (this, false));
-//  Glib::signal_idle().connect (sigc::mem_fun(*this, &Crop::refreshSpins));
+    g_idle_add (refreshSpinsUI, new RefreshSpinHelper (this, false));
 }
 
 void Crop::cropWidth2Resized (int &X, int &Y, int &W, int &H) {
 
-//  X = x->get_value ();
-//  Y = y->get_value ();
-  X = nx;
-  Y = ny;
+    if (W < 0) W = 0;
+    if (W > maxw - nx) W = maxw - nx;
+    if (fixr->get_active()) {
+        double r = getRatio();
+        H = (int)round(W / r);
+        if (H > maxh) {
+            H = maxh;
+            W = H * r;
+        }
+        ny = ny - (H - nh) / 2.0;
+        if (ny < 0) ny = 0;
+        if (ny + H > maxh) ny = maxh - H;
+    }
+    X = nx;
+    Y = ny;
+    nw = W;
+    nh = H;
 
-  if (W<0)
-    W = 0;
-  if (H<0)
-    H = 0;
-
-  if (W>maxw-X)
-    W = maxw-X;
-
-  if (fixr->get_active()) {
-    double r = getRatio();
-    int W2max = (int)round(r*(maxh-Y));
-    if (W>W2max)
-      W = W2max;  
-    H = (int)round(W / r);
-  }
-
-  nx = X;
-  ny = Y;
-  nw = W;
-  nh = H;
-
-  g_idle_add (refreshSpinsUI, new RefreshSpinHelper (this, false));
-//  Glib::signal_idle().connect (sigc::mem_fun(*this, &Crop::refreshSpins));
+    g_idle_add (refreshSpinsUI, new RefreshSpinHelper (this, false));
 }
 
 void Crop::cropHeight1Resized (int &X, int &Y, int &W, int &H) {
 
-  if (W<0)
-    W = 0;
-  if (H<0)
-    H = 0;
-
-  if (Y<0) {
-    H += Y;
-    Y = 0;
-  }
-
-  if (fixr->get_active()) {
-    double r = getRatio();
-    int H2max = (int)round((maxw-X) / r);
-    if (H>H2max) {
-      Y += H - H2max;
-      H = H2max;  
+    int oldYB = ny + nh;
+    if (H < 0) H = 0;
+    if (H > oldYB) H = oldYB;
+    if (fixr->get_active()) {
+        double r = getRatio();
+        W = (int)round(H * r);
+        if (W > maxw) {
+            W = maxw;
+            H = W / r;
+        }
+        nx = nx - (W - nw) / 2.0;
+        if (nx < 0) nx = 0;
+        if (nx + W > maxw) nx = maxw - W;
     }
-    W = (int)round(H * r);
-  }
+    X = nx;
+    Y = oldYB - H;
+    ny = Y;
+    nw = W;
+    nh = H;
 
-  nx = X;
-  ny = Y;
-  nw = W;
-  nh = H;
-
-  g_idle_add (refreshSpinsUI, new RefreshSpinHelper (this, false));
-//  Glib::signal_idle().connect (sigc::mem_fun(*this, &Crop::refreshSpins));
+    g_idle_add (refreshSpinsUI, new RefreshSpinHelper (this, false));
 }
 
 void Crop::cropHeight2Resized (int &X, int &Y, int &W, int &H) {
 
-//  X = x->get_value ();
-//  Y = y->get_value ();
-  X = nx;
-  Y = ny;
+    if (H < 0) H = 0;
+    if (H > maxh - ny) H = maxh - ny;
+    if (fixr->get_active()) {
+        double r = getRatio();
+        W = (int)round(H * r);
+        if (W > maxw) {
+            W = maxw;
+            H = W / r;
+        }
+        nx = nx - (W - nw) / 2.0; // nx must be floating point to avoid drifting
+        if (nx < 0) nx = 0;
+        if (nx + W > maxw) nx = maxw - W;
+    }
+    X = nx;
+    Y = ny;
+    nw = W;
+    nh = H;
 
-  if (W<0)
-    W = 0;
-  if (H<0)
-    H = 0;
-  int H1max = maxh-Y;
-  if (H>H1max)
-    H = H1max;
-  if (fixr->get_active()) {
-    double r = getRatio ();
-    int H2max = (int)round ((maxw-X) / r);
-    if (H>H2max)
-      H = H2max;  
-    W = (int)round(H * r);
-  }
+    g_idle_add (refreshSpinsUI, new RefreshSpinHelper (this, false));
+}
 
-  nx = X;
-  ny = Y;
-  nw = W;
-  nh = H;
+void Crop::cropTopLeftResized (int &X, int &Y, int &W, int &H) {
 
-  g_idle_add (refreshSpinsUI, new RefreshSpinHelper (this, false));
-//  Glib::signal_idle().connect (sigc::mem_fun(*this, &Crop::refreshSpins));
+    int oldXR = nx + nw; // right side
+    int oldYB = ny + nh; // bottom side
+    if (W < 0) W = 0;
+    if (H < 0) H = 0;
+    if (W > oldXR) W = oldXR;
+    if (H > oldYB) H = oldYB;
+    if (fixr->get_active()) {
+        double r = getRatio();
+        W = (int)round(H * r);
+        if (W > oldXR) {
+            W = oldXR;
+            H = (int)round(W / r);
+        }
+    }
+    X = oldXR - W;
+    Y = oldYB - H;
+    nx = X;
+    ny = Y;
+    nw = W;
+    nh = H;
+
+    g_idle_add (refreshSpinsUI, new RefreshSpinHelper (this, false));
+}
+
+void Crop::cropTopRightResized (int &X, int &Y, int &W, int &H) {
+
+    int oldYB = ny + nh;
+    if (W < 0) W = 0;
+    if (H < 0) H = 0;
+    if (W > maxw - nx) W = maxw - nx;
+    if (H > oldYB) H = oldYB;
+    if (fixr->get_active()) {
+        double r = getRatio();
+        W = (int)round(H * r);
+        if (W > maxw - nx) {
+            W = maxw - nx;
+            H = (int)round(W / r);
+        }
+    }
+    X = nx;
+    Y = oldYB - H;
+    ny = Y;
+    nw = W;
+    nh = H;
+
+    g_idle_add (refreshSpinsUI, new RefreshSpinHelper (this, false));
+}
+
+void Crop::cropBottomLeftResized (int &X, int &Y, int &W, int &H) {
+
+    int oldXR = nx + nw;
+    if (W < 0) W = 0;
+    if (H < 0) H = 0;
+    if (W > oldXR) W = oldXR;
+    if (H > maxh - ny) H = maxh - ny;
+    if (fixr->get_active()) {
+        double r = getRatio();
+        W = (int)round(H * r);
+        if (W > oldXR) {
+            W = oldXR;
+            H = (int)round(W / r);
+        }
+    }
+    X = oldXR - W;
+    Y = ny;
+    nx = X;
+    nw = W;
+    nh = H;
+
+    g_idle_add (refreshSpinsUI, new RefreshSpinHelper (this, false));
+}
+
+void Crop::cropBottomRightResized (int &X, int &Y, int &W, int &H) {
+
+    if (W < 0) W = 0;
+    if (H < 0) H = 0;
+    if (W > maxw - nx) W = maxw - nx;
+    if (H > maxh - ny) H = maxh - ny;
+    if (fixr->get_active()) {
+        double r = getRatio();
+        W = (int)round(H * r);
+        if (W > maxw - nx) {
+            W = maxw - nx;
+            H = (int)round(W / r);
+        }
+    }
+    X = nx;
+    Y = ny;
+    nw = W;
+    nh = H;
+
+    g_idle_add (refreshSpinsUI, new RefreshSpinHelper (this, false));
 }
 
 void Crop::cropInit (int &x, int &y, int &w, int &h) {
@@ -958,12 +1029,11 @@ void Crop::cropResized (int &x, int &y, int& x2, int& y2) {
   nh = H;
 
   g_idle_add (refreshSpinsUI, new RefreshSpinHelper (this, false));
-//  Glib::signal_idle().connect (sigc::mem_fun(*this, &Crop::refreshSpins));
 }
 
 void Crop::cropManipReady () {
 
-    g_idle_add (notifyListenerUI, this);
+  g_idle_add (notifyListenerUI, this);
 }
 
 double Crop::getRatio () {
