@@ -27,6 +27,10 @@ class CropGUIListener {
     virtual void cropWidth2Resized  (int &x, int &y, int &w, int &h) =0;
     virtual void cropHeight1Resized (int &x, int &y, int &w, int &h) =0;
     virtual void cropHeight2Resized (int &x, int &y, int &w, int &h) =0;
+    virtual void cropTopLeftResized     (int &x, int &y, int &w, int &h) =0;
+    virtual void cropTopRightResized    (int &x, int &y, int &w, int &h) =0;
+    virtual void cropBottomLeftResized  (int &x, int &y, int &w, int &h) =0;
+    virtual void cropBottomRightResized (int &x, int &y, int &w, int &h) =0;
     virtual void cropInit           (int &x, int &y, int &w, int &h) =0;
     virtual void cropResized        (int &x, int &y, int& x2, int& y2) =0;
     virtual void cropManipReady     () =0;
