@@ -21,6 +21,7 @@
 #include "cropwindow.h"
 #include "options.h"
 #include "guiutils.h"
+#include "threadutils.h"
 #include "../rtengine/mytime.h"
 #include "imagearea.h"
 #include "cursormanager.h"
@@ -36,6 +37,8 @@ struct ZoomStep {
 };
 
 ZoomStep zoomSteps[] = {
+		                {"  1%",  0.01,    100},
+		                {"  2%",  0.02,    50},
 		                {"  5%",  0.05,    20},
 		                {"6.7%",  1.0/15.0,15},
 		                {"  8%",  1.0/12.0,12},
@@ -55,8 +58,8 @@ ZoomStep zoomSteps[] = {
                         {"600%",  6.0,     6000},
                         {"700%",  7.0,     7000},
                         {"800%",  8.0,     8000}};
-#define MAXZOOMSTEPS 18
-#define ZOOM11INDEX  11
+#define MAXZOOMSTEPS 20
+#define ZOOM11INDEX  13
 
 CropWindow::CropWindow (ImageArea* parent, rtengine::StagedImageProcessor* ipc_, bool isLowUpdatePriority_) 
     : onResizeArea(false), deleted(false), fitZoomEnabled(true), fitZoom(false), isLowUpdatePriority(isLowUpdatePriority_),
@@ -72,6 +75,7 @@ CropWindow::CropWindow (ImageArea* parent, rtengine::StagedImageProcessor* ipc_,
     context->set_font_description (fontd);
     cropLabel = "100%";
     Glib::RefPtr<Pango::Layout> cllayout = parent->create_pango_layout("1000%");
+    exposeVersion = zoomVersion = 0;
  
     int iw, ih;
     cllayout->get_pixel_size (iw, ih);
@@ -230,6 +234,8 @@ void CropWindow::buttonPress (int button, int type, int bstate, int x, int y) {
     if (button==1 && type==GDK_2BUTTON_PRESS && onArea (CropImage, x, y) && (state==SNormal || state==SCropImgMove)) {
         if (fitZoomEnabled) {
             if (fitZoom) {
+                state = SNormal;
+                zoomVersion = exposeVersion;
                 translateCoord (x, y, action_x, action_y);
                 changeZoom (ZOOM11INDEX, true, action_x, action_y);
                 fitZoom = false;
@@ -263,7 +269,35 @@ void CropWindow::buttonPress (int button, int type, int bstate, int x, int y) {
         press_y = height;
     }
     else if (button==1 && type==GDK_BUTTON_PRESS && state==SNormal && onArea (CropImage, x, y)) {
-        if (onArea (CropTop, x, y)) {
+        if (onArea (CropTopLeft, x, y)) {
+            state = SResizeTL;
+            press_x = x;
+            action_x = cropHandler.cropParams.x;
+            press_y = y;
+            action_y = cropHandler.cropParams.y;
+        }
+        else if (onArea (CropTopRight, x, y)) {
+            state = SResizeTR;
+            press_x = x;
+            action_x = cropHandler.cropParams.w;
+            press_y = y;
+            action_y = cropHandler.cropParams.y;
+        }
+        else if (onArea (CropBottomLeft, x, y)) {
+            state = SResizeBL;
+            press_x = x;
+            action_x = cropHandler.cropParams.x;
+            press_y = y;
+            action_y = cropHandler.cropParams.h;
+        }
+        else if (onArea (CropBottomRight, x, y)) {
+            state = SResizeBR;
+            press_x = x;
+            action_x = cropHandler.cropParams.w;
+            press_y = y;
+            action_y = cropHandler.cropParams.h;
+        }
+        else if (onArea (CropTop, x, y)) {
             state = SResizeH1;
             press_y = y;
             action_y = cropHandler.cropParams.y;
@@ -361,7 +395,8 @@ void CropWindow::buttonRelease (int button, int num, int bstate, int x, int y) {
         observedCropWin->remoteMoveReady ();
         state = SNormal;
     }
-    if (cropgl && (state==SCropSelecting || state==SResizeH1 || state==SResizeH2 || state==SResizeW1 || state==SResizeW2 || state==SCropMove)) {
+    if (cropgl && (state==SCropSelecting || state==SResizeH1 || state==SResizeH2 || state==SResizeW1 || state==SResizeW2 || state==SResizeTL || state==SResizeTR || state==SResizeBL || state==SResizeBR || state==SCropMove))
+    {
         cropgl->cropManipReady ();
         iarea->setToolHand ();
     }
@@ -435,6 +470,38 @@ void CropWindow::pointerMoved (int x, int y) {
         cropgl->cropWidth2Resized (cropHandler.cropParams.x, cropHandler.cropParams.y, cropHandler.cropParams.w, cropHandler.cropParams.h);
         iarea->redraw ();
     }
+    else if (state==SResizeTL && cropgl) { 
+        int ox = cropHandler.cropParams.x;
+        cropHandler.cropParams.x = action_x + (x-press_x) / zoomSteps[cropZoom].zoom;
+        cropHandler.cropParams.w += ox - cropHandler.cropParams.x;
+        int oy = cropHandler.cropParams.y;
+        cropHandler.cropParams.y = action_y + (y-press_y) / zoomSteps[cropZoom].zoom;
+        cropHandler.cropParams.h += oy - cropHandler.cropParams.y;
+        cropgl->cropTopLeftResized (cropHandler.cropParams.x, cropHandler.cropParams.y, cropHandler.cropParams.w, cropHandler.cropParams.h);
+        iarea->redraw ();
+    }
+    else if (state==SResizeTR && cropgl) { 
+        cropHandler.cropParams.w = action_x + (x-press_x) / zoomSteps[cropZoom].zoom;
+        int oy = cropHandler.cropParams.y;
+        cropHandler.cropParams.y = action_y + (y-press_y) / zoomSteps[cropZoom].zoom;
+        cropHandler.cropParams.h += oy - cropHandler.cropParams.y;
+        cropgl->cropTopRightResized (cropHandler.cropParams.x, cropHandler.cropParams.y, cropHandler.cropParams.w, cropHandler.cropParams.h);
+        iarea->redraw ();
+    }
+    else if (state==SResizeBL && cropgl) { 
+        int ox = cropHandler.cropParams.x;
+        cropHandler.cropParams.x = action_x + (x-press_x) / zoomSteps[cropZoom].zoom;
+        cropHandler.cropParams.w += ox - cropHandler.cropParams.x;
+        cropHandler.cropParams.h = action_y + (y-press_y) / zoomSteps[cropZoom].zoom;
+        cropgl->cropBottomLeftResized (cropHandler.cropParams.x, cropHandler.cropParams.y, cropHandler.cropParams.w, cropHandler.cropParams.h);
+        iarea->redraw ();
+    }
+    else if (state==SResizeBR && cropgl) { 
+        cropHandler.cropParams.w = action_x + (x-press_x) / zoomSteps[cropZoom].zoom;
+        cropHandler.cropParams.h = action_y + (y-press_y) / zoomSteps[cropZoom].zoom;
+        cropgl->cropBottomRightResized (cropHandler.cropParams.x, cropHandler.cropParams.y, cropHandler.cropParams.w, cropHandler.cropParams.h);
+        iarea->redraw ();
+    }
     else if (state==SCropMove && cropgl) { 
         cropHandler.cropParams.x = action_x + (x-press_x) / zoomSteps[cropZoom].zoom;
         cropHandler.cropParams.y = action_y + (y-press_y) / zoomSteps[cropZoom].zoom;
@@ -488,7 +555,7 @@ void CropWindow::pointerMoved (int x, int y) {
             if (pmhlistener) pmhlistener->pointerMoved (false, cropHandler.colorParams.working, mx, my, -1, -1, -1);
                 }
 		else {
-			/*Glib::Mutex::Lock lock(cropHandler.cimg);
+			/*MyMutex::MyLock lock(cropHandler.cimg);
 
 			int vx = x - xpos - imgX;
 			int vy = y - ypos - imgY;
@@ -534,34 +601,74 @@ bool CropWindow::onArea (CursorArea a, int x, int y) {
             return 
                 (x>=xpos+imgAreaX && y>=ypos+imgAreaY && x<xpos+imgAreaX+imgAreaW && y<ypos+imgAreaY+imgAreaH) &&
                 !(x>=xpos+imgX && y>=ypos+imgY && x<xpos+imgX+imgW && y<ypos+imgY+imgH);
+        case CropTopLeft:
+            translateCoord (x, y, x1, y1);
+            return cropHandler.cropParams.enabled && 
+                y1>=cropHandler.cropParams.y-CROPRESIZEBORDER && 
+                y1<=cropHandler.cropParams.y+CROPRESIZEBORDER &&
+                y>=ypos+imgY &&
+                x1>=cropHandler.cropParams.x-CROPRESIZEBORDER && 
+                x1<=cropHandler.cropParams.x+CROPRESIZEBORDER &&
+                x>=xpos+imgX;
+        case CropTopRight:
+            translateCoord (x, y, x1, y1);
+            return cropHandler.cropParams.enabled && 
+                y1>=cropHandler.cropParams.y-CROPRESIZEBORDER && 
+                y1<=cropHandler.cropParams.y+CROPRESIZEBORDER &&
+                y>=ypos+imgY &&
+                x1>=cropHandler.cropParams.x+cropHandler.cropParams.w-1-CROPRESIZEBORDER && 
+                x1<=cropHandler.cropParams.x+cropHandler.cropParams.w-1+CROPRESIZEBORDER &&
+                x<xpos+imgX+imgW;
+        case CropBottomLeft:
+            translateCoord (x, y, x1, y1);
+            return cropHandler.cropParams.enabled && 
+                y1>=cropHandler.cropParams.y+cropHandler.cropParams.h-1-CROPRESIZEBORDER && 
+                y1<=cropHandler.cropParams.y+cropHandler.cropParams.h-1+CROPRESIZEBORDER &&
+                y<ypos+imgY+imgH &&
+                x1>=cropHandler.cropParams.x-CROPRESIZEBORDER && 
+                x1<=cropHandler.cropParams.x+CROPRESIZEBORDER &&
+                x>=xpos+imgX;
+        case CropBottomRight:
+            translateCoord (x, y, x1, y1);
+            return cropHandler.cropParams.enabled && 
+                y1>=cropHandler.cropParams.y+cropHandler.cropParams.h-1-CROPRESIZEBORDER && 
+                y1<=cropHandler.cropParams.y+cropHandler.cropParams.h-1+CROPRESIZEBORDER &&
+                y<ypos+imgY+imgH &&
+                x1>=cropHandler.cropParams.x+cropHandler.cropParams.w-1-CROPRESIZEBORDER && 
+                x1<=cropHandler.cropParams.x+cropHandler.cropParams.w-1+CROPRESIZEBORDER &&
+                x<xpos+imgX+imgW;
         case CropTop:
             translateCoord (x, y, x1, y1);
             return cropHandler.cropParams.enabled && 
                 x1>cropHandler.cropParams.x+CROPRESIZEBORDER && 
                 x1<cropHandler.cropParams.x+cropHandler.cropParams.w-1-CROPRESIZEBORDER && 
                 y1>cropHandler.cropParams.y-CROPRESIZEBORDER && 
-                y1<cropHandler.cropParams.y+CROPRESIZEBORDER;
+                y1<cropHandler.cropParams.y+CROPRESIZEBORDER &&
+                y>=ypos+imgY;
         case CropBottom:
             translateCoord (x, y, x1, y1);
             return cropHandler.cropParams.enabled && 
                 x1>cropHandler.cropParams.x+CROPRESIZEBORDER && 
                 x1<cropHandler.cropParams.x+cropHandler.cropParams.w-1-CROPRESIZEBORDER && 
                 y1>cropHandler.cropParams.y+cropHandler.cropParams.h-1-CROPRESIZEBORDER && 
-                y1<cropHandler.cropParams.y+cropHandler.cropParams.h-1+CROPRESIZEBORDER;
+                y1<cropHandler.cropParams.y+cropHandler.cropParams.h-1+CROPRESIZEBORDER &&
+                y<ypos+imgY+imgH;
         case CropLeft:
             translateCoord (x, y, x1, y1);
             return cropHandler.cropParams.enabled && 
                 y1>cropHandler.cropParams.y+CROPRESIZEBORDER && 
                 y1<cropHandler.cropParams.y+cropHandler.cropParams.h-1-CROPRESIZEBORDER && 
                 x1>cropHandler.cropParams.x-CROPRESIZEBORDER && 
-                x1<cropHandler.cropParams.x+CROPRESIZEBORDER;
+                x1<cropHandler.cropParams.x+CROPRESIZEBORDER &&
+                x>=xpos+imgX;
         case CropRight:
             translateCoord (x, y, x1, y1);
             return cropHandler.cropParams.enabled && 
                 y1>cropHandler.cropParams.y+CROPRESIZEBORDER && 
                 y1<cropHandler.cropParams.y+cropHandler.cropParams.h-1-CROPRESIZEBORDER && 
                 x1>cropHandler.cropParams.x+cropHandler.cropParams.w-1-CROPRESIZEBORDER && 
-                x1<cropHandler.cropParams.x+cropHandler.cropParams.w-1+CROPRESIZEBORDER;
+                x1<cropHandler.cropParams.x+cropHandler.cropParams.w-1+CROPRESIZEBORDER &&
+                x<xpos+imgX+imgW;
         case CropInside:
             translateCoord (x, y, x1, y1);
             return cropHandler.cropParams.enabled && 
@@ -592,6 +699,14 @@ void CropWindow::updateCursor (int x, int y) {
             cursorManager.setCursor (iarea->get_window(), CSMove);
         else if (onArea (CropResize, x, y)) 
             cursorManager.setCursor (iarea->get_window(), CSResizeDiagonal);
+        else if (tm==TMHand && (onArea (CropTopLeft, x, y))) 
+            cursorManager.setCursor (iarea->get_window(), CSResizeTopLeft);
+        else if (tm==TMHand && (onArea (CropTopRight, x, y))) 
+            cursorManager.setCursor (iarea->get_window(), CSResizeTopRight);
+        else if (tm==TMHand && (onArea (CropBottomLeft, x, y))) 
+            cursorManager.setCursor (iarea->get_window(), CSResizeBottomLeft);
+        else if (tm==TMHand && (onArea (CropBottomRight, x, y))) 
+            cursorManager.setCursor (iarea->get_window(), CSResizeBottomRight);
         else if (tm==TMHand && (onArea (CropTop, x, y) || onArea (CropBottom, x, y))) 
             cursorManager.setCursor (iarea->get_window(), CSResizeHeight);
         else if (tm==TMHand && (onArea (CropLeft, x, y) || onArea (CropRight, x, y))) 
@@ -625,12 +740,20 @@ void CropWindow::updateCursor (int x, int y) {
         cursorManager.setCursor (iarea->get_window(), CSResizeWidth);
     else if (state==SResizeH1 || state==SResizeH2)
         cursorManager.setCursor (iarea->get_window(), CSResizeHeight);
+    else if (state==SResizeTL)
+        cursorManager.setCursor (iarea->get_window(), CSResizeTopLeft);
+    else if (state==SResizeTR)
+        cursorManager.setCursor (iarea->get_window(), CSResizeTopRight);
+    else if (state==SResizeBL)
+        cursorManager.setCursor (iarea->get_window(), CSResizeBottomLeft);
+    else if (state==SResizeBR)
+        cursorManager.setCursor (iarea->get_window(), CSResizeBottomRight);
     else if (state==SCropWinResize)
         cursorManager.setCursor (iarea->get_window(), CSResizeDiagonal);
 }
 
 void CropWindow::expose (Cairo::RefPtr<Cairo::Context> cr) {
-	Glib::Mutex::Lock lock(cropHandler.cimg);
+    MyMutex::MyLock lock(cropHandler.cimg);
 
     //MyTime t1, t2, t3, t4;
     
@@ -683,6 +806,7 @@ void CropWindow::expose (Cairo::RefPtr<Cairo::Context> cr) {
             imgH = cropHandler.cropPixbuf->get_height ();
             imgX = imgAreaX + (imgAreaW-imgW)/2;
             imgY = imgAreaY + (imgAreaH-imgH)/2;
+            exposeVersion++;
 // PERFORMANCE BOTTLENECK STARTS HERE
     //t3.set ();
 			bool showcs = iarea->indClippedPanel->showClippedShadows();
@@ -940,7 +1064,6 @@ void CropWindow::expose (Cairo::RefPtr<Cairo::Context> cr) {
         drawSpotWBRectangle (cr);
 
     //t2.set ();
-
 //    printf ("etime --> %d, %d\n", t2.etime (t1), t4.etime (t3));
 }
 
@@ -972,10 +1095,44 @@ void CropWindow::zoomIn (bool toCursor, int cursorX, int cursorY) {
     if (toCursor) {
         x = cursorX;
         y = cursorY;
+    } else {
+        if (zoomSteps[cropZoom].zoom <= cropHandler.getFitZoom()) {
+            if (cropHandler.cropParams.enabled) {
+                x = cropHandler.cropParams.x + cropHandler.cropParams.w / 2;
+                y = cropHandler.cropParams.y + cropHandler.cropParams.h / 2;
+            } else {
+                int fw, fh;
+                cropHandler.getFullImageSize(fw, fh);
+                x = fw / 2;
+                y = fh / 2;
+            }
+            zoomVersion = exposeVersion;
+        } else if (zoomVersion != exposeVersion) {
+            translateCoord(xpos+imgX+imgW/2, ypos+imgY+imgH/2, x, y);
+            if (cropHandler.cropParams.enabled) {
+                // add some gravity towards crop center
+                int x1 = cropHandler.cropParams.x + cropHandler.cropParams.w / 2;
+                int y1 = cropHandler.cropParams.y + cropHandler.cropParams.h / 2;
+                double cropd = sqrt(cropHandler.cropParams.h*cropHandler.cropParams.h+cropHandler.cropParams.w*cropHandler.cropParams.w) * zoomSteps[cropZoom].zoom;
+                double imd = sqrt(imgW*imgW+imgH+imgH);
+                double d;
+                // the more we can see of the crop, the more gravity towards crop center
+                if (cropd > imd) {
+                    d = 0.8;
+                } else if (cropd < imd * 0.5) {
+                    d = 0.0;
+                } else {
+                    d = 1.6 * (cropd - imd * 0.5) / imd;
+                }
+                x = d * x + (1.0 - d) * x1;
+                y = d * y + (1.0 - d) * y1;
+            }
+            zoomVersion = exposeVersion;
+        }
     }
 
-	changeZoom (cropZoom+1, true, x, y);
-	fitZoom = false;
+    changeZoom (cropZoom+1, true, x, y);
+    fitZoom = false;
 }
 
 void CropWindow::zoomOut (bool toCursor, int cursorX, int cursorY) {
@@ -988,13 +1145,28 @@ void CropWindow::zoomOut (bool toCursor, int cursorX, int cursorY) {
         y = cursorY;
     }
 
+    zoomVersion = exposeVersion;
     changeZoom (cropZoom-1, true, x, y);
     fitZoom = false;
 }
 
 void CropWindow::zoom11 () {
 
-    changeZoom (ZOOM11INDEX);
+    int x = -1;
+    int y = -1;
+    if (zoomSteps[cropZoom].zoom <= cropHandler.getFitZoom()) {
+        if (cropHandler.cropParams.enabled) {
+            x = cropHandler.cropParams.x + cropHandler.cropParams.w / 2;
+            y = cropHandler.cropParams.y + cropHandler.cropParams.h / 2;
+        } else {
+            int fw, fh;
+            cropHandler.getFullImageSize(fw, fh);
+            x = fw/2;
+            y = fh/2;
+        }
+        zoomVersion = exposeVersion;
+    }
+    changeZoom (ZOOM11INDEX, true, x, y);
     fitZoom = false;
 }
 
@@ -1036,6 +1208,7 @@ void CropWindow::zoomFit () {
                 cz = i;
                 break;
             }
+    zoomVersion = exposeVersion;
     changeZoom (cz);
     fitZoom = true;
 }
