@@ -27,9 +27,7 @@
 #include "../rtengine/safegtk.h"
 #include "rtimage.h"
 
-extern Glib::ustring argv0;
-
-CurveEditorGroup::CurveEditorGroup (Glib::ustring& curveDir, Glib::ustring groupLabel) : curveDir(curveDir), cl(NULL), cp(NULL) {
+CurveEditorGroup::CurveEditorGroup (Glib::ustring& curveDir, Glib::ustring groupLabel) : curveDir(curveDir), cl(NULL) {
 	curveEditors.clear();
 	displayedCurve = 0;
 	numberOfPackedCurve = 0;
@@ -58,24 +56,45 @@ void CurveEditorGroup::hideCurrentCurve() {
 /*
  * Add a new curve to the curves list
  *
- * The "periodic" parameter is only used by flat curve editors
+ * Parameters:
+ *     cType:         enum saying which kind of curve type has to be created
+ *     curveLabel:    Name of the curve that will be inserted in the toggle button, before the image.
+ *                    If empty, no text will prepend the image
+ *     relatedWidget: pointer to a widget (or NULL) that will be inserted next to the curve's toggle button.
+ *                    if a smart pointer created by Gtk::manage is passed in, the widget will be deleted by the destructor,
+ *                    otherwise it'll have to be delete it manually
+ *     periodic:      for FlatCurve only, ask the curve to be periodic (default: True)
+ *
  */
-CurveEditor* CurveEditorGroup::addCurve(CurveType cType, Glib::ustring curveLabel, bool periodic) {
+CurveEditor* CurveEditorGroup::addCurve(CurveType cType, Glib::ustring curveLabel, Gtk::Widget *relatedWidget, bool periodic) {
 	switch (cType) {
 	case (CT_Diagonal):
+	{
 		if (!diagonalSubGroup) {
 			diagonalSubGroup = new DiagonalCurveEditorSubGroup(this, curveDir);
 		}
-		return (static_cast<CurveEditor*>(diagonalSubGroup->addCurve(curveLabel)));
+		// We add it to the curve editor list
+		DiagonalCurveEditor* newCE = diagonalSubGroup->addCurve(curveLabel);
+		newCE->relatedWidget = relatedWidget;
+		curveEditors.push_back(newCE);
+		return (newCE);
+	}
 	case (CT_Flat):
+	{
 		if (!flatSubGroup) {
 			flatSubGroup = new FlatCurveEditorSubGroup(this, curveDir);
 		}
-		return (static_cast<CurveEditor*>(flatSubGroup->addCurve(curveLabel, periodic)));
+		// We add it to the curve editor list
+		FlatCurveEditor* newCE = flatSubGroup->addCurve(curveLabel, periodic);
+		newCE->relatedWidget = relatedWidget;
+		curveEditors.push_back(newCE);
+		return (newCE);
+	}
 	default:
 		return (static_cast<CurveEditor*>(NULL));
 		break;
 	}
+	return NULL; // to avoid complains from Gcc
 }
 
 /*
@@ -101,9 +120,15 @@ void CurveEditorGroup::newLine() {
 		}
 
 		int j = numberOfPackedCurve;
-		for (int i = (int)(curveEditors.size())-1; i >= j; i--)
-		{
-			headerBox->pack_end (*curveEditors[i]->curveType->buttonGroup, Gtk::PACK_EXPAND_WIDGET, 2);
+		bool hasRelatedWidget = false;
+		for (int i = (int)(curveEditors.size())-1; i >= j; i--) {
+			if (curveEditors[i]->relatedWidget != NULL)
+				hasRelatedWidget = true;
+		}
+		for (int i = (int)(curveEditors.size())-1; i >= j; i--) {
+			if (curveEditors[i]->relatedWidget != NULL)
+				headerBox->pack_end (*curveEditors[i]->relatedWidget, Gtk::PACK_EXPAND_WIDGET, 2);
+			headerBox->pack_end (*curveEditors[i]->curveType->buttonGroup, hasRelatedWidget ? Gtk::PACK_SHRINK : Gtk::PACK_EXPAND_WIDGET, 2);
 			numberOfPackedCurve++;
 		}
 
@@ -121,12 +146,6 @@ void CurveEditorGroup::newLine() {
 void CurveEditorGroup::curveListComplete() {
 	newLine();
 
-	// Set the color provider
-	if (cp) {
-		if (flatSubGroup) flatSubGroup->setColorProvider(cp);
-		if (diagonalSubGroup) diagonalSubGroup->setColorProvider(cp);
-	}
-
 	// We check the length of the label ; if it contains only one char (':'), we set it to the right default string
 	if (curveGroupLabel->get_label().size()==1)
 		curveGroupLabel->set_label(M(curveEditors.size() > 1 ? "CURVEEDITOR_CURVES" : "CURVEEDITOR_CURVE") + ":");
@@ -261,12 +280,19 @@ void CurveEditorGroup::curveChanged () {
  */
 void CurveEditorGroup::curveResetPressed () {
 	if (displayedCurve) {
-		if (displayedCurve->subGroup->curveReset(displayedCurve->selected)) {
+		if (displayedCurve->subGroup->curveReset(displayedCurve->selected, displayedCurve->getIdentityValue())) {
 			curveChanged();
 		}
 	}
 }
 
+/*
+ * Set the tooltip text of the label of the curve group
+ */
+void CurveEditorGroup::setTooltip( Glib::ustring ttip) {
+	curveGroupLabel->set_tooltip_text( ttip );
+}
+
 void CurveEditorGroup::setBatchMode (bool batchMode) {
 	for (std::vector<CurveEditor*>::iterator i = curveEditors.begin(); i != curveEditors.end(); ++i) {
 		(*i)->curveType->addEntry("curveType-unchanged.png", M("GENERAL_UNCHANGED"));
@@ -294,14 +320,21 @@ void CurveEditorGroup::setUnChanged (bool uc, CurveEditor* ce) {
 	}
 }
 
-CurveEditorSubGroup::CurveEditorSubGroup(Glib::ustring& curveDir) :
-		curveDir(curveDir) {
+CurveEditorSubGroup::CurveEditorSubGroup(Glib::ustring& curveDir) : curveDir(curveDir), lastFilename("") {
+	leftBar = NULL;
+	bottomBar = NULL;
+}
+
+CurveEditorSubGroup::~CurveEditorSubGroup() {
+	if (leftBar) delete leftBar;
+	if (bottomBar) delete bottomBar;
 }
 
 Glib::ustring CurveEditorSubGroup::outputFile () {
 
     Gtk::FileChooserDialog dialog(M("CURVEEDITOR_SAVEDLGLABEL"), Gtk::FILE_CHOOSER_ACTION_SAVE);
     FileChooserLastFolderPersister persister(&dialog, curveDir);
+    dialog.set_current_name (lastFilename);
 
     dialog.add_button(Gtk::StockID("gtk-cancel"), Gtk::RESPONSE_CANCEL);
     dialog.add_button(Gtk::StockID("gtk-save"), Gtk::RESPONSE_APPLY);
@@ -316,33 +349,22 @@ Glib::ustring CurveEditorSubGroup::outputFile () {
     filter_any.add_pattern("*");
     dialog.add_filter(filter_any);
 
-    dialog.set_do_overwrite_confirmation (true);
+    //dialog.set_do_overwrite_confirmation (true);
 
-	Glib::ustring fname;
+    Glib::ustring fname;
     do {
-		int result = dialog.run();
-
-		fname = dialog.get_filename();
-
-		if (result==Gtk::RESPONSE_APPLY) {
-
-			if (getExtension (fname)!="rtc")
-				fname = fname + ".rtc";
-
-			if (safe_file_test (fname, Glib::FILE_TEST_EXISTS)) {
-				Glib::ustring msg_ = Glib::ustring("<b>") + fname + ": " + M("MAIN_MSG_ALREADYEXISTS") + "\n" + M("MAIN_MSG_QOVERWRITE") + "</b>";
-				Gtk::MessageDialog msgd (msg_, true, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_YES_NO, true);
-				int response = msgd.run ();
-				if (response==Gtk::RESPONSE_YES)
-					break;
-			}
-			else
-				break;
-		}
-		else {
-			fname = "";
-			break;
-		}
+        if (dialog.run() == Gtk::RESPONSE_APPLY) {
+            fname = dialog.get_filename();
+            if (getExtension (fname) != "rtc")
+                fname += ".rtc";
+            if (confirmOverwrite (dialog, fname)) {
+                lastFilename = Glib::path_get_basename (fname);
+                break;
+            }
+        } else {
+            fname = "";
+            break;
+        }
     } while (1);
 
     return fname;
