@@ -29,9 +29,12 @@
 #include "rtimage.h"
 
 #define CHECKTIME 5000
-extern Glib::ustring argv0;
 
-DirBrowser::DirBrowser () {
+DirBrowser::DirBrowser () : expandSuccess(false)
+                            #ifdef WIN32
+                            , volumes(0)
+                            #endif
+{
 
    dirtree = Gtk::manage ( new Gtk::TreeView() );
    scrolledwindow4 = Gtk::manage ( new Gtk::ScrolledWindow() );
@@ -144,7 +147,7 @@ void DirBrowser::updateVolumes () {
 
     int nvolumes = GetLogicalDrives ();
     if (nvolumes!=volumes) {
-		GThreadLock lock;
+        GThreadLock lock;
 
         for (int i=0; i<32; i++) 
             if (((volumes >> i) & 1) && !((nvolumes >> i) & 1)) { // volume i has been deleted
@@ -219,8 +222,6 @@ void DirBrowser::row_expanded (const Gtk::TreeModel::iterator& iter, const Gtk::
 #ifdef WIN32
   Glib::RefPtr<WinDirMonitor> monitor = Glib::RefPtr<WinDirMonitor>(new WinDirMonitor (iter->get_value (dtColumns.dirname), this));
   iter->set_value (dtColumns.monitor, monitor);
-#elif defined __APPLE__
-  printf("TODO fix dir->monitor_directory () for OSX\n"); 
 #else
   Glib::RefPtr<Gio::FileMonitor> monitor = dir->monitor_directory ();
   iter->set_value (dtColumns.monitor, monitor);
@@ -237,23 +238,26 @@ void DirBrowser::updateDir (const Gtk::TreeModel::iterator& iter) {
         for (Gtk::TreeModel::iterator it=iter->children().begin(); it!=iter->children().end(); it++)
             if (!safe_file_test (it->get_value (dtColumns.dirname), Glib::FILE_TEST_EXISTS) 
              || !safe_file_test (it->get_value (dtColumns.dirname), Glib::FILE_TEST_IS_DIR)) {
+                GThreadLock lock;
                 dirTreeModel->erase (it);
                 change = true;
                 break;
             }
     }
     // test if new files are created
-		std::vector<Glib::ustring> subDirs;
+    std::vector<Glib::ustring> subDirs;
     Glib::RefPtr<Gio::File> dir = Gio::File::create_for_path (iter->get_value (dtColumns.dirname));
-		safe_build_subdir_list (dir, subDirs, options.fbShowHidden);
+    safe_build_subdir_list (dir, subDirs, options.fbShowHidden);
 
     for (int i=0; i<subDirs.size(); i++) {
         bool found = false;
         for (Gtk::TreeModel::iterator it=iter->children().begin(); it!=iter->children().end() && !found ; it++) 
             found = (it->get_value (dtColumns.filename)==subDirs[i]);
 
-        if (!found)
+        if (!found) {
+            GThreadLock lock;
             addDir (iter, subDirs[i]);
+        }
     }
 }
 
@@ -336,6 +340,7 @@ void DirBrowser::open (const Glib::ustring& dirname, const Glib::ustring& fileNa
   
     dirtree->collapse_all ();
 
+    // WARNING & TODO: One should test here if the directory/file has R/W access permission to avoid crash
 
     Glib::RefPtr<Gio::File> dir = Gio::File::create_for_path(dirname);
     if( !dir->query_exists())
@@ -344,8 +349,12 @@ void DirBrowser::open (const Glib::ustring& dirname, const Glib::ustring& fileNa
     Gtk::TreePath path = expandToDir (absDirPath);
 	dirtree->scroll_to_row (path);
 	dirtree->get_selection()->select (path);
-	for (size_t i=0; i<dllisteners.size(); i++)
-		dllisteners[i]->dirSelected (absDirPath, Glib::build_filename (absDirPath, fileName));
+	Glib::ustring absFilePath;
+	if (!fileName.empty())
+		absFilePath = Glib::build_filename (absDirPath, fileName);
+	for (size_t i=0; i<dllisteners.size(); i++) {
+		dllisteners[i]->dirSelected (absDirPath, absFilePath);
+	}
 }
 
 void DirBrowser::file_changed (const Glib::RefPtr<Gio::File>& file, const Glib::RefPtr<Gio::File>& other_file, Gio::FileMonitorEvent event_type, const Gtk::TreeModel::iterator& iter, const Glib::ustring& dirName) {
@@ -353,9 +362,7 @@ void DirBrowser::file_changed (const Glib::RefPtr<Gio::File>& file, const Glib::
     if (!file || !safe_file_test (dirName, Glib::FILE_TEST_IS_DIR) || event_type==Gio::FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED) 
         return;
 
-    gdk_threads_enter();
     updateDir (iter);
-    gdk_threads_leave();
 }
 
 void DirBrowser::selectDir (Glib::ustring dir) {
