@@ -597,6 +597,7 @@ struct spparams {
 
 int setprogressStrUI( void *p )
 {
+	GThreadLock lock; // All GUI acces from idle_add callbacks or separate thread HAVE to be protected
 	spparams *s= static_cast<spparams*>(p);
 
 	if( ! s->str.empty() )
@@ -662,6 +663,7 @@ void EditorPanel::refreshProcessingState (bool inProcessingP) {
     	s->val = 1.0;
 
 #ifdef WIN32
+    // Maybe accessing "parent", which is a Gtk object, can justify to get the Gtk lock...
 	if (!firstProcessingDone && static_cast<RTWindow*>(parent)->getIsFullscreen()) { parent->fullscreen(); }
 #endif
         firstProcessingDone = true;
@@ -722,6 +724,7 @@ void EditorPanel::info_toggled () {
     Glib::ustring infoString1; //1-st line
     Glib::ustring infoString2; //2-nd line
     Glib::ustring infoString3; //3-rd line
+    Glib::ustring infoString4; //4-th line
     Glib::ustring expcomp;
 
     if (!ipc || !openThm) return;
@@ -735,7 +738,7 @@ void EditorPanel::info_toggled () {
         		Glib::ustring(idata->apertureToString(idata->getFNumber())),
 				Glib::ustring(idata->shutterToString(idata->getShutterSpeed())),
 				M("QINFO_ISO"), idata->getISOSpeed(),
-				idata->getFocalLen());
+				Glib::ustring::format(std::setw(3), std::fixed, std::setprecision(2), idata->getFocalLen()));
 
         expcomp = Glib::ustring(idata->expcompToString(idata->getExpComp(),true)); // maskZeroexpcomp
         if (expcomp!=""){
@@ -748,7 +751,12 @@ void EditorPanel::info_toggled () {
         				escapeHtmlChars(Glib::path_get_dirname(openThm->getFileName())) + G_DIR_SEPARATOR_S,
         				escapeHtmlChars(Glib::path_get_basename(openThm->getFileName()))  );
 
-        infoString = Glib::ustring::compose ("%1\n%2\n%3",infoString1, infoString2, infoString3);
+        int ww= ipc->getFullWidth();
+        int hh= ipc->getFullHeight();
+        //megapixels
+        infoString4 = Glib::ustring::compose ("<span size=\"small\">%1 MP (%2x%3)</span>", Glib::ustring::format(std::setw(4), std::fixed, std::setprecision(1), (float)ww*hh/1000000), ww, hh);
+
+        infoString = Glib::ustring::compose ("%1\n%2\n%3\n%4",infoString1, infoString2, infoString3, infoString4);
     }
     else
         infoString = M("QINFO_NOEXIF");
@@ -836,7 +844,11 @@ bool EditorPanel::handleShortcutKey (GdkEventKey* event) {
     bool ctrl = event->state & GDK_CONTROL_MASK;
     bool shift = event->state & GDK_SHIFT_MASK;
     bool alt = event->state & GDK_MOD1_MASK;
-
+#ifdef __WIN32__
+    bool altgr = event->state & GDK_MOD2_MASK;
+#else
+    bool altgr = event->state & GDK_MOD5_MASK;
+#endif
 
     // Editor Layout
     switch(event->keyval) {
@@ -875,7 +887,17 @@ bool EditorPanel::handleShortcutKey (GdkEventKey* event) {
         	}
 			break;
     }
-
+#ifdef __WIN32__
+    if (!alt && !ctrl && !altgr && event->hardware_keycode == 0x39 ) {
+		iareapanel->imageArea->previewModePanel->togglebackColor();
+		return true;
+    }
+#else
+    if (!alt && !ctrl && !altgr && event->hardware_keycode == 0x12 ) {
+		iareapanel->imageArea->previewModePanel->togglebackColor();
+		return true;
+    }
+#endif
     if (!alt){
 		if (!ctrl) {
 			// Normal
@@ -886,7 +908,6 @@ bool EditorPanel::handleShortcutKey (GdkEventKey* event) {
 				case GDK_bracketleft:
 					tpc->coarse->rotateLeft();
 					return true;
-
 				case GDK_i:
 				case GDK_I:
 					info->set_active (!info->get_active());
@@ -905,17 +926,13 @@ bool EditorPanel::handleShortcutKey (GdkEventKey* event) {
 				case GDK_z://GDK_1
                     iareapanel->imageArea->zoomPanel->zoom11Clicked();
 					return true;
-
-				case GDK_8: //background color of the preview 0
-				    iareapanel->imageArea->previewModePanel->togglebackColor0();
-				    return true;
-				case GDK_9: //background color of the preview 1
-					iareapanel->imageArea->previewModePanel->togglebackColor1();
+/*
+#ifndef __WIN32__
+				case GDK_9: // toggle background color of the preview
+					iareapanel->imageArea->previewModePanel->togglebackColor();
 					return true;
-				case GDK_0: //background color of the preview 2
-					iareapanel->imageArea->previewModePanel->togglebackColor2();
-					return true;
-
+#endif
+*/
 				case GDK_r: //preview mode Red
                     iareapanel->imageArea->previewModePanel->toggleR();
 					return true;
@@ -969,7 +986,7 @@ bool EditorPanel::handleShortcutKey (GdkEventKey* event) {
 				case GDK_s:
 					saveAsPressed();
 					return true;
-				case GDK_q:
+				case GDK_b:
 					if (!simpleEditor)
 						queueImgPressed();
 					return true;
@@ -988,7 +1005,7 @@ bool EditorPanel::handleShortcutKey (GdkEventKey* event) {
 			}
 		} //if (!ctrl)
     } //if (!alt)
-    
+
     if (alt){
     	switch (event->keyval) {
 			case GDK_s:
@@ -1108,19 +1125,11 @@ BatchQueueEntry* EditorPanel::createBatchQueueEntry () {
     ipc->getParams (&pparams);
     //rtengine::ProcessingJob* job = rtengine::ProcessingJob::create (ipc->getInitialImage(), pparams);
     rtengine::ProcessingJob* job = rtengine::ProcessingJob::create (openThm->getFileName (), openThm->getType()==FT_Raw, pparams);
-    int prevh = options.maxThumbnailHeight;
-    int prevw = prevh;
-    guint8* prev = NULL;//(guint8*) previewHandler->getImagePreview (prevw, prevh);
-    double tmpscale;
-    rtengine::IImage8* img = openThm->processThumbImage (pparams, options.maxThumbnailHeight, tmpscale);
-    if (img) {
-    	prevw = img->getWidth ();
-    	prevh = img->getHeight ();
-        prev = new guint8 [prevw*prevh*3];
-        memcpy (prev, img->getData (), prevw*prevh*3);
-        img->free();
-    }
-    return new BatchQueueEntry (job, pparams, openThm->getFileName(), prev, prevw, prevh, openThm);
+    int fullW=0, fullH=0;
+    isrc->getImageSource()->getFullSize(fullW, fullH, pparams.coarse.rotate==90 || pparams.coarse.rotate==270 ? TR_R90 : TR_NONE);
+    int prevh = BatchQueue::calcMaxThumbnailHeight();
+    int prevw = int((size_t)fullW * (size_t)prevh / (size_t)fullH);
+    return new BatchQueueEntry (job, pparams, openThm->getFileName(), prevw, prevh, openThm);
 }
 
 
@@ -1487,11 +1496,11 @@ void EditorPanel::tbBeforeLock_toggled () {
 	tbBeforeLock->get_active()? tbBeforeLock->set_image (*iBeforeLockON) : tbBeforeLock->set_image (*iBeforeLockOFF);
 }
 
-void EditorPanel::histogramChanged (LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma, LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histLCAM, LUTu & histCCAM,
+void EditorPanel::histogramChanged (LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma, LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histCLurve, LUTu & histLLCurve, LUTu & histLCAM, LUTu & histCCAM,
     LUTu & histRedRaw, LUTu & histGreenRaw, LUTu & histBlueRaw , LUTu & histChroma) {
 
     if (histogramPanel) histogramPanel->histogramChanged (histRed, histGreen, histBlue, histLuma, histRedRaw, histGreenRaw, histBlueRaw, histChroma);
-    tpc->updateCurveBackgroundHistogram (histToneCurve, histLCurve, histCCurve, histLCAM, histCCAM, histRed, histGreen, histBlue, histLuma);
+    tpc->updateCurveBackgroundHistogram (histToneCurve, histLCurve, histCCurve, histCLurve,  histLLCurve, histLCAM, histCCAM, histRed, histGreen, histBlue, histLuma);
 }
 
 bool EditorPanel::CheckSidePanelsVisibility() {
