@@ -35,17 +35,10 @@ ExportPanel::ExportPanel () : listener (NULL) {
 
 	Gtk::Label* labExportTitle = Gtk::manage ( new Gtk::Label (M("EXPORT_FASTEXPORTOPTIONS")) );
 	labExportTitle->set_use_markup (true);
+	labExportTitle->set_tooltip_text (M("EXPORT_INSTRUCTIONS"));
 	labExportTitle->set_alignment(Gtk::ALIGN_LEFT);
 	pack_start(*labExportTitle, Gtk::PACK_SHRINK, 4);
 
-	Gtk::Label* labInstructions = Gtk::manage ( new Gtk::Label (M("EXPORT_INSTRUCTIONS")) );
-	labInstructions->set_use_markup (true);
-	labInstructions->set_line_wrap (true);
-	labInstructions->set_alignment(Gtk::ALIGN_LEFT);
-	Gtk::HBox* hbInstructions = Gtk::manage (new Gtk::HBox ());
-	hbInstructions->pack_start(*labInstructions, Gtk::PACK_SHRINK, 4);
-	pack_start(*hbInstructions, Gtk::PACK_SHRINK, 4);
-
 	bypass_ALL              = Gtk::manage ( new Gtk::CheckButton (M("EXPORT_BYPASS_ALL")));
 	bypass_sharpening       = Gtk::manage ( new Gtk::CheckButton (M("EXPORT_BYPASS_SHARPENING")));
 	bypass_sharpenEdge      = Gtk::manage ( new Gtk::CheckButton (M("EXPORT_BYPASS_SHARPENEDGE")));
@@ -56,7 +49,7 @@ ExportPanel::ExportPanel () : listener (NULL) {
 	bypass_dirpyrDenoise    = Gtk::manage ( new Gtk::CheckButton (M("EXPORT_BYPASS_DIRPYRDENOISE")));
 	bypass_sh_hq            = Gtk::manage ( new Gtk::CheckButton (M("EXPORT_BYPASS_SH_HQ")));
 	bypass_dirpyrequalizer  = Gtk::manage ( new Gtk::CheckButton (M("EXPORT_BYPASS_DIRPYREQUALIZER")));
-	bypass_raw_all_enhance  = Gtk::manage ( new Gtk::CheckButton (M("EXPORT_BYPASS_RAW_ALL_ENHANCE")));
+	//bypass_raw_all_enhance  = Gtk::manage ( new Gtk::CheckButton (M("EXPORT_BYPASS_RAW_ALL_ENHANCE")));
 	bypass_raw_ccSteps      = Gtk::manage ( new Gtk::CheckButton (M("EXPORT_BYPASS_RAW_CCSTEPS")));
 	bypass_raw_linenoise    = Gtk::manage ( new Gtk::CheckButton (M("EXPORT_BYPASS_RAW_LINENOISE")));
 	bypass_raw_greenthresh  = Gtk::manage ( new Gtk::CheckButton (M("EXPORT_BYPASS_RAW_GREENTHRESH")));
@@ -65,6 +58,7 @@ ExportPanel::ExportPanel () : listener (NULL) {
 	bypass_raw_ff           = Gtk::manage ( new Gtk::CheckButton (M("EXPORT_BYPASS_RAW_FF")));
 	bypass_raw_dcb_iterations = Gtk::manage ( new Gtk::CheckButton (M("EXPORT_BYPASS_RAW_DCB_ITERATIONS")));
 	bypass_raw_dcb_enhance    = Gtk::manage ( new Gtk::CheckButton (M("EXPORT_BYPASS_RAW_DCB_ENHANCE")));
+	bypass_raw_lmmse_iterations = Gtk::manage ( new Gtk::CheckButton (M("EXPORT_BYPASS_RAW_LMMSE_ITERATIONS")));
 
 	Gtk::HBox* hb_raw_dmethod = Gtk::manage (new Gtk::HBox ());
 	hb_raw_dmethod->pack_start (*Gtk::manage (new Gtk::Label ( M("EXPORT_RAW_DMETHOD") +": ")),Gtk::PACK_SHRINK, 4);
@@ -89,11 +83,12 @@ ExportPanel::ExportPanel () : listener (NULL) {
 	pack_start(*bypass_dirpyrequalizer , Gtk::PACK_SHRINK, 4);
 
 	pack_start(*hb_raw_dmethod         , Gtk::PACK_SHRINK, 4);
-	pack_start(*bypass_raw_all_enhance , Gtk::PACK_SHRINK, 4);
+	//pack_start(*bypass_raw_all_enhance , Gtk::PACK_SHRINK, 4);
 	pack_start(*bypass_raw_ccSteps     , Gtk::PACK_SHRINK, 4);
 	pack_start(*bypass_raw_dcb_iterations, Gtk::PACK_SHRINK, 4);
 	pack_start(*bypass_raw_dcb_enhance   , Gtk::PACK_SHRINK, 4);
 	pack_start(*bypass_raw_ca         , Gtk::PACK_SHRINK, 4);
+	pack_start(*bypass_raw_lmmse_iterations, Gtk::PACK_SHRINK, 4);
 
 	pack_start(*bypass_raw_linenoise   , Gtk::PACK_SHRINK, 4);
 	pack_start(*bypass_raw_greenthresh , Gtk::PACK_SHRINK, 4);
@@ -168,10 +163,11 @@ ExportPanel::ExportPanel () : listener (NULL) {
 	bypass_dirpyrDenoiseConn      = bypass_dirpyrDenoise->signal_toggled().connect (sigc::bind (sigc::mem_fun(*bypass_ALL, &Gtk::CheckButton::set_inconsistent), true));
 	bypass_sh_hqConn              = bypass_sh_hq->signal_toggled().connect (sigc::bind (sigc::mem_fun(*bypass_ALL, &Gtk::CheckButton::set_inconsistent), true));
 	bypass_dirpyrequalizerConn    = bypass_dirpyrequalizer->signal_toggled().connect (sigc::bind (sigc::mem_fun(*bypass_ALL, &Gtk::CheckButton::set_inconsistent), true));
-	bypass_raw_all_enhanceConn    = bypass_raw_all_enhance->signal_toggled().connect (sigc::bind (sigc::mem_fun(*bypass_ALL, &Gtk::CheckButton::set_inconsistent), true));
+	//bypass_raw_all_enhanceConn    = bypass_raw_all_enhance->signal_toggled().connect (sigc::bind (sigc::mem_fun(*bypass_ALL, &Gtk::CheckButton::set_inconsistent), true));
 	bypass_raw_ccStepsConn        = bypass_raw_ccSteps->signal_toggled().connect (sigc::bind (sigc::mem_fun(*bypass_ALL, &Gtk::CheckButton::set_inconsistent), true));
 	bypass_raw_dcb_iterationsConn = bypass_raw_dcb_iterations->signal_toggled().connect (sigc::bind (sigc::mem_fun(*bypass_ALL, &Gtk::CheckButton::set_inconsistent), true));
 	bypass_raw_dcb_enhanceConn    = bypass_raw_dcb_enhance->signal_toggled().connect (sigc::bind (sigc::mem_fun(*bypass_ALL, &Gtk::CheckButton::set_inconsistent), true));
+	bypass_raw_lmmse_iterationsConn = bypass_raw_lmmse_iterations->signal_toggled().connect (sigc::bind (sigc::mem_fun(*bypass_ALL, &Gtk::CheckButton::set_inconsistent), true));
 	bypass_raw_caConn             = bypass_raw_ca->signal_toggled().connect (sigc::bind (sigc::mem_fun(*bypass_ALL, &Gtk::CheckButton::set_inconsistent), true));
 	bypass_raw_linenoiseConn      = bypass_raw_linenoise->signal_toggled().connect (sigc::bind (sigc::mem_fun(*bypass_ALL, &Gtk::CheckButton::set_inconsistent), true));
 	bypass_raw_greenthreshConn    = bypass_raw_greenthresh->signal_toggled().connect (sigc::bind (sigc::mem_fun(*bypass_ALL, &Gtk::CheckButton::set_inconsistent), true));
@@ -207,10 +203,11 @@ void ExportPanel::SaveSettingsAsDefault(){
 	options.fastexport_bypass_dirpyrDenoise      = bypass_dirpyrDenoise->get_active     ();
 	options.fastexport_bypass_sh_hq              = bypass_sh_hq->get_active             ();
 	options.fastexport_bypass_dirpyrequalizer    = bypass_dirpyrequalizer->get_active   ();
-	options.fastexport_bypass_raw_all_enhance    = bypass_raw_all_enhance->get_active   ();
+	//options.fastexport_bypass_raw_all_enhance    = bypass_raw_all_enhance->get_active   ();
 	options.fastexport_bypass_raw_ccSteps        = bypass_raw_ccSteps->get_active       ();
 	options.fastexport_bypass_raw_dcb_iterations = bypass_raw_dcb_iterations->get_active();
 	options.fastexport_bypass_raw_dcb_enhance    = bypass_raw_dcb_enhance->get_active   ();
+	options.fastexport_bypass_raw_lmmse_iterations = bypass_raw_lmmse_iterations->get_active();
 	options.fastexport_bypass_raw_ca             = bypass_raw_ca->get_active            ();
 	options.fastexport_bypass_raw_linenoise      = bypass_raw_linenoise->get_active     ();
 	options.fastexport_bypass_raw_greenthresh    = bypass_raw_greenthresh->get_active   ();
@@ -260,10 +257,11 @@ void ExportPanel::LoadDefaultSettings(){
 	bypass_dirpyrDenoise->set_active     (options.fastexport_bypass_dirpyrDenoise      );
 	bypass_sh_hq->set_active             (options.fastexport_bypass_sh_hq              );
 	bypass_dirpyrequalizer->set_active   (options.fastexport_bypass_dirpyrequalizer    );
-	bypass_raw_all_enhance->set_active   (options.fastexport_bypass_raw_all_enhance    );
+	//bypass_raw_all_enhance->set_active   (options.fastexport_bypass_raw_all_enhance    );
 	bypass_raw_ccSteps->set_active       (options.fastexport_bypass_raw_ccSteps        );
 	bypass_raw_dcb_iterations->set_active(options.fastexport_bypass_raw_dcb_iterations );
 	bypass_raw_dcb_enhance->set_active   (options.fastexport_bypass_raw_dcb_enhance    );
+	bypass_raw_lmmse_iterations->set_active(options.fastexport_bypass_raw_lmmse_iterations );
 	bypass_raw_ca->set_active            (options.fastexport_bypass_raw_ca             );
 	bypass_raw_linenoise->set_active     (options.fastexport_bypass_raw_linenoise      );
 	bypass_raw_greenthresh->set_active   (options.fastexport_bypass_raw_greenthresh    );
@@ -329,10 +327,11 @@ void ExportPanel::bypassALL_Toggled(){
 	bypass_dirpyrDenoiseConn.block      (true);
 	bypass_sh_hqConn.block              (true);
 	bypass_dirpyrequalizerConn.block    (true);
-	bypass_raw_all_enhanceConn.block    (true);
+	//bypass_raw_all_enhanceConn.block    (true);
 	bypass_raw_ccStepsConn.block        (true);
 	bypass_raw_dcb_iterationsConn.block (true);
 	bypass_raw_dcb_enhanceConn.block    (true);
+	bypass_raw_lmmse_iterationsConn.block (true);
 	bypass_raw_caConn.block             (true);
 	bypass_raw_linenoiseConn.block      (true);
 	bypass_raw_greenthreshConn.block    (true);
@@ -350,10 +349,11 @@ void ExportPanel::bypassALL_Toggled(){
 	bypass_dirpyrDenoise->set_active(bypass_ALL->get_active());
 	bypass_sh_hq->set_active(bypass_ALL->get_active());
 	bypass_dirpyrequalizer->set_active(bypass_ALL->get_active());
-	bypass_raw_all_enhance->set_active(bypass_ALL->get_active());
+	//bypass_raw_all_enhance->set_active(bypass_ALL->get_active());
 	bypass_raw_ccSteps->set_active(bypass_ALL->get_active());
 	bypass_raw_dcb_iterations->set_active(bypass_ALL->get_active());
 	bypass_raw_dcb_enhance->set_active(bypass_ALL->get_active());
+	bypass_raw_lmmse_iterations->set_active(bypass_ALL->get_active());
 	bypass_raw_ca->set_active(bypass_ALL->get_active());
 	bypass_raw_linenoise->set_active(bypass_ALL->get_active());
 	bypass_raw_greenthresh->set_active(bypass_ALL->get_active());
@@ -369,10 +369,11 @@ void ExportPanel::bypassALL_Toggled(){
 	bypass_dirpyrDenoiseConn.block      (false);
 	bypass_sh_hqConn.block              (false);
 	bypass_dirpyrequalizerConn.block    (false);
-	bypass_raw_all_enhanceConn.block    (false);
+	//bypass_raw_all_enhanceConn.block    (false);
 	bypass_raw_ccStepsConn.block        (false);
 	bypass_raw_dcb_iterationsConn.block (false);
 	bypass_raw_dcb_enhanceConn.block    (false);
+	bypass_raw_lmmse_iterationsConn.block (false);
 	bypass_raw_caConn.block             (false);
 	bypass_raw_linenoiseConn.block      (false);
 	bypass_raw_greenthreshConn.block    (false);
