@@ -25,15 +25,19 @@
 #include "exiffiltersettings.h"
 #include "filebrowserentry.h"
 #include "browserfilter.h"
+#include "pparamschangelistener.h"
 #include "partialpastedlg.h"
 #include "exportpanel.h"
 #include "extprog.h"
+#include "profilestore.h"
 
+class ProfileStoreLabel;
 class FileBrowser;
 class FileBrowserEntry;
 class FileBrowserListener {
 
     public:
+        virtual     ~FileBrowserListener    () {}
         virtual void openRequested          (std::vector<Thumbnail*> tbe) {}
         virtual void developRequested       (std::vector<FileBrowserEntry*> tbe, bool fastmode) {}
         virtual void renameRequested        (std::vector<FileBrowserEntry*> tbe) {}
@@ -54,7 +58,8 @@ struct FileBrowserIdleHelper {
  */
 class FileBrowser  : public ThumbBrowserBase,
                      public LWButtonListener,
-                     public ExportPanelListener{
+                     public ExportPanelListener,
+                     public ProfileStoreListener {
 
     typedef sigc::signal<void> type_trash_changed;
 
@@ -112,6 +117,7 @@ class FileBrowser  : public ThumbBrowserBase,
 
     Glib::RefPtr<Gtk::AccelGroup> pmaccelgroup;
 
+    BatchPParamsChangeListener* bppcl;
     FileBrowserListener* tbl;
     BrowserFilter filter;
     int numFiltered;
@@ -140,11 +146,12 @@ class FileBrowser  : public ThumbBrowserBase,
     FileBrowserEntry*  delEntry (const Glib::ustring& fname);    // return the entry if found here return NULL otherwise
     void close ();
     
+    void setBatchPParamsChangeListener (BatchPParamsChangeListener* l) { bppcl = l; }
     void setFileBrowserListener (FileBrowserListener* l) { tbl = l; }
      
     void menuItemActivated (Gtk::MenuItem* m);
-    void applyMenuItemActivated (Glib::ustring ppname);
-    void applyPartialMenuItemActivated (Glib::ustring ppname);
+    void applyMenuItemActivated (ProfileStoreLabel *label);
+    void applyPartialMenuItemActivated (ProfileStoreLabel *label);
 
     void applyFilter (const BrowserFilter& filter);
     int getNumFiltered(){ return numFiltered;}
@@ -156,6 +163,9 @@ class FileBrowser  : public ThumbBrowserBase,
     void doubleClicked (ThumbBrowserEntryBase* entry);
     bool keyPressed (GdkEventKey* event);
 
+    void saveThumbnailHeight (int height);
+    int  getThumbnailHeight ();
+
     void openNextImage ();
     void openPrevImage ();
     void copyProfile ();
@@ -175,6 +185,8 @@ class FileBrowser  : public ThumbBrowserBase,
 	// exportpanel interface
 	void exportRequested();
 
+    void updateProfileList ();
+
     type_trash_changed trash_changed();
 };
 
