@@ -38,8 +38,6 @@ using namespace std;
 
 #define CHECKTIME 2000
 
-extern Glib::ustring argv0;
-
 FileCatalog::FileCatalog (CoarsePanel* cp, ToolBar* tb, FilePanel* filepanel) :
 		filepanel(filepanel),
 		selectedDirectoryId(1),
@@ -144,7 +142,7 @@ FileCatalog::FileCatalog (CoarsePanel* cp, ToolBar* tb, FilePanel* filepanel) :
     igFilterClear = new RTImage ("filter.png");
     bFilterClear = Gtk::manage(new Gtk::ToggleButton ());
     bFilterClear->set_active (true);
-    bFilterClear->set_image(*iFilterClear);//(*Gtk::manage(new RTImage ("filterclear.png")));
+    bFilterClear->set_image(*iFilterClear);// (*Gtk::manage(new RTImage ("filterclear.png")));
     bFilterClear->set_relief (Gtk::RELIEF_NONE);
     bFilterClear->set_tooltip_markup (M("FILEBROWSER_SHOWDIRHINT"));
     bFilterClear->signal_button_press_event().connect (sigc::mem_fun(*this, &FileCatalog::capture_event),false);
@@ -462,9 +460,12 @@ void FileCatalog::closeDir () {
     // remove entries
     selectedDirectory = "";
     fileBrowser->close ();  
-	fileNameList.clear ();
-	
+    fileNameList.clear ();
+
+    {
+    MyMutex::MyLock lock(dirEFSMutex);
     dirEFS.clear ();
+    }
     hasValidCurrentEFS = false;
     redrawAll ();
 }
@@ -512,9 +513,7 @@ void FileCatalog::dirSelected (const Glib::ustring& dirname, const Glib::ustring
 
 #ifdef WIN32
       wdMonitor = new WinDirMonitor (selectedDirectory, this);
-#elif defined __APPLE__
-      printf("TODO fix dir->monitor_directory () for OSX\n");
-#else  
+#else
         dirMonitor = dir->monitor_directory ();
         dirMonitor->signal_changed().connect (sigc::bind(sigc::mem_fun(*this, &FileCatalog::on_dir_changed), false));
 #endif
@@ -537,13 +536,15 @@ void FileCatalog::_refreshProgressBar () {
     // Also mention that this progress bar only measures the FIRST pass (quick thumbnails)
     // The second, usually longer pass is done multithreaded down in the single entries and is NOT measured by this
     if (!inTabMode) {
-    	Gtk::Notebook *nb =(Gtk::Notebook *)(filepanel->get_parent());
-    	Gtk::Box* hbb=NULL;
-    	Gtk::Label *label=NULL;
-    	if( options.mainNBVertical )
-           hbb = Gtk::manage (new Gtk::VBox ());
-    	else
-    	   hbb = Gtk::manage (new Gtk::HBox ());
+        GThreadLock lock; // All GUI acces from idle_add callbacks or separate thread HAVE to be protected
+
+        Gtk::Notebook *nb =(Gtk::Notebook *)(filepanel->get_parent());
+        Gtk::Box* hbb=NULL;
+        Gtk::Label *label=NULL;
+        if( options.mainNBVertical )
+            hbb = Gtk::manage (new Gtk::VBox ());
+        else
+            hbb = Gtk::manage (new Gtk::HBox ());
         if (!previewsToLoad ) {
             hbb->pack_start (*Gtk::manage (new Gtk::Image (Gtk::Stock::DIRECTORY, Gtk::ICON_SIZE_MENU)));
             int filteredCount=fileBrowser->getNumFiltered();
@@ -573,17 +574,9 @@ int refreshProgressBarUI (void* data) {
 }
 
 void FileCatalog::previewReady (int dir_id, FileBrowserEntry* fdn) {
-    GThreadLock lock;
-    previewReadyUI (dir_id,fdn);
-}
 
-// Called WITHIN gtk thread
-void FileCatalog::previewReadyUI (int dir_id, FileBrowserEntry* fdn) {
-
-	if ( dir_id != selectedDirectoryId )
-	{
-		return;
-	}
+    if ( dir_id != selectedDirectoryId )
+        return;
 
     // put it into the "full directory" browser
     fdn->setImageAreaToolListener (iatlistener);
@@ -591,6 +584,9 @@ void FileCatalog::previewReadyUI (int dir_id, FileBrowserEntry* fdn) {
     
     // update exif filter settings (minimal & maximal values of exif tags, cameras, lenses, etc...)
     const CacheImageData* cfs = fdn->thumbnail->getCacheImageData();
+
+    {
+    MyMutex::MyLock lock(dirEFSMutex);
     if (cfs->exifValid) {
         if (cfs->fnumber < dirEFS.fnumberFrom)
             dirEFS.fnumberFrom = cfs->fnumber;
@@ -610,16 +606,17 @@ void FileCatalog::previewReadyUI (int dir_id, FileBrowserEntry* fdn) {
             dirEFS.focalTo = cfs->focalLen;
     }
     dirEFS.filetypes.insert (cfs->filetype);
-    dirEFS.cameras.insert (cfs->camera);
+    dirEFS.cameras.insert (cfs->getCamera());
     dirEFS.lenses.insert (cfs->lens);
     dirEFS.expcomp.insert (cfs->expcomp);
+    }
+
     previewsLoaded++;
 
     g_idle_add (refreshProgressBarUI, this);
 }
 
 int prevfinished (void* data) {
- 	GThreadLock lock;
     (static_cast<FileCatalog*>(data))->previewsFinishedUI ();
     return 0;
 }
@@ -627,26 +624,30 @@ int prevfinished (void* data) {
 // Called within GTK UI thread
 void FileCatalog::previewsFinishedUI () {
 
-    redrawAll ();
-    previewsToLoad = 0;
+    {
+        GThreadLock lock; // All GUI acces from idle_add callbacks or separate thread HAVE to be protected
+        redrawAll ();
+        previewsToLoad = 0;
 
-	if (filterPanel) {
-		filterPanel->set_sensitive (true);
-	    if ( !hasValidCurrentEFS ){
-	        currentEFS = dirEFS;
-		    filterPanel->setFilter ( dirEFS,true );
-	    }else {
-		    filterPanel->setFilter ( currentEFS,false );
-	    }
-	}
-	
-	if (exportPanel)
-		exportPanel->set_sensitive (true);
+        if (filterPanel) {
+            filterPanel->set_sensitive (true);
+            if ( !hasValidCurrentEFS ){
+                MyMutex::MyLock lock(dirEFSMutex);
+                currentEFS = dirEFS;
+                filterPanel->setFilter ( dirEFS,true );
+            }else {
+                filterPanel->setFilter ( currentEFS,false );
+            }
+        }
 
- 	// restart anything that might have been loaded low quality
- 	fileBrowser->refreshQuickThumbImages();
- 	fileBrowser->applyFilter (getFilter());  // refresh total image count
-    _refreshProgressBar();
+        if (exportPanel)
+            exportPanel->set_sensitive (true);
+
+        // restart anything that might have been loaded low quality
+        fileBrowser->refreshQuickThumbImages();
+        fileBrowser->applyFilter (getFilter());  // refresh total image count
+        _refreshProgressBar();
+    }
     filepanel->loadingThumbs(M("PROGRESSBAR_READY"),0);
 
     if (!imageToSelect_fname.empty()){
@@ -668,8 +669,10 @@ void FileCatalog::previewsFinished (int dir_id) {
 		return;
 	}
 
-    if (!hasValidCurrentEFS) 
+    if (!hasValidCurrentEFS) {
+        MyMutex::MyLock lock(dirEFSMutex);
         currentEFS = dirEFS;
+    }
 
     g_idle_add (prevfinished, this);
 }
@@ -689,9 +692,9 @@ void FileCatalog::refreshThumbImages () {
 void FileCatalog::refreshHeight () {
     int newHeight=fileBrowser->getEffectiveHeight() + buttonBar->get_height();
     if (!options.FileBrowserToolbarSingleRow) {
-    	newHeight += hbToolBar1->get_height();
+        newHeight += hbToolBar1->get_height();
     }
-    set_size_request(0, newHeight);
+    set_size_request(0, newHeight+2); // HOMBRE: yeah, +2, there's always 2 pixels missing... sorry for this dirty hack O:)
 }
 
 void FileCatalog::_openImage (std::vector<Thumbnail*> tmb) {
@@ -879,11 +882,14 @@ void FileCatalog::developRequested (std::vector<FileBrowserEntry*> tbe, bool fas
     if (listener) {
     	std::vector<BatchQueueEntry*> entries;
 
-        #pragma omp parallel for ordered
+        // TODO: (HOMBRE) should we still use parallelization here, now that thumbnails are processed asynchronously...?
+        //#pragma omp parallel for ordered
         for (size_t i=0; i<tbe.size(); i++) {
-            rtengine::procparams::ProcParams params = tbe[i]->thumbnail->getProcParams();
+            FileBrowserEntry* fbe = tbe[i];
+            Thumbnail* th = fbe->thumbnail;
+            rtengine::procparams::ProcParams params = th->getProcParams();
 
-            // if fast mode is selected, override (disable) prams
+            // if fast mode is selected, override (disable) params
             // controlling time and resource consuming tasks
             // and also those which effect is not pronounced after reducing the image size
             // TODO!!! could expose selections below via preferences
@@ -921,30 +927,19 @@ void FileCatalog::developRequested (std::vector<FileBrowserEntry*> tbe, bool fas
 				params.resize.height     = options.fastexport_resize_height   ;
             }
 
-            rtengine::ProcessingJob* pjob = rtengine::ProcessingJob::create (tbe[i]->filename, tbe[i]->thumbnail->getType()==FT_Raw, params);
-            double tmpscale;
-            rtengine::IImage8* img = tbe[i]->thumbnail->processThumbImage (params, BatchQueue::calcMaxThumbnailHeight(), tmpscale);
+            rtengine::ProcessingJob* pjob = rtengine::ProcessingJob::create (fbe->filename, th->getType()==FT_Raw, params);
 
-            int pw, ph;
-            guint8* prev=NULL;
-
-            if (img) {
-                pw = img->getWidth ();
-                ph = img->getHeight ();
-                prev = new guint8 [pw*ph*3];
-                memcpy (prev, img->getData (), pw*ph*3);
-                img->free();
- 
-            } else {
-                tbe[i]->thumbnail->getThumbnailSize (pw, ph);
-            }
+            int pw;
+            int ph = BatchQueue::calcMaxThumbnailHeight();
+            th->getThumbnailSize (pw, ph);
 
             // processThumbImage is the processing intensive part, but adding to queue must be ordered
-            #pragma omp ordered
-            {
-                entries.push_back(new BatchQueueEntry (pjob, params, tbe[i]->filename, prev, pw, ph, tbe[i]->thumbnail));
-            }
-            }
+            //#pragma omp ordered
+            //{
+                BatchQueueEntry* bqh = new BatchQueueEntry (pjob, params, fbe->filename, pw, ph, th);
+                entries.push_back(bqh);
+            //}
+        }
 
         listener->addBatchQueueJobs( entries );
     }
@@ -1324,8 +1319,10 @@ BrowserFilter FileCatalog::getFilter () {
     if (!filterPanel)
 		filter.exifFilterEnabled = false;
 	else {
-		if (!hasValidCurrentEFS)
+		if (!hasValidCurrentEFS) {
+			MyMutex::MyLock lock(dirEFSMutex);
 			filter.exifFilter = dirEFS;
+		}
 		else
 			filter.exifFilter = currentEFS;
 		filter.exifFilterEnabled = filterPanel->isEnabled ();
@@ -1408,15 +1405,14 @@ void FileCatalog::winDirChanged () {
 
 void FileCatalog::on_dir_changed (const Glib::RefPtr<Gio::File>& file, const Glib::RefPtr<Gio::File>& other_file, Gio::FileMonitorEvent event_type, bool internal) {
 
-	if (options.has_retained_extention(file->get_parse_name())) {
-		if (!internal)
-			gdk_threads_enter();
-
-		if (event_type == Gio::FILE_MONITOR_EVENT_CREATED || event_type == Gio::FILE_MONITOR_EVENT_DELETED || event_type == Gio::FILE_MONITOR_EVENT_CHANGED)
-			reparseDirectory ();
-
-		if (!internal)
-			gdk_threads_leave();
+	if (options.has_retained_extention(file->get_parse_name())
+	&& (event_type == Gio::FILE_MONITOR_EVENT_CREATED || event_type == Gio::FILE_MONITOR_EVENT_DELETED || event_type == Gio::FILE_MONITOR_EVENT_CHANGED)) {
+			if (!internal) {
+				GThreadLock lock;
+				reparseDirectory ();
+			}
+			else
+				reparseDirectory ();
 	}
 }
 
@@ -1454,7 +1450,7 @@ void FileCatalog::addAndOpenFile (const Glib::ustring& fname) {
         Thumbnail* tmb = cacheMgr->getEntry (file->get_parse_name());
         if (tmb) {
             FileBrowserEntry* entry = new FileBrowserEntry (tmb, file->get_parse_name());
-			previewReadyUI (selectedDirectoryId,entry);
+            previewReady (selectedDirectoryId,entry);
             // open the file
             FCOIParams* params = new FCOIParams;
             params->catalog = this;
@@ -1509,9 +1505,10 @@ void FileCatalog::selectionChanged (std::vector<Thumbnail*> tbe) {
         fslistener->selectionChanged (tbe);
 }
 
+// Called within GTK UI thread
 void FileCatalog::exifFilterChanged () {
 
-	currentEFS = filterPanel->getFilter ();
+    currentEFS = filterPanel->getFilter ();
     hasValidCurrentEFS = true;
     fileBrowser->applyFilter (getFilter ());
     _refreshProgressBar();
@@ -1531,11 +1528,14 @@ void FileCatalog::trashChanged () {
         bTrash->set_image(*iTrashFull);
     }
 }
+
+// Called within GTK UI thread
 void FileCatalog::buttonQueryClearPressed () {
 	Query->set_text("");
 	FileCatalog::executeQuery ();
 }
 
+// Called within GTK UI thread
 void FileCatalog::executeQuery(){
 	// if BrowsePath text was changed, do a full browse;
 	// otherwise filter only
@@ -1557,9 +1557,11 @@ bool FileCatalog::Query_key_pressed (GdkEventKey *event){
             FileCatalog::buttonQueryClearPressed ();
             return true;
         }
+        break;
     default:
-        return false;
+        break;
     }
+    return false;
 }
 
 void FileCatalog::updateFBQueryTB (bool singleRow) {
@@ -1625,9 +1627,11 @@ bool FileCatalog::BrowsePath_key_pressed (GdkEventKey *event){
             BrowsePath->select_region(BrowsePath->get_text_length(), BrowsePath->get_text_length());
             return true;
         }
+        break;
     default:
-        return false;
+        break;
     }
+    return false;
 }
 
 void FileCatalog::tbLeftPanel_1_visible (bool visible){
@@ -1744,7 +1748,11 @@ bool FileCatalog::handleShortcutKey (GdkEventKey* event) {
     bool ctrl = event->state & GDK_CONTROL_MASK;
     bool shift = event->state & GDK_SHIFT_MASK;
     bool alt = event->state & GDK_MOD1_MASK;
-    
+#ifdef __WIN32__
+    bool altgr = event->state & GDK_MOD2_MASK;
+#else
+    bool altgr = event->state & GDK_MOD5_MASK;
+#endif
     modifierKey = event->state;
     
     // GUI Layout
@@ -1773,32 +1781,37 @@ bool FileCatalog::handleShortcutKey (GdkEventKey* event) {
         }
     }
 
-    if (!alt && !shift) { // shift is reserved for ranking
-        switch(event->keyval) {
-			case GDK_grave:
+#ifdef __WIN32__
+	if (!alt && !shift && !altgr) { // shift is reserved for ranking
+		switch(event->hardware_keycode) {
+			case 0x30:
 				categoryButtonToggled(bUnRanked,false);
 				return true;
-            case GDK_1:
-                categoryButtonToggled(bRank[0],false);
-                return true;
-            case GDK_2:
-                categoryButtonToggled(bRank[1],false);
-                return true;
-            case GDK_3:
-                categoryButtonToggled(bRank[2],false);
-                return true;
-            case GDK_4:
-                categoryButtonToggled(bRank[3],false);
-                return true;
-            case GDK_5:
-                categoryButtonToggled(bRank[4],false);
-                return true;
-            case GDK_6:
-                categoryButtonToggled(bEdited[0],false);
-                return true;
-            case GDK_7:
-                categoryButtonToggled(bEdited[1],false);
-                return true;
+			case 0x31:
+				categoryButtonToggled(bRank[0],false);
+				return true;
+			case 0x32:
+				categoryButtonToggled(bRank[1],false);
+				return true;
+			case 0x33:
+				categoryButtonToggled(bRank[2],false);
+				return true;
+			case 0x34:
+				categoryButtonToggled(bRank[3],false);
+				return true;
+			case 0x35:
+				categoryButtonToggled(bRank[4],false);
+				return true;
+			case 0x36:
+				categoryButtonToggled(bEdited[0],false);
+				return true;
+			case 0x37:
+				categoryButtonToggled(bEdited[1],false);
+				return true;
+		}
+	}
+    if (!alt && !shift) {
+        switch(event->keyval) {
 
             case GDK_Return:
             case GDK_KP_Enter:
@@ -1806,38 +1819,109 @@ bool FileCatalog::handleShortcutKey (GdkEventKey* event) {
                     FileCatalog::buttonBrowsePathPressed ();
                     return true;
                 }
+                break;
         }
     }
     
     if (alt && !shift) { // shift is reserved for color labeling
-        switch(event->keyval) {
-            case GDK_grave:
+        switch(event->hardware_keycode) {
+            case 0x30:
                  categoryButtonToggled(bUnCLabeled,false);
                  return true;
-            case GDK_1:
+            case 0x31:
                 categoryButtonToggled(bCLabel[0],false);
                 return true;
-            case GDK_2:
+            case 0x32:
                 categoryButtonToggled(bCLabel[1],false);
                 return true;
-            case GDK_3:
+            case 0x33:
                 categoryButtonToggled(bCLabel[2],false);
                 return true;
-            case GDK_4:
+            case 0x34:
                 categoryButtonToggled(bCLabel[3],false);
                 return true;
-            case GDK_5:
+            case 0x35:
                 categoryButtonToggled(bCLabel[4],false);
                 return true;
-            case GDK_6:
+            case 0x36:
                 categoryButtonToggled(bRecentlySaved[0],false);
                 return true;
-            case GDK_7:
+            case 0x37:
                 categoryButtonToggled(bRecentlySaved[1],false);
                 return true;
         }
     }
+#else
+	if (!alt && !shift && !altgr) { // shift is reserved for ranking
+		switch(event->hardware_keycode) {
+			case 0x13:
+				categoryButtonToggled(bUnRanked,false);
+				return true;
+			case 0x0a:
+				categoryButtonToggled(bRank[0],false);
+				return true;
+			case 0x0b:
+				categoryButtonToggled(bRank[1],false);
+				return true;
+			case 0x0c:
+				categoryButtonToggled(bRank[2],false);
+				return true;
+			case 0x0d:
+				categoryButtonToggled(bRank[3],false);
+				return true;
+			case 0x0e:
+				categoryButtonToggled(bRank[4],false);
+				return true;
+			case 0x0f:
+				categoryButtonToggled(bEdited[0],false);
+				return true;
+			case 0x10:
+				categoryButtonToggled(bEdited[1],false);
+				return true;
+		}
+	}
+    if (!alt && !shift) {
+        switch(event->keyval) {
+
+            case GDK_Return:
+            case GDK_KP_Enter:
+                if (BrowsePath->is_focus()){
+                    FileCatalog::buttonBrowsePathPressed ();
+                    return true;
+                }
+                break;
+        }
+    }
     
+    if (alt && !shift) { // shift is reserved for color labeling
+        switch(event->hardware_keycode) {
+            case 0x13:
+                 categoryButtonToggled(bUnCLabeled,false);
+                 return true;
+            case 0x0a:
+                categoryButtonToggled(bCLabel[0],false);
+                return true;
+            case 0x0b:
+                categoryButtonToggled(bCLabel[1],false);
+                return true;
+            case 0x0c:
+                categoryButtonToggled(bCLabel[2],false);
+                return true;
+            case 0x0d:
+                categoryButtonToggled(bCLabel[3],false);
+                return true;
+            case 0x0e:
+                categoryButtonToggled(bCLabel[4],false);
+                return true;
+            case 0x0f:
+                categoryButtonToggled(bRecentlySaved[0],false);
+                return true;
+            case 0x10:
+                categoryButtonToggled(bRecentlySaved[1],false);
+                return true;
+        }
+    }
+#endif
     if (!ctrl && !alt) {
         switch(event->keyval) {
 			case GDK_d:
