@@ -17,6 +17,7 @@
  *  along with RawTherapee.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "clipboard.h"
 #include <gtkmm.h>
 #include <fstream>
 #include <string>
@@ -37,20 +38,27 @@ FlatCurveEditorSubGroup::FlatCurveEditorSubGroup (CurveEditorGroup* prt, Glib::u
 	parent = prt;
 
 	// ControlPoints curve
-	CPointsCurveBox = new Gtk::HBox ();
+	CPointsCurveBox = new Gtk::VBox ();
 	CPointsCurveBox->set_spacing(4);
 	CPointsCurve = Gtk::manage (new MyFlatCurve ());
-	//CPointsCurve->set_size_request (GRAPH_SIZE+2*RADIUS+1, GRAPH_SIZE+2*RADIUS+1);
+	CPointsCurve->set_size_request (GRAPH_SIZE+2*RADIUS+1, GRAPH_SIZE+2*RADIUS+1);
 	CPointsCurve->setType (FCT_MinMaxCPoints);
 	CPointsCurveBox->pack_start (*CPointsCurve, Gtk::PACK_EXPAND_WIDGET, 0);
 
-	Gtk::VBox* CPointsbbox = Gtk::manage (new Gtk::VBox ());
+	Gtk::HBox* CPointsbbox = Gtk::manage (new Gtk::HBox ());
 	CPointsbbox->set_spacing(4);
-	saveCPoints = Gtk::manage (new Gtk::Button ());
-	saveCPoints->add (*Gtk::manage (new Gtk::Image (Gtk::StockID("gtk-save"), Gtk::ICON_SIZE_BUTTON)));
-	loadCPoints = Gtk::manage (new Gtk::Button ());
-	loadCPoints->add (*Gtk::manage (new Gtk::Image (Gtk::StockID("gtk-open"), Gtk::ICON_SIZE_BUTTON)));
 
+	pasteCPoints = Gtk::manage (new Gtk::Button ());
+	pasteCPoints->add (*Gtk::manage (new RTImage ("edit-paste.png")));
+	copyCPoints = Gtk::manage (new Gtk::Button ());
+	copyCPoints->add (*Gtk::manage (new RTImage ("edit-copy.png")));
+	saveCPoints = Gtk::manage (new Gtk::Button ());
+	saveCPoints->add (*Gtk::manage (new RTImage ("gtk-save-large.png")));
+	loadCPoints = Gtk::manage (new Gtk::Button ());
+	loadCPoints->add (*Gtk::manage (new RTImage ("gtk-open.png")));
+
+	CPointsbbox->pack_end (*pasteCPoints, Gtk::PACK_SHRINK, 0);
+	CPointsbbox->pack_end (*copyCPoints, Gtk::PACK_SHRINK, 0);
 	CPointsbbox->pack_end (*saveCPoints, Gtk::PACK_SHRINK, 0);
 	CPointsbbox->pack_end (*loadCPoints, Gtk::PACK_SHRINK, 0);
 
@@ -59,8 +67,13 @@ FlatCurveEditorSubGroup::FlatCurveEditorSubGroup (CurveEditorGroup* prt, Glib::u
 
 	saveCPoints->signal_clicked().connect( sigc::mem_fun(*this, &FlatCurveEditorSubGroup::savePressed) );
 	loadCPoints->signal_clicked().connect( sigc::mem_fun(*this, &FlatCurveEditorSubGroup::loadPressed) );
+	copyCPoints->signal_clicked().connect( sigc::mem_fun(*this, &FlatCurveEditorSubGroup::copyPressed) );
+	pasteCPoints->signal_clicked().connect( sigc::mem_fun(*this, &FlatCurveEditorSubGroup::pastePressed) );
+
 	saveCPoints->set_tooltip_text (M("CURVEEDITOR_TOOLTIPSAVE"));
 	loadCPoints->set_tooltip_text (M("CURVEEDITOR_TOOLTIPLOAD"));
+	copyCPoints->set_tooltip_text (M("CURVEEDITOR_TOOLTIPCOPY"));
+	pasteCPoints->set_tooltip_text (M("CURVEEDITOR_TOOLTIPPASTE"));
 
 	CPointsCurve->setCurveListener (parent); // Send the message directly to the parent
 }
@@ -77,12 +90,25 @@ FlatCurveEditor* FlatCurveEditorSubGroup::addCurve(Glib::ustring curveLabel, boo
 
 	// Initialization of the new curve
 	storeCurveValues(newCE, getCurveFromGUI(FCT_MinMaxCPoints));
-
-	// We add it to the curve editor list
-	parent->curveEditors.push_back(newCE);
 	return newCE;
 }
 
+/*
+ * Force the resize of the curve editor, if the displayed one is the requested one
+ */
+void FlatCurveEditorSubGroup::refresh(CurveEditor *curveToRefresh) {
+	if (curveToRefresh != NULL && curveToRefresh == static_cast<FlatCurveEditor*>(parent->displayedCurve)) {
+		switch(FlatCurveType(curveToRefresh->curveType->getSelected())) {
+		case (FCT_MinMaxCPoints):
+			CPointsCurve->refresh();
+			break;
+		default:	// (DCT_Linear, DCT_Unchanged)
+			// ... do nothing
+			break;
+		}
+	}
+}
+
 /*
  * Switch the editor widgets to the currently edited curve
  */
@@ -97,13 +123,67 @@ void FlatCurveEditorSubGroup::switchGUI() {
 		// Initializing GUI values + repacking the appropriated widget
 		//dCurve->typeconn.block(true);
 
+		// first we update the colored bar
+
+		ColorProvider *barColorProvider = dCurve->getLeftBarColorProvider();
+		std::vector<GradientMilestone>  bgGradient = dCurve->getLeftBarBgGradient();
+		if (barColorProvider == NULL && bgGradient.size() == 0) {
+			// dCurve has no left colored bar, so we delete the object
+			if (leftBar) {
+				delete leftBar;
+				leftBar = NULL;
+			}
+		}
+		else {
+			// dCurve has a ColorProvider or a background gradient defined, so we create/update the object
+			if (!leftBar) {
+				leftBar = new ColoredBar(RTO_Bottom2Top);
+			}
+			if (barColorProvider) {
+				bgGradient.clear();
+				leftBar->setColorProvider(barColorProvider, dCurve->getLeftBarCallerId());
+				leftBar->setBgGradient (bgGradient);
+			}
+			else {
+				leftBar->setColorProvider(NULL, -1);
+				leftBar->setBgGradient (bgGradient);
+			}
+		}
+
+		barColorProvider = dCurve->getBottomBarColorProvider();
+		bgGradient = dCurve->getBottomBarBgGradient();
+		if (barColorProvider == NULL && bgGradient.size() == 0) {
+			// dCurve has no bottom colored bar, so we delete the object
+			if (bottomBar) {
+				delete bottomBar;
+				bottomBar = NULL;
+			}
+		}
+		else {
+			// dCurve ave a ColorProvider or a background gradient defined, so we create/update the object
+			if (!bottomBar) {
+				bottomBar = new ColoredBar(RTO_Left2Right);
+			}
+			if (barColorProvider) {
+				bgGradient.clear();
+				bottomBar->setColorProvider(barColorProvider, dCurve->getBottomBarCallerId());
+				bottomBar->setBgGradient (bgGradient);
+			}
+			else {
+				bottomBar->setColorProvider(NULL, -1);
+				bottomBar->setBgGradient (bgGradient);
+			}
+		}
+
 		switch((FlatCurveType)(dCurve->curveType->getSelected())) {
 		case (FCT_MinMaxCPoints):
 			CPointsCurve->setPeriodicity(dCurve->periodic);		// Setting Periodicity before setting points
 			CPointsCurve->setPoints (dCurve->controlPointsCurveEd);
+			CPointsCurve->setColorProvider(dCurve->getCurveColorProvider(), dCurve->getCurveCallerId());
+			CPointsCurve->setColoredBar(leftBar, bottomBar);
+			CPointsCurve->forceResize();
 			parent->pack_start (*CPointsCurveBox);
 			CPointsCurveBox->check_resize();
-			CPointsCurve->forceResize();
 			break;
 		default:	// (DCT_Linear, DCT_Unchanged)
 			// ... do nothing
@@ -132,9 +212,9 @@ void FlatCurveEditorSubGroup::savePressed () {
 
 		int ix = 0;
 		if (p[ix]==(double)(FCT_Linear))
-			f << "Linear\n";
+			f << "Linear" << std::endl;
 		else if (p[ix]==(double)(FCT_MinMaxCPoints))
-			f << "ControlPoints\n";
+			f << "ControlPoints" << std::endl;
 		ix++;
 		for (unsigned int i=0; i<p.size()/2; i++, ix+=2)
 			f << p[ix] << ' ' << p[ix+1] << std::endl;
@@ -171,6 +251,46 @@ void FlatCurveEditorSubGroup::loadPressed () {
     }
 }
 
+void FlatCurveEditorSubGroup::copyPressed () {
+// For compatibility use enum FlatCurveType here
+
+	std::vector<double> curve;
+
+	switch (parent->displayedCurve->selected) {
+	case FCT_MinMaxCPoints:                // custom
+		curve = CPointsCurve->getPoints ();
+		clipboard.setFlatCurveData (curve,FCT_MinMaxCPoints);
+		break;
+	default:                       // (DCT_Linear, DCT_Unchanged)
+		// ... do nothing
+		break;
+	}
+}
+
+void FlatCurveEditorSubGroup::pastePressed () {
+// For compatibility use enum FlatCurveType here
+
+	std::vector<double> curve;
+	FlatCurveType type;
+
+	type = clipboard.hasFlatCurveData();
+
+	if (type == (FlatCurveType)parent->displayedCurve->selected) {
+		curve = clipboard.getFlatCurveData ();
+		switch (type) {
+		case FCT_MinMaxCPoints:    // min/max control points
+			CPointsCurve->setPoints (curve);
+			CPointsCurve->queue_draw ();
+			CPointsCurve->notifyListener ();
+			break;
+		default:                   // (FCT_Linear, FCT_Unchanged)
+			// ... do nothing
+			break;
+		}
+	}
+	return;
+}
+
 /*
  * Store the curves of the currently displayed type from the widgets to the CurveEditor object
  */
@@ -203,7 +323,6 @@ void FlatCurveEditorSubGroup::restoreDisplayedHistogram() {
 void FlatCurveEditorSubGroup::storeCurveValues (CurveEditor* ce, const std::vector<double>& p) {
 	if (!p.empty()) {
 		FlatCurveType t = static_cast<FlatCurveType>(p[0]);
-		for (int i=0; i<(int)p.size(); i++)
 
 		switch (t) {
 		case (FCT_MinMaxCPoints):
@@ -221,7 +340,7 @@ void FlatCurveEditorSubGroup::storeCurveValues (CurveEditor* ce, const std::vect
 const std::vector<double> FlatCurveEditorSubGroup::getCurveFromGUI (int type) {
 	switch ((FlatCurveType)type) {
 	case (FCT_MinMaxCPoints):
-        return CPointsCurve->getPoints ();
+		return CPointsCurve->getPoints ();
 	default: {
 		// linear and other solutions
 		std::vector<double> lcurve (1);
@@ -238,10 +357,10 @@ void FlatCurveEditorSubGroup::removeEditor () {
     removeIfThere (parent, CPointsCurveBox, false);
 }
 
-bool FlatCurveEditorSubGroup::curveReset(int cType) {
+bool FlatCurveEditorSubGroup::curveReset(int cType, double iValue) {
 	switch ((FlatCurveType) cType) {
 	case (FCT_MinMaxCPoints) :	// = Control cage
-		CPointsCurve->reset ();
+		CPointsCurve->reset (iValue);
 		return true;
 		break;
 	/*case (FCT_Parametric) :
@@ -257,10 +376,7 @@ bool FlatCurveEditorSubGroup::curveReset(int cType) {
 		return false;
 		break;
 	}
-}
-
-void FlatCurveEditorSubGroup::setColorProvider (ColorProvider* p) {
-	CPointsCurve->setColorProvider(p);
+	return true;
 }
 
 /*void FlatCurveEditorSubGroup::updateBackgroundHistogram (CurveEditor* ce) {
