@@ -21,6 +21,8 @@
 
 #include <gtkmm.h>
 #include "../rtengine/rtengine.h"
+#include <sstream>
+#include <iostream>
 
 Glib::ustring escapeHtmlChars(const Glib::ustring &src);
 bool removeIfThere (Gtk::Container* cont, Gtk::Widget* w, bool increference=true);
