@@ -18,7 +18,7 @@
  */
 
 #include "hsvequalizer.h"
-#include "../rtengine/utils.h"
+#include "../rtengine/color.h"
 
 using namespace rtengine;
 using namespace rtengine::procparams;
@@ -27,13 +27,34 @@ using namespace rtengine::procparams;
 
 HSVEqualizer::HSVEqualizer () : Gtk::VBox(), FoldableToolPanel(this) {
 	
+	set_border_width(4);
+
+	std::vector<GradientMilestone> bottomMilestones;
+	float R, G, B;
+	// -0.1 rad < Hue < 1.6 rad
+	for (int i=0; i<7; i++) {
+		float x = float(i)*(1.0f/6.0);
+		Color::hsv2rgb01(x, 0.5f, 0.5f, R, G, B);
+		bottomMilestones.push_back( GradientMilestone(double(x), double(R), double(G), double(B)) );
+	}
+
 	curveEditorG = new CurveEditorGroup (options.lastHsvCurvesDir, M("TP_HSVEQUALIZER_CHANNEL"));
 	curveEditorG->setCurveListener (this);
-	curveEditorG->setColorProvider (this);
 
 	hshape = static_cast<FlatCurveEditor*>(curveEditorG->addCurve(CT_Flat, M("TP_HSVEQUALIZER_HUE")));
+	hshape->setBottomBarBgGradient(bottomMilestones);
+	//hshape->setLeftBarColorProvider(this);  Not working yet
+	hshape->setCurveColorProvider(this, 1);
+
 	sshape = static_cast<FlatCurveEditor*>(curveEditorG->addCurve(CT_Flat, M("TP_HSVEQUALIZER_SAT")));
+	sshape->setBottomBarBgGradient(bottomMilestones);
+	//sshape->setLeftBarColorProvider(this);  Not working yet
+	sshape->setCurveColorProvider(this, 2);
+
 	vshape = static_cast<FlatCurveEditor*>(curveEditorG->addCurve(CT_Flat, M("TP_HSVEQUALIZER_VAL")));
+	vshape->setBottomBarBgGradient(bottomMilestones);
+	//vshape->setLeftBarColorProvider(this);  Not working yet
+	vshape->setCurveColorProvider(this, 3);
 
 	// This will add the reset button at the end of the curveType buttons
 	curveEditorG->curveListComplete();
@@ -64,12 +85,14 @@ void HSVEqualizer::read (const ProcParams* pp, const ParamsEdited* pedited) {
 	sshape->setCurve         (pp->hsvequalizer.scurve);
     vshape->setCurve         (pp->hsvequalizer.vcurve);
 	
+    enableListener ();
+}
+
+void HSVEqualizer::autoOpenCurve () {
     // Open up the first curve if selected
     bool active = hshape->openIfNonlinear();
     if (!active) sshape->openIfNonlinear();
     if (!active) vshape->openIfNonlinear();
-
-    enableListener ();
 }
 
 void HSVEqualizer::write (ProcParams* pp, ParamsEdited* pedited) {
@@ -182,35 +205,33 @@ void HSVEqualizer::adjusterChanged (Adjuster* a, double newval) {
 }
 */
 
-void HSVEqualizer::colorForValue (double valX, double valY) {
+void HSVEqualizer::colorForValue (double valX, double valY, int callerId, ColorCaller* caller) {
 
 	float r, g, b;
 
-	CurveEditor* ce = curveEditorG->getDisplayedCurve();
+	if (callerId == 1) {        // Hue = f(Hue)
 
-	if (ce == hshape) {        // Hue = f(Hue)
-
-		float h = (float)((valY - 0.5) * 2. + valX);
-		if (h > 1.0)
-			h -= 1.0;
-		else if (h < 0.0)
-			h += 1.0;
-		hsv2rgb(h, (float)0.5, (float)0.5, r, g, b);
-		red = (double)r;
-		green = (double)g;
-		blue = (double)b;
+		float h = float((valY - 0.5) * 2. + valX);
+		if (h > 1.0f)
+			h -= 1.0f;
+		else if (h < 0.0f)
+			h += 1.0f;
+		Color::hsv2rgb01(h, 0.5f, 0.5f, r, g, b);
+		caller->ccRed = double(r);
+		caller->ccGreen = double(g);
+		caller->ccBlue = double(b);
 	}
-	else if (ce == sshape) {   // Saturation = f(Hue)
-		hsv2rgb((float)valX, (float)valY, (float)0.5, r, g, b);
-		red = (double)r;
-		green = (double)g;
-		blue = (double)b;
+	else if (callerId == 2) {   // Saturation = f(Hue)
+		Color::hsv2rgb01(float(valX), float(valY), 0.5f, r, g, b);
+		caller->ccRed = double(r);
+		caller->ccGreen = double(g);
+		caller->ccBlue = double(b);
 	}
-	else if (ce == vshape) {   // Value = f(Hue)
-		hsv2rgb((float)valX, (float)0.5, (float)valY, r, g, b);
-		red = (double)r;
-		green = (double)g;
-		blue = (double)b;
+	else if (callerId == 3) {   // Value = f(Hue)
+		Color::hsv2rgb01(float(valX), 0.5f, float(valY), r, g, b);
+		caller->ccRed = double(r);
+		caller->ccGreen = double(g);
+		caller->ccBlue = double(b);
 	}
 	else {
 		printf("Error: no curve displayed!\n");
