@@ -32,6 +32,7 @@ LCurve::LCurve () : Gtk::VBox(), FoldableToolPanel(this) {
 	brightness = Gtk::manage (new Adjuster (M("TP_LABCURVE_BRIGHTNESS"), -100., 100., 1., 0.));
 	contrast   = Gtk::manage (new Adjuster (M("TP_LABCURVE_CONTRAST"), -100., 100., 1., 0.));
 	chromaticity   = Gtk::manage (new Adjuster (M("TP_LABCURVE_CHROMATICITY"), -100., 100., 1., 0.));
+	chromaticity->set_tooltip_markup(M("TP_LABCURVE_CHROMA_TOOLTIP"));
 
 	pack_start (*brightness);
 	brightness->show ();
@@ -51,10 +52,6 @@ LCurve::LCurve () : Gtk::VBox(), FoldableToolPanel(this) {
 	hsep2->show ();
 	pack_start (*hsep2, Gtk::PACK_EXPAND_WIDGET, 4);
 	
-	bwtoning = Gtk::manage (new Gtk::CheckButton (M("TP_LABCURVE_BWTONING")));
-	bwtoning->set_tooltip_markup (M("TP_LABCURVE_BWTONING_TIP"));
-	pack_start (*bwtoning);
-
 	avoidcolorshift = Gtk::manage (new Gtk::CheckButton (M("TP_LABCURVE_AVOIDCOLORSHIFT")));
 	avoidcolorshift->set_tooltip_text (M("TP_LABCURVE_AVOIDCOLORSHIFT_TOOLTIP"));
 	pack_start (*avoidcolorshift, Gtk::PACK_SHRINK, 4);
@@ -70,7 +67,6 @@ LCurve::LCurve () : Gtk::VBox(), FoldableToolPanel(this) {
 	rstprotection->setAdjusterListener (this);
 	rstprotection->set_tooltip_text (M("TP_LABCURVE_RSTPRO_TOOLTIP"));
 	
-	bwtconn= bwtoning->signal_toggled().connect( sigc::mem_fun(*this, &LCurve::bwtoning_toggled) );
 	acconn = avoidcolorshift->signal_toggled().connect( sigc::mem_fun(*this, &LCurve::avoidcolorshift_toggled) );
 	lcconn = lcredsk->signal_toggled().connect( sigc::mem_fun(*this, &LCurve::lcredsk_toggled) );
 	
@@ -113,6 +109,22 @@ LCurve::LCurve () : Gtk::VBox(), FoldableToolPanel(this) {
 	milestones.clear();
 
 	curveEditorG->newLine();  //  ------------------------------------------------ second line
+	
+	lhshape = static_cast<FlatCurveEditor*>(curveEditorG->addCurve(CT_Flat, M("TP_LABCURVE_CURVEEDITOR_LH")));
+	lhshape->setTooltip(M("TP_LABCURVE_CURVEEDITOR_LH_TOOLTIP"));
+	lhshape->setCurveColorProvider(this, 4);
+
+
+	chshape = static_cast<FlatCurveEditor*>(curveEditorG->addCurve(CT_Flat, M("TP_LABCURVE_CURVEEDITOR_CH")));
+	chshape->setTooltip(M("TP_LABCURVE_CURVEEDITOR_CH_TOOLTIP"));
+	chshape->setCurveColorProvider(this, 1);
+
+
+	hhshape = static_cast<FlatCurveEditor*>(curveEditorG->addCurve(CT_Flat, M("TP_LABCURVE_CURVEEDITOR_HH")));
+	hhshape->setTooltip(M("TP_LABCURVE_CURVEEDITOR_HH_TOOLTIP"));
+	hhshape->setCurveColorProvider(this, 5);
+	
+	curveEditorG->newLine();  //  ------------------------------------------------ 3rd line
 
 	ccshape = static_cast<DiagonalCurveEditor*>(curveEditorG->addCurve(CT_Diagonal, M("TP_LABCURVE_CURVEEDITOR_CC")));
 	ccshape->setTooltip(M("TP_LABCURVE_CURVEEDITOR_CC_TOOLTIP"));
@@ -120,31 +132,44 @@ LCurve::LCurve () : Gtk::VBox(), FoldableToolPanel(this) {
 			M("TP_LABCURVE_CURVEEDITOR_CC_RANGE1"), M("TP_LABCURVE_CURVEEDITOR_CC_RANGE2"),
 			M("TP_LABCURVE_CURVEEDITOR_CC_RANGE3"), M("TP_LABCURVE_CURVEEDITOR_CC_RANGE4")
 	);
+	
 	ccshape->setBottomBarColorProvider(this, 2);
 	ccshape->setLeftBarColorProvider(this, 2);
 	ccshape->setRangeDefaultMilestones(0.05, 0.2, 0.58);
-
-	chshape = static_cast<FlatCurveEditor*>(curveEditorG->addCurve(CT_Flat, M("TP_LABCURVE_CURVEEDITOR_CH")));
-	chshape->setTooltip(M("TP_LABCURVE_CURVEEDITOR_CH_TOOLTIP"));
-	chshape->setCurveColorProvider(this, 1);
 	
 	lcshape = static_cast<DiagonalCurveEditor*>(curveEditorG->addCurve(CT_Diagonal, M("TP_LABCURVE_CURVEEDITOR_LC")));
 	lcshape->setTooltip(M("TP_LABCURVE_CURVEEDITOR_LC_TOOLTIP"));
 	// left and bottom bar uses the same caller id because the will display the same content
-	lcshape->setBottomBarColorProvider(this, 3);
+	lcshape->setBottomBarColorProvider(this, 2);
 	lcshape->setRangeLabels(
 			M("TP_LABCURVE_CURVEEDITOR_CC_RANGE1"), M("TP_LABCURVE_CURVEEDITOR_CC_RANGE2"),
 			M("TP_LABCURVE_CURVEEDITOR_CC_RANGE3"), M("TP_LABCURVE_CURVEEDITOR_CC_RANGE4")
 	);
-	lcshape->setRangeDefaultMilestones(0.15, 0.3, 0.6);
+	lcshape->setRangeDefaultMilestones(0.05, 0.2, 0.58);
 
+	clshape = static_cast<DiagonalCurveEditor*>(curveEditorG->addCurve(CT_Diagonal, M("TP_LABCURVE_CURVEEDITOR_CL")));
+	clshape->setTooltip(M("TP_LABCURVE_CURVEEDITOR_CL_TOOLTIP"));
+	clshape->setLeftBarColorProvider(this, 2);
+	clshape->setRangeDefaultMilestones(0.25, 0.5, 0.75);
+	milestones.push_back( GradientMilestone(0., 0., 0., 0.) );
+	milestones.push_back( GradientMilestone(1., 1., 1., 1.) );
+	
+	clshape->setBottomBarBgGradient(milestones);
+	
+	
 	// Setting the gradient milestones
 
 	// from black to white
 	milestones.push_back( GradientMilestone(0., 0., 0., 0.) );
 	milestones.push_back( GradientMilestone(1., 1., 1., 1.) );
-	lshape->setBottomBarBgGradient(milestones);
+	lshape->setBottomBarBgGradient(milestones);	
 	lshape->setLeftBarBgGradient(milestones);
+	milestones.push_back( GradientMilestone(0., 0., 0., 0.) );
+	milestones.push_back( GradientMilestone(1., 1., 1., 1.) );
+	lcshape->setRangeDefaultMilestones(0.05, 0.2, 0.58);
+	
+	lcshape->setBottomBarBgGradient(milestones);
+	
 	milestones.at(0).r = milestones.at(0).g = milestones.at(0).b = 0.1;
 	milestones.at(1).r = milestones.at(1).g = milestones.at(1).b = 0.8;
 	lcshape->setLeftBarBgGradient(milestones);
@@ -158,6 +183,8 @@ LCurve::LCurve () : Gtk::VBox(), FoldableToolPanel(this) {
 		milestones.push_back( GradientMilestone(double(x), double(R), double(G), double(B)) );
 	}
 	chshape->setBottomBarBgGradient(milestones);
+	lhshape->setBottomBarBgGradient(milestones);
+	hhshape->setBottomBarBgGradient(milestones);
 
 
 	// This will add the reset button at the end of the curveType buttons
@@ -174,8 +201,6 @@ LCurve::~LCurve () {
 void LCurve::read (const ProcParams* pp, const ParamsEdited* pedited) {
 
     disableListener ();
-	//	if(!pp->labCurve.cccurve.empty()) printf("plein"); else printf("vide");
-	//	if(pp->labCurve.cccurve[0] !=0) printf(" pp %i\n,pp->labCurve.cccurve[0] ");
 
     if (pedited) {
         brightness->setEditedState (pedited->labCurve.brightness ? Edited : UnEdited);
@@ -184,7 +209,6 @@ void LCurve::read (const ProcParams* pp, const ParamsEdited* pedited) {
 		
 		//%%%%%%%%%%%%%%%%%%%%%%
 		rstprotection->setEditedState (pedited->labCurve.rstprotection ? Edited : UnEdited);
-		bwtoning->set_inconsistent (!pedited->labCurve.bwtoning);
         avoidcolorshift->set_inconsistent (!pedited->labCurve.avoidcolorshift);
 		lcredsk->set_inconsistent (!pedited->labCurve.lcredsk);
 		
@@ -195,33 +219,23 @@ void LCurve::read (const ProcParams* pp, const ParamsEdited* pedited) {
         bshape->setUnChanged   (!pedited->labCurve.bcurve);
         ccshape->setUnChanged  (!pedited->labCurve.cccurve);
         chshape->setUnChanged  (!pedited->labCurve.chcurve);
+        lhshape->setUnChanged  (!pedited->labCurve.lhcurve);
+        hhshape->setUnChanged  (!pedited->labCurve.hhcurve);
         lcshape->setUnChanged  (!pedited->labCurve.lccurve);
-    }
-    else {
-        //if bwtoning is enabled, chromaticity value, avoid color shift and rstprotection has no effect
-        //ccshape->set_sensitive(!(!pp->labCurve.bwtoning));
-        //chshape->set_sensitive(!(!pp->labCurve.bwtoning));
-        chromaticity->set_sensitive(!pp->labCurve.bwtoning);
-        rstprotection->set_sensitive( !pp->labCurve.bwtoning /*&& pp->labCurve.chromaticity!=0*/ );//no reason for grey rstprotection
-        avoidcolorshift->set_sensitive(!pp->labCurve.bwtoning);
-        lcredsk->set_sensitive(!pp->labCurve.bwtoning);
-
-        std::vector<GradientMilestone> milestones;
-        lcshape->setBottomBarBgGradient(milestones);
-        lcshape->refresh();
+        clshape->setUnChanged  (!pedited->labCurve.clcurve);
     }
 
     brightness->setValue    (pp->labCurve.brightness);
     contrast->setValue      (pp->labCurve.contrast);
-	chromaticity->setValue  (pp->labCurve.chromaticity);
-	
+    chromaticity->setValue  (pp->labCurve.chromaticity);
+    adjusterChanged(chromaticity, pp->labCurve.chromaticity); // To update the GUI sensitiveness
+
 	//%%%%%%%%%%%%%%%%%%%%%%
 	rstprotection->setValue (pp->labCurve.rstprotection);
 
     bwtconn.block (true);
     acconn.block (true);
 	lcconn.block (true);
-    bwtoning->set_active (pp->labCurve.bwtoning);
     avoidcolorshift->set_active (pp->labCurve.avoidcolorshift);
     lcredsk->set_active (pp->labCurve.lcredsk);
 	
@@ -229,7 +243,6 @@ void LCurve::read (const ProcParams* pp, const ParamsEdited* pedited) {
     acconn.block (false);
 	lcconn.block (false);
 	
-    lastBWTVal = pp->labCurve.bwtoning;
     lastACVal = pp->labCurve.avoidcolorshift;
 	lastLCVal = pp->labCurve.lcredsk;
 	//%%%%%%%%%%%%%%%%%%%%%%
@@ -239,7 +252,10 @@ void LCurve::read (const ProcParams* pp, const ParamsEdited* pedited) {
     bshape->setCurve   (pp->labCurve.bcurve);
     ccshape->setCurve  (pp->labCurve.cccurve);
     chshape->setCurve  (pp->labCurve.chcurve);
+    lhshape->setCurve  (pp->labCurve.lhcurve);
+    hhshape->setCurve  (pp->labCurve.hhcurve);
     lcshape->setCurve  (pp->labCurve.lccurve);
+    clshape->setCurve  (pp->labCurve.clcurve);
 
     queue_draw();
 
@@ -253,8 +269,10 @@ void LCurve::autoOpenCurve () {
     if (!active) bshape->openIfNonlinear();
     if (!active) ccshape->openIfNonlinear();
     if (!active) chshape->openIfNonlinear();
+    if (!active) lhshape->openIfNonlinear();
+    if (!active) hhshape->openIfNonlinear();
     if (!active) lcshape->openIfNonlinear();
-	
+    if (!active) clshape->openIfNonlinear();
 }
 
 void LCurve::write (ProcParams* pp, ParamsEdited* pedited) {
@@ -264,19 +282,21 @@ void LCurve::write (ProcParams* pp, ParamsEdited* pedited) {
     pp->labCurve.chromaticity  = (int)chromaticity->getValue ();
 
     //%%%%%%%%%%%%%%%%%%%%%%
-    pp->labCurve.bwtoning        = bwtoning->get_active ();
     pp->labCurve.avoidcolorshift = avoidcolorshift->get_active ();
     pp->labCurve.lcredsk         = lcredsk->get_active ();
 
     pp->labCurve.rstprotection   = rstprotection->getValue ();
-	//%%%%%%%%%%%%%%%%%%%%%%
+    //%%%%%%%%%%%%%%%%%%%%%%
 
     pp->labCurve.lcurve  = lshape->getCurve ();
     pp->labCurve.acurve  = ashape->getCurve ();
     pp->labCurve.bcurve  = bshape->getCurve ();
     pp->labCurve.cccurve = ccshape->getCurve ();
     pp->labCurve.chcurve = chshape->getCurve ();
+    pp->labCurve.lhcurve = lhshape->getCurve ();
+    pp->labCurve.hhcurve = hhshape->getCurve ();
     pp->labCurve.lccurve = lcshape->getCurve ();
+    pp->labCurve.clcurve = clshape->getCurve ();
 
     if (pedited) {
         pedited->labCurve.brightness   = brightness->getEditedState ();
@@ -284,7 +304,6 @@ void LCurve::write (ProcParams* pp, ParamsEdited* pedited) {
         pedited->labCurve.chromaticity = chromaticity->getEditedState ();
 
         //%%%%%%%%%%%%%%%%%%%%%%
-        pedited->labCurve.bwtoning        = !bwtoning->get_inconsistent();
         pedited->labCurve.avoidcolorshift = !avoidcolorshift->get_inconsistent();
         pedited->labCurve.lcredsk         = !lcredsk->get_inconsistent();
 
@@ -296,7 +315,10 @@ void LCurve::write (ProcParams* pp, ParamsEdited* pedited) {
         pedited->labCurve.bcurve    = !bshape->isUnChanged ();
         pedited->labCurve.cccurve   = !ccshape->isUnChanged ();
         pedited->labCurve.chcurve   = !chshape->isUnChanged ();
+        pedited->labCurve.lhcurve   = !lhshape->isUnChanged ();
+        pedited->labCurve.hhcurve   = !hhshape->isUnChanged ();
         pedited->labCurve.lccurve   = !lcshape->isUnChanged ();
+        pedited->labCurve.clcurve   = !clshape->isUnChanged ();
     }
 }
 
@@ -304,7 +326,7 @@ void LCurve::setDefaults (const ProcParams* defParams, const ParamsEdited* pedit
 
     brightness->setDefault (defParams->labCurve.brightness);
     contrast->setDefault (defParams->labCurve.contrast);
-	chromaticity->setDefault (defParams->labCurve.chromaticity);
+    chromaticity->setDefault (defParams->labCurve.chromaticity);
     rstprotection->setDefault (defParams->labCurve.rstprotection);
 
     if (pedited) {
@@ -373,41 +395,6 @@ void LCurve::lcredsk_toggled () {
     }
 }
 
-
-//%%%%%%%%%%%%%%%%%%%%%%
-//BW toning control changed
-void LCurve::bwtoning_toggled () {
-
-    if (batchMode) {
-        if (bwtoning->get_inconsistent()) {
-        	bwtoning->set_inconsistent (false);
-        	bwtconn.block (true);
-            bwtoning->set_active (false);
-            bwtconn.block (false);
-        }
-        else if (lastBWTVal)
-        	bwtoning->set_inconsistent (true);
-
-        lastBWTVal = bwtoning->get_active ();
-    }
-    else {
-        //ccshape->set_sensitive(!(!pp->labCurve.bwtoning));
-        //chshape->set_sensitive(!(!pp->labCurve.bwtoning));
-        chromaticity->set_sensitive( !(bwtoning->get_active ()) );
-        rstprotection->set_sensitive( !(bwtoning->get_active ()) /*&& chromaticity->getIntValue()!=0*/);
-        avoidcolorshift->set_sensitive( !(bwtoning->get_active ()) );
-        lcredsk->set_sensitive( !(bwtoning->get_active ()) );
-		
-    }
-
-    if (listener) {
-    	if (bwtoning->get_active ())
-        	listener->panelChanged (EvLBWtoning, M("GENERAL_ENABLED"));
-        else
-            listener->panelChanged (EvLBWtoning, M("GENERAL_DISABLED"));
-    }
-}
-
 //%%%%%%%%%%%%%%%%%%%%%%
 
 /*
@@ -429,36 +416,51 @@ void LCurve::curveChanged (CurveEditor* ce) {
            listener->panelChanged (EvLCCCurve, M("HISTORY_CUSTOMCURVE"));
         if (ce == chshape)
             listener->panelChanged (EvLCHCurve, M("HISTORY_CUSTOMCURVE"));
+        if (ce == lhshape)
+            listener->panelChanged (EvLLHCurve, M("HISTORY_CUSTOMCURVE"));
+        if (ce == hhshape)
+            listener->panelChanged (EvLHHCurve, M("HISTORY_CUSTOMCURVE"));
         if (ce == lcshape)
             listener->panelChanged (EvLLCCurve, M("HISTORY_CUSTOMCURVE"));
+        if (ce == clshape)
+            listener->panelChanged (EvLCLCurve, M("HISTORY_CUSTOMCURVE"));
     }
 }
 
 void LCurve::adjusterChanged (Adjuster* a, double newval) {
 
-    if (!listener)
-        return;
-
     Glib::ustring costr;
     if (a==brightness)
         costr = Glib::ustring::format (std::setw(3), std::fixed, std::setprecision(2), a->getValue());
-	else if (a==rstprotection)
+    else if (a==rstprotection)
         costr = Glib::ustring::format (std::setw(3), std::fixed, std::setprecision(1), a->getValue());
     else
         costr = Glib::ustring::format ((int)a->getValue());
 
-    if (a==brightness)
-        listener->panelChanged (EvLBrightness, costr);
-    else if (a==contrast)
-        listener->panelChanged (EvLContrast, costr);
-	else if (a==chromaticity) {
-		if (!batchMode) {
-	        rstprotection->set_sensitive( !(bwtoning->get_active ())/* && chromaticity->getIntValue()!=0*/ );
-		}
-        listener->panelChanged (EvLSaturation, costr);
-	}
-	else if (a==rstprotection)
-        listener->panelChanged (EvLRSTProtection, costr);
+    if (a==brightness) {
+        if (listener) listener->panelChanged (EvLBrightness, costr);
+    }
+    else if (a==contrast) {
+        if (listener) listener->panelChanged (EvLContrast, costr);
+    }
+    else if (a==rstprotection) {
+        if (listener) listener->panelChanged (EvLRSTProtection, costr);
+    }
+    else if (a==chromaticity) {
+        if (multiImage) {
+            //if chromaticity==-100 (lowest value), we enter the B&W mode and avoid color shift and rstprotection has no effect
+            rstprotection->set_sensitive( true );
+            avoidcolorshift->set_sensitive( true );
+            lcredsk->set_sensitive( true );
+        }
+        else {
+            //if chromaticity==-100 (lowest value), we enter the B&W mode and avoid color shift and rstprotection has no effect
+            rstprotection->set_sensitive( int(newval)> -100 );//no reason for grey rstprotection
+            avoidcolorshift->set_sensitive( int(newval)> -100 );
+            lcredsk->set_sensitive( int(newval)> -100 );
+        }
+        if (listener) listener->panelChanged (EvLSaturation, costr);
+    }
 }
 
 void LCurve::colorForValue (double valX, double valY, int callerId, ColorCaller *caller) {
@@ -493,6 +495,17 @@ void LCurve::colorForValue (double valX, double valY, int callerId, ColorCaller
 			Color::hsv2rgb01(float(valY), float(valX), value, R, G, B);
 		}
 	}
+	else if (callerId == 4) {    // LH - bottom bar
+		Color::hsv2rgb01(float(valX), 0.5f, float(valY), R, G, B);
+	}
+	else if (callerId == 5) {    // HH - bottom bar
+		float h = float((valY - 0.5) * 0.3 + valX);
+		if (h > 1.0f)
+			h -= 1.0f;
+		else if (h < 0.0f)
+			h += 1.0f;
+		Color::hsv2rgb01(h, 0.5f, 0.5f, R, G, B);
+	}
 	caller->ccRed = double(R);
 	caller->ccGreen = double(G);
 	caller->ccBlue = double(B);
@@ -512,10 +525,12 @@ void LCurve::setBatchMode (bool batchMode) {
 }
 
 
-void LCurve::updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histLCAM,  LUTu & histCCAM, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma){
+void LCurve::updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histCLurve, LUTu & histLLCurve, LUTu & histLCAM,  LUTu & histCCAM, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma){
 
 	lshape->updateBackgroundHistogram (histLCurve);
 	ccshape->updateBackgroundHistogram (histCCurve);
+	clshape->updateBackgroundHistogram (histCLurve);
+	lcshape->updateBackgroundHistogram (histLLCurve);
 	
 }
 
