@@ -39,15 +39,18 @@ class LCurve : public Gtk::VBox, public AdjusterListener, public FoldableToolPan
     DiagonalCurveEditor* ccshape;
     DiagonalCurveEditor* lcshape;
     FlatCurveEditor*   chshape;
+    FlatCurveEditor*   lhshape;
+    FlatCurveEditor*   hhshape;
+	
+    DiagonalCurveEditor* clshape;
 
     //%%%%%%%%%%%%%%%%
     Gtk::CheckButton* avoidcolorshift;
-    Gtk::CheckButton* bwtoning;
     Gtk::CheckButton* lcredsk;
 	
     Adjuster* rstprotection;
     sigc::connection  bwtconn, acconn, lcconn;
-    bool lastBWTVal, lastACVal, lastLCVal;
+    bool lastACVal, lastLCVal;
 	
     //%%%%%%%%%%%%%%%%
 
@@ -67,10 +70,9 @@ class LCurve : public Gtk::VBox, public AdjusterListener, public FoldableToolPan
     void curveChanged (CurveEditor* ce);
     void adjusterChanged (Adjuster* a, double newval);
 	void avoidcolorshift_toggled ();
-    void bwtoning_toggled();
     void lcredsk_toggled();
 	
-    void updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histLCAM,  LUTu & histCCAM, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma);
+    void updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histCLurve,  LUTu & histLLCurve, LUTu & histLCAM,  LUTu & histCCAM, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma);
 
     virtual void colorForValue (double valX, double valY, int callerId, ColorCaller* caller);
 };
