@@ -38,7 +38,13 @@ MyCurve::MyCurve () : listener(NULL) {
     curveIsDirty = true;
 
     set_extension_events(Gdk::EXTENSION_EVENTS_ALL);
+#if defined (__APPLE__)
+    // Workaround: disabling POINTER_MOTION_HINT_MASK as for gtk 2.24.22 the get_pointer() function is buggy for quartz and modifier mask is not updated correctly.
+    // This workaround should be removed when bug is fixed in GTK2 or when migrating to GTK3
+    add_events(Gdk::EXPOSURE_MASK |	Gdk::POINTER_MOTION_MASK |	Gdk::ENTER_NOTIFY_MASK | Gdk::LEAVE_NOTIFY_MASK | Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK | Gdk::BUTTON1_MOTION_MASK);
+#else
     add_events(Gdk::EXPOSURE_MASK |	Gdk::POINTER_MOTION_MASK |	Gdk::POINTER_MOTION_HINT_MASK |	Gdk::ENTER_NOTIFY_MASK | Gdk::LEAVE_NOTIFY_MASK | Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK | Gdk::BUTTON1_MOTION_MASK);
+#endif
     signal_style_changed().connect( sigc::mem_fun(*this, &MyCurve::styleChanged) );
 
     mcih = new MyCurveIdleHelper;
@@ -83,14 +89,27 @@ void MyCurve::notifyListener () {
         listener->curveChanged ();
 }
 
-bool MyCurve::snapCoordinate(double testedVal, double realVal) {
+bool MyCurve::snapCoordinateX(double testedVal, double realVal) {
 
-	double distY = realVal - testedVal;
+	double dist = realVal - testedVal;
 
-	if (distY < 0.) distY = -distY;
-	if (distY < snapToMinDist) {
-		snapToMinDist = distY;
-		snapToVal = testedVal;
+	if (dist < 0.) dist = -dist;
+	if (dist < snapToMinDistX) {
+		snapToMinDistX = dist;
+		snapToValX = testedVal;
+		return true;
+	}
+	return false;
+}
+
+bool MyCurve::snapCoordinateY(double testedVal, double realVal) {
+
+	double dist = realVal - testedVal;
+
+	if (dist < 0.) dist = -dist;
+	if (dist < snapToMinDistY) {
+		snapToMinDistY = dist;
+		snapToValY = testedVal;
 		return true;
 	}
 	return false;
