@@ -77,7 +77,7 @@ class MyDiagonalCurve : public MyCurve {
 		void setType (DiagonalCurveType t);
 		bool handleEvents (GdkEvent* event);
 		void setActiveParam (int ac);
-		void reset ();
+		void reset (double identityValue=0.5);
 		void updateBackgroundHistogram (LUTu & hist);
 };
 
