@@ -88,19 +88,17 @@ void Options::updatePaths() {
     if (Glib::path_is_absolute(profilePath)) {
         // absolute path
         if (!checkDirPath (profilePath, "")) {
-            int retVal = safe_g_mkdir_with_parents (profilePath, 511);
-            if (!retVal)
+            safe_g_mkdir_with_parents (profilePath, 511);
+            if (!checkDirPath (profilePath, "")) // had problems with mkdir_with_parents return value on OS X, just check dir again
             	printf("Error: user's profiles' directory \"%s\" creation failed\n", profilePath.c_str());
         }
         if (checkDirPath (profilePath, "Error: the specified user's profiles' path doesn't point to a directory or doesn't exist!\n")) {
             if (multiUser) {
                 userProfilePath = profilePath;
-                if (useBundledProfiles) {
-                    tmpPath = Glib::build_filename(argv0, "profiles");
-                    if(checkDirPath (tmpPath, "Error: the global's profiles' path doesn't point to a directory or doesn't exist!\n")) {
-                        if (userProfilePath != tmpPath)
-                            globalProfilePath = tmpPath;
-                    }
+                tmpPath = Glib::build_filename(argv0, "profiles");
+                if(checkDirPath (tmpPath, "Error: the global's profiles' path doesn't point to a directory or doesn't exist!\n")) {
+                    if (userProfilePath != tmpPath)
+                        globalProfilePath = tmpPath;
                 }
             }
             else {
@@ -119,18 +117,16 @@ void Options::updatePaths() {
         if (multiUser) {
             tmpPath = Glib::build_filename(rtdir, profilePath);
             if (!checkDirPath (tmpPath, "")) {
-                int retVal = safe_g_mkdir_with_parents (tmpPath, 511);
-                if (!retVal)
-                	printf("Error: user's profiles' directory \"%s\" creation failed\n", tmpPath.c_str());
+                safe_g_mkdir_with_parents (tmpPath, 511);
+                if (!checkDirPath (tmpPath, ""))
+                    printf("Error: user's profiles' directory \"%s\" creation failed\n", tmpPath.c_str());
             }
             if(checkDirPath (tmpPath, "Error: the specified user's profiles' path doesn't point to a directory!\n")) {
                	userProfilePath = tmpPath;
             }
-            if (useBundledProfiles) {
-                tmpPath = Glib::build_filename(argv0, "profiles");
-                if(checkDirPath (tmpPath, "Error: the specified user's profiles' path doesn't point to a directory or doesn't exist!\n")) {
-                    globalProfilePath = tmpPath;
-                }
+            tmpPath = Glib::build_filename(argv0, "profiles");
+            if(checkDirPath (tmpPath, "Error: the specified user's profiles' path doesn't point to a directory or doesn't exist!\n")) {
+                globalProfilePath = tmpPath;
             }
         }
         else {
@@ -161,6 +157,9 @@ void Options::updatePaths() {
         lastVibranceCurvesDir = preferredPath;
     if (loadSaveProfilePath.empty() || !safe_file_test (loadSaveProfilePath, Glib::FILE_TEST_EXISTS) || !safe_file_test (loadSaveProfilePath, Glib::FILE_TEST_IS_DIR))
         loadSaveProfilePath = preferredPath;
+    if (lastBWCurvesDir.empty() || !safe_file_test (lastBWCurvesDir, Glib::FILE_TEST_EXISTS) || !safe_file_test (lastBWCurvesDir, Glib::FILE_TEST_IS_DIR))
+        lastBWCurvesDir = preferredPath;
+		
 }
 
 Glib::ustring Options::getPreferredProfilePath() {
@@ -172,32 +171,64 @@ Glib::ustring Options::getPreferredProfilePath() {
         return "";
 }
 
+/** @brief Get the absolute path of the given filename or the "Neutral" special value
+  *
+  *@param profName  path + filename of the procparam to look for. A filename without path can be provided for backward compatibility.
+  *                 In this case, this parameter will be update with the new format.
+  *@return Send back the absolute path of the given filename or "Neutral" if "Neutral" has been set to profName. Implementor will have
+  *        to test for this particular value. If the absolute path is invalid (e.g. the file doesn't exist), it will return an empty string.
+  */
 Glib::ustring Options::findProfilePath(Glib::ustring &profName) {
     if (profName.empty())
         return "";
 
-    Glib::ustring p = getUserProfilePath();
-    Glib::ustring fullPath = Glib::build_filename(p, profName + paramFileExtension);
-    if (!p.empty() && safe_file_test (fullPath, Glib::FILE_TEST_EXISTS))
-        return p;
+    if (profName == DEFPROFILE_INTERNAL)
+        return profName;
 
-    p = getGlobalProfilePath();
-    fullPath = Glib::build_filename(p, profName + paramFileExtension);
-    if (!p.empty() && safe_file_test (fullPath, Glib::FILE_TEST_EXISTS))
-        return p;
-    else
-    	return "";
+    Glib::ustring p = profName.substr(0, 4);
+    if (p=="${U}") {
+        // the path starts by the User virtual path
+        p = getUserProfilePath();
+        Glib::ustring fullPath = Glib::build_filename(p, profName.substr(5) + paramFileExtension);
+        if (!p.empty() && safe_file_test (fullPath, Glib::FILE_TEST_EXISTS))
+            return Glib::path_get_dirname(fullPath);
+    }
+    else if (p=="${G}") {
+        // the path starts by the User virtual path
+        p = getGlobalProfilePath();
+        Glib::ustring fullPath = Glib::build_filename(p, profName.substr(5) + paramFileExtension);
+        if (!p.empty() && safe_file_test (fullPath, Glib::FILE_TEST_EXISTS))
+            return Glib::path_get_dirname(fullPath);
+    }
+    else {
+        // compatibility case -> convert the path to the new format
+        p = getUserProfilePath();
+        Glib::ustring fullPath = Glib::build_filename(p, profName + paramFileExtension);
+        if (!p.empty() && safe_file_test (fullPath, Glib::FILE_TEST_EXISTS)) {
+            // update the profile path
+            profName = Glib::build_filename("${U}", profName);
+            return Glib::path_get_dirname(fullPath);
+        }
+
+        p = getGlobalProfilePath();
+        fullPath = Glib::build_filename(p, profName + paramFileExtension);
+        if (!p.empty() && safe_file_test (fullPath, Glib::FILE_TEST_EXISTS)) {
+            profName = Glib::build_filename("${G}", profName);
+            return Glib::path_get_dirname(fullPath);
+        }
+    }
+    return "";
 
 }
 
 void Options::setDefaults () {
 
     font = "sans, 8";
-    windowWidth = 900;
-    windowHeight = 560;
-    windowMaximized = false;
-    saveAsDialogWidth = 600;
-    saveAsDialogHeight = 600;
+    windowWidth = 1200;
+    windowHeight = 680;
+    windowMaximized = true;
+    saveAsDialogWidth = 920;
+    saveAsDialogHeight = 680;
     savesParamsAtExit = true;
     saveFormat.format = "jpg";
     saveFormat.jpegQuality = 90;
@@ -213,7 +244,7 @@ void Options::setDefaults () {
     saveFormatBatch.jpegSubSamp = 2;
     saveFormatBatch.pngCompression = 6;
     saveFormatBatch.pngBits = 8;
-    saveFormatBatch.tiffBits = 8;
+    saveFormatBatch.tiffBits = 16;
     saveFormatBatch.tiffUncompressed = true;
     saveFormatBatch.saveParams = true;
 
@@ -227,21 +258,21 @@ void Options::setDefaults () {
     startupDir = STARTUPDIR_LAST;
     startupPath = "";
     useBundledProfiles = true;
-    dirBrowserWidth = 200;
-    dirBrowserHeight = 150;
-    preferencesWidth = 0;
+    dirBrowserWidth = 260;
+    dirBrowserHeight = 350;
+    preferencesWidth = 800;
     preferencesHeight = 0;
-    toolPanelWidth = 300;
-    browserToolPanelWidth = 300;
-    browserToolPanelHeight = 300;
-    historyPanelWidth = 230;
+    toolPanelWidth = 390;
+    browserToolPanelWidth = 430;
+    browserToolPanelHeight = 600;
+    historyPanelWidth = 330;
     lastScale = 5;
     panAccelFactor = 5;
     lastCropSize = 1;
     fbOnlyRaw = false;
     fbShowDateTime = true;
     fbShowBasicExif = true;
-    fbShowExpComp = true;
+    fbShowExpComp = false;
     fbShowHidden = false;
     fbArrangement = 2;					// was 0
     multiUser = true;
@@ -249,7 +280,9 @@ void Options::setDefaults () {
     loadSaveProfilePath = "";			// will be corrected in load as otherwise construction fails
     version = "0.0.0.0";				// temporary value; will be correctly set in RTWindow::on_realize
     thumbSize = 240;
-    thumbSizeTab = 80;
+    thumbSizeTab = 180;
+    thumbSizeQueue = 160;
+    sameThumbSize = false;				// preferring speed of switch between file browser and single editor tab
     showHistory = true;
     showFilePanelState = 0;				// Not used anymore ; was the thumb strip state
     showInfo = true;
@@ -270,8 +303,8 @@ void Options::setDefaults () {
     maxThumbnailHeight = 400;
     maxCacheEntries = 20000;
     thumbInterp = 1;
-    autoSuffix = false;
-    forceFormatOpts = false;
+    autoSuffix = true;
+    forceFormatOpts = true;
     saveMethodNum = 0;				// 0->immediate, 1->putToQueuHead, 2->putToQueueTail
     saveParamsFile = true;				// was false, but saving the procparams files next to the file make more sense when reorganizing file tree than in a cache
     saveParamsCache = false;			// there's no need to save the procparams files in a cache if saveParamsFile is true
@@ -297,19 +330,20 @@ void Options::setDefaults () {
     thumbnailZoomRatios.push_back (0.6);
     thumbnailZoomRatios.push_back (0.8);
     thumbnailZoomRatios.push_back (1.0);
-    overlayedFileNames = true;
+    overlayedFileNames = false;
     internalThumbIfUntouched = true; 	// if TRUE, only fast, internal preview images are taken if the image is not edited yet
     showFileNames = true;
-    tabbedUI = true;
+    tabbedUI = false;
+    mainNBVertical = true;
     multiDisplayMode = 0;
-    tunnelMetaData = false;
-    histogramPosition = 2;
+    tunnelMetaData = true;
+    histogramPosition = 1;
     histogramBar = true;
     histogramFullMode = false;
 
     rgbDenoiseThreadLimit = 0;
 
-    filledProfile = false;
+    filledProfile = true;
 
     showProfileSelector = true;
     FileBrowserToolbarSingleRow = false;
@@ -344,7 +378,7 @@ void Options::setDefaults () {
     fastexport_bypass_raw_ff             = true;
     fastexport_raw_dmethod               = "fast";
     fastexport_icm_input                 = "(camera)";
-    fastexport_icm_working               = "sRGB";
+    fastexport_icm_working               = "ProPhoto";
     fastexport_icm_output                = "RT_sRGB";
     fastexport_icm_gamma                 = "default";
     fastexport_resize_enabled            = true;
@@ -352,8 +386,8 @@ void Options::setDefaults () {
     fastexport_resize_appliesTo          = "Cropped area";
     fastexport_resize_method             = "Lanczos";
     fastexport_resize_dataspec           = 3;
-    fastexport_resize_width              = 1000;
-    fastexport_resize_height             = 1000;
+    fastexport_resize_width              = 900;
+    fastexport_resize_height             = 900;
 
     cutOverlayBrush = std::vector<double> (4);
     cutOverlayBrush[3] = 0.667;  // :-p
@@ -393,13 +427,15 @@ void Options::setDefaults () {
 			0,  // ADDSET_TC_HLCOMPTHRESH
 			0,  // ADDSET_TC_SHCOMP
 			0,  // ADDSET_DIRPYREQ
-			0,  // ADDSET_DIRPYRDN_LUMA	
-			0,  // ADDSET_DIRPYRDN_LUDET			
+			0,  // ADDSET_DIRPYRDN_LUMA
+			0,  // ADDSET_DIRPYRDN_LUDET
 			0,  // ADDSET_DIRPYRDN_CHROMA
 			0,  // ADDSET_DIRPYRDN_CHROMARED
-			0,  // ADDSET_DIRPYRDN_CHROMABLUE			
+			0,  // ADDSET_DIRPYRDN_CHROMABLUE
 			0,  // ADDSET_DIRPYRDN_GAMMA
 			0,  // ADDSET_CHMIXER
+			0,  // ADDSET_BLACKWHITE_HUES
+			0,  // ADDSET_BLACKWHITE_GAMMA
 			0,  // ADDSET_PREPROCESS_GREENEQUIL
 			0,  // ADDSET_PREPROCESS_LINEDENOISE
 			0,  // ADDSET_RAWCACORR
@@ -421,6 +457,14 @@ void Options::setDefaults () {
 			0,  // ADDSET_CAT_JLIGHT
 			0,  // ADDSET_CAT_CHROMA
  			0,  // ADDSET_CAT_CONTRAST
+			0,  // ADDSET_WB_EQUAL
+			0,  // ADDSET_GRADIENT_DEGREE
+			0,  // ADDSET_GRADIENT_FEATHER
+			0,  // ADDSET_GRADIENT_STRENGTH
+			0,  // ADDSET_GRADIENT_CENTER
+			0,  // ADDSET_PCVIGNETTE_STRENGTH
+			0,  // ADDSET_PCVIGNETTE_FEATHER
+			0,  // ADDSET_PCVIGNETTE_ROUNDNESS
 
 	};
     baBehav = std::vector<int> (babehav, babehav+ADDSET_PARAM_NUM);
@@ -456,6 +500,8 @@ void Options::setDefaults () {
     rtSettings.verbose = false;
     rtSettings.gamutICC = true;
     rtSettings.gamutLch = true;
+    rtSettings.amchroma = 40;//between 20 and 140   low values increase effect..and also artefacts, high values reduces
+	
     rtSettings.ciecamfloat = true;
     rtSettings.protectred = 60;
     rtSettings.protectredh = 0.3;
@@ -466,7 +512,7 @@ void Options::setDefaults () {
 	lastIccDir = rtSettings.iccDirectory;
 	lastDarkframeDir = rtSettings.darkFramesPath;
 	lastFlatfieldDir = rtSettings.flatFieldsPath;
-
+//	rtSettings.bw_complementary = true;
 	// There is no reasonable default for curves. We can still suppose that they will take place
 	// in a subdirectory of the user's own ProcParams presets, i.e. in a subdirectory
 	// of the one pointed to by the "profile" field.
@@ -479,6 +525,8 @@ void Options::setDefaults () {
 	lastToneCurvesDir = "";
 	lastVibranceCurvesDir = "";
 	lastProfilingReferenceDir = "";
+	lastBWCurvesDir = "";
+	
 }
 
 Options* Options::copyFrom (Options* other) {
@@ -496,22 +544,20 @@ int Options::readFromFile (Glib::ustring fname) {
 
     rtengine::SafeKeyFile keyFile;
 
-    try {
-    	if( !safe_file_test(fname,Glib::FILE_TEST_EXISTS))
-    		return 1;
-        if (!keyFile.load_from_file (fname)) 
-            return 1;
-    }
-    catch (Glib::FileError &err) {
+    if( !safe_file_test(fname,Glib::FILE_TEST_EXISTS))
         return 1;
-    }
 
-    setDefaults ();
+    try {
+        if (keyFile.load_from_file (fname)) {
+
+            setDefaults ();
     
+// --------------------------------------------------------------------------------------------------------
+
 if (keyFile.has_group ("General")) {
     if (keyFile.has_key ("General", "TabbedEditor"))    tabbedUI= keyFile.get_boolean ("General", "TabbedEditor");    
     if (keyFile.has_key ("General", "StartupDirectory")){
-    	if( keyFile.get_string ("General", "StartupDirectory") == "home")          startupDir = STARTUPDIR_HOME;
+        if      ( keyFile.get_string ("General", "StartupDirectory") == "home")    startupDir = STARTUPDIR_HOME;
         else if ( keyFile.get_string ("General", "StartupDirectory") == "current") startupDir = STARTUPDIR_CURRENT;
         else if ( keyFile.get_string ("General", "StartupDirectory") == "last")    startupDir = STARTUPDIR_LAST;
         else if ( keyFile.get_string ("General", "StartupDirectory") == "custom")  startupDir = STARTUPDIR_CUSTOM;
@@ -573,21 +619,25 @@ if (keyFile.has_group ("Output")) {
 }
 
 if (keyFile.has_group ("Profiles")) { 
-    if (keyFile.has_key ("Profiles", "Directory"))              profilePath          = keyFile.get_string  ("Profiles", "Directory");
-    if (keyFile.has_key ("Profiles", "UseBundledProfiles"))     useBundledProfiles   = keyFile.get_boolean ("Profiles", "UseBundledProfiles");
-    if (keyFile.has_key ("Profiles", "LoadSaveProfilePath"))    loadSaveProfilePath  = keyFile.get_string  ("Profiles", "LoadSaveProfilePath");
-    if (keyFile.has_key ("Profiles", "RawDefault"))             defProfRaw           = keyFile.get_string  ("Profiles", "RawDefault");
-    if (keyFile.has_key ("Profiles", "ImgDefault"))             defProfImg           = keyFile.get_string  ("Profiles", "ImgDefault");
-    if (keyFile.has_key ("Profiles", "FilledProfile"))          filledProfile        = keyFile.get_boolean ("Profiles", "FilledProfile");
-    if (keyFile.has_key ("Profiles", "SaveParamsWithFile"))     saveParamsFile       = keyFile.get_boolean ("Profiles", "SaveParamsWithFile");
-    if (keyFile.has_key ("Profiles", "SaveParamsToCache"))      saveParamsCache      = keyFile.get_boolean ("Profiles", "SaveParamsToCache");
-    if (keyFile.has_key ("Profiles", "LoadParamsFromLocation")) paramsLoadLocation   = (PPLoadLocation)keyFile.get_integer ("Profiles", "LoadParamsFromLocation");
-    if (keyFile.has_key ("Profiles", "CustomProfileBuilder"))   customProfileBuilder = keyFile.get_string  ("Profiles", "CustomProfileBuilder");
+    if (keyFile.has_key ("Profiles", "Directory"))                profilePath          = keyFile.get_string  ("Profiles", "Directory");
+    if (keyFile.has_key ("Profiles", "UseBundledProfiles"))       useBundledProfiles   = keyFile.get_boolean ("Profiles", "UseBundledProfiles");
+    if (keyFile.has_key ("Profiles", "LoadSaveProfilePath"))      loadSaveProfilePath  = keyFile.get_string  ("Profiles", "LoadSaveProfilePath");
+    if (keyFile.has_key ("Profiles", "RawDefault"))               defProfRaw           = keyFile.get_string  ("Profiles", "RawDefault");
+    if (keyFile.has_key ("Profiles", "ImgDefault"))               defProfImg           = keyFile.get_string  ("Profiles", "ImgDefault");
+    if (keyFile.has_key ("Profiles", "FilledProfile"))            filledProfile        = keyFile.get_boolean ("Profiles", "FilledProfile");
+    if (keyFile.has_key ("Profiles", "SaveParamsWithFile"))       saveParamsFile       = keyFile.get_boolean ("Profiles", "SaveParamsWithFile");
+    if (keyFile.has_key ("Profiles", "SaveParamsToCache"))        saveParamsCache      = keyFile.get_boolean ("Profiles", "SaveParamsToCache");
+    if (keyFile.has_key ("Profiles", "LoadParamsFromLocation"))   paramsLoadLocation   = (PPLoadLocation)keyFile.get_integer ("Profiles", "LoadParamsFromLocation");
+    if (keyFile.has_key ("Profiles", "CustomProfileBuilder"))     CPBPath              = keyFile.get_string  ("Profiles", "CustomProfileBuilder"); // for backward compatibility only
+    if (keyFile.has_key ("Profiles", "CustomProfileBuilderPath")) CPBPath              = keyFile.get_string  ("Profiles", "CustomProfileBuilderPath");
+    if (keyFile.has_key ("Profiles", "CustomProfileBuilderKeys")) CPBKeys              = (CPBKeyType)keyFile.get_integer ("Profiles", "CustomProfileBuilderKeys");
 }
 
 if (keyFile.has_group ("File Browser")) { 
     if (keyFile.has_key ("File Browser", "ThumbnailSize"))      thumbSize          = keyFile.get_integer ("File Browser", "ThumbnailSize");
     if (keyFile.has_key ("File Browser", "ThumbnailSizeTab"))   thumbSizeTab       = keyFile.get_integer ("File Browser", "ThumbnailSizeTab");
+    if (keyFile.has_key ("File Browser", "ThumbnailSizeQueue")) thumbSizeQueue     = keyFile.get_integer ("File Browser", "ThumbnailSizeQueue");
+    if (keyFile.has_key ("File Browser", "SameThumbSize"))      sameThumbSize      = keyFile.get_integer ("File Browser", "SameThumbSize");
     if (keyFile.has_key ("File Browser", "BrowseOnlyRaw"))      fbOnlyRaw          = keyFile.get_boolean ("File Browser", "BrowseOnlyRaw");
     if (keyFile.has_key ("File Browser", "BrowserShowsDate"))   fbShowDateTime     = keyFile.get_boolean ("File Browser", "BrowserShowsDate");
     if (keyFile.has_key ("File Browser", "BrowserShowsExif"))   fbShowBasicExif    = keyFile.get_boolean ("File Browser", "BrowserShowsExif");
@@ -664,8 +714,6 @@ if (keyFile.has_group ("GUI")) {
     if (keyFile.has_key ("GUI", "UseIconNoText"))    UseIconNoText    = keyFile.get_boolean ("GUI", "UseIconNoText");
 }
 
-
-
 if (keyFile.has_group ("Crop Settings")) { 
     if (keyFile.has_key ("Crop Settings", "PPI"))       cropPPI      = keyFile.get_integer ("Crop Settings", "PPI");
 }
@@ -675,7 +723,6 @@ if (keyFile.has_group ("Color Management")) {
     if (keyFile.has_key ("Color Management", "MonitorProfile")) rtSettings.monitorProfile       = keyFile.get_string ("Color Management", "MonitorProfile");
     if (keyFile.has_key ("Color Management", "AutoMonitorProfile")) rtSettings.autoMonitorProfile = keyFile.get_boolean ("Color Management", "AutoMonitorProfile");
     if (keyFile.has_key ("Color Management", "Autocielab")) rtSettings.autocielab = keyFile.get_boolean ("Color Management", "Autocielab");
-    if (keyFile.has_key ("Color Management", "Ciencamfloat")) rtSettings.ciecamfloat = keyFile.get_boolean ("Color Management", "Ciecamfloat");
     if (keyFile.has_key ("Color Management", "RGBcurvesLumamode_Gamut")) rtSettings.rgbcurveslumamode_gamut = keyFile.get_boolean ("Color Management", "RGBcurvesLumamode_Gamut");
 
     if (keyFile.has_key ("Color Management", "Intent"))         rtSettings.colorimetricIntent   = keyFile.get_integer("Color Management", "Intent");
@@ -685,6 +732,7 @@ if (keyFile.has_group ("Color Management")) {
 
     if (keyFile.has_key ("Color Management", "WhiteBalanceSpotSize")) whiteBalanceSpotSize      = keyFile.get_integer("Color Management", "WhiteBalanceSpotSize");
     if( keyFile.has_key ("Color Management", "GamutICC"))       rtSettings.gamutICC             = keyFile.get_boolean("Color Management", "GamutICC");
+ //   if( keyFile.has_key ("Color Management", "BWcomplement"))   rtSettings.bw_complementary     = keyFile.get_boolean("Color Management", "BWcomplement");
     if( keyFile.has_key ("Color Management", "Ciecamfloat"))    rtSettings.ciecamfloat          = keyFile.get_boolean("Color Management", "Ciecamfloat");
     if( keyFile.has_key ("Color Management", "AdobeRGB"))       rtSettings.adobe                = keyFile.get_string("Color Management", "AdobeRGB");
     if( keyFile.has_key ("Color Management", "ProPhoto"))       rtSettings.prophoto             = keyFile.get_string("Color Management", "ProPhoto");
@@ -698,8 +746,9 @@ if (keyFile.has_group ("Color Management")) {
     if( keyFile.has_key ("Color Management", "GamutLch"))       rtSettings.gamutLch             = keyFile.get_boolean("Color Management", "GamutLch");
     if( keyFile.has_key ("Color Management", "ProtectRed"))     rtSettings.protectred           = keyFile.get_integer("Color Management", "ProtectRed");
     if( keyFile.has_key ("Color Management", "ProtectRedH"))    rtSettings.protectredh          = keyFile.get_double("Color Management", "ProtectRedH");
+    if( keyFile.has_key ("Color Management", "Amountchroma"))    rtSettings.amchroma            = keyFile.get_integer("Color Management", "Amountchroma");
 //    if( keyFile.has_key ("Color Management", "Ciebadpixgauss")) rtSettings.ciebadpixgauss       = keyFile.get_boolean("Color Management", "Ciebadpixgauss");
-	
+
 }
 
 if (keyFile.has_group ("Batch Processing")) { 
@@ -755,14 +804,32 @@ if (keyFile.has_group ("Dialogs")) {
     safeDirGet(keyFile, "Dialogs", "LastLabCurvesDir", lastLabCurvesDir);
     safeDirGet(keyFile, "Dialogs", "LastPFCurvesDir", lastPFCurvesDir);
     safeDirGet(keyFile, "Dialogs", "LastHsvCurvesDir", lastHsvCurvesDir);
+    safeDirGet(keyFile, "Dialogs", "LastBWCurvesDir", lastBWCurvesDir);
+	
     safeDirGet(keyFile, "Dialogs", "LastToneCurvesDir", lastToneCurvesDir);
     safeDirGet(keyFile, "Dialogs", "LastVibranceCurvesDir", lastVibranceCurvesDir);
     safeDirGet(keyFile, "Dialogs", "LastProfilingReferenceDir", lastProfilingReferenceDir);
 }
 
-        filterOutParsedExtensions ();
+// --------------------------------------------------------------------------------------------------------
+
+            filterOutParsedExtensions ();
+
+            return 0;
+
+        }
+    }
+    catch (Glib::Error &err) {
+        if (options.rtSettings.verbose)
+            printf("Options::readFromFile / Error code %d while reading values from \"%s\":\n%s\n", err.code(), fname.c_str(), err.what().c_str());
+    }
+    catch (...) {
+        if (options.rtSettings.verbose)
+            printf("Options::readFromFile / Unknown exception while trying to load \"%s\"!\n", fname.c_str());
+    }
+
+    return 1;
 
-        return 0;
 }
 
 bool Options::safeDirGet(const rtengine::SafeKeyFile& keyFile, const Glib::ustring& section,
@@ -816,6 +883,8 @@ int Options::saveToFile (Glib::ustring fname) {
     keyFile.set_boolean ("File Browser", "BrowserShowsHidden", fbShowHidden);
     keyFile.set_integer ("File Browser", "ThumbnailSize", thumbSize);
     keyFile.set_integer ("File Browser", "ThumbnailSizeTab", thumbSizeTab);
+    keyFile.set_integer ("File Browser", "ThumbnailSizeQueue", thumbSizeQueue);
+    keyFile.set_integer ("File Browser", "SameThumbSize", sameThumbSize);
     keyFile.set_integer ("File Browser", "MaxPreviewHeight", maxThumbnailHeight);
     keyFile.set_integer ("File Browser", "MaxCacheEntries", maxCacheEntries);
     Glib::ArrayHandle<Glib::ustring> pext = parseExtensions;
@@ -884,7 +953,8 @@ int Options::saveToFile (Glib::ustring fname) {
     keyFile.set_boolean ("Profiles", "SaveParamsWithFile", saveParamsFile);
     keyFile.set_boolean ("Profiles", "SaveParamsToCache", saveParamsCache);
     keyFile.set_integer ("Profiles", "LoadParamsFromLocation", paramsLoadLocation);
-    keyFile.set_string  ("Profiles", "CustomProfileBuilder", customProfileBuilder);
+    keyFile.set_string  ("Profiles", "CustomProfileBuilderPath", CPBPath);
+    keyFile.set_integer ("Profiles", "CustomProfileBuilderKeys", CPBKeys);
     
     keyFile.set_string  ("GUI", "Font", font);
     keyFile.set_integer ("GUI", "WindowWidth", windowWidth);
@@ -949,9 +1019,11 @@ int Options::saveToFile (Glib::ustring fname) {
     keyFile.set_string  ("Color Management", "Bruce", rtSettings.bruce);
     keyFile.set_integer ("Color Management", "WhiteBalanceSpotSize", whiteBalanceSpotSize);
     keyFile.set_boolean ("Color Management", "GamutICC", rtSettings.gamutICC);
+ //   keyFile.set_boolean ("Color Management", "BWcomplement", rtSettings.bw_complementary);
     keyFile.set_boolean ("Color Management", "Ciecamfloat", rtSettings.ciecamfloat);
     keyFile.set_boolean ("Color Management", "GamutLch", rtSettings.gamutLch);
     keyFile.set_integer ("Color Management", "ProtectRed", rtSettings.protectred);
+    keyFile.set_integer ("Color Management", "Amountchroma", rtSettings.amchroma);
     keyFile.set_double  ("Color Management", "ProtectRedH", rtSettings.protectredh);
     keyFile.set_integer ("Color Management", "CRI", rtSettings.CRI_color);
 //    keyFile.set_boolean ("Color Management", "Ciebadpixgauss", rtSettings.ciebadpixgauss);
@@ -1004,13 +1076,17 @@ int Options::saveToFile (Glib::ustring fname) {
     keyFile.set_string ("Dialogs", "LastLabCurvesDir", lastLabCurvesDir);
     keyFile.set_string ("Dialogs", "LastPFCurvesDir", lastPFCurvesDir);
     keyFile.set_string ("Dialogs", "LastHsvCurvesDir", lastHsvCurvesDir);
+    keyFile.set_string ("Dialogs", "LastBWCurvesDir", lastBWCurvesDir);
     keyFile.set_string ("Dialogs", "LastToneCurvesDir", lastToneCurvesDir);
     keyFile.set_string ("Dialogs", "LastVibranceCurvesDir", lastVibranceCurvesDir);
     keyFile.set_string ("Dialogs", "LastProfilingReferenceDir", lastProfilingReferenceDir);
 
     FILE *f = safe_g_fopen (fname, "wt");
-    if (f==NULL)
+    if (f==NULL) {
+        if (options.rtSettings.verbose)
+            printf("Options::saveToFile / Error: unable to open file \"%s\" with write access!\n", fname.c_str());
         return 1;
+    }
     else {
         fprintf (f, "%s", keyFile.to_data().c_str());
         fclose (f);
@@ -1034,34 +1110,39 @@ void Options::load () {
 
         if (SHGetSpecialFolderPathW(NULL,pathW,CSIDL_LOCAL_APPDATA,false)) {
             WideCharToMultiByte(CP_UTF8,0,pathW,-1,pathA,MAX_PATH,0,0);
-            rtdir = Glib::ustring(pathA) + Glib::ustring("\\") + Glib::ustring(CACHEFOLDERNAME);
+            rtdir = Glib::build_filename(Glib::ustring(pathA), Glib::ustring(CACHEFOLDERNAME));
 		}
 	}
 #else
-    rtdir = Glib::ustring(g_get_user_config_dir ()) + Glib::ustring("/") + Glib::ustring(CACHEFOLDERNAME);
+    rtdir = Glib::build_filename(Glib::ustring(g_get_user_config_dir ()), Glib::ustring(CACHEFOLDERNAME));
 #endif
 
     // Set the cache folder in RT's base folder
-    cacheBaseDir = argv0 + "/cache";
+    cacheBaseDir = Glib::build_filename(argv0, "cache");
 
     // Read the global option file (the one located in the application's base folder)
-    options.readFromFile (argv0+"/options");
+    options.readFromFile (Glib::build_filename(argv0, "options"));
 
     // Check if RT is installed in Multi-User mode
     if (options.multiUser) {
         // Read the user option file (the one located somewhere in the user's home folder)
         // Those values supersets those of the global option file
-        int r = options.readFromFile (rtdir + "/options");
+        int r = options.readFromFile (Glib::build_filename(rtdir, "options"));
         // If the local option file does not exist or is broken, and the local cache folder does not exist, recreate it
         if (r && !safe_g_mkdir_with_parents (rtdir, 511)) {
             // Save the option file
-        	options.saveToFile (rtdir + "/options");
+            options.saveToFile (Glib::build_filename(rtdir, "options"));
         }
         // Modify the path of the cache folder to the user's personal folder
 #ifdef WIN32
-        cacheBaseDir = rtdir + "/cache";
+        cacheBaseDir = Glib::build_filename(rtdir, "cache");
 #else
-        cacheBaseDir = Glib::ustring(g_get_user_cache_dir()) + Glib::ustring("/") + Glib::ustring(CACHEFOLDERNAME);
+        cacheBaseDir = Glib::build_filename(Glib::ustring(g_get_user_cache_dir()), Glib::ustring(CACHEFOLDERNAME));
+#endif
+
+#ifdef __APPLE__
+        // make sure .local/share exists on OS X so we don't get problems with recently-used.xbel
+        safe_g_mkdir_with_parents (g_get_user_data_dir(), 511);
 #endif
     }
 
@@ -1070,7 +1151,7 @@ void Options::load () {
 
     // Check default Raw and Img procparams existence
     if (options.defProfRaw.empty())
-    	options.defProfRaw = DEFPROFILE_INTERNAL;
+        options.defProfRaw = DEFPROFILE_INTERNAL;
     else {
         Glib::ustring tmpFName = options.findProfilePath(options.defProfRaw);
         if (!tmpFName.empty()) {
@@ -1130,16 +1211,16 @@ void Options::load () {
 
 	langMgr.load(localeTranslation, new MultiLangMgr(languageTranslation, new MultiLangMgr(defaultTranslation)));
 
-	rtengine::init (&options.rtSettings, argv0);
+	rtengine::init (&options.rtSettings, argv0, rtdir);
 }
 
 void Options::save () {
 
     if (options.multiUser==false) {
-        options.saveToFile (argv0+"/options");
+        options.saveToFile (Glib::build_filename(argv0, "options"));
     }
     else {
-        options.saveToFile (rtdir + "/options");
+        options.saveToFile (Glib::build_filename(rtdir, "options"));
     }
 }
 
