@@ -28,11 +28,15 @@
 #define STARTUPDIR_LAST    3
 
 // Default bundled profile name to use for Raw images
-#define DEFPROFILE_RAW      "Default"
+#ifdef WIN32
+#define DEFPROFILE_RAW      "${G}\\Default"
+#else
+#define DEFPROFILE_RAW      "${G}/Default"
+#endif
 // Default bundled profile name to use for Standard images
 #define DEFPROFILE_IMG      "Neutral"
 // Profile name to use for internal values' profile
-#define DEFPROFILE_INTERNAL "Internal"
+#define DEFPROFILE_INTERNAL "Neutral"
 
 class SaveFormat {
 
@@ -49,6 +53,7 @@ class SaveFormat {
 
 enum ThFileType {FT_Invalid=-1, FT_None=0, FT_Raw=1, FT_Jpeg=2, FT_Tiff=3, FT_Png=4, FT_Custom=5, FT_Tiff16=6, FT_Png16=7, FT_Custom16=8}; 
 enum PPLoadLocation {PLL_Cache=0, PLL_Input=1};
+enum CPBKeyType {CPBKT_TID=0, CPBKT_NAME=1, CPBKT_TID_NAME=2};
 
 namespace rtengine {
 	class SafeKeyFile;
@@ -122,7 +127,8 @@ class Options {
     bool multiUser;
     static Glib::ustring rtdir;
     Glib::ustring version;
-    int thumbSize,thumbSizeTab;
+    int thumbSize,thumbSizeTab, thumbSizeQueue;
+    bool sameThumbSize;     // Will use only one thumb size for the file browser and the single editor tab, and avoid recomputing them
     bool showHistory;
     int showFilePanelState; // 0: normal, 1: maximized, 2: normal, 3: hidden
     bool showInfo;
@@ -150,7 +156,8 @@ class Options {
     Glib::ustring gimpDir;
     Glib::ustring psDir;
     Glib::ustring customEditorProg;
-    Glib::ustring customProfileBuilder;
+    Glib::ustring CPBPath; // Custom Profile Builder's path
+    CPBKeyType CPBKeys; // Custom Profile Builder's key type
     int editorToSendTo;
     int maxThumbnailHeight;
     std::size_t maxCacheEntries;
@@ -250,6 +257,7 @@ class Options {
     Glib::ustring lastToneCurvesDir;
     Glib::ustring lastVibranceCurvesDir;
     Glib::ustring lastProfilingReferenceDir;
+    Glib::ustring lastBWCurvesDir;
 
     Options ();
 
@@ -270,6 +278,8 @@ class Options {
     bool        is_extention_enabled(Glib::ustring ext);
     bool        is_defProfRawMissing() { return defProfRawMissing; }
     bool        is_defProfImgMissing() { return defProfImgMissing; }
+    void        setDefProfRawMissing(bool value) { defProfRawMissing = value; }
+    void        setDefProfImgMissing(bool value) { defProfImgMissing = value; }
 };
 
 extern Options options;
