@@ -46,19 +46,23 @@ void ParamsEdited::set (bool v) {
 	toneCurve.autoexp    = v;
 	toneCurve.clip       = v;
 	toneCurve.expcomp    = v;
+	toneCurve.hrenabled   = v;
+	toneCurve.method    = v;
 	labCurve.lcurve      = v;
 	labCurve.acurve      = v;
 	labCurve.bcurve      = v;
 	labCurve.cccurve     = v;
 	labCurve.chcurve     = v;
+	labCurve.lhcurve     = v;
+	labCurve.hhcurve     = v;
 	labCurve.lccurve    = v;
+	labCurve.clcurve    = v;
 	labCurve.brightness  = v;
 	labCurve.contrast    = v;
 	labCurve.chromaticity    = v;
 	labCurve.avoidcolorshift = v;
 	labCurve.rstprotection   = v;
-	labCurve.bwtoning        = v;
-	labCurve.lcredsk        = v;
+	labCurve.lcredsk         = v;
 	rgbCurves.lumamode       = v;
 	rgbCurves.rcurve         = v;
 	rgbCurves.gcurve         = v;
@@ -133,6 +137,7 @@ void ParamsEdited::set (bool v) {
 	wb.method                  = v;
 	wb.green                   = v;
 	wb.temperature             = v;
+	wb.equal                   = v;
 	//colorShift.a               = v;
 	//colorShift.b               = v;
 	//lumaDenoise.enabled        = v;
@@ -147,6 +152,7 @@ void ParamsEdited::set (bool v) {
 	impulseDenoise.enabled     = v;
 	impulseDenoise.thresh      = v;
 	dirpyrDenoise.enabled      = v;
+	dirpyrDenoise.enhance      = v;
 //	dirpyrDenoise.perform      = v;
 	dirpyrDenoise.luma         = v;
 	dirpyrDenoise.Ldetail      = v;
@@ -189,6 +195,16 @@ void ParamsEdited::set (bool v) {
     lensProf.useCA = v;
 	perspective.horizontal = v;
 	perspective.vertical = v;
+	gradient.enabled = v;
+	gradient.degree = v;
+	gradient.feather = v;
+	gradient.strength = v;
+	gradient.centerX = v;
+	gradient.centerY = v;
+	pcvignette.enabled = v;
+	pcvignette.strength = v;
+	pcvignette.feather = v;
+	pcvignette.roundness = v;
 	cacorrection.red = v;
 	cacorrection.blue = v;
 	vignetting.amount = v;
@@ -205,8 +221,29 @@ void ParamsEdited::set (bool v) {
 	chmixer.blue[0] = v;
 	chmixer.blue[1] = v;
 	chmixer.blue[2] = v;
-	hlrecovery.enabled   = v;
-	hlrecovery.method    = v;
+	blackwhite.enabled   = v;
+	blackwhite.enabledcc   = v;
+	blackwhite.mixerRed   = v;
+	blackwhite.mixerOrange   = v;
+	blackwhite.mixerYellow   = v;
+	blackwhite.mixerGreen   = v;
+	blackwhite.mixerCyan   = v;
+	blackwhite.mixerBlue   = v;
+	blackwhite.mixerMagenta   = v;
+	blackwhite.mixerPurple   = v;
+	blackwhite.gammaRed   = v;
+	blackwhite.gammaGreen   = v;
+	blackwhite.gammaBlue   = v;
+	blackwhite.filter   = v;
+	blackwhite.setting   = v;
+	blackwhite.method   = v;
+	blackwhite.luminanceCurve = v;
+	blackwhite.beforeCurve      = v;
+	blackwhite.beforeCurveMode  = v;
+	blackwhite.afterCurve      = v;
+	blackwhite.afterCurveMode  = v;
+	blackwhite.autoc    = v;
+
 	resize.scale     = v;
 	resize.appliesTo = v;
 	resize.method    = v;
@@ -217,13 +254,13 @@ void ParamsEdited::set (bool v) {
 	icm.input        = v;
     icm.toneCurve = v;
     icm.blendCMSMatrix = v;
-    icm.preferredProfile = v;
+    icm.dcpIlluminant = v;
 	icm.working      = v;
 	icm.output       = v;
 	icm.gamma		= v;
 	icm.freegamma		= v;
 	icm.gampos		= v;
-	icm.slpos		= v;	
+	icm.slpos		= v;
 	raw.ccSteps = v;
 	raw.dmethod = v;
 	raw.dcbIterations = v;
@@ -256,6 +293,7 @@ void ParamsEdited::set (bool v) {
 	for(int i = 0; i < 5; i++) {
 		dirpyrequalizer.mult[i] = v;
 	}
+	dirpyrequalizer.threshold = v;
 	hsvequalizer.hcurve = v;
 	hsvequalizer.scurve = v;
 	hsvequalizer.vcurve = v;
@@ -289,18 +327,22 @@ void ParamsEdited::initFrom (const std::vector<rtengine::procparams::ProcParams>
         toneCurve.autoexp = toneCurve.autoexp && p.toneCurve.autoexp == other.toneCurve.autoexp;
         toneCurve.clip = toneCurve.clip && p.toneCurve.clip == other.toneCurve.clip;
         toneCurve.expcomp = toneCurve.expcomp && p.toneCurve.expcomp == other.toneCurve.expcomp;
+        toneCurve.hrenabled = toneCurve.hrenabled && p.toneCurve.hrenabled == other.toneCurve.hrenabled;
+        toneCurve.method = toneCurve.method && p.toneCurve.method == other.toneCurve.method;
         labCurve.lcurve = labCurve.lcurve && p.labCurve.lcurve == other.labCurve.lcurve;
         labCurve.acurve = labCurve.acurve && p.labCurve.acurve == other.labCurve.acurve;
         labCurve.bcurve = labCurve.bcurve && p.labCurve.bcurve == other.labCurve.bcurve;
         labCurve.cccurve = labCurve.cccurve && p.labCurve.cccurve == other.labCurve.cccurve;
         labCurve.chcurve = labCurve.chcurve && p.labCurve.chcurve == other.labCurve.chcurve;
+        labCurve.lhcurve = labCurve.lhcurve && p.labCurve.lhcurve == other.labCurve.lhcurve;
+        labCurve.hhcurve = labCurve.hhcurve && p.labCurve.hhcurve == other.labCurve.hhcurve;
         labCurve.lccurve = labCurve.lccurve && p.labCurve.lccurve == other.labCurve.lccurve;
+        labCurve.clcurve = labCurve.clcurve && p.labCurve.clcurve == other.labCurve.clcurve;
         labCurve.brightness = labCurve.brightness && p.labCurve.brightness == other.labCurve.brightness;
         labCurve.contrast = labCurve.contrast && p.labCurve.contrast == other.labCurve.contrast;
         labCurve.chromaticity = labCurve.chromaticity && p.labCurve.chromaticity == other.labCurve.chromaticity;
         labCurve.avoidcolorshift = labCurve.avoidcolorshift && p.labCurve.avoidcolorshift == other.labCurve.avoidcolorshift;
         labCurve.rstprotection = labCurve.rstprotection && p.labCurve.rstprotection == other.labCurve.rstprotection;
-        labCurve.bwtoning = labCurve.bwtoning && p.labCurve.bwtoning == other.labCurve.bwtoning;
         labCurve.lcredsk = labCurve.lcredsk && p.labCurve.lcredsk == other.labCurve.lcredsk;
         rgbCurves.lumamode = rgbCurves.lumamode && p.rgbCurves.lumamode == other.rgbCurves.lumamode;
         rgbCurves.rcurve = rgbCurves.rcurve && p.rgbCurves.rcurve == other.rgbCurves.rcurve;
@@ -362,9 +404,9 @@ void ParamsEdited::initFrom (const std::vector<rtengine::procparams::ProcParams>
         colorappearance.datacie = colorappearance.datacie && p.colorappearance.datacie == other.colorappearance.datacie;
         colorappearance.tonecie = colorappearance.tonecie && p.colorappearance.tonecie == other.colorappearance.tonecie;
    //     colorappearance.sharpcie = colorappearance.sharpcie && p.colorappearance.sharpcie == other.colorappearance.sharpcie;
-		colorappearance.curve = colorappearance.curve && p.colorappearance.curve == other.colorappearance.curve;
+        colorappearance.curve = colorappearance.curve && p.colorappearance.curve == other.colorappearance.curve;
         colorappearance.curve3 = colorappearance.curve3 && p.colorappearance.curve3 == other.colorappearance.curve3;
-		colorappearance.curve2 = colorappearance.curve2 && p.colorappearance.curve2 == other.colorappearance.curve2;
+        colorappearance.curve2 = colorappearance.curve2 && p.colorappearance.curve2 == other.colorappearance.curve2;
         colorappearance.curveMode = colorappearance.curveMode && p.colorappearance.curveMode == other.colorappearance.curveMode;
         colorappearance.curveMode2 = colorappearance.curveMode2 && p.colorappearance.curveMode2 == other.colorappearance.curveMode2;
         colorappearance.curveMode3 = colorappearance.curveMode3 && p.colorappearance.curveMode3 == other.colorappearance.curveMode3;
@@ -375,6 +417,7 @@ void ParamsEdited::initFrom (const std::vector<rtengine::procparams::ProcParams>
         //colorBoost.saturationlimit = colorBoost.saturationlimit && p.colorBoost.saturationlimit == other.colorBoost.saturationlimit;
         wb.method = wb.method && p.wb.method == other.wb.method;
         wb.green = wb.green && p.wb.green == other.wb.green;
+        wb.equal = wb.equal && p.wb.equal == other.wb.equal;
         wb.temperature = wb.temperature && p.wb.temperature == other.wb.temperature;
         //colorShift.a = colorShift.a && p.colorShift.a == other.colorShift.a;
         //colorShift.b = colorShift.b && p.colorShift.b == other.colorShift.b;
@@ -392,11 +435,12 @@ void ParamsEdited::initFrom (const std::vector<rtengine::procparams::ProcParams>
         impulseDenoise.thresh = impulseDenoise.thresh && p.impulseDenoise.thresh == other.impulseDenoise.thresh;
 
         dirpyrDenoise.enabled = dirpyrDenoise.enabled && p.dirpyrDenoise.enabled == other.dirpyrDenoise.enabled;
+        dirpyrDenoise.enhance = dirpyrDenoise.enhance && p.dirpyrDenoise.enhance == other.dirpyrDenoise.enhance;
  //       dirpyrDenoise.perform = dirpyrDenoise.perform && p.dirpyrDenoise.perform == other.dirpyrDenoise.perform;
         dirpyrDenoise.luma = dirpyrDenoise.luma && p.dirpyrDenoise.luma == other.dirpyrDenoise.luma;
         dirpyrDenoise.Ldetail = dirpyrDenoise.Ldetail && p.dirpyrDenoise.Ldetail == other.dirpyrDenoise.Ldetail;
         dirpyrDenoise.chroma = dirpyrDenoise.chroma && p.dirpyrDenoise.chroma == other.dirpyrDenoise.chroma;
-		dirpyrDenoise.redchro = dirpyrDenoise.redchro && p.dirpyrDenoise.redchro == other.dirpyrDenoise.redchro;
+        dirpyrDenoise.redchro = dirpyrDenoise.redchro && p.dirpyrDenoise.redchro == other.dirpyrDenoise.redchro;
         dirpyrDenoise.bluechro = dirpyrDenoise.bluechro && p.dirpyrDenoise.bluechro == other.dirpyrDenoise.bluechro;	
         dirpyrDenoise.gamma = dirpyrDenoise.gamma && p.dirpyrDenoise.gamma == other.dirpyrDenoise.gamma;
         dirpyrDenoise.dmethod = dirpyrDenoise.dmethod && p.dirpyrDenoise.dmethod == other.dirpyrDenoise.dmethod;
@@ -436,6 +480,16 @@ void ParamsEdited::initFrom (const std::vector<rtengine::procparams::ProcParams>
         lensProf.useCA = lensProf.useCA && p.lensProf.useCA == other.lensProf.useCA;
         perspective.horizontal = perspective.horizontal && p.perspective.horizontal == other.perspective.horizontal;
         perspective.vertical = perspective.vertical && p.perspective.vertical == other.perspective.vertical;
+        gradient.enabled = gradient.enabled && p.gradient.enabled == other.gradient.enabled;
+        gradient.degree = gradient.degree && p.gradient.degree == other.gradient.degree;
+        gradient.feather = gradient.feather && p.gradient.feather == other.gradient.feather;
+        gradient.strength = gradient.strength && p.gradient.strength == other.gradient.strength;
+        gradient.centerX = gradient.centerX && p.gradient.centerX == other.gradient.centerX;
+        gradient.centerY = gradient.centerY && p.gradient.centerY == other.gradient.centerY;
+        pcvignette.enabled = pcvignette.enabled && p.pcvignette.enabled == other.pcvignette.enabled;
+        pcvignette.strength = pcvignette.strength && p.pcvignette.strength == other.pcvignette.strength;
+        pcvignette.feather = pcvignette.feather && p.pcvignette.feather == other.pcvignette.feather;
+        pcvignette.roundness = pcvignette.roundness && p.pcvignette.roundness == other.pcvignette.roundness;
         cacorrection.red = cacorrection.red && p.cacorrection.red == other.cacorrection.red;
         cacorrection.blue = cacorrection.blue && p.cacorrection.blue == other.cacorrection.blue;
         vignetting.amount = vignetting.amount && p.vignetting.amount == other.vignetting.amount;
@@ -452,8 +506,28 @@ void ParamsEdited::initFrom (const std::vector<rtengine::procparams::ProcParams>
         chmixer.blue[0] = chmixer.blue[0] && p.chmixer.blue[0] == other.chmixer.blue[0];
         chmixer.blue[1] = chmixer.blue[1] && p.chmixer.blue[1] == other.chmixer.blue[1];
         chmixer.blue[2] = chmixer.blue[2] && p.chmixer.blue[2] == other.chmixer.blue[2];
-        hlrecovery.enabled = hlrecovery.enabled && p.hlrecovery.enabled == other.hlrecovery.enabled;
-        hlrecovery.method = hlrecovery.method && p.hlrecovery.method == other.hlrecovery.method;
+        blackwhite.enabledcc = blackwhite.enabledcc && p.blackwhite.enabledcc == other.blackwhite.enabledcc;
+        blackwhite.enabled = blackwhite.enabled && p.blackwhite.enabled == other.blackwhite.enabled;
+        blackwhite.mixerRed = blackwhite.mixerRed && p.blackwhite.mixerRed == other.blackwhite.mixerRed;
+        blackwhite.mixerOrange = blackwhite.mixerOrange && p.blackwhite.mixerOrange == other.blackwhite.mixerOrange;
+        blackwhite.mixerYellow = blackwhite.mixerYellow && p.blackwhite.mixerYellow == other.blackwhite.mixerYellow;
+        blackwhite.mixerGreen = blackwhite.mixerGreen && p.blackwhite.mixerGreen == other.blackwhite.mixerGreen;
+        blackwhite.mixerCyan = blackwhite.mixerCyan && p.blackwhite.mixerCyan == other.blackwhite.mixerCyan;
+        blackwhite.mixerBlue = blackwhite.mixerBlue && p.blackwhite.mixerBlue == other.blackwhite.mixerBlue;
+        blackwhite.mixerMagenta = blackwhite.mixerMagenta && p.blackwhite.mixerMagenta == other.blackwhite.mixerMagenta;
+        blackwhite.mixerPurple = blackwhite.mixerPurple && p.blackwhite.mixerPurple == other.blackwhite.mixerPurple;
+        blackwhite.gammaRed = blackwhite.gammaRed && p.blackwhite.gammaRed == other.blackwhite.gammaRed;
+        blackwhite.gammaGreen = blackwhite.gammaGreen && p.blackwhite.gammaGreen == other.blackwhite.gammaGreen;
+        blackwhite.gammaBlue = blackwhite.gammaBlue && p.blackwhite.gammaBlue == other.blackwhite.gammaBlue;
+        blackwhite.filter = blackwhite.filter && p.blackwhite.filter == other.blackwhite.filter;
+        blackwhite.setting = blackwhite.setting && p.blackwhite.setting == other.blackwhite.setting;
+        blackwhite.luminanceCurve = blackwhite.luminanceCurve && p.blackwhite.luminanceCurve == other.blackwhite.luminanceCurve;
+        blackwhite.method = blackwhite.method && p.blackwhite.method == other.blackwhite.method;
+        blackwhite.beforeCurve = blackwhite.beforeCurve && p.blackwhite.beforeCurve == other.blackwhite.beforeCurve;
+        blackwhite.beforeCurveMode = blackwhite.beforeCurveMode && p.blackwhite.beforeCurveMode == other.blackwhite.beforeCurveMode;
+        blackwhite.afterCurve = blackwhite.afterCurve && p.blackwhite.afterCurve == other.blackwhite.afterCurve;
+        blackwhite.afterCurveMode = blackwhite.afterCurveMode && p.blackwhite.afterCurveMode == other.blackwhite.afterCurveMode;
+        blackwhite.autoc = blackwhite.autoc && p.blackwhite.autoc == other.blackwhite.autoc;
         resize.scale = resize.scale && p.resize.scale == other.resize.scale;
         resize.appliesTo = resize.appliesTo && p.resize.appliesTo == other.resize.appliesTo;
         resize.method = resize.method && p.resize.method == other.resize.method;
@@ -464,11 +538,11 @@ void ParamsEdited::initFrom (const std::vector<rtengine::procparams::ProcParams>
         icm.input = icm.input && p.icm.input == other.icm.input;
         icm.toneCurve = icm.toneCurve && p.icm.toneCurve == other.icm.toneCurve;
         icm.blendCMSMatrix = icm.blendCMSMatrix && p.icm.blendCMSMatrix == other.icm.blendCMSMatrix;
-        icm.preferredProfile = icm.preferredProfile && p.icm.preferredProfile == other.icm.preferredProfile;
+        icm.dcpIlluminant = icm.dcpIlluminant && p.icm.dcpIlluminant == other.icm.dcpIlluminant;
         icm.working = icm.working && p.icm.working == other.icm.working;
         icm.output = icm.output && p.icm.output == other.icm.output;
         icm.gamma = icm.gamma && p.icm.gamma == other.icm.gamma;
-        icm.freegamma = icm.freegamma && p.icm.freegamma == other.icm.freegamma;		
+        icm.freegamma = icm.freegamma && p.icm.freegamma == other.icm.freegamma;
         icm.gampos = icm.gampos && p.icm.gampos == other.icm.gampos;
         icm.slpos = icm.slpos && p.icm.slpos == other.icm.slpos;
         raw.ccSteps = raw.ccSteps && p.raw.ccSteps == other.raw.ccSteps;
@@ -499,9 +573,10 @@ void ParamsEdited::initFrom (const std::vector<rtengine::procparams::ProcParams>
         raw.exTwoGreen = p.raw.twogreen == other.raw.twogreen;
 
         dirpyrequalizer.enabled = dirpyrequalizer.enabled && p.dirpyrequalizer.enabled == other.dirpyrequalizer.enabled;
-        for(int i = 0; i < 8; i++) {
+        for(int i = 0; i < 5; i++) {
             dirpyrequalizer.mult[i] = dirpyrequalizer.mult[i] && p.dirpyrequalizer.mult[i] == other.dirpyrequalizer.mult[i];
-        }		
+        }
+        dirpyrequalizer.threshold = dirpyrequalizer.threshold && p.dirpyrequalizer.threshold == other.dirpyrequalizer.threshold;
         hsvequalizer.hcurve = hsvequalizer.hcurve && p.hsvequalizer.hcurve == other.hsvequalizer.hcurve;
         hsvequalizer.scurve = hsvequalizer.scurve && p.hsvequalizer.scurve == other.hsvequalizer.scurve;
         hsvequalizer.vcurve = hsvequalizer.vcurve && p.hsvequalizer.vcurve == other.hsvequalizer.vcurve;
@@ -530,19 +605,23 @@ void ParamsEdited::combine (rtengine::procparams::ProcParams& toEdit, const rten
 	if (toneCurve.expcomp)		toEdit.toneCurve.expcomp 	= dontforceSet && options.baBehav[ADDSET_TC_EXPCOMP] ? toEdit.toneCurve.expcomp + mods.toneCurve.expcomp : mods.toneCurve.expcomp;
 	if (toneCurve.hlcompr)		toEdit.toneCurve.hlcompr 	= dontforceSet && options.baBehav[ADDSET_TC_HLCOMPAMOUNT] ? toEdit.toneCurve.hlcompr + mods.toneCurve.hlcompr : mods.toneCurve.hlcompr;
 	if (toneCurve.hlcomprthresh) toEdit.toneCurve.hlcomprthresh	= dontforceSet && options.baBehav[ADDSET_TC_HLCOMPTHRESH] ? toEdit.toneCurve.hlcomprthresh + mods.toneCurve.hlcomprthresh : mods.toneCurve.hlcomprthresh;
+	if (toneCurve.hrenabled)	toEdit.toneCurve.hrenabled 	= mods.toneCurve.hrenabled;
+	if (toneCurve.method)		toEdit.toneCurve.method 	= mods.toneCurve.method;
 	if (labCurve.lcurve)		toEdit.labCurve.lcurve 	    = mods.labCurve.lcurve;
 	if (labCurve.acurve)		toEdit.labCurve.acurve 	    = mods.labCurve.acurve;
 	if (labCurve.bcurve)		toEdit.labCurve.bcurve 	    = mods.labCurve.bcurve;
 	if (labCurve.cccurve)		toEdit.labCurve.cccurve     = mods.labCurve.cccurve;
 	if (labCurve.chcurve)		toEdit.labCurve.chcurve     = mods.labCurve.chcurve;
+	if (labCurve.lhcurve)		toEdit.labCurve.lhcurve     = mods.labCurve.lhcurve;
+	if (labCurve.hhcurve)		toEdit.labCurve.hhcurve     = mods.labCurve.hhcurve;
 	if (labCurve.lccurve)		toEdit.labCurve.lccurve    = mods.labCurve.lccurve;
+	if (labCurve.clcurve)		toEdit.labCurve.clcurve    = mods.labCurve.clcurve;
 	if (labCurve.brightness)	toEdit.labCurve.brightness   = dontforceSet && options.baBehav[ADDSET_LC_BRIGHTNESS] ? toEdit.labCurve.brightness + mods.labCurve.brightness : mods.labCurve.brightness;
 	if (labCurve.contrast)		toEdit.labCurve.contrast 	 = dontforceSet && options.baBehav[ADDSET_LC_CONTRAST] ? toEdit.labCurve.contrast + mods.labCurve.contrast : mods.labCurve.contrast;
 	if (labCurve.chromaticity)	toEdit.labCurve.chromaticity = dontforceSet && options.baBehav[ADDSET_LC_CHROMATICITY] ? toEdit.labCurve.chromaticity + mods.labCurve.chromaticity : mods.labCurve.chromaticity;
 	if (labCurve.avoidcolorshift)	toEdit.labCurve.avoidcolorshift		= mods.labCurve.avoidcolorshift;
 	if (labCurve.rstprotection)		toEdit.labCurve.rstprotection		= mods.labCurve.rstprotection;
-	if (labCurve.bwtoning)			toEdit.labCurve.bwtoning			= mods.labCurve.bwtoning;
-	if (labCurve.lcredsk)			toEdit.labCurve.lcredsk			= mods.labCurve.lcredsk;
+	if (labCurve.lcredsk)			toEdit.labCurve.lcredsk				= mods.labCurve.lcredsk;
 
 	if (rgbCurves.lumamode)					toEdit.rgbCurves.lumamode   = mods.rgbCurves.lumamode;
 	if (rgbCurves.rcurve)					toEdit.rgbCurves.rcurve     = mods.rgbCurves.rcurve;
@@ -561,14 +640,6 @@ void ParamsEdited::combine (rtengine::procparams::ProcParams& toEdit, const rten
 	if (sharpening.radius)					toEdit.sharpening.radius 	= mods.sharpening.radius;
 	if (sharpening.amount)					toEdit.sharpening.amount 	= dontforceSet && options.baBehav[ADDSET_SHARP_AMOUNT] ? toEdit.sharpening.amount + mods.sharpening.amount : mods.sharpening.amount;
 	if (sharpening.threshold)				toEdit.sharpening.threshold = mods.sharpening.threshold;
-
-	for (int i=0; i<3; i++) {
-		if (chmixer.red[i])		toEdit.chmixer.red[i] 	= dontforceSet && options.baBehav[ADDSET_CHMIXER] ? toEdit.chmixer.red[i] + mods.chmixer.red[i] : mods.chmixer.red[i];
-		if (chmixer.green[i])	toEdit.chmixer.green[i]	= dontforceSet && options.baBehav[ADDSET_CHMIXER] ? toEdit.chmixer.green[i] + mods.chmixer.green[i] : mods.chmixer.green[i];
-		if (chmixer.blue[i])	toEdit.chmixer.blue[i] 	= dontforceSet && options.baBehav[ADDSET_CHMIXER] ? toEdit.chmixer.blue[i] + mods.chmixer.blue[i] : mods.chmixer.blue[i];
-	}
-
-
 	if (sharpening.edgesonly)				toEdit.sharpening.edgesonly 	= mods.sharpening.edgesonly;
 	if (sharpening.edges_radius)			toEdit.sharpening.edges_radius 	= mods.sharpening.edges_radius;
 	if (sharpening.edges_tolerance)			toEdit.sharpening.edges_tolerance	 = mods.sharpening.edges_tolerance;
@@ -593,6 +664,7 @@ void ParamsEdited::combine (rtengine::procparams::ProcParams& toEdit, const rten
 	//if (colorBoost.enable_saturationlimiter)toEdit.colorBoost.enable_saturationlimiter 	= mods.colorBoost.enable_saturationlimiter;
 	//if (colorBoost.saturationlimit)			toEdit.colorBoost.saturationlimit 	= mods.colorBoost.saturationlimit;
 	if (wb.method)							toEdit.wb.method 	= mods.wb.method;
+	if (wb.equal)							toEdit.wb.equal 	= dontforceSet && options.baBehav[ADDSET_WB_EQUAL] ? toEdit.wb.equal + mods.wb.equal : mods.wb.equal;
 	if (wb.green)							toEdit.wb.green 	= dontforceSet && options.baBehav[ADDSET_WB_GREEN] ? toEdit.wb.green + mods.wb.green : mods.wb.green;
 	if (wb.temperature)						toEdit.wb.temperature 	= dontforceSet && options.baBehav[ADDSET_WB_TEMPERATURE] ? toEdit.wb.temperature + mods.wb.temperature : mods.wb.temperature;
 	//if (colorShift.a)						toEdit.colorShift.a 	= dontforceSet && options.baBehav[ADDSET_CS_BLUEYELLOW] ? toEdit.colorShift.a + mods.colorShift.a : mods.colorShift.a;
@@ -645,6 +717,7 @@ void ParamsEdited::combine (rtengine::procparams::ProcParams& toEdit, const rten
 	if (impulseDenoise.thresh)				toEdit.impulseDenoise.thresh 	= mods.impulseDenoise.thresh;
 
 	if (dirpyrDenoise.enabled)				toEdit.dirpyrDenoise.enabled 	= mods.dirpyrDenoise.enabled;
+	if (dirpyrDenoise.enhance)				toEdit.dirpyrDenoise.enhance 	= mods.dirpyrDenoise.enhance;
 	if (dirpyrDenoise.luma)					toEdit.dirpyrDenoise.luma		= dontforceSet && options.baBehav[ADDSET_DIRPYRDN_LUMA] ? toEdit.dirpyrDenoise.luma + mods.dirpyrDenoise.luma : mods.dirpyrDenoise.luma;
 	if (dirpyrDenoise.Ldetail)				toEdit.dirpyrDenoise.Ldetail	= dontforceSet && options.baBehav[ADDSET_DIRPYRDN_LUMDET] ? toEdit.dirpyrDenoise.Ldetail + mods.dirpyrDenoise.Ldetail : mods.dirpyrDenoise.Ldetail;
 	if (dirpyrDenoise.chroma)				toEdit.dirpyrDenoise.chroma		= dontforceSet && options.baBehav[ADDSET_DIRPYRDN_CHROMA] ? toEdit.dirpyrDenoise.chroma + mods.dirpyrDenoise.chroma : mods.dirpyrDenoise.chroma;
@@ -690,6 +763,16 @@ void ParamsEdited::combine (rtengine::procparams::ProcParams& toEdit, const rten
 
 	if (perspective.horizontal)				toEdit.perspective.horizontal 	= dontforceSet && options.baBehav[ADDSET_PERSPECTIVE] ? toEdit.perspective.horizontal + mods.perspective.horizontal : mods.perspective.horizontal;
 	if (perspective.vertical)				toEdit.perspective.vertical 	= dontforceSet && options.baBehav[ADDSET_PERSPECTIVE] ? toEdit.perspective.vertical + mods.perspective.vertical : mods.perspective.vertical;
+	if (gradient.enabled)					toEdit.gradient.enabled 	= mods.gradient.enabled;
+	if (gradient.degree)					toEdit.gradient.degree		= dontforceSet && options.baBehav[ADDSET_GRADIENT_DEGREE] ? toEdit.gradient.degree + mods.gradient.degree : mods.gradient.degree;
+	if (gradient.feather)					toEdit.gradient.feather 	= mods.gradient.feather;
+	if (gradient.strength)					toEdit.gradient.strength 	= mods.gradient.strength;
+	if (gradient.centerX)					toEdit.gradient.centerX 	= mods.gradient.centerX;
+	if (gradient.centerY)					toEdit.gradient.centerY 	= mods.gradient.centerY;
+	if (pcvignette.enabled)					toEdit.pcvignette.enabled 	= mods.pcvignette.enabled;
+	if (pcvignette.strength)				toEdit.pcvignette.strength 	= mods.pcvignette.strength;
+	if (pcvignette.feather)					toEdit.pcvignette.feather 	= mods.pcvignette.feather;
+	if (pcvignette.roundness)				toEdit.pcvignette.roundness = mods.pcvignette.roundness;
 	if (cacorrection.red)					toEdit.cacorrection.red 	= dontforceSet && options.baBehav[ADDSET_CA] ? toEdit.cacorrection.red + mods.cacorrection.red : mods.cacorrection.red;
 	if (cacorrection.blue)					toEdit.cacorrection.blue 	= dontforceSet && options.baBehav[ADDSET_CA] ? toEdit.cacorrection.blue + mods.cacorrection.blue : mods.cacorrection.blue;
 	if (vignetting.amount)					toEdit.vignetting.amount 	= dontforceSet && options.baBehav[ADDSET_VIGN_AMOUNT] ? toEdit.vignetting.amount + mods.vignetting.amount : mods.vignetting.amount;
@@ -702,8 +785,29 @@ void ParamsEdited::combine (rtengine::procparams::ProcParams& toEdit, const rten
 		if (chmixer.green[i])	toEdit.chmixer.green[i]	= dontforceSet && options.baBehav[ADDSET_CHMIXER] ? toEdit.chmixer.green[i] + mods.chmixer.green[i] : mods.chmixer.green[i];
 		if (chmixer.blue[i])	toEdit.chmixer.blue[i] 	= dontforceSet && options.baBehav[ADDSET_CHMIXER] ? toEdit.chmixer.blue[i] + mods.chmixer.blue[i] : mods.chmixer.blue[i];
 	}
-	if (hlrecovery.enabled)	toEdit.hlrecovery.enabled 	= mods.hlrecovery.enabled;
-	if (hlrecovery.method)	toEdit.hlrecovery.method 	= mods.hlrecovery.method;
+	if (blackwhite.enabled)			toEdit.blackwhite.enabled			= mods.blackwhite.enabled;
+	if (blackwhite.method)			toEdit.blackwhite.method			= mods.blackwhite.method;
+	if (blackwhite.luminanceCurve)	toEdit.blackwhite.luminanceCurve	= mods.blackwhite.luminanceCurve;
+	if (blackwhite.autoc)			toEdit.blackwhite.autoc				= mods.blackwhite.autoc;
+	if (blackwhite.setting)			toEdit.blackwhite.setting			= mods.blackwhite.setting;
+	if (blackwhite.enabledcc)		toEdit.blackwhite.enabledcc			= mods.blackwhite.enabledcc;
+	if (blackwhite.filter)			toEdit.blackwhite.filter			= mods.blackwhite.filter;
+	if (blackwhite.mixerRed)		toEdit.blackwhite.mixerRed 			= dontforceSet && options.baBehav[ADDSET_BLACKWHITE_HUES] ? toEdit.blackwhite.mixerRed + mods.blackwhite.mixerRed : mods.blackwhite.mixerRed;
+	if (blackwhite.mixerOrange)		toEdit.blackwhite.mixerOrange 		= dontforceSet && options.baBehav[ADDSET_BLACKWHITE_HUES] ? toEdit.blackwhite.mixerOrange + mods.blackwhite.mixerOrange : mods.blackwhite.mixerOrange;
+	if (blackwhite.mixerYellow)		toEdit.blackwhite.mixerYellow 		= dontforceSet && options.baBehav[ADDSET_BLACKWHITE_HUES] ? toEdit.blackwhite.mixerYellow + mods.blackwhite.mixerYellow : mods.blackwhite.mixerYellow;
+	if (blackwhite.mixerGreen)		toEdit.blackwhite.mixerGreen 		= dontforceSet && options.baBehav[ADDSET_BLACKWHITE_HUES] ? toEdit.blackwhite.mixerGreen + mods.blackwhite.mixerGreen : mods.blackwhite.mixerGreen;
+	if (blackwhite.mixerCyan)		toEdit.blackwhite.mixerCyan 		= dontforceSet && options.baBehav[ADDSET_BLACKWHITE_HUES] ? toEdit.blackwhite.mixerCyan + mods.blackwhite.mixerCyan : mods.blackwhite.mixerCyan;
+	if (blackwhite.mixerBlue)		toEdit.blackwhite.mixerBlue 		= dontforceSet && options.baBehav[ADDSET_BLACKWHITE_HUES] ? toEdit.blackwhite.mixerBlue + mods.blackwhite.mixerBlue : mods.blackwhite.mixerBlue;
+	if (blackwhite.mixerMagenta)	toEdit.blackwhite.mixerMagenta 		= dontforceSet && options.baBehav[ADDSET_BLACKWHITE_HUES] ? toEdit.blackwhite.mixerMagenta + mods.blackwhite.mixerMagenta : mods.blackwhite.mixerMagenta;
+	if (blackwhite.mixerPurple)		toEdit.blackwhite.mixerPurple 		= dontforceSet && options.baBehav[ADDSET_BLACKWHITE_HUES] ? toEdit.blackwhite.mixerPurple + mods.blackwhite.mixerPurple : mods.blackwhite.mixerPurple;
+	if (blackwhite.gammaRed)		toEdit.blackwhite.gammaRed 			= dontforceSet && options.baBehav[ADDSET_BLACKWHITE_GAMMA] ? toEdit.blackwhite.gammaRed + mods.blackwhite.gammaRed : mods.blackwhite.gammaRed;
+	if (blackwhite.gammaGreen)		toEdit.blackwhite.gammaGreen 		= dontforceSet && options.baBehav[ADDSET_BLACKWHITE_GAMMA] ? toEdit.blackwhite.gammaGreen + mods.blackwhite.gammaGreen : mods.blackwhite.gammaGreen;
+	if (blackwhite.gammaBlue)		toEdit.blackwhite.gammaBlue 		= dontforceSet && options.baBehav[ADDSET_BLACKWHITE_GAMMA] ? toEdit.blackwhite.gammaBlue + mods.blackwhite.gammaBlue : mods.blackwhite.gammaBlue;
+	if (blackwhite.beforeCurve)		toEdit.blackwhite.beforeCurve		= mods.blackwhite.beforeCurve;
+	if (blackwhite.beforeCurveMode)	toEdit.blackwhite.beforeCurveMode	= mods.blackwhite.beforeCurveMode;
+	if (blackwhite.afterCurve)		toEdit.blackwhite.afterCurve		= mods.blackwhite.afterCurve;
+	if (blackwhite.afterCurveMode)	toEdit.blackwhite.afterCurveMode	= mods.blackwhite.afterCurveMode;
+	
 	if (resize.scale)		toEdit.resize.scale 	= mods.resize.scale;
 	if (resize.appliesTo)	toEdit.resize.appliesTo = mods.resize.appliesTo;
 	if (resize.method)		toEdit.resize.method 	= mods.resize.method;
@@ -714,7 +818,7 @@ void ParamsEdited::combine (rtengine::procparams::ProcParams& toEdit, const rten
 	if (icm.input)		    toEdit.icm.input 	    = mods.icm.input;
     if (icm.toneCurve)      toEdit.icm.toneCurve = mods.icm.toneCurve;
     if (icm.blendCMSMatrix)	toEdit.icm.blendCMSMatrix = mods.icm.blendCMSMatrix;
-    if (icm.preferredProfile) toEdit.icm.preferredProfile = mods.icm.preferredProfile;
+    if (icm.dcpIlluminant) toEdit.icm.dcpIlluminant = mods.icm.dcpIlluminant;
 	if (icm.working)		toEdit.icm.working 	    = mods.icm.working;
 	if (icm.output)		    toEdit.icm.output       = mods.icm.output;
 	//if (icm.gampos)		    toEdit.icm.gampos       = mods.icm.gampos;
@@ -748,15 +852,16 @@ void ParamsEdited::combine (rtengine::procparams::ProcParams& toEdit, const rten
     if (raw.darkFrame)          toEdit.raw.dark_frame        = mods.raw.dark_frame;
     if (raw.dfAuto)             toEdit.raw.df_autoselect     = mods.raw.df_autoselect;
 
-    if (raw.ff_file)        toEdit.raw.ff_file         = mods.raw.ff_file;         
+    if (raw.ff_file)        toEdit.raw.ff_file         = mods.raw.ff_file;
     if (raw.ff_AutoSelect)  toEdit.raw.ff_AutoSelect   = mods.raw.ff_AutoSelect;
     if (raw.ff_BlurRadius)  toEdit.raw.ff_BlurRadius   = mods.raw.ff_BlurRadius;
-    if (raw.ff_BlurType)    toEdit.raw.ff_BlurType     = mods.raw.ff_BlurType;      
+    if (raw.ff_BlurType)    toEdit.raw.ff_BlurType     = mods.raw.ff_BlurType;
 
 	if (dirpyrequalizer.enabled)	toEdit.dirpyrequalizer.enabled	= mods.dirpyrequalizer.enabled;
 	for(int i = 0; i < 5; i++) {
 		if(dirpyrequalizer.mult[i])	toEdit.dirpyrequalizer.mult[i]	= dontforceSet && options.baBehav[ADDSET_DIRPYREQ] ? toEdit.dirpyrequalizer.mult[i] + mods.dirpyrequalizer.mult[i] : mods.dirpyrequalizer.mult[i];
 	}
+	if (dirpyrequalizer.threshold)  toEdit.dirpyrequalizer.threshold= dontforceSet && options.baBehav[ADDSET_DIRPYREQ_THRESHOLD] ? toEdit.dirpyrequalizer.threshold + mods.dirpyrequalizer.threshold : mods.dirpyrequalizer.threshold;
 	if (hsvequalizer.hcurve)		toEdit.hsvequalizer.hcurve		= mods.hsvequalizer.hcurve;
 	if (hsvequalizer.scurve)		toEdit.hsvequalizer.scurve		= mods.hsvequalizer.scurve;
 	if (hsvequalizer.vcurve)		toEdit.hsvequalizer.vcurve		= mods.hsvequalizer.vcurve;
@@ -777,7 +882,7 @@ void ParamsEdited::combine (rtengine::procparams::ProcParams& toEdit, const rten
 bool RAWParamsEdited::isUnchanged() const {
     return ccSteps && dmethod && dcbIterations && dcbEnhance && lmmseIterations/*&& allEnhance*/ && caCorrection && caRed && caBlue && greenEq
         && hotDeadPixelFilter && hotDeadPixelThresh && linenoise && darkFrame && dfAuto && ff_file && ff_AutoSelect && ff_BlurRadius && ff_BlurType
-	    && exPos && exPreser && exBlackzero && exBlackone && exBlacktwo && exBlackthree && exTwoGreen;
+        && exPos && exPreser && exBlackzero && exBlackone && exBlacktwo && exBlackthree && exTwoGreen;
 }
 
 bool LensProfParamsEdited::isUnchanged() const {
