@@ -49,6 +49,9 @@ class ToneCurveParamsEdited {
         bool autoexp;
         bool clip;
         bool expcomp;
+        bool hrenabled;
+        bool method;
+		
 };
 
 class LCurveParamsEdited {
@@ -62,11 +65,17 @@ class LCurveParamsEdited {
         bool lcurve;
         bool acurve;
         bool bcurve;
-        bool bwtoning;
-		bool lcredsk;
+        bool lcredsk;
         bool cccurve;
         bool chcurve;
+        bool lhcurve;
+        bool hhcurve;
         bool lccurve;
+        bool clcurve;
+		
+        bool enabled;
+        bool method;
+		
 };
 
 class RGBCurvesParamsEdited {
@@ -143,6 +152,7 @@ class WBParamsEdited {
         bool method;
         bool temperature;
         bool green;
+        bool equal;
 };
 
 /*class ColorShiftParamsEdited {
@@ -168,7 +178,7 @@ class WBParamsEdited {
 };*/
 
 class DefringeParamsEdited {
-	
+
 public:
     bool enabled;
     bool radius;
@@ -188,20 +198,20 @@ class ColorAppearanceParamsEdited {
 public:
     bool curve;
     bool curve2;
-	bool curve3;
+    bool curve3;
     bool curveMode;
     bool curveMode2;
     bool curveMode3;
     bool enabled;
     bool degree;
     bool autodegree;
-	bool autoadapscen;
+    bool autoadapscen;
     bool surround;
     bool adapscen;
     bool adaplum;
-	bool badpixsl;
+    bool badpixsl;
     bool wbmodel;
-	bool algo;
+    bool algo;
     bool jlight;
     bool qbright;
     bool chroma;
@@ -210,26 +220,25 @@ public:
     bool contrast;
     bool qcontrast;
     bool colorh;
-	bool rstprotection;
-	bool surrsource;
-	bool gamut;
-//	bool badpix;
-	bool datacie;
-	bool tonecie;
-//	bool sharpcie;
-	
+    bool rstprotection;
+    bool surrsource;
+    bool gamut;
+//  bool badpix;
+    bool datacie;
+    bool tonecie;
+//  bool sharpcie;
 };
 
-
 class DirPyrDenoiseParamsEdited {
 
 public:
     bool enabled;
+    bool enhance;
     bool Ldetail;
     bool luma;
     bool chroma;
-	bool redchro;
-	bool bluechro;
+    bool redchro;
+    bool bluechro;
     bool gamma;
 //    bool perform;
 	bool dmethod;
@@ -282,8 +291,8 @@ class CoarseTransformParamsEdited {
 
 class CommonTransformParamsEdited {
 
-	public:
-		bool autofill;
+   public:
+        bool autofill;
 };
 
 class RotateParamsEdited {
@@ -307,9 +316,29 @@ class LensProfParamsEdited {
 
 class PerspectiveParamsEdited {
 
-	public:
-		bool horizontal;
-		bool vertical;
+    public:
+        bool horizontal;
+        bool vertical;
+};
+
+class GradientParamsEdited {
+
+    public:
+        bool enabled;
+        bool degree;
+        bool feather;
+        bool strength;
+        bool centerX;
+        bool centerY;
+};
+
+class PCVignetteParamsEdited {
+
+    public:
+        bool enabled;
+        bool strength;
+        bool feather;
+        bool roundness;
 };
 
 class VignettingParamsEdited {
@@ -328,6 +357,33 @@ class ChannelMixerParamsEdited {
         bool red[3];
         bool green[3];
         bool blue[3];
+
+};
+class BlackWhiteParamsEdited {
+
+    public:
+        bool enabledcc;
+        bool enabled;
+        bool method;
+        bool filter;
+        bool setting;
+        bool mixerRed;
+        bool mixerOrange;
+        bool mixerYellow;
+        bool mixerGreen;
+        bool mixerCyan;
+        bool mixerBlue;
+        bool mixerMagenta;
+        bool mixerPurple;
+        bool gammaRed;
+        bool gammaGreen;
+        bool gammaBlue;
+        bool luminanceCurve;
+        bool beforeCurve;
+        bool beforeCurveMode;
+        bool afterCurve;
+        bool afterCurveMode;
+        bool autoc;
 };
 
 class CACorrParamsEdited {
@@ -336,14 +392,14 @@ class CACorrParamsEdited {
         bool red;
         bool blue;
 };
-
+/*
 class HRecParamsEdited {
 
     public:
         bool enabled;
-		bool method;
+        bool method;
 };
-
+*/
 class ResizeParamsEdited {
 
     public:
@@ -362,26 +418,27 @@ class ColorManagementParamsEdited {
         bool input;
         bool toneCurve;
         bool blendCMSMatrix;
-        bool preferredProfile;
+        bool dcpIlluminant;
         bool working;
         bool output;
-		bool gamma;
-		bool gampos;
-		bool slpos;
-		bool gamfree;
-		bool freegamma;		
+        bool gamma;
+        bool gampos;
+        bool slpos;
+        bool gamfree;
+        bool freegamma;
 };
 
 class DirPyrEqualizerParamsEdited {
-	
-public:
-	bool enabled;
-	bool mult[8];
+
+    public:
+        bool enabled;
+        bool mult[5];
+        bool threshold;
 };
 
 class HSVEqualizerParamsEdited {
-	
-public:
+
+    public:
         bool hcurve;
         bool scurve;
         bool vcurve;
@@ -394,7 +451,7 @@ class RAWParamsEdited {
         bool dmethod;
         bool dcbIterations;
         bool dcbEnhance;
-		bool lmmseIterations;
+        bool lmmseIterations;
         //bool allEnhance;
         bool caCorrection;
         bool caRed;
@@ -449,10 +506,12 @@ class ParamsEdited {
         DistortionParamsEdited        distortion;
         LensProfParamsEdited          lensProf;
         PerspectiveParamsEdited       perspective;
+        GradientParamsEdited          gradient;
+        PCVignetteParamsEdited        pcvignette;
         CACorrParamsEdited            cacorrection;
         VignettingParamsEdited        vignetting;
         ChannelMixerParamsEdited      chmixer;
-        HRecParamsEdited              hlrecovery;
+        BlackWhiteParamsEdited        blackwhite;
         ResizeParamsEdited            resize;
         ColorManagementParamsEdited   icm;
         RAWParamsEdited               raw;
