@@ -40,9 +40,10 @@ class PartialPasteDlg : public Gtk::Dialog {
         // options in basic:
         Gtk::CheckButton* wb;
         Gtk::CheckButton* exposure;
-        Gtk::CheckButton* hlrec;
         Gtk::CheckButton* sh;
         Gtk::CheckButton* epd;
+        Gtk::CheckButton* pcvignette;
+        Gtk::CheckButton* gradient;
         Gtk::CheckButton* labcurve;
         Gtk::CheckButton* colorappearance;
 
@@ -59,6 +60,7 @@ class PartialPasteDlg : public Gtk::Dialog {
         // options in color:
         Gtk::CheckButton* vibrance;
         Gtk::CheckButton* chmixer;
+        Gtk::CheckButton* blackwhite;
         Gtk::CheckButton* hsveq;
         Gtk::CheckButton* rgbcurves;
        // Gtk::CheckButton* icm;
@@ -109,9 +111,9 @@ class PartialPasteDlg : public Gtk::Dialog {
 
         sigc::connection everythingConn, basicConn, detailConn, colorConn, lensConn, compositionConn, metaicmConn, rawConn;
 
-        sigc::connection wbConn, exposureConn, hlrecConn, shConn, labcurveConn, colorappearanceConn;
+        sigc::connection wbConn, exposureConn, shConn, pcvignetteConn, gradientConn, labcurveConn, colorappearanceConn;
         sigc::connection sharpenConn, gradsharpenConn, microcontrastConn, impdenConn, dirpyrdenConn, waveqConn, defringeConn, epdConn, dirpyreqConn;
-        sigc::connection vibranceConn, chmixerConn, hsveqConn, rgbcurvesConn;
+        sigc::connection vibranceConn, chmixerConn, hsveqConn, rgbcurvesConn, chmixerbwConn;
         sigc::connection distortionConn, cacorrConn, vignettingConn, lcpConn;
         sigc::connection coarserotConn, finerotConn, cropConn, resizeConn, perspectiveConn, commonTransConn;
         sigc::connection exifchConn, iptcConn, icmConn, gamcsconn;
