@@ -31,7 +31,7 @@
 extern Options options;
 extern Glib::ustring argv0;
 
-Preferences::Preferences  (RTWindow *rtwindow):parent(rtwindow)  {
+Preferences::Preferences  (RTWindow *rtwindow) : rprofiles(NULL), iprofiles(NULL), parent(rtwindow) {
   
 	splash = NULL;
 
@@ -89,6 +89,8 @@ Preferences::Preferences  (RTWindow *rtwindow):parent(rtwindow)  {
 #endif
     nb->set_current_page (0);
 
+    profileStore.addListener(this);
+
     fillPreferences ();
 
     show_all_children ();
@@ -98,6 +100,7 @@ Preferences::Preferences  (RTWindow *rtwindow):parent(rtwindow)  {
 
 Preferences::~Preferences () {
 
+    profileStore.removeListener(this);
     options.preferencesWidth = get_width();
     options.preferencesHeight = get_height();
 }
@@ -201,6 +204,7 @@ Gtk::Widget* Preferences::getBatchProcPanel () {
     mi->set_value (behavColumns.label, M("TP_WBALANCE_LABEL"));
     appendBehavList (mi, M("TP_WBALANCE_TEMPERATURE"), ADDSET_WB_TEMPERATURE, true);
     appendBehavList (mi, M("TP_WBALANCE_GREEN"), ADDSET_WB_GREEN, true);
+    appendBehavList (mi, M("TP_WBALANCE_EQBLUERED"), ADDSET_WB_EQUAL, true);
 
     mi = behModel->append ();
     mi->set_value (behavColumns.label, M("TP_COLORAPP_LABEL"));
@@ -231,6 +235,11 @@ Gtk::Widget* Preferences::getBatchProcPanel () {
     mi = behModel->append ();
     mi->set_value (behavColumns.label, M("TP_CHMIXER_LABEL"));
     appendBehavList (mi, M("TP_CHMIXER_RED")+", "+M("TP_CHMIXER_GREEN")+", "+M("TP_CHMIXER_BLUE"), ADDSET_CHMIXER, false);
+    
+	mi = behModel->append ();
+    mi->set_value (behavColumns.label, M("TP_BWMIX_LABEL"));
+    appendBehavList (mi, M("TP_BWMIX_MIXC"), ADDSET_BLACKWHITE_HUES, false);
+    appendBehavList (mi, M("TP_BWMIX_GAMMA"), ADDSET_BLACKWHITE_GAMMA, false);
 
     mi = behModel->append ();
     mi->set_value (behavColumns.label, M("TP_ROTATE_LABEL"));
@@ -244,6 +253,19 @@ Gtk::Widget* Preferences::getBatchProcPanel () {
     mi->set_value (behavColumns.label, M("TP_PERSPECTIVE_LABEL"));
     appendBehavList (mi, M("TP_PERSPECTIVE_HORIZONTAL")+", "+M("TP_PERSPECTIVE_VERTICAL"), ADDSET_PERSPECTIVE, false);
 
+    mi = behModel->append ();
+    mi->set_value (behavColumns.label, M("TP_GRADIENT_LABEL"));
+    appendBehavList (mi, M("TP_GRADIENT_DEGREE"), ADDSET_GRADIENT_DEGREE, false);
+    appendBehavList (mi, M("TP_GRADIENT_FEATHER"), ADDSET_GRADIENT_FEATHER, false);
+    appendBehavList (mi, M("TP_GRADIENT_STRENGTH"), ADDSET_GRADIENT_STRENGTH, false);
+    appendBehavList (mi, M("TP_GRADIENT_CENTER_X")+", "+M("TP_GRADIENT_CENTER_Y"), ADDSET_GRADIENT_CENTER, false);
+
+    mi = behModel->append ();
+    mi->set_value (behavColumns.label, M("TP_PCVIGNETTE_LABEL"));
+    appendBehavList (mi, M("TP_PCVIGNETTE_STRENGTH"), ADDSET_PCVIGNETTE_STRENGTH, false);
+    appendBehavList (mi, M("TP_PCVIGNETTE_FEATHER"), ADDSET_PCVIGNETTE_FEATHER, false);
+    appendBehavList (mi, M("TP_PCVIGNETTE_ROUNDNESS"), ADDSET_PCVIGNETTE_ROUNDNESS, false);
+
     mi = behModel->append ();
     mi->set_value (behavColumns.label, M("TP_CACORRECTION_LABEL"));
     appendBehavList (mi, M("TP_CACORRECTION_BLUE")+", "+M("TP_CACORRECTION_RED"), ADDSET_CA, true);
@@ -257,7 +279,8 @@ Gtk::Widget* Preferences::getBatchProcPanel () {
 
     mi = behModel->append ();
     mi->set_value (behavColumns.label, M("TP_DIRPYREQUALIZER_LABEL"));
-    appendBehavList (mi, M("TP_EXPOSURE_CONTRAST")+", "+M("TP_DIRPYREQUALIZER_THRESHOLD"), ADDSET_DIRPYREQ, true);
+    appendBehavList (mi, M("TP_EXPOSURE_CONTRAST"), ADDSET_DIRPYREQ, true);
+    appendBehavList (mi, M("TP_DIRPYREQUALIZER_THRESHOLD"), ADDSET_DIRPYREQ_THRESHOLD, true);
 
     mi = behModel->append ();
     mi->set_value (behavColumns.label, M("TP_PREPROCESS_LABEL"));
@@ -327,25 +350,47 @@ Gtk::Widget* Preferences::getProcParamsPanel () {
     Gtk::VBox* mvbpp = Gtk::manage (new Gtk::VBox ());
 
     Gtk::Frame* fpp = Gtk::manage (new Gtk::Frame (M("PREFERENCES_IMPROCPARAMS")));
+    Gtk::VBox* vbpp = Gtk::manage (new Gtk::VBox ());
+    vbpp->set_border_width(4);
     Gtk::Label* drlab = Gtk::manage (new Gtk::Label (M("PREFERENCES_FORRAW")+":", Gtk::ALIGN_LEFT));
-    rprofiles = Gtk::manage (new Gtk::ComboBoxText ());
+    rprofiles = Gtk::manage (new ProfileStoreComboBox ());
     rprofiles->set_size_request(50, -1);
-    rprofiles->signal_changed().connect( sigc::mem_fun(*this, &Preferences::forRAWComboChanged) );
-    forRAWComboChanged(); // update the tooltip
+    rpconn = rprofiles->signal_changed().connect( sigc::mem_fun(*this, &Preferences::forRAWComboChanged) );
     Gtk::Label* drimg = Gtk::manage (new Gtk::Label (M("PREFERENCES_FORIMAGE")+":", Gtk::ALIGN_LEFT));
-    iprofiles = Gtk::manage (new Gtk::ComboBoxText ());
+    iprofiles = Gtk::manage (new ProfileStoreComboBox ());
     iprofiles->set_size_request(50, -1);
-    iprofiles->signal_changed().connect( sigc::mem_fun(*this, &Preferences::forImageComboChanged) );
-    forImageComboChanged(); // update the tooltip
+    ipconn = iprofiles->signal_changed().connect( sigc::mem_fun(*this, &Preferences::forImageComboChanged) );
     Gtk::Table* defpt = Gtk::manage (new Gtk::Table (2, 2));
     defpt->attach (*drlab, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 2, 2);
     defpt->attach (*rprofiles, 1, 2, 0, 1, Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK, Gtk::SHRINK, 2, 2);
     defpt->attach (*drimg, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 2, 2);
     defpt->attach (*iprofiles, 1, 2, 1, 2, Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK, Gtk::SHRINK, 2, 2);
-    fpp->add (*defpt);
-
+    vbpp->pack_start (*defpt, Gtk::PACK_SHRINK, 4);
+    useBundledProfiles = Gtk::manage (new Gtk::CheckButton (M("PREFERENCES_USEBUNDLEDPROFILES")));
+    bpconn = useBundledProfiles->signal_clicked().connect ( sigc::mem_fun(*this, &Preferences::bundledProfilesChanged) );
+    vbpp->pack_start (*useBundledProfiles, Gtk::PACK_SHRINK, 4);
+    fpp->add (*vbpp);
     mvbpp->pack_start (*fpp, Gtk::PACK_SHRINK, 4);
 
+    // Custom profile builder box
+    Gtk::Frame* cpfrm = Gtk::manage( new Gtk::Frame (M("PREFERENCES_CUSTPROFBUILD")) );
+    Gtk::Label* cplab = Gtk::manage( new Gtk::Label (M("PREFERENCES_CUSTPROFBUILDPATH")+":", Gtk::ALIGN_LEFT) );
+    txtCustProfBuilderPath = Gtk::manage( new Gtk::Entry () );
+    txtCustProfBuilderPath->set_tooltip_markup (M("PREFERENCES_CUSTPROFBUILDHINT"));
+    Gtk::Label* cpltypelab = Gtk::manage( new Gtk::Label (M("PREFERENCES_CUSTPROFBUILDKEYFORMAT")+":", Gtk::ALIGN_LEFT) );
+    custProfBuilderLabelType = Gtk::manage (new Gtk::ComboBoxText ());
+    custProfBuilderLabelType->append_text (M("PREFERENCES_CUSTPROFBUILDKEYFORMAT_TID"));
+    custProfBuilderLabelType->append_text (M("PREFERENCES_CUSTPROFBUILDKEYFORMAT_NAME"));
+    custProfBuilderLabelType->append_text (M("PREFERENCES_CUSTPROFBUILDKEYFORMAT_TID") + "_" + M("PREFERENCES_CUSTPROFBUILDKEYFORMAT_NAME"));
+    Gtk::Table* cpbt = Gtk::manage (new Gtk::Table (2, 2));
+    cpbt->set_border_width(4);
+    cpbt->attach (*cplab, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 2, 2);
+    cpbt->attach (*txtCustProfBuilderPath, 1, 2, 0, 1, Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK, Gtk::SHRINK, 2, 2);
+    cpbt->attach (*cpltypelab, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 2, 2);
+    cpbt->attach (*custProfBuilderLabelType, 1, 2, 1, 2, Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK, Gtk::SHRINK, 2, 2);
+    cpfrm->add (*cpbt);
+    mvbpp->pack_start (*cpfrm, Gtk::PACK_SHRINK, 4);
+
     Gtk::Frame* fdp = Gtk::manage (new Gtk::Frame (M("PREFERENCES_PROFILEHANDLING")));
     Gtk::VBox* vbdp = Gtk::manage (new Gtk::VBox ());
     vbdp->set_border_width (4);
@@ -399,14 +444,6 @@ Gtk::Widget* Preferences::getProcParamsPanel () {
 
     //ffconn = flatFieldDir->signal_file_set().connect ( sigc::mem_fun(*this, &Preferences::flatFieldChanged), true);
     ffconn = flatFieldDir->signal_current_folder_changed().connect ( sigc::mem_fun(*this, &Preferences::flatFieldChanged), true);
-	
-    std::vector<Glib::ustring> pnames;
-    parseDir (options.getUserProfilePath(), pnames, paramFileExtension);
-    parseDir (options.getGlobalProfilePath(), pnames, paramFileExtension);
-    for (size_t i=0; i<pnames.size(); i++) {
-        rprofiles->append_text (pnames[i]);
-        iprofiles->append_text (pnames[i]);
-    }
 
     Gtk::Frame* fmd = Gtk::manage (new Gtk::Frame (M("PREFERENCES_METADATA")));
     Gtk::VBox* vbmd = Gtk::manage (new Gtk::VBox ());
@@ -470,9 +507,11 @@ Gtk::Widget* Preferences::getColorManagementPanel () {
     colt->attach (*intent, 1, 2, 0, 1, Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK, Gtk::SHRINK, 2, 2);
     colt->attach (*pdlabel, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 2, 2);
     colt->attach (*iccDir, 1, 2, 1, 2, Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK, Gtk::SHRINK, 2, 2);
+#if !defined(__APPLE__) // monitor profile not supported on apple
     colt->attach (*mplabel, 0, 1, 2, 3, Gtk::FILL, Gtk::SHRINK, 2, 2);
     colt->attach (*monProfile, 1, 2, 2, 3, Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK, Gtk::SHRINK, 2, 2);
     colt->attach (*cbAutoMonProfile, 1, 2, 3, 4, Gtk::EXPAND | Gtk::FILL | Gtk::SHRINK, Gtk::SHRINK, 2, 2);
+#endif
     mvbcm->pack_start (*colt, Gtk::PACK_SHRINK, 4);
 
     autoMonProfileToggled();
@@ -711,7 +750,6 @@ Gtk::Widget* Preferences::getGeneralPanel () {
     hb6->set_border_width (4);
     hb6->set_spacing (4);
     Gtk::Label* dflab = Gtk::manage( new Gtk::Label (M("PREFERENCES_DATEFORMAT")+":", Gtk::ALIGN_LEFT));
-    hb6->pack_start (*dflab, Gtk::PACK_SHRINK,4);
     dateformat = Gtk::manage( new Gtk::Entry () );
     dateformat->set_tooltip_markup (M("PREFERENCES_DATEFORMATHINT"));
     dflab->set_tooltip_markup (M("PREFERENCES_DATEFORMATHINT"));
@@ -795,26 +833,6 @@ Gtk::Widget* Preferences::getGeneralPanel () {
     mvbsd->pack_start (*fdg, Gtk::PACK_SHRINK, 4);
 
 
-    // Custom profile builder box
-    Gtk::Frame* cpfrm = Gtk::manage( new Gtk::Frame (M("PREFERENCES_CUSTPROFBUILD")) );
-
-    Gtk::HBox* cphb = Gtk::manage( new Gtk::HBox () );
-    cphb->set_border_width (4);
-    cphb->set_spacing (4);
-
-    Gtk::Label* cplab = Gtk::manage( new Gtk::Label (M("PREFERENCES_CUSTPROFBUILDPATH")+":") );
-    cphb->pack_start (*cplab, Gtk::PACK_SHRINK,4);
-
-    txtCustProfBuilderPath = Gtk::manage( new Gtk::Entry () );
-    txtCustProfBuilderPath->set_tooltip_markup (M("PREFERENCES_CUSTPROFBUILDHINT"));
-    cphb->set_tooltip_markup (M("PREFERENCES_CUSTPROFBUILDHINT"));
-    cphb->pack_start (*txtCustProfBuilderPath);
-    
-    cpfrm->add (*cphb);
-
-    mvbsd->pack_start (*cpfrm, Gtk::PACK_SHRINK, 4);
-
-
     mvbsd->set_border_width (4);
 
     tconn = theme->signal_changed().connect( sigc::mem_fun(*this, &Preferences::themeChanged) );
@@ -1067,15 +1085,20 @@ void Preferences::parseDir (Glib::ustring dirname, std::vector<Glib::ustring>& i
       if (!safe_file_test (fname, Glib::FILE_TEST_IS_DIR) && sname.size() >= ext.size() && sname.substr (sname.size()-ext.size(), ext.size()).casefold() == ext) 
             items.push_back (sname.substr(0,sname.size()-ext.size()));
     }
+    std::sort(items.begin(), items.end());
     delete dir;
 }
 
 void Preferences::storePreferences () {
 
-    moptions.defProfRaw          = rprofiles->get_active_text();
-    if (moptions.defProfRaw.empty()) moptions.defProfRaw = DEFPROFILE_RAW;
-    moptions.defProfImg          = iprofiles->get_active_text();
-    if (moptions.defProfImg.empty()) moptions.defProfImg = DEFPROFILE_IMG;
+    // With the new mechanism, we can't be sure of the availability of the DEFPROFILE_RAW & DEFPROFILE_IMG profiles,
+    // because useBundledProfiles may be false. We're now using DEFPROFILE_INTERNAL instead, which is always available.
+    moptions.defProfRaw          = rprofiles->getFullPathFromActiveRow();
+    if (moptions.defProfRaw.empty()) moptions.defProfRaw = DEFPROFILE_INTERNAL;
+    moptions.defProfImg          = iprofiles->getFullPathFromActiveRow();
+    if (moptions.defProfImg.empty()) moptions.defProfImg = DEFPROFILE_INTERNAL;
+
+
     moptions.dateFormat          = dateformat->get_text();
     moptions.panAccelFactor      = (int)panFactor->get_value();
     moptions.fbShowDateTime  = showDateTime->get_active ();
@@ -1121,7 +1144,8 @@ void Preferences::storePreferences () {
     else if (edOther->get_active ())
         moptions.editorToSendTo = 3;
 
-    moptions.customProfileBuilder = txtCustProfBuilderPath->get_text();
+    moptions.CPBPath = txtCustProfBuilderPath->get_text();
+    moptions.CPBKeys = CPBKeyType(custProfBuilderLabelType->get_active_row_number());
 
     moptions.rtSettings.monitorProfile      = monProfile->get_filename ();
     moptions.rtSettings.autoMonitorProfile  = cbAutoMonProfile->get_active ();
@@ -1159,6 +1183,7 @@ void Preferences::storePreferences () {
     moptions.saveParamsFile = saveParamsFile->get_active ();
     moptions.saveParamsCache = saveParamsCache->get_active ();
     moptions.paramsLoadLocation = (PPLoadLocation)loadParamsPreference->get_active_row_number ();
+    moptions.useBundledProfiles = useBundledProfiles->get_active ();
 
     moptions.tunnelMetaData = ckbTunnelMetaData->get_active ();
 
@@ -1200,16 +1225,23 @@ void Preferences::fillPreferences () {
     sconn.block (true);
     dfconn.block (true);
     ffconn.block (true);
+    rpconn.block(true);
+    ipconn.block(true);
+    bpconn.block(true);
 
-    rprofiles->set_active_text (moptions.defProfRaw);
-    iprofiles->set_active_text (moptions.defProfImg);
+    rprofiles->setActiveRowFromFullPath (moptions.defProfRaw);
+    forRAWComboChanged(); // update the tooltip
+    iprofiles->setActiveRowFromFullPath (moptions.defProfImg);
+    forImageComboChanged(); // update the tooltip
     dateformat->set_text (moptions.dateFormat);
     panFactor->set_value(moptions.panAccelFactor);
+#if !defined(__APPLE__) // monitor profile not supported on apple
     if (safe_file_test (moptions.rtSettings.monitorProfile, Glib::FILE_TEST_EXISTS)) 
         monProfile->set_filename (moptions.rtSettings.monitorProfile);
     if (moptions.rtSettings.monitorProfile.empty())
         monProfile->set_current_folder (moptions.rtSettings.iccDirectory);
     cbAutoMonProfile->set_active(moptions.rtSettings.autoMonitorProfile);
+#endif
 
     if (Glib::file_test (moptions.rtSettings.iccDirectory, Glib::FILE_TEST_IS_DIR)) 
         iccDir->set_current_folder (moptions.rtSettings.iccDirectory);
@@ -1258,7 +1290,9 @@ void Preferences::fillPreferences () {
 #endif
     editorToSendTo->set_text (moptions.customEditorProg);
 
-    txtCustProfBuilderPath->set_text(moptions.customProfileBuilder);
+    txtCustProfBuilderPath->set_text(moptions.CPBPath);
+    custProfBuilderLabelType->set_active(moptions.CPBKeys);
+
 
     if (moptions.startupDir==STARTUPDIR_CURRENT) 
         sdcurrent->set_active ();
@@ -1285,7 +1319,8 @@ void Preferences::fillPreferences () {
     
     saveParamsFile->set_active (moptions.saveParamsFile);
     saveParamsCache->set_active (moptions.saveParamsCache);
-    loadParamsPreference->set_active (moptions.paramsLoadLocation);    
+    loadParamsPreference->set_active (moptions.paramsLoadLocation);
+    useBundledProfiles->set_active (moptions.useBundledProfiles);
 
     ckbTunnelMetaData->set_active (moptions.tunnelMetaData); 
 
@@ -1331,6 +1366,9 @@ void Preferences::fillPreferences () {
     sconn.block (false);
     dfconn.block (false);
     ffconn.block (false);
+    rpconn.block(true);
+    ipconn.block(true);
+    bpconn.block(false);
 
     chOverwriteOutputFile->set_active (moptions.overwriteOutputFile);
 
@@ -1398,7 +1436,17 @@ void Preferences::cancelPressed () {
 	// set the initial font back
 	if (fontbutton->get_font_name() != options.font)
 		switchFontTo(options.font);
-    hide ();
+
+	// update the profileStore
+	if (useBundledProfiles->get_active () != options.useBundledProfiles) {
+		// we have to rescan with the old value;
+		bpconn.block(true);
+		useBundledProfiles->set_active (false);
+		bundledProfilesChanged();
+		bpconn.block(false);
+	}
+
+	hide ();
 }
 
 void Preferences::selectStartupDir () {
@@ -1434,17 +1482,94 @@ void Preferences::themeChanged () {
 }
 
 void Preferences::forRAWComboChanged () {
-	rprofiles->set_tooltip_text(rprofiles->get_active_text());
+	if (!rprofiles)
+		return;
+	const ProfileStoreEntry *selectedEntry = rprofiles->getSelectedEntry();
+	if (!selectedEntry)
+		return;
+
+	if (selectedEntry->type == PSET_FOLDER) {
+		rpconn.block(true);
+		rprofiles->set_active(currRawRow);
+		rpconn.block(false);
+	}
+	else
+		currRawRow = rprofiles->get_active();
+
+	rprofiles->set_tooltip_text(selectedEntry->label);
 }
 
 void Preferences::forImageComboChanged () {
-	iprofiles->set_tooltip_text(iprofiles->get_active_text());
+	if (!iprofiles)
+		return;
+	const ProfileStoreEntry *selectedEntry = iprofiles->getSelectedEntry();
+	if (!selectedEntry)
+		return;
+
+	if (selectedEntry->type == PSET_FOLDER) {
+		ipconn.block(true);
+		iprofiles->set_active(currImgRow);
+		ipconn.block(false);
+	}
+	else
+		currImgRow = rprofiles->get_active();
+	iprofiles->set_tooltip_text(iprofiles->getSelectedEntry()->label);
 }
 
 void Preferences::layoutComboChanged () {
 	editorLayout->set_tooltip_text(editorLayout->get_active_text());
 }
 
+void Preferences::bundledProfilesChanged () {
+	rpconn.block (true);
+	ipconn.block (true);
+
+	// parseProfiles does use options.useBundledProfiles, so we temporarily change its value
+	bool currValue = options.useBundledProfiles;
+	options.useBundledProfiles = useBundledProfiles->get_active ();
+
+	// rescan the file's tree
+	profileStore.parseProfiles(); // This will call Preferences::updateProfileList in return
+
+	// restoring back the old value
+	options.useBundledProfiles = currValue;
+
+	ipconn.block (false);
+	rpconn.block (false);
+}
+
+void Preferences::storeCurrentValue() {
+	// TODO: Find a way to get and restore the current selection; the following line can't work anymore
+	storedValueRaw = rprofiles->getFullPathFromActiveRow();
+	storedValueImg = iprofiles->getFullPathFromActiveRow();
+}
+
+void Preferences::updateProfileList() {
+	rprofiles->updateProfileList();
+	iprofiles->updateProfileList();
+}
+
+void Preferences::restoreValue() {
+	if (!rprofiles->setActiveRowFromFullPath(storedValueRaw)) {
+		moptions.defProfRaw = DEFPROFILE_INTERNAL;
+		rpconn.block(true);
+		rprofiles->setInternalEntry();
+		rpconn.block(false);
+	}
+	currRawRow = rprofiles->get_active();
+
+	if (!iprofiles->setActiveRowFromFullPath(storedValueImg)) {
+		moptions.defProfImg = DEFPROFILE_INTERNAL;
+		ipconn.block(true);
+		iprofiles->setInternalEntry();
+		ipconn.block(false);
+	}
+	currImgRow = iprofiles->get_active();
+
+	storedValueRaw = "";
+	storedValueImg = "";
+}
+
 void Preferences::fontChanged () {
 
 	switchFontTo(fontbutton->get_font_name());
