@@ -25,9 +25,19 @@
 class ProfileChangeListener {
 
     public:
+        virtual     ~ProfileChangeListener() {}
         virtual void profileChange  (const rtengine::procparams::PartialProfile* nparams, rtengine::ProcEvent event, const Glib::ustring& descr, const ParamsEdited* paramsEdited=NULL) {}
         virtual void setDefaults    (rtengine::procparams::ProcParams* defparams) {}
 };
 
+class BatchProfileChangeListener {
+
+    public:
+        virtual     ~BatchProfileChangeListener() {}
+        virtual void beginBatchProfileChange(int numberOfEntries) {}
+        virtual void endBatchProfileChange() {}
+};
+
+
 #endif
 
