@@ -32,20 +32,20 @@ ShadowsHighlights::ShadowsHighlights () : Gtk::VBox(), FoldableToolPanel(this) {
 
   pack_start (*Gtk::manage (new  Gtk::HSeparator()));
 
-  hq = Gtk::manage (new Gtk::CheckButton (M("GENERAL_HIGH_QUALITY")));
+  hq = Gtk::manage (new Gtk::CheckButton (M("TP_SHADOWSHLIGHTS_SHARPMASK")));
   hq->set_active (false);
   pack_start (*hq);
   hqConn = hq->signal_toggled().connect( sigc::mem_fun(*this, &ShadowsHighlights::hqChanged) );
 
   pack_start (*Gtk::manage (new  Gtk::HSeparator()));
-  highlights   = Gtk::manage (new Adjuster (M("TP_SHADOWSHLIGHTS_HIGHLIGHTS"), 0, 100, 1, 10));
+  highlights   = Gtk::manage (new Adjuster (M("TP_SHADOWSHLIGHTS_HIGHLIGHTS"), 0, 100, 1, 0));
   h_tonalwidth = Gtk::manage (new Adjuster (M("TP_SHADOWSHLIGHTS_HLTONALW"), 10, 100, 1, 80));
   pack_start (*highlights);
   pack_start (*h_tonalwidth);
 
   pack_start (*Gtk::manage (new  Gtk::HSeparator()));
 
-  shadows      = Gtk::manage (new Adjuster (M("TP_SHADOWSHLIGHTS_SHADOWS"), 0, 100, 1, 10));
+  shadows      = Gtk::manage (new Adjuster (M("TP_SHADOWSHLIGHTS_SHADOWS"), 0, 100, 1, 0));
   s_tonalwidth = Gtk::manage (new Adjuster (M("TP_SHADOWSHLIGHTS_SHTONALW"), 10, 100, 1, 80));
   pack_start (*shadows);
   pack_start (*s_tonalwidth);
