@@ -28,12 +28,21 @@ using namespace rtengine::procparams;
 
 
 SharpenMicro::SharpenMicro () : Gtk::VBox(), FoldableToolPanel(this) {
+
+	set_border_width(4);
+
 	enabled = Gtk::manage (new Gtk::CheckButton (M("GENERAL_ENABLED")));
 	enabled->set_active (true);
+	enabled->set_tooltip_markup (M("TP_SHARPENING_TOOLTIP"));
+	
 	pack_start(*enabled, Gtk::PACK_SHRINK, 0);
 	enabled->show ();
 
-	amount= Gtk::manage(new Adjuster (M("TP_SHARPENMICRO_AMOUNT"),0,100,1,25));
+	Gtk::HSeparator *hsep1 = Gtk::manage (new  Gtk::HSeparator());
+	hsep1->show ();
+	pack_start (*hsep1);
+
+	amount= Gtk::manage(new Adjuster (M("TP_SHARPENMICRO_AMOUNT"),0,100,1,20));
 	amount->setAdjusterListener (this);
 	if (amount->delay < 1000) amount->delay = 1000;
 	amount->show();
