@@ -25,7 +25,7 @@
 using namespace std;
 
 ThumbBrowserBase::ThumbBrowserBase () 
-    : lastClicked(NULL), previewHeight(options.thumbSize) {
+    : lastClicked(NULL), previewHeight(options.thumbSize), numOfCols(1) {
     inTabMode=false;  // corresponding to take thumbSize
     inW = -1; inH = -1;
 
@@ -58,19 +58,25 @@ ThumbBrowserBase::ThumbBrowserBase ()
 }
 
 void ThumbBrowserBase::scrollChanged () {
-	for (size_t i=0; i<fd.size(); i++)
+    {
+    #if PROTECT_VECTORS
+    MYWRITERLOCK(l, entryRW);
+    #endif
+
+    for (size_t i=0; i<fd.size(); i++)
         fd[i]->setOffset ((int)(hscroll.get_value()), (int)(vscroll.get_value()));
+    }
 
     internal.setPosition ((int)(hscroll.get_value()), (int)(vscroll.get_value()));
 
-    if (!internal.isDirty()) {    
+    if (!internal.isDirty()) {
         internal.setDirty ();
         internal.queue_draw ();
-//        gdk_window_process_updates (get_window()->gobj(), true);
     }
 }
 
 void ThumbBrowserBase::scroll (int direction) {
+    // GUI already acquired when here
     if (arrangement==TB_Vertical)
         vscroll.set_value (vscroll.get_value() + (direction==GDK_SCROLL_DOWN ? +1 : -1) * vscroll.get_adjustment()->get_step_increment());
     else
@@ -78,12 +84,300 @@ void ThumbBrowserBase::scroll (int direction) {
 }
 
 void ThumbBrowserBase::scrollPage (int direction) {
+    // GUI already acquired when here
     if (arrangement==TB_Vertical)
         vscroll.set_value (vscroll.get_value() + (direction==GDK_SCROLL_DOWN ? +1 : -1) * vscroll.get_adjustment()->get_page_increment());
     else
         hscroll.set_value (hscroll.get_value() + (direction==GDK_SCROLL_DOWN ? +1 : -1) * hscroll.get_adjustment()->get_page_increment());
 }
 
+static void scrollToEntry (double& h, double& v, int iw, int ih, ThumbBrowserEntryBase* entry) {
+    const int hmin = entry->getX ();
+    const int hmax = hmin + entry->getEffectiveWidth () - iw;
+    const int vmin = entry->getY ();
+    const int vmax = vmin + entry->getEffectiveHeight () - ih;
+
+    if (hmin < 0)
+        h += hmin;
+    else if (hmax > 0)
+        h += hmax;
+
+    if(vmin < 0)
+        v += vmin;
+    else if (vmax > 0)
+        v += vmax;
+}
+
+void ThumbBrowserBase::selectPrev (int distance, bool enlarge) {
+    double h, v;
+    getScrollPosition (h, v);
+
+    {
+    #if PROTECT_VECTORS
+    MYWRITERLOCK(l, entryRW);
+    #endif
+
+    if (!selected.empty ()) {
+        std::vector<ThumbBrowserEntryBase*>::iterator front = std::find (fd.begin (), fd.end (), selected.front ());
+        std::vector<ThumbBrowserEntryBase*>::iterator back = std::find (fd.begin (), fd.end (), selected.back ());
+        std::vector<ThumbBrowserEntryBase*>::iterator last = std::find (fd.begin (), fd.end (), lastClicked);
+
+        if (front > back)
+            std::swap(front, back);
+
+        std::vector<ThumbBrowserEntryBase*>::iterator& curr = last == front ? front : back;
+
+        // find next thumbnail at filtered distance before current
+        for (; curr >= fd.begin (); --curr) {
+            if (!(*curr)->filtered) {
+                if (distance-- == 0) {
+                    // clear current selection
+                    for (size_t i=0; i<selected.size (); ++i) {
+                        selected[i]->selected = false;
+                        redrawNeeded (selected[i]);
+                    }
+                    selected.clear ();
+
+                    // make sure the newly selected thumbnail is visible and make it current
+                    scrollToEntry (h, v, internal.get_width (), internal.get_height (), *curr);
+                    lastClicked = *curr;
+
+                    // either enlarge current selection or set new selection
+                    if(enlarge) {
+                        // reverse direction if distance is too large
+                        if(front > back)
+                            std::swap(front, back);
+
+                        for (; front <= back; ++front) {
+                            if (!(*front)->filtered) {
+                                (*front)->selected = true;
+                                redrawNeeded (*front);
+                                selected.push_back (*front);
+                            }
+                        }
+                    }
+                    else {
+                        (*curr)->selected = true;
+                        redrawNeeded (*curr);
+                        selected.push_back (*curr);
+                    }
+
+                    break;
+                }
+            }
+        }
+    }
+
+    #if PROTECT_VECTORS
+    MYWRITERLOCK_RELEASE(l);
+    #endif
+    selectionChanged ();
+    }
+
+    setScrollPosition (h, v);
+}
+
+void ThumbBrowserBase::selectNext (int distance, bool enlarge) {
+    double h, v;
+    getScrollPosition (h, v);
+
+    {
+    #if PROTECT_VECTORS
+    MYWRITERLOCK(l, entryRW);
+    #endif
+
+    if (!selected.empty ()) {
+        std::vector<ThumbBrowserEntryBase*>::iterator front = std::find (fd.begin (), fd.end (), selected.front ());
+        std::vector<ThumbBrowserEntryBase*>::iterator back = std::find (fd.begin (), fd.end (), selected.back ());
+        std::vector<ThumbBrowserEntryBase*>::iterator last = std::find (fd.begin (), fd.end (), lastClicked);
+
+        if (front > back)
+            std::swap(front, back);
+
+        std::vector<ThumbBrowserEntryBase*>::iterator& curr = last == back ? back : front;
+
+        // find next thumbnail at filtered distance after current
+        for (; curr < fd.end (); ++curr) {
+            if (!(*curr)->filtered) {
+                if (distance-- == 0) {
+                    // clear current selection
+                    for (size_t i=0; i<selected.size (); ++i) {
+                        selected[i]->selected = false;
+                        redrawNeeded (selected[i]);
+                    }
+                    selected.clear ();
+
+                    // make sure the newly selected thumbnail is visible and make it current
+                    scrollToEntry (h, v, internal.get_width (), internal.get_height (), *curr);
+                    lastClicked = *curr;
+
+                    // either enlarge current selection or set new selection
+                    if(enlarge) {
+                        // reverse direction if distance is too large
+                        if(front > back)
+                            std::swap(front, back);
+
+                        for (; front <= back; ++front) {
+                            if (!(*front)->filtered) {
+                                (*front)->selected = true;
+                                redrawNeeded (*front);
+                                selected.push_back (*front);
+                            }
+                        }
+                    }
+                    else {
+                        (*curr)->selected = true;
+                        redrawNeeded (*curr);
+                        selected.push_back (*curr);
+                    }
+
+                    break;
+                }
+            }
+        }
+    }
+
+    #if PROTECT_VECTORS
+    MYWRITERLOCK_RELEASE(l);
+    #endif
+    selectionChanged ();
+    }
+
+    setScrollPosition (h, v);
+}
+
+void ThumbBrowserBase::selectFirst (bool enlarge) {
+    double h, v;
+    getScrollPosition (h, v);
+
+    {
+    #if PROTECT_VECTORS
+    MYWRITERLOCK(l, entryRW);
+    #endif
+
+    if (!fd.empty ()) {
+        // find first unfiltered entry
+        std::vector<ThumbBrowserEntryBase*>::iterator first = fd.begin ();
+
+        for (; first < fd.end (); ++first) {
+            if (!(*first)->filtered) {
+                break;
+            }
+        }
+
+        scrollToEntry (h, v, internal.get_width (), internal.get_height (), *first);
+
+        ThumbBrowserEntryBase* lastEntry = lastClicked;
+        lastClicked = *first;
+
+        if(selected.empty ()) {
+            (*first)->selected = true;
+            redrawNeeded (*first);
+            selected.push_back (*first);
+        }
+        else {
+            std::vector<ThumbBrowserEntryBase*>::iterator back = std::find (fd.begin (), fd.end (), lastEntry ? lastEntry : selected.back ());
+
+            if (first > back)
+                std::swap(first, back);
+
+            // clear current selection
+            for (size_t i=0; i<selected.size (); ++i) {
+                selected[i]->selected = false;
+                redrawNeeded (selected[i]);
+            }
+            selected.clear ();
+
+            // either enlarge current selection or set new selection
+            for (; first <= back; ++first) {
+                if (!(*first)->filtered) {
+                    (*first)->selected = true;
+                    redrawNeeded (*first);
+                    selected.push_back (*first);
+                }
+
+                if (!enlarge)
+                    break;
+            }
+        }
+    }
+
+    #if PROTECT_VECTORS
+    MYWRITERLOCK_RELEASE(l);
+    #endif
+    selectionChanged ();
+    }
+
+    setScrollPosition (h, v);
+}
+
+void ThumbBrowserBase::selectLast (bool enlarge) {
+    double h, v;
+    getScrollPosition (h, v);
+
+    {
+    #if PROTECT_VECTORS
+    MYWRITERLOCK(l, entryRW);
+    #endif
+
+    if (!fd.empty ()) {
+        // find last unfiltered entry
+        std::vector<ThumbBrowserEntryBase*>::iterator last = fd.end () - 1;
+
+        for (; last >= fd.begin (); --last) {
+            if (!(*last)->filtered) {
+                break;
+            }
+        }
+
+        scrollToEntry (h, v, internal.get_width (), internal.get_height (), *last);
+
+        ThumbBrowserEntryBase* lastEntry = lastClicked;
+        lastClicked = *last;
+
+        if(selected.empty()) {
+            (*last)->selected = true;
+            redrawNeeded (*last);
+            selected.push_back (*last);
+        }
+        else {
+            std::vector<ThumbBrowserEntryBase*>::iterator front = std::find (fd.begin (), fd.end (), lastEntry ? lastEntry : selected.front ());
+
+            if (last < front)
+                std::swap(last, front);
+
+            // clear current selection
+            for (size_t i=0; i<selected.size (); ++i) {
+                selected[i]->selected = false;
+                redrawNeeded (selected[i]);
+            }
+            selected.clear ();
+
+            // either enlarge current selection or set new selection
+            for (; front <= last; --last) {
+                if (!(*last)->filtered) {
+                    (*last)->selected = true;
+                    redrawNeeded (*last);
+                    selected.push_back (*last);
+                }
+
+                if (!enlarge)
+                    break;
+            }
+
+            std::reverse(selected.begin (), selected.end ());
+        }
+    }
+
+    #if PROTECT_VECTORS
+    MYWRITERLOCK_RELEASE(l);
+    #endif
+    selectionChanged ();
+    }
+
+    setScrollPosition (h, v);
+}
+
 void ThumbBrowserBase::resizeThumbnailArea (int w, int h) {
 
     inW = w;
@@ -107,6 +401,9 @@ void ThumbBrowserBase::internalAreaResized (Gtk::Allocation& req) {
 
 void ThumbBrowserBase::configScrollBars () {
 
+    // HOMBRE:DELETE ME?
+    GThreadLock tLock; // Acquire the GUI
+
     if (inW>0 && inH>0) {
   
         int iw = internal.get_width ();
@@ -136,10 +433,14 @@ void ThumbBrowserBase::configScrollBars () {
 }
 
 void ThumbBrowserBase::arrangeFiles () {
-	// TODO: Check for Linux
-	#ifdef WIN32
-	Glib::RWLock::ReaderLock l(entryRW);
-	#endif
+
+    #if PROTECT_VECTORS
+    MYREADERLOCK(l, entryRW);
+    #endif
+
+    // GUI already locked by ::redraw, the only caller of this method for now.
+    // We could lock it one more time, there's no harm excepted (negligible) speed penalty
+    //GThreadLock lock;
 
     int N = fd.size ();
     // apply filter
@@ -152,8 +453,8 @@ void ThumbBrowserBase::arrangeFiles () {
         if (!fd[i]->filtered && fd[i]->getMinimalHeight() > rowHeight)
             rowHeight = fd[i]->getMinimalHeight ();
     
-    if (arrangement==TB_Horizontal) {            
-
+    if (arrangement==TB_Horizontal) {
+        numOfCols = 1;
         int numOfRows = 1;
 //        if (rowHeight>0) {
 //            numOfRows = (internal.get_height()+rowHeight/2)/rowHeight;
@@ -183,12 +484,16 @@ void ThumbBrowserBase::arrangeFiles () {
             }
             currx += maxw;
         }
+        #if PROTECT_VECTORS
+        MYREADERLOCK_RELEASE(l);
+        #endif
+        // This will require a Writer access
         resizeThumbnailArea (currx, numOfRows*rowHeight);
     }
     else {
         int availWidth = internal.get_width();
         // initial number of columns
-        int numOfCols = 0;
+        numOfCols = 0;
         int colsWidth = 0;
         for (int i=0; i<N; i++)
             if (!fd[i]->filtered && colsWidth + fd[i]->getMinimalWidth() <= availWidth) {
@@ -234,13 +539,17 @@ void ThumbBrowserBase::arrangeFiles () {
             if (currx>0) // there were thumbnails placed in the row
                 curry += rowHeight;
         }
+        #if PROTECT_VECTORS
+        MYREADERLOCK_RELEASE(l);
+        #endif
+        // This will require a Writer access
         resizeThumbnailArea (colsWidth, curry);
     }
 }
 
 
-void ThumbBrowserBase::Internal::on_realize()
-{
+void ThumbBrowserBase::Internal::on_realize() {
+  // Gtk signals automatically acquire the GUI (i.e. this method is enclosed by gdk_thread_enter and gdk_thread_leave)
   Cairo::FontOptions cfo;
   cfo.set_antialias (Cairo::ANTIALIAS_SUBPIXEL);
   get_pango_context()->set_cairo_font_options (cfo);
@@ -255,14 +564,22 @@ void ThumbBrowserBase::Internal::on_realize()
 }
 
 bool ThumbBrowserBase::Internal::on_query_tooltip (int x, int y, bool keyboard_tooltip, const Glib::RefPtr<Gtk::Tooltip>& tooltip) {
+    // Gtk signals automatically acquire the GUI (i.e. this method is enclosed by gdk_thread_enter and gdk_thread_leave)
     Glib::ustring ttip = "";
+
+    {
+    #if PROTECT_VECTORS
+    MYREADERLOCK(l, parent->entryRW);
+    #endif
+
     for (size_t i=0; i<parent->fd.size(); i++)
         if (parent->fd[i]->drawable && parent->fd[i]->inside (x, y)) {
             ttip = parent->fd[i]->getToolTip (x, y);
             break;
         }
+    }
     if (ttip!="") {
-        tooltip->set_text (ttip);
+        tooltip->set_markup (ttip);
         return true;
     }
     else
@@ -270,8 +587,8 @@ bool ThumbBrowserBase::Internal::on_query_tooltip (int x, int y, bool keyboard_t
 }
 
 void ThumbBrowserBase::styleChanged (const Glib::RefPtr<Gtk::Style>& style) {
-
-  refreshThumbImages ();
+    // GUI will be acquired by refreshThumbImages
+    refreshThumbImages ();
 }
 
 ThumbBrowserBase::Internal::Internal () : ofsX(0), ofsY(0), parent(NULL), dirty(true) {
@@ -287,10 +604,12 @@ void ThumbBrowserBase::Internal::setPosition (int x, int y) {
 }
 
 bool ThumbBrowserBase::Internal::on_key_press_event (GdkEventKey* event) {
+    // Gtk signals automatically acquire the GUI (i.e. this method is enclosed by gdk_thread_enter and gdk_thread_leave)
     return parent->keyPressed (event);
 }
 
 bool ThumbBrowserBase::Internal::on_button_press_event (GdkEventButton* event) {
+    // Gtk signals automatically acquire the GUI (i.e. this method is enclosed by gdk_thread_enter and gdk_thread_leave)
     grab_focus ();
 
     parent->eventTime = event->time;
@@ -310,30 +629,43 @@ bool ThumbBrowserBase::Internal::on_button_press_event (GdkEventButton* event) {
 }
 
 void ThumbBrowserBase::buttonPressed (int x, int y, int button, GdkEventType type, int state, int clx, int cly, int clw, int clh) {
+    // GUI already acquired
+
     ThumbBrowserEntryBase* fileDescr = NULL;
     bool handled = false;
-	
+
     {
-	    for (size_t i=0; i<fd.size(); i++)
+    #if PROTECT_VECTORS
+    MYREADERLOCK(l, entryRW);
+    #endif
+        for (size_t i=0; i<fd.size(); i++)
             if (fd[i]->drawable) {
-                if (fd[i]->inside (x, y) && fd[i]->insideWindow (clx, cly, clw, clh))            
+                if (fd[i]->inside (x, y) && fd[i]->insideWindow (clx, cly, clw, clh))
                     fileDescr = fd[i];
                 bool b = fd[i]->pressNotify (button, type, state, x, y);
                 handled = handled || b;
-            }   
+            }
     }
 
     if (handled || (fileDescr && fileDescr->processing))
         return;
-        
+
+    {
+    #if PROTECT_VECTORS
+    MYWRITERLOCK(l, entryRW);
+    #endif
+
     if (selected.size()==1 && type==GDK_2BUTTON_PRESS && button==1) 
         doubleClicked (selected[0]);
     else if (button==1 && type==GDK_BUTTON_PRESS) {
-        if (fileDescr && state & GDK_SHIFT_MASK) {
+        if (fileDescr && (state & GDK_SHIFT_MASK)) {
             if (selected.empty()) {
                 selected.push_back (fileDescr);
                 fileDescr->selected = true;
                 lastClicked = fileDescr;
+                #if PROTECT_VECTORS
+                MYWRITERLOCK_RELEASE(l);
+                #endif
                 selectionChanged ();
             }
             else {
@@ -363,16 +695,20 @@ void ThumbBrowserBase::buttonPressed (int x, int y, int button, GdkEventType typ
                     selected[i]->selected = false;
                 selected.clear ();
                 // select thumbnails in the interval
-		for (size_t i=startx; i<=endx; i++) {
+                for (size_t i=startx; i<=endx; i++) {
                     if (!fd[i]->filtered) {
                         fd[i]->selected = true;
                         selected.push_back (fd[i]);
                     }
                 }
+                lastClicked = fileDescr;
+                #if PROTECT_VECTORS
+                MYWRITERLOCK_RELEASE(l);
+                #endif
                 selectionChanged ();
             }
         }
-        else if (fileDescr && state & GDK_CONTROL_MASK) {
+        else if (fileDescr && (state & GDK_CONTROL_MASK)) {
             std::vector<ThumbBrowserEntryBase*>::iterator i = std::find (selected.begin(), selected.end(), fileDescr);
             if (i!=selected.end()) {
                 (*i)->selected = false;
@@ -383,10 +719,13 @@ void ThumbBrowserBase::buttonPressed (int x, int y, int button, GdkEventType typ
                 fileDescr->selected = true;
             }
             lastClicked = fileDescr;
+            #if PROTECT_VECTORS
+            MYWRITERLOCK_RELEASE(l);
+            #endif
             selectionChanged ();
         }
         else {
-		for (size_t i=0; i<selected.size(); i++)
+            for (size_t i=0; i<selected.size(); i++)
                 selected[i]->selected = false;
             selected.clear ();
             if (fileDescr) {
@@ -394,28 +733,36 @@ void ThumbBrowserBase::buttonPressed (int x, int y, int button, GdkEventType typ
                 fileDescr->selected = true;
             }
             lastClicked = fileDescr;
+            #if PROTECT_VECTORS
+            MYWRITERLOCK_RELEASE(l);
+            #endif
             selectionChanged ();
         }
     }
     else if (fileDescr && button==3 && type==GDK_BUTTON_PRESS) {
         if (!fileDescr->selected) {
-		for (size_t i=0; i<selected.size(); i++)
+            for (size_t i=0; i<selected.size(); i++)
                 selected[i]->selected = false;
             selected.clear ();
             fileDescr->selected = true;
             selected.push_back (fileDescr);
             lastClicked = fileDescr;
+            #if PROTECT_VECTORS
+            MYWRITERLOCK_RELEASE(l);
+            #endif
             selectionChanged ();
         }
+        #if PROTECT_VECTORS
+        MYWRITERLOCK_RELEASE(l);
+        #endif
         rightClicked (fileDescr);
     }
+    } // end of MYWRITERLOCK(l, entryRW);
+
 }
 
 bool ThumbBrowserBase::Internal::on_expose_event(GdkEventExpose* event) {
-    // TODO: Check for Linux
-	#ifdef WIN32
-	Glib::RWLock::ReaderLock l(parent->entryRW);
-	#endif
+    // Gtk signals automatically acquire the GUI (i.e. this method is enclosed by gdk_thread_enter and gdk_thread_leave)
 
     dirty = false;
 
@@ -428,6 +775,12 @@ bool ThumbBrowserBase::Internal::on_expose_event(GdkEventExpose* event) {
     // draw thumbnails
     Glib::RefPtr<Pango::Context> context = get_pango_context ();
     context->set_font_description (get_style()->get_font());
+
+    {
+    #if PROTECT_VECTORS
+    MYWRITERLOCK(l, parent->entryRW);
+    #endif
+
     for (size_t i=0; i<parent->fd.size() && !dirty; i++) {  // if dirty meanwhile, cancel and wait for next redraw
         if (!parent->fd[i]->drawable || !parent->fd[i]->insideWindow (0, 0, w, h))
             parent->fd[i]->updatepriority = false;
@@ -436,36 +789,56 @@ bool ThumbBrowserBase::Internal::on_expose_event(GdkEventExpose* event) {
             parent->fd[i]->draw ();
         }
     }
+    }
 
     return true;
 }
 
 bool ThumbBrowserBase::Internal::on_button_release_event (GdkEventButton* event) {
+    // Gtk signals automatically acquire the GUI (i.e. this method is enclosed by gdk_thread_enter and gdk_thread_leave)
     int w = get_width();
     int h = get_height();
 
+    #if PROTECT_VECTORS
+    MYREADERLOCK(l, parent->entryRW);
+    #endif
+
     for (size_t i=0; i<parent->fd.size(); i++)
         if (parent->fd[i]->drawable && parent->fd[i]->insideWindow (0, 0, w, h)) {
-            parent->fd[i]->releaseNotify (event->button, event->type, event->state, (int)event->x, (int)event->y);
+            ThumbBrowserEntryBase* tbe = parent->fd[i];
+            #if PROTECT_VECTORS
+            MYREADERLOCK_RELEASE(l);
+            #endif
+            // This will require a Writer access...
+            tbe->releaseNotify (event->button, event->type, event->state, (int)event->x, (int)event->y);
+            #if PROTECT_VECTORS
+            MYREADERLOCK_ACQUIRE(l);
+            #endif
         }
     return true;
 }
 
 bool ThumbBrowserBase::Internal::on_motion_notify_event (GdkEventMotion* event) {
+    // Gtk signals automatically acquire the GUI (i.e. this method is enclosed by gdk_thread_enter and gdk_thread_leave)
     int w = get_width();
     int h = get_height();
 
+    #if PROTECT_VECTORS
+    MYREADERLOCK(l, parent->entryRW);
+    #endif
+
     for (size_t i=0; i<parent->fd.size(); i++)
         if (parent->fd[i]->drawable && parent->fd[i]->insideWindow (0, 0, w, h)) {
-            #ifdef WIN32
-	        //l.release();  // motionNotify calls the queue, which locks
-	        #endif
+            /*#if PROTECT_VECTORS
+            MYREADERLOCK_RELEASE(l); // motionNotify calls the queue, which locks
+            #endif*/
             parent->fd[i]->motionNotify ((int)event->x, (int)event->y);
         }
     return true;
 }
 
 bool ThumbBrowserBase::Internal::on_scroll_event (GdkEventScroll* event) {
+    // Gtk signals automatically acquire the GUI (i.e. this method is enclosed by gdk_thread_enter and gdk_thread_leave)
 
     parent->scroll (event->direction);
     return true;
@@ -474,6 +847,7 @@ bool ThumbBrowserBase::Internal::on_scroll_event (GdkEventScroll* event) {
 
 void ThumbBrowserBase::redraw () {
 
+    GThreadLock lock;
     arrangeFiles ();
     queue_draw ();
 }
@@ -481,30 +855,31 @@ void ThumbBrowserBase::redraw () {
 void ThumbBrowserBase::zoomChanged (bool zoomIn) {
 
     int newHeight=0;
-    int optThumbSize=getCurrentThumbSize();
+    int optThumbSize=getThumbnailHeight();
     if (zoomIn)
-	    for (size_t i=0; i<options.thumbnailZoomRatios.size(); i++) {
+        for (size_t i=0; i<options.thumbnailZoomRatios.size(); i++) {
             newHeight = (int)(options.thumbnailZoomRatios[i] * getMaxThumbnailHeight());
             if (newHeight > optThumbSize)
                 break;
         }
     else
-	    for (size_t i=options.thumbnailZoomRatios.size()-1; i>0; i--) {
+        for (size_t i=options.thumbnailZoomRatios.size()-1; i>0; i--) {
             newHeight = (int)(options.thumbnailZoomRatios[i] * getMaxThumbnailHeight());
             if (newHeight < optThumbSize)
                 break;
         }
     previewHeight = newHeight;
-    if (inTabMode) options.thumbSizeTab = newHeight; else options.thumbSize = newHeight;
 
-	{
-		// TODO: Check for Linux
-		#ifdef WIN32
-		Glib::RWLock::WriterLock l(entryRW);
-		#endif
+    saveThumbnailHeight(newHeight);
 
-		for (size_t i=0; i<fd.size(); i++) fd[i]->resize (previewHeight);
-	}
+    {
+        #if PROTECT_VECTORS
+        MYWRITERLOCK(l, entryRW);
+        #endif
+
+        for (size_t i=0; i<fd.size(); i++)
+            fd[i]->resize (previewHeight);
+    }
 
     redraw ();
 #ifdef WIN32
@@ -512,44 +887,46 @@ void ThumbBrowserBase::zoomChanged (bool zoomIn) {
 #endif    
 }
 
-int ThumbBrowserBase::getCurrentThumbSize() { return inTabMode ? options.thumbSizeTab : options.thumbSize; }
-
 void ThumbBrowserBase::refreshThumbImages () {
-	{
-		// TODO: Check for Linux
-		#ifdef WIN32
-		Glib::RWLock::WriterLock l(entryRW);
-		#endif
 
-        int previewHeight = getCurrentThumbSize();
-	for (size_t i=0; i<fd.size(); i++) fd[i]->resize (previewHeight);
-	}
+    int previewHeight = getThumbnailHeight();
+    {
+        #if PROTECT_VECTORS
+        MYWRITERLOCK(l, entryRW);
+        #endif
+
+        for (size_t i=0; i<fd.size(); i++) fd[i]->resize (previewHeight);
+    }
 
     redraw ();
 }
 
 void ThumbBrowserBase::refreshQuickThumbImages () {
-	// TODO: Check for Linux
-	#ifdef WIN32
-	Glib::RWLock::WriterLock l(entryRW);
-	#endif
+    #if PROTECT_VECTORS
+    MYWRITERLOCK(l, entryRW);
+    #endif
 
-	for (size_t i=0; i<fd.size(); ++i) fd[i]->refreshQuickThumbnailImage ();
+    for (size_t i=0; i<fd.size(); ++i) fd[i]->refreshQuickThumbnailImage ();
 }
 
 void ThumbBrowserBase::refreshEditedState (const std::set<Glib::ustring>& efiles) {
 
     editedFiles = efiles;
+    {
+    #if PROTECT_VECTORS
+    MYREADERLOCK(l, entryRW);
+    #endif
+
     for (size_t i=0; i<fd.size(); i++)
         fd[i]->framed = editedFiles.find (fd[i]->filename)!=editedFiles.end();
+    }
 
     queue_draw ();
 }
-    
 
 void ThumbBrowserBase::setArrangement (Arrangement a) {
 
-    arrangement = a;    
+    arrangement = a;
     redraw ();
 }
 
@@ -557,31 +934,40 @@ void ThumbBrowserBase::enableTabMode(bool enable) {
     inTabMode = enable;
     arrangement = inTabMode ? ThumbBrowserBase::TB_Horizontal : ThumbBrowserBase::TB_Vertical;
     
-    if (options.thumbSizeTab!=options.thumbSize) {
-		// TODO: Check for Linux
-		#ifdef WIN32
-		Glib::RWLock::WriterLock l(entryRW);
-		#endif
+    if (!options.sameThumbSize && (options.thumbSizeTab!=options.thumbSize)) {
+        #if PROTECT_VECTORS
+        MYWRITERLOCK(l, entryRW);
+        #endif
 
-		for (size_t i=0; i<fd.size(); i++)
-            fd[i]->resize (getCurrentThumbSize());
+        for (size_t i=0; i<fd.size(); i++)
+            fd[i]->resize (getThumbnailHeight());
     }
 
     redraw ();
 
     // Scroll to selected position if going into ribbon mode or back
     // Tab mode is horizontal, file browser is vertical
+    {
+    #if PROTECT_VECTORS
+    MYREADERLOCK(l, entryRW);
+    #endif
+
     if (!selected.empty()) {
         if (inTabMode) {
             double h=selected[0]->getStartX();
+            #if PROTECT_VECTORS
+            MYREADERLOCK_RELEASE(l);
+            #endif
             hscroll.set_value (min(h, hscroll.get_adjustment()->get_upper()));
         } else {
             double v=selected[0]->getStartY();
+            #if PROTECT_VECTORS
+            MYREADERLOCK_RELEASE(l);
+            #endif
             vscroll.set_value (min(v, vscroll.get_adjustment()->get_upper()));
         }
     }
-
- 
+    }
 }
 
 void ThumbBrowserBase::initEntry (ThumbBrowserEntryBase* entry) {
@@ -602,6 +988,10 @@ void ThumbBrowserBase::setScrollPosition (double h, double v) {
 int ThumbBrowserBase::getEffectiveHeight() { 
     int h=hscroll.get_height() + 2;  // have 2 pixels rounding error for scroll bars to appear
 
+    #if PROTECT_VECTORS
+    MYREADERLOCK(l, entryRW);
+    #endif
+
     // Filtered items do not change in size, so take a non-filtered
     for (size_t i=0;i<fd.size();i++)
         if (!fd[i]->filtered) {
@@ -610,10 +1000,13 @@ int ThumbBrowserBase::getEffectiveHeight() {
         }
 
     return h;
-}  
+}
 
 void ThumbBrowserBase::redrawNeeded (ThumbBrowserEntryBase* entry) {
 
+    // HOMBRE:DELETE ME?
+    GThreadLock tLock; // Acquire the GUI
+
     if (entry->insideWindow (0, 0, internal.get_width(), internal.get_height())) {
         if (!internal.isDirty ()) {
             internal.setDirty ();
