@@ -22,6 +22,7 @@
 #include <gtkmm.h>
 #include "lwbuttonset.h"
 #include "thumbnail.h"
+#include "threadutils.h"
 
 class ThumbBrowserBase;
 class ThumbBrowserEntryBase {
@@ -42,7 +43,7 @@ protected:
     int lowerMargin;
 
     
-    Glib::RWLock lockRW;  // Locks access to all image thumb changing actions
+    MyRWMutex lockRW;  // Locks access to all image thumb changing actions
 
     guint8* preview;  // holds the preview image. used in updateBackBuffer. TODO Olli: Make a cache to reduce mem significantly
 
@@ -109,10 +110,13 @@ protected:
     int getMinimalHeight    () { return height; }
     int getMinimalWidth     () { return width; }
 
+    int getEffectiveWidth   () const { return exp_width; }
     int getEffectiveHeight  () const { return exp_height; }
     int getPreviewHeight    () const { return preh; }
     int getStartX           () const { return startx; }
     int getStartY           () const { return starty; }
+    int getX                () const { return ofsX+startx; }
+    int getY                () const { return ofsY+starty; }
 
     bool inside             (int x, int y);
     bool insideWindow       (int x, int y, int w, int h);
