@@ -33,13 +33,15 @@
 
 using namespace rtengine::procparams;
 
-Thumbnail::Thumbnail (CacheManager* cm, const Glib::ustring& fname, CacheImageData* cf) 
+Thumbnail::Thumbnail (CacheManager* cm, const Glib::ustring& fname, CacheImageData* cf)
     : fname(fname), cfs(*cf), cachemgr(cm), ref(1), enqueueNumber(0), tpp(NULL),
       pparamsValid(false), needsReProcessing(true),imageLoading(false), lastImg(NULL),
-		lastW(0), lastH(0), lastScale(0), initial_(false) {
+      lastW(0), lastH(0), lastScale(0), initial_(false)
+{
 
-    cfs.load (getCacheFileName ("data")+".txt");
     loadProcParams ();
+
+    // should be safe to use the unprotected version of loadThumbnail, since we are in the constructor
     _loadThumbnail ();
     generateExifDateTimeStrings ();
 
@@ -55,25 +57,26 @@ Thumbnail::Thumbnail (CacheManager* cm, const Glib::ustring& fname, CacheImageDa
         setStage(cfs.inTrashOld);
     }
 
- 	delete tpp;
- 	tpp = 0;
+    delete tpp;
+    tpp = 0;
 }
 
 Thumbnail::Thumbnail (CacheManager* cm, const Glib::ustring& fname, const std::string& md5)
     : fname(fname), cachemgr(cm), ref(1), enqueueNumber(0), tpp(NULL), pparamsValid(false),
       needsReProcessing(true),imageLoading(false), lastImg(NULL),
-		initial_(true) {
+      initial_(true)
+{
 
 
     cfs.md5 = md5;
-    _generateThumbnailImage ();
     loadProcParams ();
+    _generateThumbnailImage ();
     cfs.recentlySaved = false;
 
-	initial_ = false;
- 
- 	delete tpp;
- 	tpp = 0;
+    initial_ = false;
+
+    delete tpp;
+    tpp = 0;
 }
 
 void Thumbnail::_generateThumbnailImage () {
@@ -96,18 +99,19 @@ void Thumbnail::_generateThumbnailImage () {
 	cfs.timeValid = false;
 
 	if (ext.lowercase()=="jpg" || ext.lowercase()=="jpeg") {
-		tpp = rtengine::Thumbnail::loadFromImage (fname, tw, th, 1, infoFromImage (fname));
+		infoFromImage (fname);
+		tpp = rtengine::Thumbnail::loadFromImage (fname, tw, th, 1, pparams.wb.equal);
 		if (tpp)
 			cfs.format = FT_Jpeg;
 	}
 	else if (ext.lowercase()=="png") {
-		tpp = rtengine::Thumbnail::loadFromImage (fname, tw, th, 1);
+		tpp = rtengine::Thumbnail::loadFromImage (fname, tw, th, 1, pparams.wb.equal);
 		if (tpp)
 			cfs.format = FT_Png;
 	}
 	else if (ext.lowercase()=="tif" || ext.lowercase()=="tiff") {
-		int deg = infoFromImage (fname);
-		tpp = rtengine::Thumbnail::loadFromImage (fname, tw, th, 1, deg);
+		infoFromImage (fname);
+		tpp = rtengine::Thumbnail::loadFromImage (fname, tw, th, 1, pparams.wb.equal);
 		if (tpp)
 			cfs.format = FT_Tiff;
 	}
@@ -126,7 +130,7 @@ void Thumbnail::_generateThumbnailImage () {
 		if ( tpp == NULL )
 		{
 			quick = false;
-			tpp = rtengine::Thumbnail::loadFromRaw (fname, ri, tw, th, 1, TRUE);
+			tpp = rtengine::Thumbnail::loadFromRaw (fname, ri, tw, th, 1, pparams.wb.equal, TRUE);
 		}
 		if (tpp) {
 			cfs.format = FT_Raw;
@@ -136,7 +140,8 @@ void Thumbnail::_generateThumbnailImage () {
 	}
 
     if (tpp)
-	{
+    {
+        tpp->getAutoWBMultipliers(cfs.redAWBMul, cfs.greenAWBMul, cfs.blueAWBMul);
         _saveThumbnail ();
         cfs.supported = true;
         needsReProcessing = true;
@@ -152,11 +157,12 @@ bool Thumbnail::isSupported () {
 }
 
 const ProcParams& Thumbnail::getProcParams () {
-	// TODO: Check for Linux
-	#ifdef WIN32
-	Glib::Mutex::Lock lock(mutex);
-	#endif
+    MyMutex::MyLock lock(mutex);
+    return getProcParamsU();
+}
 
+// Unprotected version of getProcParams, when
+const ProcParams& Thumbnail::getProcParamsU () {
     if (pparamsValid)
         return pparams;
     else {
@@ -168,18 +174,29 @@ const ProcParams& Thumbnail::getProcParams () {
         }
         else if (pparams.wb.method=="Auto") {
             double ct;
-            getAutoWB (ct, pparams.wb.green);
+            getAutoWB (ct, pparams.wb.green, pparams.wb.equal);
             pparams.wb.temperature = ct;
         }
     }
     return pparams; // there is no valid pp to return, but we have to return something
 }
 
-/*
- *  Create default params on demand and returns a new updatable object
+/** @brief  Create default params on demand and returns a new updatable object
+ *
  *  The loaded profile may be partial, but it return a complete ProcParams (i.e. without ParamsEdited)
+ *
+ *  @param returnParams Ask to return a pointer to a ProcParams object if true
+ *  @param forceCPB True if the Custom Profile Builder has to be invoked, False if the CPB has to be invoked if the profile doesn't
+ *                  exist yet. It depends on other conditions too
+ *  @param flaggingMode True if the ProcParams will be created because the file browser is being flagging an image
+ *                      (rang, to trash, color labels). This parameter is passed to the CPB.
+ *
+ *  @return Return a pointer to a ProcPamas structure to be updated if returnParams is true and if everything went fine, NULL otherwise.
  */
-rtengine::procparams::ProcParams* Thumbnail::createProcParamsForUpdate(bool returnParams, bool forceCPB) {
+rtengine::procparams::ProcParams* Thumbnail::createProcParamsForUpdate(bool returnParams, bool forceCPB, bool flaggingMode) {
+
+    static int index=0; // Will act as unique identifier during the session
+
     // try to load the last saved parameters from the cache or from the paramfile file
     ProcParams* ldprof = NULL;
 
@@ -187,21 +204,44 @@ rtengine::procparams::ProcParams* Thumbnail::createProcParamsForUpdate(bool retu
 
     const CacheImageData* cfs=getCacheImageData();
     Glib::ustring defaultPparamsPath = options.findProfilePath(defProf);
-    if (!options.customProfileBuilder.empty() && !defaultPparamsPath.empty() && (!hasProcParams() || forceCPB) && cfs && cfs->exifValid) {
-        // For the filename etc. do NOT use streams, since they are not UTF8 safe
-        Glib::ustring cmdLine=Glib::ustring("\"") + options.customProfileBuilder + Glib::ustring("\" \"") + fname + Glib::ustring("\" \"")
-        + Glib::build_filename(defaultPparamsPath, defProf + paramFileExtension) + Glib::ustring("\" ");
+    if (!options.CPBPath.empty() && !defaultPparamsPath.empty() && (!hasProcParams() || forceCPB) && cfs && cfs->exifValid) {
+        // First generate the communication file, with general values and EXIF metadata
+        rtengine::ImageMetaData* imageMetaData;
+        if (getType()==FT_Raw) {
+            rtengine::RawMetaDataLocation metaData = rtengine::Thumbnail::loadMetaDataFromRaw(fname);
+            imageMetaData = rtengine::ImageMetaData::fromFile (fname, &metaData);
+        }
+        else
+            imageMetaData = rtengine::ImageMetaData::fromFile (fname, NULL);
 
-        // ustring doesn't know int etc formatting, so take these via (unsafe) stream
-        std::ostringstream strm;
-        strm << cfs->fnumber << Glib::ustring(" ") << cfs->shutter << Glib::ustring(" ");
-        strm << cfs->focalLen << Glib::ustring(" ") << cfs->iso << Glib::ustring(" \"");
-        strm << cfs->lens << Glib::ustring("\" \"") << cfs->camera << Glib::ustring("\"");
- 
-        bool success = safe_spawn_command_line_sync (cmdLine + strm.str());
+        Glib::ustring tmpFileName( Glib::build_filename(options.cacheBaseDir, Glib::ustring::compose("CPB_temp_%1.txt", index++)) );
+
+        const rtexif::TagDirectory* exifDir=NULL;
+        if (imageMetaData && (exifDir = imageMetaData->getExifData())) {
+            Glib::ustring outFName;
+            if (options.paramsLoadLocation==PLL_Input)
+                outFName = fname+paramFileExtension;
+            else
+                outFName = getCacheFileName("profiles")+paramFileExtension;
+            exifDir->CPBDump(tmpFileName, fname, outFName,
+                             defaultPparamsPath == DEFPROFILE_INTERNAL ? DEFPROFILE_INTERNAL : Glib::build_filename(defaultPparamsPath, Glib::path_get_basename(defProf) + paramFileExtension),
+                             cfs,
+                             flaggingMode);
+        }
+
+        // For the filename etc. do NOT use streams, since they are not UTF8 safe
+        Glib::ustring cmdLine = options.CPBPath + Glib::ustring(" \"") + tmpFileName + Glib::ustring("\"");
+
+        if (options.rtSettings.verbose)
+            printf("Custom profile builder's command line: %s\n", Glib::ustring(cmdLine).c_str());
+        bool success = safe_spawn_command_line_sync (cmdLine);
 
         // Now they SHOULD be there (and potentially "partial"), so try to load them and store it as a full procparam
         if (success) loadProcParams();
+
+        if (safe_file_test(tmpFileName, Glib::FILE_TEST_EXISTS )) safe_g_remove (tmpFileName);
+
+        if (imageMetaData) delete imageMetaData;
     }
 
     if (returnParams && hasProcParams()) {
@@ -226,10 +266,7 @@ void Thumbnail::notifylisterners_procParamsChanged(int whoChangedIt){
  * ProcParams (sidecar or cache file).
  */
 void Thumbnail::loadProcParams () {
-	// TODO: Check for Linux
-	#ifdef WIN32
-	Glib::Mutex::Lock lock(mutex);
-	#endif
+    MyMutex::MyLock lock(mutex);
 
     pparamsValid = false;
     pparams.setDefaults();
@@ -265,11 +302,9 @@ void Thumbnail::clearProcParams (int whoClearedIt) {
     the "clear profile" will lead to execution of ProcParams::setDefaults 
     (the CPB is NOT called) to set the params values and will preserve 
     rank/colorlabel/inTrash in the param file. */
-    
-	// TODO: Check for Linux
-	#ifdef WIN32
-	Glib::Mutex::Lock lock(mutex);
-	#endif
+
+	{
+    MyMutex::MyLock lock(mutex);
 
     // preserve rank, colorlabel and inTrash across clear
     int rank = getRank();
@@ -313,6 +348,8 @@ void Thumbnail::clearProcParams (int whoClearedIt) {
             safe_g_remove (fname_);
     }
 
+	} // end of mutex lock
+
     for (size_t i=0; i<listeners.size(); i++)
         listeners[i]->procParamsChanged (this, whoClearedIt);
 }
@@ -323,15 +360,14 @@ bool Thumbnail::hasProcParams () {
 }
 
 void Thumbnail::setProcParams (const ProcParams& pp, ParamsEdited* pe, int whoChangedIt, bool updateCacheNow) {
-	// TODO: Check for Linux
-	#ifdef WIN32
-	Glib::Mutex::Lock lock(mutex);
-	#endif
-    
-	if (pparams.sharpening.threshold.isDouble() != pp.sharpening.threshold.isDouble())
-		printf("WARNING: Sharpening different!\n");
-	if (pparams.vibrance.psthreshold.isDouble() != pp.vibrance.psthreshold.isDouble())
-		printf("WARNING: Vibrance different!\n");
+
+	{
+    MyMutex::MyLock lock(mutex);
+
+    if (pparams.sharpening.threshold.isDouble() != pp.sharpening.threshold.isDouble())
+        printf("WARNING: Sharpening different!\n");
+    if (pparams.vibrance.psthreshold.isDouble() != pp.vibrance.psthreshold.isDouble())
+        printf("WARNING: Vibrance different!\n");
 
     if (pparams!=pp) 
         cfs.recentlySaved = false;
@@ -355,6 +391,8 @@ void Thumbnail::setProcParams (const ProcParams& pp, ParamsEdited* pe, int whoCh
     if (updateCacheNow)
         updateCache ();
 
+	} // end of mutex lock
+
     for (size_t i=0; i<listeners.size(); i++)
         listeners[i]->procParamsChanged (this, whoChangedIt);
 }
@@ -388,14 +426,14 @@ bool Thumbnail::isEnqueued () {
 
 void Thumbnail::increaseRef ()
 {
-	Glib::Mutex::Lock lock(mutex);
-   	++ref;
+    MyMutex::MyLock lock(mutex);
+    ++ref;
 }
 
 void Thumbnail::decreaseRef () 
 {
 	{
-		Glib::Mutex::Lock lock(mutex);
+		MyMutex::MyLock lock(mutex);
 		if ( ref == 0 )
 		{
 			return;
@@ -408,18 +446,36 @@ void Thumbnail::decreaseRef ()
 	cachemgr->closeThumbnail (this); 
 }
 
-void Thumbnail::getThumbnailSize (int &w, int &h) {
-	if (imgRatio > 0.)
-		w = (int)(imgRatio * (float)h);
+void Thumbnail::getThumbnailSize (int &w, int &h, const rtengine::procparams::ProcParams *pparams) {
+	int tw_ = tw;
+	int th_ = th;
+	float imgRatio_ = imgRatio;
+
+	if (pparams) {
+		int ppCoarse = pparams->coarse.rotate;
+		if (ppCoarse >= 180) ppCoarse -= 180;
+
+		int thisCoarse = this->pparams.coarse.rotate;
+		if (thisCoarse >= 180) thisCoarse -= 180;
+
+		if (thisCoarse != ppCoarse) {
+			// different orientation -> swapping width & height
+			int tmp = th_;
+			th_ = tw_;
+			tw_ = tmp;
+			if (imgRatio_ >= 0.0001f)
+				imgRatio_ = 1.f/imgRatio_;
+		}
+	}
+
+	if (imgRatio_ > 0.)
+		w = (int)(imgRatio_ * (float)h);
 	else
-		w = tw * h / th;
+		w = tw_ * h / th_;
 }
 
 void Thumbnail::getFinalSize (const rtengine::procparams::ProcParams& pparams, int& w, int& h) {
-    // TODO: Check for Linux
-    #ifdef WIN32
-    Glib::Mutex::Lock lock(mutex);
-    #endif
+    MyMutex::MyLock lock(mutex);
 
     // WARNING: When downscaled, the ratio have loosed a lot of precision, so we can't get back the exact initial dimensions
     double fw = lastW*lastScale;
@@ -442,40 +498,35 @@ void Thumbnail::getFinalSize (const rtengine::procparams::ProcParams& pparams, i
 
 rtengine::IImage8* Thumbnail::processThumbImage (const rtengine::procparams::ProcParams& pparams, int h, double& scale) {
 
-	Glib::Mutex::Lock lock(mutex);
+    MyMutex::MyLock lock(mutex);
 
-     if ( tpp == 0 )
- 	{
- 		_loadThumbnail();
- 		if ( tpp == 0 )
- 		{
- 			return 0;
- 		}
- 	}
- 
- 	rtengine::IImage8* image = 0;
+    if ( tpp == 0 ) {
+        _loadThumbnail();
+        if ( tpp == 0 )
+            return 0;
+    }
 
-	if ( cfs.thumbImgType == CacheImageData::QUICK_THUMBNAIL )
-	{
-		// RAW internal thumbnail, no profile yet: just do some rotation etc.
- 		image = tpp->quickProcessImage (pparams, h, rtengine::TI_Nearest, scale);
-	}
-	else
-	{
-		// Full thumbnail: apply profile
- 		image = tpp->processImage (pparams, h, rtengine::TI_Bilinear, cfs.camera, cfs.focalLen, cfs.focalLen35mm, cfs.focusDist, cfs.shutter, cfs.fnumber, cfs.iso, cfs.expcomp, scale );
-	}
+    rtengine::IImage8* image = 0;
+
+    if ( cfs.thumbImgType == CacheImageData::QUICK_THUMBNAIL ) {
+        // RAW internal thumbnail, no profile yet: just do some rotation etc.
+        image = tpp->quickProcessImage (pparams, h, rtengine::TI_Nearest, scale);
+    }
+    else {
+        // Full thumbnail: apply profile
+        image = tpp->processImage (pparams, h, rtengine::TI_Bilinear, cfs.getCamera(), cfs.focalLen, cfs.focalLen35mm, cfs.focusDist, cfs.shutter, cfs.fnumber, cfs.iso, cfs.expcomp, scale );
+    }
 
     tpp->getDimensions(lastW,lastH,lastScale);
 
- 	delete tpp;
- 	tpp = 0;
- 	return image;
+    delete tpp;
+    tpp = 0;
+    return image;
 }
 
 rtengine::IImage8* Thumbnail::upgradeThumbImage (const rtengine::procparams::ProcParams& pparams, int h, double& scale) {
 
-	Glib::Mutex::Lock lock(mutex);
+	MyMutex::MyLock lock(mutex);
 
 	if ( cfs.thumbImgType != CacheImageData::QUICK_THUMBNAIL )
 	{
@@ -488,7 +539,7 @@ rtengine::IImage8* Thumbnail::upgradeThumbImage (const rtengine::procparams::Pro
  		return 0;
  	}
  
- 	rtengine::IImage8* image = tpp->processImage (pparams, h, rtengine::TI_Bilinear, cfs.camera, cfs.focalLen, cfs.focalLen35mm, cfs.focusDist,cfs.shutter, cfs.fnumber, cfs.iso, cfs.expcomp,  scale );
+ 	rtengine::IImage8* image = tpp->processImage (pparams, h, rtengine::TI_Bilinear, cfs.getCamera(), cfs.focalLen, cfs.focalLen35mm, cfs.focusDist,cfs.shutter, cfs.fnumber, cfs.iso, cfs.expcomp,  scale );
     tpp->getDimensions(lastW,lastH,lastScale);
  
  	delete tpp;
@@ -504,7 +555,7 @@ void Thumbnail::generateExifDateTimeStrings () {
     if (!cfs.exifValid)
         return;
 
-    exifString = Glib::ustring::compose ("f/%1 %2s %3%4 %5mm", Glib::ustring(rtengine::ImageData::apertureToString(cfs.fnumber)), Glib::ustring(rtengine::ImageData::shutterToString(cfs.shutter)), M("QINFO_ISO"), cfs.iso, cfs.focalLen);
+    exifString = Glib::ustring::compose ("f/%1 %2s %3%4 %5mm", Glib::ustring(rtengine::ImageData::apertureToString(cfs.fnumber)), Glib::ustring(rtengine::ImageData::shutterToString(cfs.shutter)), M("QINFO_ISO"), cfs.iso, Glib::ustring::format(std::setw(3), std::fixed, std::setprecision(2), cfs.focalLen));
 
     if (options.fbShowExpComp && cfs.expcomp!="0.00" && cfs.expcomp!="") // don't show exposure compensation if it is 0.00EV;old cache iles do not have ExpComp, so value will not be displayed. 
     	exifString = Glib::ustring::compose ("%1 %2EV", exifString, cfs.expcomp); // append exposure compensation to exifString
@@ -548,6 +599,17 @@ const Glib::ustring& Thumbnail::getDateTimeString () {
     return dateTimeString;
 }
 
+void Thumbnail::getAutoWB (double& temp, double& green, double equal) {
+	if (cfs.redAWBMul != -1.0) {
+		rtengine::ColorTemp ct(cfs.redAWBMul, cfs.greenAWBMul, cfs.blueAWBMul, equal);
+		temp = ct.getTemp();
+		green = ct.getGreen();
+	}
+	else
+		temp = green = -1.0;
+}
+
+
 ThFileType Thumbnail::getType () {
 
     return (ThFileType) cfs.format;
@@ -578,8 +640,9 @@ int Thumbnail::infoFromImage (const Glib::ustring& fname, rtengine::RawMetaDataL
         cfs.sec      = idata->getDateTime().tm_sec;
         cfs.timeValid = true;
         cfs.exifValid = true;
-        cfs.lens     = idata->getLens();
-        cfs.camera   = idata->getCamera();
+        cfs.lens      = idata->getLens();
+        cfs.camMake   = idata->getMake();
+        cfs.camModel  = idata->getModel();
 
         if (idata->getOrientation()=="Rotate 90 CW") {
             deg = 90;
@@ -593,7 +656,8 @@ int Thumbnail::infoFromImage (const Glib::ustring& fname, rtengine::RawMetaDataL
     }
     else {
         cfs.lens     = "Unknown";
-        cfs.camera   = "Unknown";
+        cfs.camMake  = "Unknown";
+        cfs.camModel = "Unknown";
     }
     // get image filetype
     std::string::size_type idx;
@@ -605,6 +669,14 @@ int Thumbnail::infoFromImage (const Glib::ustring& fname, rtengine::RawMetaDataL
     return deg;
 }
 
+/*
+ * Read all thumbnail's data from the cache; build and save them if doesn't exist - NON PROTECTED
+ * This includes:
+ *  - image's bitmap (*.rtti)
+ *  - auto exposure's histogram (full thumbnail only)
+ *  - embedded profile (full thumbnail only)
+ *  - LiveThumbData section of the data file
+ */
 void Thumbnail::_loadThumbnail(bool firstTrial) {
 
     needsReProcessing = true;
@@ -616,7 +688,10 @@ void Thumbnail::_loadThumbnail(bool firstTrial) {
 
     // load supplementary data
     bool succ = tpp->readData (getCacheFileName ("data")+".txt");
-    
+
+    if (succ)
+        tpp->getAutoWBMultipliers(cfs.redAWBMul, cfs.greenAWBMul, cfs.blueAWBMul);
+
     // thumbnail image
     succ = succ && tpp->readImage (getCacheFileName ("images"));
 
@@ -643,14 +718,30 @@ void Thumbnail::_loadThumbnail(bool firstTrial) {
         tpp->init ();
     }
  
-    if (!initial_ && tpp) tw = tpp->getImageWidth (getProcParams(), th, imgRatio);  // this might return 0 if image was just building
+    if (!initial_ && tpp) tw = tpp->getImageWidth (getProcParamsU(), th, imgRatio);  // this might return 0 if image was just building
 }
 
+/*
+ * Read all thumbnail's data from the cache; build and save them if doesn't exist - MUTEX PROTECTED
+ * This includes:
+ *  - image's bitmap (*.rtti)
+ *  - auto exposure's histogram (full thumbnail only)
+ *  - embedded profile (full thumbnail only)
+ *  - LiveThumbData section of the data file
+ */
 void Thumbnail::loadThumbnail (bool firstTrial) {
-    Glib::Mutex::Lock lock(mutex);
+    MyMutex::MyLock lock(mutex);
     _loadThumbnail(firstTrial);
 }
 
+/*
+ * Save thumbnail's data to the cache - NON PROTECTED
+ * This includes:
+ *  - image's bitmap (*.rtti)
+ *  - auto exposure's histogram (full thumbnail only)
+ *  - embedded profile (full thumbnail only)
+ *  - LiveThumbData section of the data file
+ */
 void Thumbnail::_saveThumbnail () {
 
     if (!tpp)
@@ -676,18 +767,33 @@ void Thumbnail::_saveThumbnail () {
     tpp->writeData (getCacheFileName ("data")+".txt");
 }
 
+/*
+ * Save thumbnail's data to the cache - MUTEX PROTECTED
+ * This includes:
+ *  - image's bitmap (*.rtti)
+ *  - auto exposure's histogram (full thumbnail only)
+ *  - embedded profile (full thumbnail only)
+ *  - LiveThumbData section of the data file
+ */
 void Thumbnail::saveThumbnail () 
 {
-   	Glib::Mutex::Lock lock(mutex);
+   	MyMutex::MyLock lock(mutex);
     _saveThumbnail();
 }
 
+/*
+ * Update the cached files
+ *  - updatePParams==true (default)        : write the procparams file (sidecar or cache, depending on the options)
+ *  - updateCacheImageData==true (default) : write the CacheImageData values in the cache folder,
+ *                                           i.e. some General, DateTime, ExifInfo, File info and ExtraRawInfo,
+ */
 void Thumbnail::updateCache (bool updatePParams, bool updateCacheImageData) {
 
     if (updatePParams && pparamsValid) {
         pparams.save (
+            options.saveParamsFile  ? fname + paramFileExtension : "",
             options.saveParamsCache ? getCacheFileName ("profiles")+paramFileExtension : "",
-            options.saveParamsFile  ? fname + paramFileExtension : ""
+            true
         );
     }
     if (updateCacheImageData)
@@ -695,13 +801,11 @@ void Thumbnail::updateCache (bool updatePParams, bool updateCacheImageData) {
 }
 
 Thumbnail::~Thumbnail () {
-    // TODO: Check for Linux
-    #ifdef WIN32
-    Glib::Mutex::Lock lock(mutex);
-    #endif
+    mutex.lock();
 
     delete [] lastImg;
     delete tpp;
+    mutex.unlock();
 }
 
 Glib::ustring Thumbnail::getCacheFileName (Glib::ustring subdir) {
@@ -795,7 +899,7 @@ bool Thumbnail::openDefaultViewer(int destination) {
 
 bool Thumbnail::imageLoad(bool loading)
 {
-    Glib::Mutex::Lock lock(mutex);
+    MyMutex::MyLock lock(mutex);
     bool previous = imageLoading;
     if( loading && !previous ){
         imageLoading = true;
