@@ -27,11 +27,12 @@
 #include "../rtengine/rtthumbnail.h"
 #include "cacheimagedata.h"
 #include "thumbnaillistener.h"
+#include "threadutils.h"
 
 class CacheManager;
 class Thumbnail {
 
-        Glib::Mutex    mutex;
+        MyMutex         mutex;
 
         Glib::ustring   fname;              // file name corresponding to the thumbnail
         CacheImageData  cfs;                // cache entry corresponding to the thumbnail
@@ -82,9 +83,10 @@ class Thumbnail {
         
         bool              hasProcParams ();
         const rtengine::procparams::ProcParams& getProcParams ();
+        const rtengine::procparams::ProcParams& getProcParamsU ();  // Unprotected version
 
-        // Use this to create params on demand for update
-        rtengine::procparams::ProcParams* createProcParamsForUpdate (bool returnParams, bool forceCPB);
+        // Use this to create params on demand for update ; if flaggingMode=true, the procparams is created for a file being flagged (inTrash, rank, colorLabel)
+        rtengine::procparams::ProcParams* createProcParamsForUpdate (bool returnParams, bool forceCPB, bool flaggingMode=false);
 
         void              setProcParams (const rtengine::procparams::ProcParams& pp, ParamsEdited* pe=NULL, int whoChangedIt=-1, bool updateCacheNow=true);
         void              clearProcParams (int whoClearedIt=-1);
@@ -103,13 +105,13 @@ class Thumbnail {
 //        unsigned char*  getThumbnailImage (int &w, int &h, int fixwh=1); // fixwh = 0: fix w and calculate h, =1: fix h and calculate w
         rtengine::IImage8* processThumbImage    (const rtengine::procparams::ProcParams& pparams, int h, double& scale);
         rtengine::IImage8* upgradeThumbImage    (const rtengine::procparams::ProcParams& pparams, int h, double& scale);
-        void            getThumbnailSize        (int &w, int &h);
+        void            getThumbnailSize        (int &w, int &h, const rtengine::procparams::ProcParams *pparams=NULL);
         void            getFinalSize            (const rtengine::procparams::ProcParams& pparams, int& w, int& h);
 
         const Glib::ustring&  getExifString ();
         const Glib::ustring&  getDateTimeString ();
         void                  getCamWB  (double& temp, double& green) { if (tpp) tpp->getCamWB  (temp, green); else temp = green = -1.0; }
-        void                  getAutoWB (double& temp, double& green) { if (tpp) tpp->getAutoWB (temp, green); else temp = green = -1.0; }
+        void                  getAutoWB (double& temp, double& green, double equal);
         void                  getSpotWB (int x, int y, int rect, double& temp, double& green) { if (tpp) tpp->getSpotWB (getProcParams(), x, y, rect, temp, green); else temp = green = -1.0; }
         void                  applyAutoExp (rtengine::procparams::ProcParams& pparams) { if (tpp) tpp->applyAutoExp (pparams); }
         
