@@ -28,6 +28,7 @@ using namespace rtengine::procparams;
 ToneCurve::ToneCurve () : Gtk::VBox(), FoldableToolPanel(this) {
 
   set_border_width(4);
+  set_spacing(4);
 
   CurveListener::setMulti(true);
 
@@ -44,10 +45,10 @@ ToneCurve::ToneCurve () : Gtk::VBox(), FoldableToolPanel(this) {
   autoconn = autolevels->signal_toggled().connect( sigc::mem_fun(*this, &ToneCurve::autolevels_toggled) );
 
   sclip = Gtk::manage (new MySpinButton ());
-  sclip->set_range (0.0, 0.9999);
-  sclip->set_increments (0.001, 0.01);
-  sclip->set_value (0.002);
-  sclip->set_digits (4);
+  sclip->set_range (0.0, 0.99);
+  sclip->set_increments (0.01, 0.01);
+  sclip->set_value (0.02);
+  sclip->set_digits (2);
   sclip->set_tooltip_text (M("TP_EXPOSURE_CLIP_TIP"));
   sclip->signal_value_changed().connect( sigc::mem_fun(*this, &ToneCurve::clip_changed) );
 
@@ -64,17 +65,43 @@ ToneCurve::ToneCurve () : Gtk::VBox(), FoldableToolPanel(this) {
   abox->pack_end (*Gtk::manage (new Gtk::Label (M("TP_EXPOSURE_CLIP"))));
   pack_start (*abox);
 
+//-------------- Highlight Reconstruction -----------------
   pack_start (*Gtk::manage (new  Gtk::HSeparator()));
 
-//----------- Exposure Compensation ------------------------
-  expcomp   = Gtk::manage (new Adjuster (M("TP_EXPOSURE_EXPCOMP"), -5, 10, 0.05, 0));
+  hrenabled = Gtk::manage (new Gtk::CheckButton (M("TP_HLREC_LABEL")));
+  hrenabled->set_active (false);
+  hrenabled->set_tooltip_markup (M("TP_HLREC_ENA_TOOLTIP"));
+  pack_start (*hrenabled);
+
+  method = Gtk::manage (new MyComboBoxText ());
+  method->append_text (M("TP_HLREC_LUMINANCE"));
+  method->append_text (M("TP_HLREC_CIELAB"));
+  method->append_text (M("TP_HLREC_COLOR"));
+  method->append_text (M("TP_HLREC_BLEND"));
+
+  method->set_active (0);
+  hlrbox = Gtk::manage (new Gtk::HBox ());
+  Gtk::Label* lab = Gtk::manage (new Gtk::Label (M("TP_HLREC_METHOD")));
+  hlrbox->pack_start (*lab, Gtk::PACK_SHRINK, 4);
+  hlrbox->pack_start (*method);
+  pack_start (*hlrbox);
+
+  enaconn  = hrenabled->signal_toggled().connect( sigc::mem_fun(*this, &ToneCurve::hrenabledChanged) );
+  methconn = method->signal_changed().connect ( sigc::mem_fun(*this, &ToneCurve::methodChanged) );
+
+  //----------- Exposure Compensation ---------------------
+  pack_start (*Gtk::manage (new  Gtk::HSeparator()));
+
+  expcomp   = Gtk::manage (new Adjuster (M("TP_EXPOSURE_EXPCOMP"), -5, 12, 0.05, 0));
   pack_start (*expcomp);
-  hlcompr = Gtk::manage (new Adjuster (M("TP_EXPOSURE_COMPRHIGHLIGHTS"), 0, 500, 1, 70));
+
+  //----------- Highlight recovery & threshold -------------
+  hlcompr = Gtk::manage (new Adjuster (M("TP_EXPOSURE_COMPRHIGHLIGHTS"), 0, 500, 1, 0));
   pack_start (*hlcompr);
-  hlcomprthresh = Gtk::manage (new Adjuster (M("TP_EXPOSURE_COMPRHIGHLIGHTSTHRESHOLD"), 0, 100, 1, 0));
+  hlcomprthresh = Gtk::manage (new Adjuster (M("TP_EXPOSURE_COMPRHIGHLIGHTSTHRESHOLD"), 0, 100, 1, 33));
   pack_start (*hlcomprthresh);
 
-//----------- Black Level ----------------------------------
+//----------- Black Level & Compression -------------------
   black = Gtk::manage (new Adjuster (M("TP_EXPOSURE_BLACKLEVEL"), -16384, 32768, 50, 0));
   pack_start (*black);
   shcompr = Gtk::manage (new Adjuster (M("TP_EXPOSURE_COMPRSHADOWS"), 0, 100, 1, 50));
@@ -89,7 +116,7 @@ ToneCurve::ToneCurve () : Gtk::VBox(), FoldableToolPanel(this) {
   pack_start (*contrast);
   saturation = Gtk::manage (new Adjuster (M("TP_EXPOSURE_SATURATION"), -100, 100, 1, 0));
   pack_start (*saturation);
-  
+
 //----------- Curve 1 ------------------------------
   pack_start (*Gtk::manage (new  Gtk::HSeparator()));
 
@@ -203,6 +230,30 @@ void ToneCurve::read (const ProcParams* pp, const ParamsEdited* pedited) {
             toneCurveMode2->set_active(4);
         }
     }
+    if (pedited)
+        hrenabled->set_inconsistent (!pedited->toneCurve.hrenabled);
+    enaconn.block (true);
+    hrenabled->set_active  (pp->toneCurve.hrenabled);
+    enaconn.block (false);
+
+    if (pedited && !pedited->toneCurve.method) 
+        method->set_active (4);
+    else if (pp->toneCurve.method=="Luminance")
+        method->set_active (0);
+    else if (pp->toneCurve.method=="CIELab blending")
+        method->set_active (1);
+    else if (pp->toneCurve.method=="Color")
+        method->set_active (2);
+    else if (pp->toneCurve.method=="Blend")
+        method->set_active (3);
+
+    if (!batchMode) {
+        if (hrenabled->get_active())
+            hlrbox->show();
+        else
+            hlrbox->hide();
+    }
+    lasthrEnabled = pp->toneCurve.hrenabled;
 
     autoconn.block (false);
     tcmode2conn.block(false);
@@ -212,8 +263,8 @@ void ToneCurve::read (const ProcParams* pp, const ParamsEdited* pedited) {
 }
 
 void ToneCurve::autoOpenCurve  () {
-		shape->openIfNonlinear();
-		shape2->openIfNonlinear();
+    shape->openIfNonlinear();
+    shape2->openIfNonlinear();
 }
 
 void ToneCurve::write (ProcParams* pp, ParamsEdited* pedited) {
@@ -243,7 +294,6 @@ void ToneCurve::write (ProcParams* pp, ParamsEdited* pedited) {
     else if (tcMode == 2) pp->toneCurve.curveMode2 = ToneCurveParams::TC_MODE_FILMLIKE;
     else if (tcMode == 3) pp->toneCurve.curveMode2 = ToneCurveParams::TC_MODE_SATANDVALBLENDING;
 
-
     if (pedited) {
         pedited->toneCurve.expcomp    = expcomp->getEditedState ();
         pedited->toneCurve.black      = black->getEditedState ();
@@ -260,8 +310,74 @@ void ToneCurve::write (ProcParams* pp, ParamsEdited* pedited) {
         pedited->toneCurve.curveMode  = toneCurveMode->get_active_row_number() != 4;
         pedited->toneCurve.curveMode2 = toneCurveMode2->get_active_row_number() != 4;
     }
+    if (pedited) {
+        pedited->toneCurve.method     = method->get_active_row_number()!=4;
+        pedited->toneCurve.hrenabled  = !hrenabled->get_inconsistent();
+    }
+
+    pp->toneCurve.hrenabled = hrenabled->get_active();
+    if (method->get_active_row_number()==0)
+        pp->toneCurve.method = "Luminance";
+    else if (method->get_active_row_number()==1)
+        pp->toneCurve.method = "CIELab blending";
+    else if (method->get_active_row_number()==2)
+        pp->toneCurve.method = "Color";
+    else if (method->get_active_row_number()==3)
+        pp->toneCurve.method = "Blend";
 }
 
+void ToneCurve::hrenabledChanged () {
+
+    if (multiImage) {
+        if (hrenabled->get_inconsistent()) {
+            hrenabled->set_inconsistent (false);
+            enaconn.block (true);
+            hrenabled->set_active (false);
+            enaconn.block (false);
+        }
+        else if (lasthrEnabled)
+            hrenabled->set_inconsistent (true);
+
+        lasthrEnabled = hrenabled->get_active ();
+    }
+
+    if (!batchMode) {
+        if (hrenabled->get_active())
+            hlrbox->show();
+        else
+            hlrbox->hide();
+    }
+
+    if (listener) {
+        // Switch off auto exposure if user changes enabled manually
+        if (autolevels->get_active() ) {
+            autoconn.block(true);
+            autolevels->set_active (false);
+            autoconn.block(false);
+            autolevels->set_inconsistent (false);
+        }
+        if (hrenabled->get_active ())
+            listener->panelChanged (EvHREnabled, M("GENERAL_ENABLED"));
+        else
+            listener->panelChanged (EvHREnabled, M("GENERAL_DISABLED"));
+    }  
+}
+void ToneCurve::methodChanged () {
+
+    if (listener) {
+        if (hrenabled->get_active ())
+            listener->panelChanged (EvHRMethod, method->get_active_text ());
+    }
+}
+void ToneCurve::setRaw (bool raw) {
+
+    disableListener ();
+    method->set_sensitive (raw);
+    hrenabled->set_sensitive (raw);
+    enableListener ();
+}
+
+
 void ToneCurve::setDefaults (const ProcParams* defParams, const ParamsEdited* pedited) {
 
     expcomp->setDefault (defParams->toneCurve.expcomp);
@@ -343,7 +459,7 @@ void ToneCurve::adjusterChanged (Adjuster* a, double newval) {
         costr = Glib::ustring::format (std::setw(3), std::fixed, std::setprecision(2), a->getValue());
     else
         costr = Glib::ustring::format ((int)a->getValue());
-    
+
     if (a==expcomp) 
         listener->panelChanged (EvExpComp, costr);
     else if (a==brightness) 
@@ -354,7 +470,7 @@ void ToneCurve::adjusterChanged (Adjuster* a, double newval) {
     }
     else if (a==contrast)
         listener->panelChanged (EvContrast, costr);
-	else if (a==saturation)
+    else if (a==saturation)
         listener->panelChanged (EvSaturation, costr);
     else if (a==hlcompr)
         listener->panelChanged (EvHLCompr, costr);
@@ -365,7 +481,7 @@ void ToneCurve::adjusterChanged (Adjuster* a, double newval) {
 }
 
 void ToneCurve::neutral_pressed () {
-// This method deselects auto levels 
+// This method deselects auto levels and HL reconstruction auto
 // and sets neutral values to params in exposure panel
 
     if (batchMode) {
@@ -380,17 +496,22 @@ void ToneCurve::neutral_pressed () {
         autolevels->set_active (false);
         autolevels->set_inconsistent (false);
     }
-    
+
     expcomp->setValue(0);
     hlcompr->setValue(0);
     hlcomprthresh->setValue(0);
     brightness->setValue(0);
     black->setValue(0);
     shcompr->setValue(50);
+    enaconn.block (true);
+    hrenabled->set_active (false);
+    enaconn.block (false);
+    if (!batchMode)
+        hlrbox->hide();
     if (!black->getAddMode()) shcompr->set_sensitive(!((int)black->getValue ()==0)); //at black=0 shcompr value has no effect
     contrast->setValue(0);
     //saturation->setValue(0);
-    
+
     listener->panelChanged (EvNeutralExp, M("GENERAL_ENABLED"));
 }
 void ToneCurve::autolevels_toggled () {
@@ -425,6 +546,7 @@ void ToneCurve::autolevels_toggled () {
             hlcompr->setValue (0);
         if (hlcomprthresh->getAddMode())
             hlcomprthresh->setValue (0);
+
         if (listener) {
             if (!autolevels->get_inconsistent()) {
                 if (autolevels->get_active ())
@@ -444,7 +566,6 @@ void ToneCurve::autolevels_toggled () {
             listener->panelChanged (EvFixedExp, M("GENERAL_DISABLED"));
         }
     }
-
 }
 
 void ToneCurve::clip_changed () {
@@ -478,6 +599,8 @@ void ToneCurve::waitForAutoExp () {
     saturation->setEnabled (false);
     curveEditorG->set_sensitive (false);
     toneCurveMode->set_sensitive (false);
+    hrenabled->set_sensitive(false);
+    method->set_sensitive(false);
 }
 
 int autoExpChangedUI (void* data) {
@@ -485,7 +608,7 @@ int autoExpChangedUI (void* data) {
     return 0;
 }
 
-void ToneCurve::autoExpChanged (double expcomp, int bright, int contr, int black, int hlcompr, int hlcomprthresh) {
+void ToneCurve::autoExpChanged (double expcomp, int bright, int contr, int black, int hlcompr, int hlcomprthresh, bool hlrecons) {
 
     nextBlack = black;
     nextExpcomp = expcomp;
@@ -493,9 +616,8 @@ void ToneCurve::autoExpChanged (double expcomp, int bright, int contr, int black
     nextContrast = contr;
     nextHlcompr = hlcompr;
     nextHlcomprthresh = hlcomprthresh;
+    nextHLRecons = hlrecons;
     g_idle_add (autoExpChangedUI, this);
-
-//    Glib::signal_idle().connect (sigc::mem_fun(*this, &ToneCurve::autoExpComputed_));
 }
 
 void ToneCurve::enableAll () {
@@ -511,10 +633,13 @@ void ToneCurve::enableAll () {
     saturation->setEnabled (true);
     curveEditorG->set_sensitive (true);
     toneCurveMode->set_sensitive (true);
+    hrenabled->set_sensitive(true);
+    method->set_sensitive(true);
 }
 
 bool ToneCurve::autoExpComputed_ () {
 
+    GThreadLock lock; // All GUI acces from idle_add callbacks or separate thread HAVE to be protected
     disableListener ();
     enableAll ();
     expcomp->setValue (nextExpcomp);
@@ -523,6 +648,13 @@ bool ToneCurve::autoExpComputed_ () {
     black->setValue (nextBlack);
     hlcompr->setValue (nextHlcompr);
     hlcomprthresh->setValue (nextHlcomprthresh);
+    enaconn.block (true);
+    hrenabled->set_active (nextHLRecons);
+    enaconn.block (false);
+    if (nextHLRecons)
+        hlrbox->show();
+    else if (!batchMode)
+        hlrbox->hide();
     if (!black->getAddMode()) shcompr->set_sensitive(!((int)black->getValue ()==0)); //at black=0 shcompr value has no effect
     enableListener ();
 
@@ -530,6 +662,8 @@ bool ToneCurve::autoExpComputed_ () {
 }
 
 void ToneCurve::setBatchMode (bool batchMode) {
+    ToolPanel::setBatchMode (batchMode);
+    method->append_text (M("GENERAL_UNCHANGED"));
 
     removeIfThere (abox, autolevels, false);
     autolevels = Gtk::manage (new Gtk::CheckButton (M("TP_EXPOSURE_AUTOLEVELS")));
@@ -545,38 +679,38 @@ void ToneCurve::setBatchMode (bool batchMode) {
     shcompr->showEditedCB ();
     brightness->showEditedCB ();
     contrast->showEditedCB ();
-	saturation->showEditedCB ();
+    saturation->showEditedCB ();
 
-	toneCurveMode->append_text (M("GENERAL_UNCHANGED"));
+    toneCurveMode->append_text (M("GENERAL_UNCHANGED"));
 
     curveEditorG->setBatchMode (batchMode);
 }
 
 void ToneCurve::setAdjusterBehavior (bool expadd, bool hlcompadd, bool hlcompthreshadd, bool bradd, bool blackadd, bool shcompadd, bool contradd, bool satadd) {
 
-	expcomp->setAddMode(expadd);
-	hlcompr->setAddMode(hlcompadd);
-	hlcomprthresh->setAddMode(hlcompthreshadd);
-	brightness->setAddMode(bradd);
-	black->setAddMode(blackadd);
-	shcompr->setAddMode(shcompadd);
-	contrast->setAddMode(contradd);
-	saturation->setAddMode(satadd);
+    expcomp->setAddMode(expadd);
+    hlcompr->setAddMode(hlcompadd);
+    hlcomprthresh->setAddMode(hlcompthreshadd);
+    brightness->setAddMode(bradd);
+    black->setAddMode(blackadd);
+    shcompr->setAddMode(shcompadd);
+    contrast->setAddMode(contradd);
+    saturation->setAddMode(satadd);
 }
 
 void ToneCurve::trimValues (rtengine::procparams::ProcParams* pp) {
 
-	expcomp->trimValue(pp->toneCurve.expcomp);
-	hlcompr->trimValue(pp->toneCurve.hlcompr);
-	hlcomprthresh->trimValue(pp->toneCurve.hlcomprthresh);
-	brightness->trimValue(pp->toneCurve.brightness);
-	black->trimValue(pp->toneCurve.black);
-	shcompr->trimValue(pp->toneCurve.shcompr);
-	contrast->trimValue(pp->toneCurve.contrast);
-	saturation->trimValue(pp->toneCurve.saturation);
+    expcomp->trimValue(pp->toneCurve.expcomp);
+    hlcompr->trimValue(pp->toneCurve.hlcompr);
+    hlcomprthresh->trimValue(pp->toneCurve.hlcomprthresh);
+    brightness->trimValue(pp->toneCurve.brightness);
+    black->trimValue(pp->toneCurve.black);
+    shcompr->trimValue(pp->toneCurve.shcompr);
+    contrast->trimValue(pp->toneCurve.contrast);
+    saturation->trimValue(pp->toneCurve.saturation);
 }
 
-void ToneCurve::updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histLCAM, LUTu & histCCAM, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma) {
+void ToneCurve::updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve, LUTu & histCCurve, LUTu & histCLurve, LUTu & histLLCurve, LUTu & histLCAM, LUTu & histCCAM, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma) {
 
     shape->updateBackgroundHistogram (histToneCurve);
 }
