@@ -44,16 +44,19 @@
 #include "icmpanel.h"
 #include "resize.h"
 #include "chmixer.h"
-#include "hlrec.h"
+#include "blackwhite.h"
 #include "cacorrection.h"
 #include "lensprofile.h"
 #include "distortion.h"
 #include "perspective.h"
 #include "rotate.h"
 #include "vignetting.h"
+#include "gradient.h"
+#include "pcvignette.h"
 #include "toolbar.h"
 #include "lensgeom.h"
 #include "lensgeomlistener.h"
+#include "dirselectionlistener.h"
 #include "dirpyrequalizer.h"
 #include "hsvequalizer.h"
 #include "rawprocess.h"
@@ -78,22 +81,25 @@ class ToolPanelCoordinator :    public ToolPanelListener,
                                 public SpotWBListener,
                                 public CropPanelListener,
                                 public ICMPanelListener,
+                                public DirSelectionListener,
                                 public ImageAreaToolListener {
 
     protected:
 
         WhiteBalance* whitebalance;
         Vignetting* vignetting;
+        Gradient* gradient;
+        PCVignette* pcvignette;
         LensGeometry* lensgeom;
         LensProfilePanel* lensProf;
         Rotate* rotate;
         Distortion* distortion;
         PerspCorrection* perspective;
         CACorrection* cacorrection;
-        HLRecovery* hlrecovery;
         ColorAppearance* colorappearance;
         Vibrance* vibrance;
         ChMixer* chmixer;
+        BlackWhite* blackwhite;
         Resize* resize;
         ICMPanel* icm;
         Crop* crop;
@@ -165,7 +171,7 @@ class ToolPanelCoordinator :    public ToolPanelListener,
         
         bool hasChanged;
 
-        void addPanel (Gtk::Box* where, FoldableToolPanel* panel, Glib::ustring label);
+        void addPanel (Gtk::Box* where, FoldableToolPanel* panel, Glib::ustring label, bool need100Percent=false);
         void foldThemAll (GdkEventButton* event);
         void updateVScrollbars (bool hide);
         void updateTabsHeader (bool useIcons);
@@ -179,7 +185,7 @@ class ToolPanelCoordinator :    public ToolPanelListener,
         virtual ~ToolPanelCoordinator ();
 
         bool getChangedState                ()                                      { return hasChanged; }
-		void updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve,LUTu & histCCurve, LUTu & histLCAM,  LUTu & histCCAM, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma);
+        void updateCurveBackgroundHistogram (LUTu & histToneCurve, LUTu & histLCurve,LUTu & histCCurve, LUTu & histCLurve, LUTu & histLLCurve, LUTu & histLCAM,  LUTu & histCCAM, LUTu & histRed, LUTu & histGreen, LUTu & histBlue, LUTu & histLuma);
         void foldAllButOne (Gtk::Box* parent, FoldableToolPanel* openedSection);
 
         // multiple listeners can be added that are notified on changes (typical: profile panel and the history)
@@ -192,19 +198,25 @@ class ToolPanelCoordinator :    public ToolPanelListener,
         void profileChange  (const rtengine::procparams::PartialProfile* nparams, rtengine::ProcEvent event, const Glib::ustring& descr, const ParamsEdited* paramsEdited=NULL);
         void setDefaults    (rtengine::procparams::ProcParams* defparams);
 
+        // DirSelectionListener interface
+        void dirSelected (const Glib::ustring& dirname, const Glib::ustring& openfile="");
+
         // to support the GUI:
         CropGUIListener* getCropGUIListener (); // through the CropGUIListener the editor area can notify the "crop" ToolPanel when the crop selection changes
         
-        // init the toolpanelcoordinator with an image & close it        
+        // init the toolpanelcoordinator with an image & close it
         void initImage          (rtengine::StagedImageProcessor* ipc_, bool israw);
         void closeImage         ();
 
         // read/write the "expanded" state of the expanders & read/write the crop panel settings (ratio, guide type, etc.)
         void readOptions        ();
-        void writeOptions       ();       
+        void writeOptions       ();
 
         // wbprovider interface
-        void getAutoWB (double& temp, double& green) { if (ipc) ipc->getAutoWB (temp, green); }
+        void getAutoWB (double& temp, double& green, double equal) {
+            if (ipc)
+                ipc->getAutoWB (temp, green, equal);
+        }
         void getCamWB (double& temp, double& green)  { if (ipc) ipc->getCamWB (temp, green); }
 
         //DFProvider interface
